#include <click/config.h>
#include "icingcommon.hh"
#include "PocomonGatewayProcessor.hh"
#include "json/json.h"
#include <string>
#include <iostream>
#include <fstream>

CLICK_DECLS


PocomonGatewayProcessor::PocomonGatewayProcessor() 
{
}

PocomonGatewayProcessor::~PocomonGatewayProcessor()
{
}

int PocomonGatewayProcessor::configure(Vector<String> &conf, ErrorHandler *errh) {
  IPAddress mask;
  String config_hostname;
  String config_server;
  String config_configFile;

  if (cp_va_kparse(conf, this, errh,
		   "NEXTHOP", cpkP+cpkM, cpIPAddressOrPrefix, &_nextHop, &mask, //IP Addr of next hop
		   "HOSTNAME", cpkP+cpkM, cpString, &config_hostname, //Hostname to lookup via pathserver
		   "SERVER", cpkP+cpkM, cpString, &config_server, // Pathserver addr
		   "PORT", cpkP+cpkM, cpUDPPort, &port, // Pathserver port
		   "CONFIGFILE", cpkP+cpkM, cpString, &config_configFile, // JSON config file to use
       "USE_MIN_PATH", cpkP+cpkM, cpBool, &_use_min_path,	     
                        cpEnd) < 0)
           return -1;

  _default_hostname = config_hostname.c_str();
  server = config_server.c_str();

  // Now, connect to pathserver
  _connected_to_pathserver = false;
  _errh = router()->chatter_channel("default");
  config(config_configFile.c_str());

  tcpconnect(str(server.c_str()), port, wrap(this, &PocomonGatewayProcessor::connected));
  while(!_connected_to_pathserver){
    acheck();
  }

         return 0;
}

void
PocomonGatewayProcessor::drop_it(Packet* p)
{
    if (noutputs() > 1)
	output(1).push(p);
    else
	p->kill();
}

Packet* 
PocomonGatewayProcessor::simple_action(Packet* p_in)
{
  _have_pocs_for_packet = false;
  
  IPAddress dst_ip = p_in->dst_ip_anno();
  
  // XXX: MM: This destination prefix mapping is hardcoded; just working to get something done before deadline.
  IPAddress testMask("128.0.0.0");
  if(dst_ip.mask_as_specific(testMask)){
    //If we match the test mask, forward to austin
    hostname =_default_hostname;
  }
  else { 
    //    warn << "Unkown dest IP prefix\n";
    return NULL;
  }
  data = (char *)p_in->data(); //The data we need to encap in an icing packet
  dataLen = p_in->length(); // length of data to encap
  
  
  make_call(); // to get paths from pathserver
  
  //Keep executing event loop until we get back the path
  while(!_have_pocs_for_packet){
    acheck();
  }
  
  //At this point, the encapsulated ICING packet is stored at outputData, and the size is outputLen
  WritablePacket *p = Packet::make(outputData, outputLen);
  
  // Now, we need to set the destination annotations to direct the UDP packet
  // XXX: MM: The below is hardcoded for the deadline. Should have some sort of table lookup in the future

  //IPAddress nextHop("184.72.26.204"); //Assume that gateway is always forwarding to icing1
  p->set_dst_ip_anno(_nextHop);
  set_port_anno(p, htons(7662));
  
  // Delete the original packet, since we've replaced it with our own
  p_in->kill();


  delete outputData;
  
  return p;
}


// Crap below this line taken from i2tx. Done for USEC11 deadline; needs to be refactored.

Path *
PocomonGatewayProcessor::convert_to_path(icp_path &path)
{
	RealmVoq *realms = New RealmVoq[path.waypoints.size()];

	for(uint32_t i = 0; i < path.waypoints.size(); i++)
	{
		memcpy(realms[i].realmid.bytes, path.waypoints[i].pubkey.base(), RealmIDWire::SIZE);
		realms[i].voq = htonl(path.waypoints[i].vq);
	}
	Path *wirepath = reinterpret_cast<Path *>(realms);
	return wirepath;
}

POC *
PocomonGatewayProcessor::get_pocs(icp_path &path)
{
	uint32_t poclen = path.waypoints.size() -1;
	POC *pocs = New POC[poclen];
	bzero(pocs, sizeof(POC) * poclen);

	for(uint32_t i = 0; i < poclen; i++)
		memcpy(pocs[i], path.pocs[i].base(), sizeof(POC));
	return pocs;
}


uint32_t
PocomonGatewayProcessor::build_packet(icp_path &path, uint8_t *payload, uint32_t payloadSize, uint8_t **outPkt)
{
	uint32_t pathlen = path.waypoints.size();
	//	uint32_t hdrlen = icing_hdrlen(pathlen);

	uint32_t requiredSize = payloadSize + icing_hdrlen(path.pocs.size());
	*outPkt = new uint8_t[requiredSize];

	IcingHdr1 *hdr1 = reinterpret_cast<IcingHdr1 *>(*outPkt);
	hdr1->icing_ver = ICING_CURRENT_VERSION;
	hdr1->path_len = static_cast<uint8_t>(pathlen);
	hdr1->path_idx = 0;
	for (uint32_t i = 0; i < IV_SIZE; i++)
		hdr1->init_vec[i] = (i+1) & 0xff;

	IcingHdr2 *hdr2 = get_icinghdr2(hdr1);
	hdr2->proto = 8;
	hdr2->pkt_len = htons(requiredSize);

	Path *wirepath = get_path(hdr1);
	Path *temppath = convert_to_path(path);
	memcpy(wirepath, temppath, pathlen * sizeof(RealmVoq));
	delete[] temppath;

	/*
	uint8_t *currentPos = *outPkt + icing_hdrlen(path.pocs.size());
	for(uint32_t i = pathlen - 1; i < path.pocs.size(); i++)
	{
		memcpy(currentPos, path.pocs[i].base(), sizeof(POC));
		currentPos += sizeof(POC);
	}
	*/


	
	memcpy(get_payload(hdr1), payload, payloadSize);
	POC *pocs = get_pocs(path);
	init_vtags(hdr1, pocs, _privk);
	delete[] pocs;
		hdr1->path_idx = 1;
  return requiredSize;
}



void
PocomonGatewayProcessor::handle_result(ptr<ps_reply> reply, uint64_t reqt)
{
	switch(reply->status)
	{
		case PS_PATH_FOUND:
		{
			icp_path_vec &paths = *reply->paths;

#ifdef DEBUG
			warn << "Paths(s) returned\n";
#endif
			if(paths.size() == 0){
			  warn << "No paths recieved!\n";
			  break;
			}
			icp_path &path = paths[0];
if(_use_min_path){
			unsigned min_length = paths[0].waypoints.size();
			for(unsigned i=0; i<paths.size(); i++){
			  if(paths[i].waypoints.size() < min_length){
			    path = paths[i];
			    min_length = paths[i].waypoints.size();
			  }
			}
}
else{
			// Maximum path length
			unsigned max_length = 0;
			for(unsigned i=0; i<paths.size(); i++){
			  if(paths[i].waypoints.size() > max_length){
			    path = paths[i];
			    max_length = paths[i].waypoints.size();
			  }
			}
}
			//			warn << "Number of paths: " << paths.size() << "\n";
			outputLen = build_packet(path, (uint8_t *)data, dataLen, (uint8_t **)&outputData);
      _have_pocs_for_packet = true;
      
			break;
		}
		case PS_SERVER_FOUND:
		{
			ps_nextps &res = *reply->nextps;

			warn << "Pathserver: " << res.host << ":\n\n";

			//print_paths(res.paths, &keydb);
			break;
		}
		case PS_UNKNOWN_HOST:
			warn << "Unknown host\n";
			break;
		case PS_INVALID_ARG:
			warn << "Invalid arguments\n";
			break;
		case PS_NO_PATH:
			warn << "No path\n";
			break;
		case PS_UNKNOWN_DOMAIN:
			warn << "Unknown domain\n";
			break;
		default:
			warn << "Unknown error\n";
	}
}
void
PocomonGatewayProcessor::get_reply(ptr<ps_reply> reply, uint64_t reqt, clnt_stat err)
{
	switch(err)
	{
		case RPC_SUCCESS:
			handle_result(reply, reqt);
			break;
		default:
			fatal << "ERROR\n";
	}
}

void
PocomonGatewayProcessor::make_call()
{
	ptr<ps_reply> reply = New refcounted<ps_reply>();

	ps_getargs args;
	args.host = str(hostname.c_str());
	args.partialpaths = partialpaths;

	clnt->call(
			FIND_PATHTO_HOST,
			&args,
			reply,
			wrap(this, &PocomonGatewayProcessor::get_reply, reply, 0));

}

void
PocomonGatewayProcessor::connected(int32_t fd)
{
	if(fd < 0)
		fatal("Could not open socket.\nerrno: %d - %m\n", errno);
	clnt = init_tcp_client(fd, icing_pathserver_1);
  _connected_to_pathserver = true;
}
void
PocomonGatewayProcessor::add_waypoint(Json::Value& waypoint, icp_path& path){
  std::string pubkey = waypoint["pubkey"].asString();
  ptr<keydb_rec> *rec = keydb[str(pubkey.c_str())];
  if(!rec)
    fatal("No record with public key %s\n",pubkey.c_str());

  icp_realm realm;
  realm.pubkey = (*rec)->key;

  realm.vq = (uint32_t)waypoint["tag"].asInt();

  path.waypoints.push_back(realm);
}

void PocomonGatewayProcessor::add_cert(Json::Value& certData, icp_path& path){
  ptr<keydb_rec> *rec;

  icp_certificate cert;
  
  std::string current_realm_id = certData["current_realm_id"].asString();
  uint32_t current_realm_tag = certData["current_realm_tag"].asInt();
  rec = keydb[str(current_realm_id.c_str())];
  if(!rec)
    fatal("No record fouund for realmid %s\n",current_realm_id.c_str());

  cert.realm.pubkey = (*rec)->key;
  cert.realm.vq = htonl(current_realm_tag);

  int32_t keysize = cert.realm.pubkey.size();

  bool prev_is_wildcard = certData["prev_is_wildcard"].asBool();
  bool next_is_wildcard = certData["next_is_wildcard"].asBool();
  
  std::string prev_realm_id;
  uint32_t prev_realm_tag = 0;
  if(!prev_is_wildcard){
    prev_realm_id = certData["prev_realm_id"].asString();
    prev_realm_tag = certData["prev_realm_tag"].asInt();
    rec = keydb[str(prev_realm_id.c_str())];
    if(!rec)
      fatal("No record found in add_cert\n");

    cert.prev.pubkey = (*rec)->key;
    cert.prev.vq = htonl(prev_realm_tag);
  }
  else { 
    // XXX: MM: Need to handle case where realmid is all ones;
    // this is not a wildcard, but "use class/vnode" pattern
    // See icp_cert_includes
    memset(cert.prev.pubkey.base(), 0, keysize);
    cert.prev.vq = htonl(4062);
  }
  std::string next_realm_id;
  uint32_t next_realm_tag = 0;
  if(!next_is_wildcard){
    next_realm_id = certData["next_realm_id"].asString();
    next_realm_tag = certData["next_realm_tag"].asInt();
    rec = keydb[str(next_realm_id.c_str())];
    if(!rec)
      fatal("No record found in add_cert\n");

    cert.next.pubkey = (*rec)->key;
    cert.next.vq = htonl(next_realm_tag);;
  }
  else {
    // XXX: MM: Same thing about handling case of all ones as above
    memset(cert.next.pubkey.base(), 0, keysize);
    cert.next.vq = htonl(4062);
  }

  cert.metric = 4062;

  cert.creation = time(NULL);
  cert.expiry = cert.creation + 120;

  path.certs.push_back(cert);
}
void
PocomonGatewayProcessor::add_path(Json::Value& pathData, ps_partialpath &partial)
{
  icp_path path;
  
  Json::Value waypoints = pathData["waypoints"];
  for(unsigned i=0; i<waypoints.size(); i++){
    add_waypoint(waypoints[i], path);
  }

  Json::Value certs = pathData["certs"];
  for(unsigned i=0; i<certs.size(); i++){
    add_cert(certs[i], path);
  }

  if(!pathData["memcerts"].isNull())
    fatal("Memcert support not implemented?\n"); // XXX: MM: Investivate
  partial.paths.push_back(path);
}

void
PocomonGatewayProcessor::add_partial(Json::Value& partialData)
{
  std::string end;
	ps_partialpath partial;

	//Ignoring start
	end = partialData["end"].asString();	
	add_path(partialData["path"], partial);      

	ptr<keydb_rec> *rec;
	rec = (keydb)[str(end.c_str())];
	if(!rec)
		return;

	//ptr<rabin_priv> priv = (*rec)->priv;
	partial.realm = (*rec)->key; //priv->n.getraw();
	partialpaths.push_back(partial);
}
void PocomonGatewayProcessor::add_key(Json::Value& keyArray){
  std::string id, pr, aes;
  id = keyArray["id"].asString();
  pr = keyArray["pr"].asString();
  aes = keyArray["aes"].asString();

  ptr<keydb_rec> rec = New refcounted<keydb_rec>();
  
  rec->id = str(id.c_str());

  char hex[3];
  hex[2] = '\0';
  for(uint32_t i = 0; i < id.length()/2; i++){
    memcpy(hex, id.c_str() + (i*2), 2);
    long byte = strtol(hex, NULL, 16);
    rec->key[i] = byte & 0xFF;
  }

  for(uint32_t i = 0; i < aes.length()/2; i++){
    memcpy(hex, aes.c_str() + (i*2), 2);
    long byte = strtol(hex, NULL, 16);
    rec->k[i] = byte & 0xFF;
  }

  rpc_bytes<> b;
  for(uint32_t i = 0; i < pr.length()/2; i++){
    memcpy(hex, pr.c_str() + (i*2), 2);
    long byte = strtol(hex, NULL, 16);
    b.push_back(byte & 0xFF);
  }

  str k(b.base(), b.size());
  bigint x;
  x.setraw(k);
  ecdh_priv privk(x);

  rec->privk = privk;

  keydb.insert(rec->id, rec);
  str rawkey(rec->key.base(), rec->key.size());
  keydb.insert(rawkey, rec);
}

void
PocomonGatewayProcessor::add_privk(std::string& key){
  char hex[3];
  hex[2] = '\0';
  rpc_bytes<> b;
  for(uint32_t i=0; i<key.length()/2; i++){
    memcpy(hex, key.c_str() + (i*2), 2);
    long byte = strtol(hex, NULL, 16);
    b.push_back(byte&0xFF);
  }

  str k(b.base(), b.size());
  bigint x;
  x.setraw(k);
  ecdh_priv privk(x);
  _privk = privk;
}
void
PocomonGatewayProcessor::config(std::string filename)
{
  Json::Value root;
  Json::Reader reader;

  std::ifstream file(filename.c_str());

  if(!reader.parse(file, root)){
    std::cout << reader.getFormatedErrorMessages();
    fatal("Could not parse config file!\n");
    return;
  }
  file.close();

  // hostname = root.get("hostname",NULL).asString(); - hostname not used for gateway
  
  // server =  root["server"].asString(); - server is now passed in as click arg
  // port = root["port"].asInt(); - port is now passed in as click arg
  Json::Value keys = root["keys"];
  for(unsigned i=0; i<keys.size(); i++){
    add_key(keys[i]);
  }
  payload_len = root["payload_len"].asInt();
  delay = root["delay"].asInt();
  times_to_send = root["times_to_send"].asInt();

  std::string i2_addr;
  i2_addr = root["i2_addr"].asString();
  _i2_port = root["i2_port"].asInt();


  std::string privk = root["privk"].asString();
  add_privk(privk);

  Json::Value partials = root["partials"];
  for(unsigned i=0; i<partials.size(); i++){
    add_partial(partials[i]);
  }

  _addr.sin_family = AF_INET;
  _addr.sin_port = htons(_i2_port);
  _addr.sin_addr.s_addr = inet_addr(i2_addr.c_str());
  bzero(_addr.sin_zero, sizeof(_addr.sin_zero));
  
}

CLICK_ENDDECLS
ELEMENT_REQUIRES(icing)
EXPORT_ELEMENT(PocomonGatewayProcessor)
