#ifndef POCOMONGATEWAYPROCESSOR_HH_
#define POCOMONGATEWAYPROCESSOR_HH_

#include <click/element.hh>
#include <json/json.h>
#include <cstring>
#include <string>
#include <cstdio>
extern "C"
{
#include <arpa/inet.h>
}
#include "util.h"
#include "parseopt.h"
#include "icp_config.h"
#include "icinghdr.h"
#include "icingsec.h"

//#include "tt.h"

CLICK_DECLS

// What this processor does:
// Input: Link layer packet
// Output: IP-in-ICING-in-IP packet
// 
// - Strips the ethernet frame to get an IP packet
// - Figures out the dest IP from the IP packet
// - Maps the dest IP -> dest realm
// - Maps dest realm -> path to dest realm(including PoCs)
// - Encapsulates IP packet in a UDP packet to the next realm.
class PocomonGatewayProcessor : public Element { public:
  
    PocomonGatewayProcessor();
    ~PocomonGatewayProcessor();		

    const char *port_count() const      { return PORTS_1_1X2;   }
    const char *class_name() const	{ return "PocomonGatewayProcessor"; }
    const char *processing() const	{ return PROCESSING_A_AH; }

    Packet* simple_action(Packet *);
    int configure(Vector<String> &conf, ErrorHandler *errh);
private:

    void drop_it(Packet*);

    ErrorHandler* _errh;
    
    IPAddress _nextHop;
    
    bool _have_pocs_for_packet; //Used so we know when we're done processing the packet
    
    //Input packet, less ethernet headers
    char *data;
    uint32_t dataLen;
    
    //IP-in-ICING packet
    char *outputData;
    uint32_t outputLen;
    

    // Used to help single step using acheck()
    bool _connected_to_pathserver;
    
    std::string _default_hostname;
    
    bool _use_min_path; //else use max
//Stuff below this line ripped out from i2tx. Not clean, but working towards a deadline.
  std::string server;

  std::string  hostname;
  uint16_t port, _i2_port;
  keydb_t keydb;
  ps_partialpath_vec partialpaths;
  uint32_t payload_len, delay, times_to_send;
  int32_t _recvfd;
  sockaddr_in _addr;
  FILE *out;
  ptr<aclnt> clnt;
  ecdh_priv _privk;



  void config(std::string filename);
  void add_key(Json::Value& keyArray);
  void add_partial(Json::Value& partial);
  void add_path(Json::Value& path, ps_partialpath& partial);
  void add_cert(Json::Value& certData, icp_path& path);
  void add_waypoint(Json::Value& waypoint, icp_path& path);
  void add_privk(std::string& key);
    void connected(int32_t fd);
    void make_call();
    void get_reply(ptr<ps_reply> reply, uint64_t reqt, clnt_stat err);
    void handle_result(ptr<ps_reply> reply, uint64_t reqt);
    /**
      @brief Builds an icing packet, with the given payload, using the given path.
      
      @param path The path, including PoCs which the constructed ICING packet should use
      
      @param payload The payload to store in the ICING packet
      
      @param payloadSize The size of the payload
      
      @param outPkt On return, contains a pointer to the constructed ICING packet
      
      @returns The size of the constructed ICING packet
      
    */
  uint32_t build_packet(icp_path &path, uint8_t *payload, uint32_t payloadSize, uint8_t **outPkt);
    POC *get_pocs(icp_path &path);
    Path *convert_to_path(icp_path &path);
};

CLICK_ENDDECLS
#endif
