#include <click/config.h>
#include "icingcommon.hh"
#include "icingconfig.hh"
#include "addrealmentries.hh"
#include "realmtable.hh"

CLICK_DECLS

AddRealmEntries::AddRealmEntries() 
    : _con(NULL), _realmtable(NULL)
{
    _errh = router()->chatter_channel("default");
}

void
AddRealmEntries::error_packet(Packet* p)
{
    if (noutputs() > 1) 
	output(1).push(p);
    else
	p->kill();
}

L2LinkAddr*
hackycast(LinkAddr_constp p)
{
    IcingLinkAddr* temp = const_cast<IcingLinkAddr*>(
			    static_cast<const IcingLinkAddr*>(p));
    return static_cast<L2LinkAddr*>(temp->cast("L2LinkAddr"));
}

Packet*
AddRealmEntries::simple_action(Packet* p)
{
    // This method presumes that IcingCheck was called first
    const IcingHdr1* hdr1 = reinterpret_cast<const IcingHdr1*>(p->data());
    const Path* path = get_path(hdr1);

    uint8_t nrealms = hdr1->path_len;
    uint8_t idx = hdr1->path_idx;

    for (uint8_t i = 0; i < nrealms; i++) {

	if (i == idx)
	    continue;

	const RealmVoq* realmvoq = &path->realmvoqs[i];
	RealmID rid(realmvoq->realmid);
	
	if (_realmtable->lookup_realm(rid) == NULL)  {

	    smart_ptr<RealmInfo> rinfo = new RealmInfo;
	    rinfo->rid = rid;
		rinfo->entry_kind = RealmInfo::NON_NEIGH;
	    set_sharedkey(rinfo->sharedkey, 
			  *(rid.toWire()),
			  _con->privkey());
	    rinfo->linkaddr = NULL;

	    if (!_realmtable->add_realm(rinfo, true, NULL, _errh)) {
		_errh->debug("not enough room in realmtable; not good\n");
		error_packet(p);
		return NULL;
	    }
	}	
    }

    return p;
}

int
AddRealmEntries::configure(Vector<String> &conf, ErrorHandler *errh)
{
    int before = errh->nerrors();

    Element* elem;

    if (cp_va_kparse(conf, this, errh,
		     "CONFIG", cpkP+cpkM, cpElement, &elem,
		     cpEnd) < 0)
	return -1;

    if ( (_con = static_cast<IcingConfig*>(elem->cast("IcingConfig"))) == NULL)
	errh->error("CONFIG needs to be an IcingConfig object");

    if ( (_realmtable = _con->realmtable()) == NULL)
	errh->error("This element requires CONFIG to specify a REALMTABLE");

    return (before == errh->nerrors() ? 0 : -1);
}


CLICK_ENDDECLS
ELEMENT_REQUIRES(icing)
EXPORT_ELEMENT(AddRealmEntries)
