#include <click/config.h>
#include "icingcommon.hh"
#include "checkcurrentrealm.hh"
#include "icingconfig.hh"

CLICK_DECLS

CheckCurrentRealm::CheckCurrentRealm() : 
    _con(NULL)
{
    _errh = router()->chatter_channel("default");
}

CheckCurrentRealm::~CheckCurrentRealm()
{
}

void
CheckCurrentRealm::error_packet(Packet* p)
{
    if (noutputs() > 1) 
	output(1).push(p);
    else
	p->kill();
}

Packet* 
CheckCurrentRealm::simple_action(Packet* p)
{
    const IcingHdr1* ice_hdr = 
	    reinterpret_cast<const IcingHdr1*>(p->data());
    const Path* path = get_path(ice_hdr);
    const RealmVoq* rv = &path->realmvoqs[ice_hdr->path_idx];

    RealmID packet_rid;
    packet_rid.parse(rv->realmid);

    if (packet_rid != _con->realmid()) {
	_errh->debug("RealmIDs don't match: %s %s", 
		     packet_rid.unparse().c_str(),
		     _con->realmid().unparse().c_str());
	error_packet(p);
	return NULL;
    }
    
    // XXX: should we check the virtual queue eventually?

    return p;
}


int
CheckCurrentRealm::configure(Vector<String> &conf, ErrorHandler *errh)
{
    int before = errh->nerrors();

    Element* elem;

    if (cp_va_kparse(conf, this, errh,
		     "CONFIG", cpkP+cpkM, cpElement, &elem,
		     cpEnd) < 0)
	return -1;

    if ( (_con = static_cast<IcingConfig*>(elem->cast("IcingConfig"))) == NULL)
	errh->error("CONFIG needs to be an IcingConfig object");

    return (before == errh->nerrors() ? 0 : -1);
}

CLICK_ENDDECLS
ELEMENT_REQUIRES(icing)
EXPORT_ELEMENT(CheckCurrentRealm)
