#ifndef CHECK_NEXT_HOP_HH
#define CHECK_NEXT_HOP_HH

#include <click/element.hh>
CLICK_DECLS

class IcingConfig;
class RealmTable;

// peeks ahead to the next realm (if there is one) and verifies
// that the nextone exists. the purpose of this is, that if the next
// hop *doesn't* exist, then we can save ourselves the work of deriving
// shared keys.
//    Packet is emitted on output 0 if if the next hop exists.
//    Packet is emitted on output 1 if next hop is unknown 
//    Packet is emitted on output 2 if the packet is at its
//    destination
//
//    Output 1 is agnostic. Output 2 is push. Output 3 is agnostic.
//    
// Requires a 'CONFIG' element that in turn points to a RealmTable
class CheckNextHop : public Element { public:
  
    CheckNextHop();		
    ~CheckNextHop();		

    const char *port_count() const      { return "1/1-3";   }
    const char *class_name() const	{ return "CheckNextHop"; }
    const char *processing() const	{ return "a/aha"; }
    bool can_live_reconfigure() const   { return true; }

    void push(int, Packet*);
    Packet* pull(int);

    int configure(Vector<String> &, ErrorHandler *);

private:

    // not the same as the simple_action virtual function
    Packet* simple_action(Packet *, int*);

    Packet* handle_at_destination(Packet* p, int* outport);
    Packet* handle_unknown_next_hop(Packet* p, int* outport);

    IcingConfig* _con;
    RealmTable* _realmtable;
    ErrorHandler* _errh;
};

CLICK_ENDDECLS
#endif
