/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#ifndef _CRYPTKEY_H_
#define _CRYPTKEY_H_

#include "crypt.h"

class str;
class pubkey;
class privkey;
class rabin_pubkey;
class rabin_privkey;

// MW: XXX: can't this file be deleted?

// right now, this is nothing fancy. in particular, the private key file isn't
// password-protected and salted, but it wouldn't be hard to get
// these functions, given what DM and Max have already.

ptr<rabin_privkey> gen_rabin_privkey(int nbits);

typedef enum {CKF_B64_T, CKF_BYTES_T} cryptkeyformat_t;

class pubkey {
   
    public:
	virtual bool verify(const str& msg, const str& sig) const = 0;
	virtual bool parse_from_buf(const char* buf, int len,
				    cryptkeyformat_t) = 0;
	virtual bool parse_from_buf(const str&, cryptkeyformat_t) = 0;
	virtual str to_b64() const = 0;
	virtual str to_bytes() const = 0;

	virtual bool to_file(const str& fname) const;
	virtual ~pubkey() { }
};

class privkey {

    public:
	virtual str sign(const str& msg) const = 0;
	virtual bool parse_from_buf(const char* buf, int len, 
				    cryptkeyformat_t) = 0;
	virtual bool parse_from_buf(const str&, cryptkeyformat_t) = 0;
	virtual str to_b64() const = 0;
	virtual str to_bytes() const = 0;
	virtual ptr<pubkey> to_pubkey() const = 0;
	virtual bool to_file(const str& fname) const;

	virtual ~privkey() {}
};

class rabin_pubkey : public pubkey {

    public:
        static ptr<rabin_pubkey> from_str(const str& s, cryptkeyformat_t f);
	rabin_pubkey(ptr<rabin_pub> pubk) : pubk(pubk) {}
	explicit rabin_pubkey() : pubk(NULL) {}
	explicit rabin_pubkey(const char* p, int len, 
			      cryptkeyformat_t f) :
	    pubk(buf2rabinpub(p, len, f)) {}
	virtual ~rabin_pubkey() { }

	bool verify(const str& msg, const str& sig) const;
	bool parse_from_buf(const char* buf, int len, cryptkeyformat_t);
	bool parse_from_buf(const str&, cryptkeyformat_t);
	str  to_b64() const;
	str  to_bytes() const;

	const rabin_pubkey& operator= (const rabin_pubkey& r) {
	    pubk = r.pubk;
	    return *this;
	}
    private:
	static ptr<rabin_pub> buf2rabinpub(const char*, int,
					    cryptkeyformat_t);

	// no copying
	rabin_pubkey(const rabin_pubkey&);

	ptr<rabin_pub> pubk;
};

class rabin_privkey : public privkey {

    public:
	static ptr<rabin_privkey> from_str(const str& s, cryptkeyformat_t);
	rabin_privkey(ptr<rabin_priv> privk) : privk(privk) {}
	explicit rabin_privkey() : privk(NULL) {}
	virtual ~rabin_privkey() { }

	str sign(const str& msg) const;
	bool parse_from_buf(const char* buf, int len, cryptkeyformat_t);
	bool parse_from_buf(const str&, cryptkeyformat_t);

	str to_b64() const;
	str to_bytes() const;

	ptr<pubkey> to_pubkey() const {
	    return to_pubkey_r();
	}

	ptr<rabin_pubkey> to_pubkey_r() const {
	    // the privk is a rabin_pub object
	    return New refcounted<rabin_pubkey> (privk);
	}
    private:
	static ptr<rabin_priv> buf2rabinpriv(const char*, int, 
					       cryptkeyformat_t);

	// no copying
	rabin_privkey(const rabin_privkey&);
	const rabin_privkey& operator= (const rabin_privkey&);

	ptr<rabin_priv> privk;
};

inline bool 
pubkey::to_file(const str& fname) const
{
    str s = to_b64();
    return str2file(fname, s, 0644, true);
}

inline bool 
privkey::to_file(const str& fname) const
{
    str s = to_b64();
    return str2file(fname, s, 0600, true);
}

#endif
