#include <click/config.h>
#include "icingcommon.hh"
#include "derivekey.hh"

CLICK_DECLS

DeriveKey::DeriveKey() :
	_g(1), _v(16)
{
	_errh = router()->chatter_channel("default");
}

DeriveKey::DeriveKey(RealmSecret &k) :
    	_g(1), _v(16)
{
    memcpy(_k.bytes, k.bytes, RealmSecret::SIZE);
	buildCache();
}

int
DeriveKey::configure(Vector<String> &conf, ErrorHandler *errh)
{
	int before = errh->nerrors();
	String k_str;

	// XXX: MW: see my comment in derivekey.hh. Maybe K,G,V
	// shouldn't be mandatory. Maybe they should have sensible
	// defaults.
	if( cp_va_kparse(conf, this, errh,
			"K", cpkP+cpkM, cpString, &k_str,
			"G", cpkP, cpInteger, &_g,
			"V", cpkP, cpInteger, &_v,
			cpEnd) < 0)
		return -1;

	if(!cp_icing_realmsecret(k_str, &_k))
		errh->error("Error processing K");

	if((_g == 0) || (_g > 32) || (_g & (_g - 1)))
		errh->error("G must be a power of 2 less than or equal to 32");
	else if(_v % _g != 0)
		errh->error("V must be a multiple of G");
	else
		buildCache();

	return (errh->nerrors() == before ? 0 : 1);
}

void
DeriveKey::buildCache()
{
	for(uint32_t i = 0; i < (1 << 16); i++)
		derive_poc_key(_k, _g, _v, htonl(i << 16), _s[i]);
}

// MW: XXX: I don't understand this function. I mean, I see what it is
// doing, but it is not clear what the real point is. Maybe include some
// pseudo-code?
// AS: this emulates the behavior hardware lookups to SRAM. If v <= 16, lookups
// will entirely be in SRAM. Otherwise, the AES encryptions will need to take place.
// Note: with Jad's current implementation, V will never be greater than 16.
//
// WAIT: cache is looked up using HOST BYTE ORDER!!!
void
DeriveKey::derive(const uint32_t vnode, RealmSecret &s)
{
	if(_v <= 16)
	{
		// Do cache lookup to retrieve poc key (simulate SRAM lookup)
		uint32_t hostordered = ntohl(vnode);
		uint32_t i = hostordered >> 16;
		memcpy(s.bytes, _s[i].bytes, RealmSecret::SIZE);
	}
	else
		derive_poc_key(_k, _g, _v, vnode, s);
}

CLICK_ENDDECLS
ELEMENT_REQUIRES(icing)
EXPORT_ELEMENT(DeriveKey)
