#ifndef DERIVEKEY_HH_
#define DERIVEKEY_HH_

#include <click/element.hh>

CLICK_DECLS

/*
 * =c
 *
 * DeriveKey(K [, G, V])
 *
 * =s icing
 *
 * Handles PoC key delegation for other click elements. Emulates ICING forwarder by 
 * storing some delegated PoC keys in a cache.
 *
 * =d
 * Uses the realm secret K to derive PoC keys for a specific vnode. The PoC delegation strategy
 * can be specified by setting the delegation granularity G and V.
 */

class RealmSecret;

class DeriveKey : public Element
{
	public:
		DeriveKey();
		DeriveKey(RealmSecret &);
		~DeriveKey() { }

		const char *port_count() const { return PORTS_0_0; }
		const char *class_name() const { return "DeriveKey"; }
		const char *processing() const { return AGNOSTIC; }

		bool can_live_reconfigure() const { return true; }

		int configure(Vector<String> &, ErrorHandler *);

		const uint32_t g() const { return _g; }
		const uint32_t v() const { return _v; }
		const RealmSecret &k() { return _k; }
		
		void derive(const uint32_t, RealmSecret &);

		const RealmSecret* cache() { return _s; }

	private:
		ErrorHandler *_errh;

		uint32_t _g, _v;
		RealmSecret _k;
		RealmSecret _s[1 << 16];

		void buildCache();
};

CLICK_ENDDECLS

#endif /* DERIVEKEY_HH_ */
