#ifndef _ICINGHWINTERFACE_HH_
#define _ICINGHWINTERFACE_HH_

#include <click/element.hh>
#include "icingcommon.hh"
extern "C"
{
#include <nf2icing/nf_icing3.h>
}

CLICK_DECLS

class IcingConfig;
class RealmInfo;

class HardwareInterfaceAbstract : public Element { public:

    virtual bool set_entry(smart_ptr<const RealmInfo>) = 0;
    virtual smart_ptr<const RealmInfo> get_entry(const RealmID &) = 0;
    virtual void clear_entry(const RealmID &) = 0;

    virtual void clear() = 0;
	
	// dump will not work in icing3
    //virtual String dump() const = 0;
};

class HardwareInterface : public HardwareInterfaceAbstract { public:

    HardwareInterface();
    virtual ~HardwareInterface();

    // issues hardware reset
    void reset();

    //+ Element methods
    const char *class_name() const	{ return "HardwareInterface"; }
    int configure(Vector<String> &conf, ErrorHandler *errh);
    int configure_phase() const         { return CONFIGURE_PHASE_INFO + 1; }
    int initialize(ErrorHandler*);
    void cleanup(CleanupStage);

    bool set_entry(smart_ptr<const RealmInfo>);
    smart_ptr<const RealmInfo> get_entry(const RealmID &);
    void clear_entry(const RealmID &);
	void rehash(Vector<smart_ptr<const RealmInfo> > &);

    void clear();
    String dump() const;

 private:

    int init_nf2(ErrorHandler*);

    //void dump_table(StringAccum&) const;
    //void dump_entry(StringAccum&, uint32_t,
	//const RealmIDWire&, const SharedKey&,
	//const EtherAddress&, uint8_t outmask, 
	//uint32_t num_hits)  const;

    bool is_zeroed(const RealmIDWire&, const SharedKey&, 
		   const EtherAddress&, uint8_t outmask, uint32_t num_hits) const;

	nf_icing_realm_info_t toHWInfo(smart_ptr<const RealmInfo>);
	smart_ptr<RealmInfo> fromHWInfo(nf_icing_realm_info_t &);

    uint8_t encode_mac_fwd(uint8_t) const;
    uint8_t encode_cpu_fwd(uint8_t) const;
    int8_t decode_mac_fwd(uint8_t) const;
    int8_t decode_cpu_fwd(uint8_t) const;

    static const uint8_t NPORTS = 4;

    String   _iface_anchor;
    EtherAddress _eth_addrs[NPORTS];

    IcingConfig* _con;
    ErrorHandler* _errh;
    nf_icing_dev* _nf2;
};

CLICK_ENDDECLS
#endif
