#include <click/config.h>
#include "icingcommon.hh"
#include "icingcheck.hh"

CLICK_DECLS

IcingCheck::IcingCheck() 
{
    _errh = router()->chatter_channel("default");
}

IcingCheck::~IcingCheck()
{
}

Packet* 
IcingCheck::simple_action(Packet* p)
{
    const IcingHdr1* ice_hdr1;
    uint32_t hdrlen;
    uint16_t total_len;
    uint8_t nrealms;
    uint8_t curr_idx;

    ice_hdr1 = reinterpret_cast<const IcingHdr1*>(p->data());

    if (p->length() < sizeof(IcingHdr1)) {
	_errh->debug("Packet length smaller than sizeof(IcingHdr1)");
	goto error;
    } 

    if (ice_hdr1->icing_ver != ICING_CURRENT_VERSION) {
	_errh->debug("received unexpected version");
	goto error;
    }

    hdrlen = icing_hdrlen(ice_hdr1);
    nrealms = ice_hdr1->path_len;
    curr_idx = ice_hdr1->path_idx;
    total_len = icing_pktlen(ice_hdr1);

    if (p->length() < hdrlen) {
	_errh->debug("Packet length, %d, smaller than hdrlen, %d.",
		     p->length(), hdrlen);
	goto error;
    }

    if (curr_idx >= nrealms) {
	_errh->debug("Current realm larger than total number "
		     "of realms");
	goto error;
    }

    if (total_len < hdrlen) {
	_errh->debug("total_len, %d, smaller than hdrlen, %d.",
		     total_len, hdrlen);
	goto error;
    }

    if (p->length() < total_len) {
	_errh->debug("Packet length, %d, smaller than total_len, %d.",
		     p->length(), total_len);
	goto error;
    }


    return p;

error:

    if (noutputs() > 1) 
	output(1).push(p);
    else
	p->kill();

    return NULL;
}

CLICK_ENDDECLS
ELEMENT_REQUIRES(icing)
EXPORT_ELEMENT(IcingCheck)
