#include <click/config.h>
#include "icingcommon.hh"

CLICK_DECLS

String
myhexdump(const void* q, size_t s)
{
    StringAccum sa;
    const uint8_t* p = static_cast<const uint8_t*>(q);
    for (size_t i = 0; i < s; i++) {
	sa.snprintf(2, "%02x", p[i] & 0xff);
    }

    return sa.take_string();
}

bool
check_realmid(const RealmID& rid, const rabin_pubkey& pubkey)
{
    RealmID expected_rid(pubkey);
    return expected_rid == rid;
}

void
dump_header1(const IcingHdr1* h, StringAccum& sa)
{
    sa << "ver " << (int)h->icing_ver << ' ';
    sa << "path_len " << (int)h->path_len << ' ';
    sa << "path_idx " << (int)h->path_idx << ' ';
    sa << "init_vec " << myhexdump(h->init_vec, sizeof(h->init_vec));
}

void
dump_header2(const IcingHdr2* h, StringAccum& sa)
{
    sa << "proto " << (int)h->proto << ' ';
    sa << "total_len " << ntohs(h->pkt_len) << ' ';
}

void
dump_icing_header(const IcingHdr1* h, StringAccum& sa)
{
    uint8_t nrealms = h->path_len; 
    uint8_t idx = h->path_idx;
    const Path* path = get_path(h);
    const VtagVector* vv = get_vtagvector(h);
    const IcingHdr2* h2 = get_icinghdr2(h);

    dump_header1(h, sa);

    sa << '\n';

    for (uint32_t i = 0; i < nrealms; i++) {
	if (i == idx) 
	    sa << "*";
	sa.snprintf(32, "realm %d: ", i);

	const RealmVoq* rv = &path->realmvoqs[i];
	dump_icing_realmvoq(rv, sa);

	sa << '\n';
    }

    for (int32_t i = 0; i < nrealms - 1; i++) {

	sa.snprintf(32, "vtag %d: ", i+1);

	const Vtag* vt = &vv->vtags[i];
	dump_icing_vtag(vt, sa);

	sa << '\n';
    }

    dump_header2(h2, sa);
}

void
dump_icing_annotations(const Packet* p, StringAccum& sa)
{
    sa << "anno: " <<  p->dst_ip_anno() << ":" << ntohs(get_port_anno(p));
}

void
dump_icing_payload(const unsigned char* ice_payload, int nbytes, StringAccum& sa)
{
    for (int i = 0; i < nbytes; i++) 
	sa.snprintf(2, "%02x", ice_payload[i] & 0xff);

}

void
dump_icing_realmvoq(const RealmVoq* rv, StringAccum& sa)
{
    sa << (*rv) << ' ';
}

void
dump_icing_vtag(const Vtag* vt, StringAccum& sa)
{
    sa << "Va: " << myhexdump(vt->va, sizeof(vt->va)) 
       << " Vb: " << myhexdump(vt->vb, sizeof(vt->vb));
}

StringAccum&
operator<<(StringAccum& sa, const rabin_pubkey& pk)
{
    str s = pk.to_b64();
    return sa << String(s.cstr(), s.len());
}

StringAccum&
operator<<(StringAccum& sa, const RealmVoq& rv)
{
    sa << "realmid: " << rv.realmid << " voq: " << ntohl(rv.voq);
    return sa;
}

StringAccum&
operator<<(StringAccum& sa, const Vtag& vt)
{
    sa << "va: " << myhexdump(vt.va, sizeof(vt.va)) 
       << " vb: " << myhexdump(vt.vb, sizeof(vt.vb));
    return sa;
}

CLICK_ENDDECLS
ELEMENT_PROVIDES(IcingFuncs)
