#ifndef ICING_HH_
#define ICING_HH_

#include <click/element.hh>
#include <click/router.hh>
#include <click/packet.hh>
#include <click/timestamp.hh>
#undef fatal //beat this compiler into submission XXX: MM: This is definitely not the proper way to fix
#include <click/error.hh>
#include <click/confparse.hh>
#include <click/string.hh>
#include <click/straccum.hh>
#include <click/hashtable.hh>
#include <click/timer.hh>
#include <click/etheraddress.hh>
#include <click/packet_anno.hh>
#include "sha1.h"
#include "icinghdr.h"
#include "icingsec.h"
#include "realmid.hh"
#include "linkaddr.hh"
#include "derivekey.hh"
// sfslite headers need to be included after click headers because SFS
// has some macros, like "fatal", that correspond to tokens in the click
// code.
#include "cryptkey.hh"

typedef enum {

    MODE_UNSET   = 0,
    MODE_OVERLAY = 1,
    MODE_L3 = 2

} IcingMode;

#define DECLARE_WRITE_HANDLER(x) \
    static int x(const String &, Element *, void *, ErrorHandler *);

#define DECLARE_READ_HANDLER(x) \
    static String x(Element *, void *);

#define DECLARE_COMPREHENSIVE_HANDLER(x) \
    static int x(int, String&, Element*, const Handler*, ErrorHandler*);

const uint32_t CMACSIZE = 32;

#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif

String myhexdump(const void*, size_t);

bool check_realmid(const RealmID& rid, const rabin_pubkey& pubkey);

void dump_icing_header(const IcingHdr1*, StringAccum&);
void dump_icing_annotations(const Packet*, StringAccum&);
void dump_icing_payload(const unsigned char*, int, StringAccum&);
void dump_icing_realmvoq(const RealmVoq*, StringAccum&);
void dump_icing_vtag(const Vtag*, StringAccum&);

void set_port_anno(Packet*, uint16_t);
uint16_t get_port_anno(const Packet*);

void set_dstether_anno(Packet*, const EtherAddress&);
EtherAddress get_dstether_anno(const Packet*);
EtherAddress get_dstether_anno(Packet*);

#define DST_ETHER_ANNO_OFFSET  17
#define DST_ETHER_ANNO_SIZE     6
#define CONST_DST_ETHER_ANNO(p)      (*(reinterpret_cast<const EtherAddress*>((p)->anno_u8() + DST_ETHER_ANNO_OFFSET)))
#define DST_ETHER_ANNO(p)      (*(reinterpret_cast<EtherAddress*>((p)->anno_u8() + DST_ETHER_ANNO_OFFSET)))
#define SET_DST_ETHER_ANNO(p, v) (*(reinterpret_cast<EtherAddress*>((p)->anno_u8() + DST_ETHER_ANNO_OFFSET)) = (v))


#define ICING_PORT_ANNO_OFFSET  24
#define ICING_PORT_ANNO_SIZE     2
#define ICING_PORT_ANNO(p)      ((p)->anno_u16(ICING_PORT_ANNO_OFFSET))
#define SET_ICING_PORT_ANNO(p, v) ((p)->set_anno_u16(ICING_PORT_ANNO_OFFSET, (v)))


inline void
set_port_anno(Packet* p, uint16_t port)
{
    SET_ICING_PORT_ANNO(p, port);
}

inline uint16_t
get_port_anno(const Packet* p)
{
    return ICING_PORT_ANNO(p);
}

inline void
set_dstether_anno(Packet* p, const EtherAddress& ea)
{
    SET_DST_ETHER_ANNO(p, ea);
}

inline EtherAddress
get_dstether_anno(const Packet* p)
{
    return CONST_DST_ETHER_ANNO(p);
}

inline EtherAddress
get_dstether_anno(Packet* p)
{
    return DST_ETHER_ANNO(p);
}

inline bool
cp_icing_realmsecret(const String& s, RealmSecret* secret) 
{
    if (s.length() != static_cast<int32_t>(RealmSecret::SIZE)) {
	return false;
    }
    memcpy(secret->bytes, s.data(), RealmSecret::SIZE);
    return true;
}

// MW: XXX: can you delete this next function? probably worth doing a
// global search for 'rabin' and ripping out all of the functions.

// precondition: String has been unquoted.
inline bool
cp_icing_pubkey(const String& s, rabin_pubkey* pubkey)
{
    return pubkey->parse_from_buf(s.data(), s.length(), CKF_B64_T);
}

// precondition: String has been unquoted.
inline bool
cp_icing_privkey(const String &s, ecdh_priv &privkey, RealmID &rid)
{
	bigint x;
	x.setraw(str(s.data(), s.length()));

	ecdh_priv p(x);
	privkey = p;

	chopped_ecpk_raw raw;
	p.to_chopped_ecpk_raw(raw);
	return memcmp(raw, rid.toWire()->bytes, RealmIDWire::SIZE) == 0;
}

inline bool
cp_icing_privkey(const String &s, ecdh_priv *privkey, RealmID *id = NULL)
{
	bigint x;
	x.setraw(str(s.data(), s.length()));
	ecdh_priv p(x);
	*privkey = p;

	if(! id)
		return true;
	else
	{
		chopped_ecpk_raw chopped;
		privkey->to_chopped_ecpk_raw(chopped);

		return memcmp(chopped, id->toWire()->bytes, RealmIDWire::SIZE) == 0;
	}
}

inline bool
confstr2pubkey(const String& str, rabin_pubkey* pubk)
{
    String unquoted;

    return cp_string(str, &unquoted) &&
	   cp_icing_pubkey(unquoted, pubk);
}

inline bool
confstr2realmid(const String& str, RealmID* rid)
{
    String unquoted;

    return cp_string(str, &unquoted) && 
	   cp_icing_realmid(unquoted, rid);
}

StringAccum&
operator<<(StringAccum& sa, const rabin_pubkey& pk);

StringAccum&
operator<<(StringAccum& sa, const RealmVoq& rv);

StringAccum&
operator<<(StringAccum& sa, const Vtag& vt);

#endif
