#ifndef ICINGCONFIG_HH_
#define ICINGCONFIG_HH_

#include <click/element.hh>
#include "icingcommon.hh"

CLICK_DECLS

class RealmTable;
class HardwareInterface;

// Most of the configuration of an Icing Forwarder will be encapsulated
// in one module.
class IcingConfig : public Element { public:
  
    IcingConfig();		
    ~IcingConfig();		

    const char *port_count() const      { return PORTS_0_0; }
    const char *class_name() const	{ return "IcingConfig"; }
    const char *processing() const	{ return AGNOSTIC; }
    bool can_live_reconfigure() const   { return true; }
    int configure_phase() const         { return CONFIGURE_PHASE_INFO; }

    void add_handlers();

    int configure(Vector<String> &, ErrorHandler *);

    const RealmSecret &k();
    const RealmID & realmid()	   { return _realmid; }
    IcingMode mode()               { return _mode;    }
//    const rabin_pubkey& pubkey()   { return _pubkey;  }
    ecdh_priv& privkey() { return _privkey; }

    RealmTable*  realmtable() { return _realmtable; }
    HardwareInterface* hwinterface() { return _hwinterface; }
    DeriveKey *derivekey() { return _derivekey; }

    bool use_pocomon() const { return  _use_pocomon; }
    uint16_t pocomon_port() const { return _pocomon_port; }
private:
    DECLARE_READ_HANDLER(read_handler);

    void process_mode(const String&, ErrorHandler*);

    RealmID	    _realmid;
    RealmSecret	    _k;
    IcingMode	    _mode;
    RealmTable*  _realmtable;
    HardwareInterface* _hwinterface;
    DeriveKey *_derivekey;

    ecdh_priv   _privkey;

    struct in_addr   _daemon_ip;
    uint16_t	     _daemon_port;

    ErrorHandler* _errh;
    
    bool _use_pocomon; //Should we use pocomon to forward packets?
    uint16_t _pocomon_port; //Port in network order
};

CLICK_ENDDECLS
#endif
