#include <click/config.h>
#include "icingcommon.hh"
#include "icingprint.hh"

CLICK_DECLS

IcingPrint::IcingPrint() : 
    _label(""),
    _nbytes(-1),
    _payload(false),
    _anno(true),
    _outfile(NULL)
{}

IcingPrint::~IcingPrint()
{}

int
IcingPrint::configure(Vector<String>& conf, ErrorHandler* errh)
{
    if (cp_va_kparse(conf, this, errh,
		    "LABEL", 0, cpString, &_label,
		    "PAYLOAD", 0, cpBool, &_payload,
		    "NBYTES", 0, cpUnsigned, &_nbytes,
		    "ANNO", 0, cpBool, &_anno,
#if CLICK_USERLEVEL
		    "OUTFILE", 0, cpFilename, &_outfilename,
#endif
		    cpEnd) < 0) {
	return -1;
    }

    return 0;
}

int
IcingPrint::initialize(ErrorHandler* errh)
{
#if CLICK_USERLEVEL
    if (_outfilename) {
	_outfile = fopen(_outfilename.c_str(), "wb");
	if (!_outfile)
	    return errh->error("%s: %s\n", _outfilename.c_str(), strerror(errno));
    }
#else
    (void) errh;
#endif
    return 0;
}

void
IcingPrint::cleanup(CleanupStage)
{
#if CLICK_USERLEVEL
    if (_outfile)
	fclose(_outfile);
    _outfile = 0;
#endif
}

Packet*
IcingPrint::simple_action(Packet* p)
{
    StringAccum sa;
    const IcingHdr1* hdr = 
	reinterpret_cast<const IcingHdr1*>(p->data());

    if (_label != "")
	sa << _label << ": ";

    if (_anno)  {
	dump_icing_annotations(p, sa);
	sa << ' ';
    }

    dump_icing_header(hdr, sa);

    if (_payload) {
	uint32_t hdr_len = icing_hdrlen(hdr);
	const unsigned char* payload = p->data() + hdr_len;
	uint32_t payload_len = icing_pktlen(hdr) - hdr_len;
	int dump_bytes = payload_len;

	if (_nbytes > -1 && static_cast<uint32_t>(_nbytes) < payload_len)
	    dump_bytes = _nbytes;

	dump_icing_payload(payload, dump_bytes, sa);
    }


#if CLICK_USERLEVEL
    if (_outfile) {
	sa << '\n';
	fwrite(sa.data(), 1, sa.length(), _outfile);
    } else 
#endif

    click_chatter("%s", sa.c_str());

    return p;
}

CLICK_ENDDECLS
ELEMENT_REQUIRES(icing)
EXPORT_ELEMENT(IcingPrint)
