#ifndef ICINGPRINT_HH_
#define ICINGPRINT_HH_

#include <click/element.hh>

CLICK_DECLS

// modelled after IPPrint.
// keywords:
//    LABEL -- prints a label in front of each packet
//    PAYLOAD -- if true, prints the payload of the icing packet
//		 (default is false)
//    NBYTES -- how many bytes from the payload to print out
//    ANNO -- if true, prints the annotations
//	      (default is true)
//    OUTFILE -- (usermode, only): place the output in the given file
class IcingPrint : public Element {public:

    IcingPrint();
    ~IcingPrint();

    const char *port_count() const      { return PORTS_1_1;   }
    const char *class_name() const	{ return "IcingPrint"; }
    const char *processing() const	{ return AGNOSTIC; }

    Packet* simple_action(Packet *);

    int configure(Vector<String> &, ErrorHandler *);

    int initialize(ErrorHandler* errh);
    void cleanup(CleanupStage);

private:

    String _label;
    int _nbytes;
    bool _payload;
    bool _anno;
    // copying this from IPPrint
#if CLICK_USERLEVEL
    String _outfilename;
    FILE* _outfile;
#endif
};

CLICK_ENDDECLS

#endif
