#ifndef ICINGSOURCE_HH 
#define ICINGSOURCE_HH 

#include <click/element.hh>
CLICK_DECLS

// Constructs a sample ICING packet based on various parameters in the 
// configuration string.
// IcingSource2 constructs an ICING packet based on the realm list that
// it is configured with. This crudely models a source being given a
// path by a path server and then placing the path
// in its packets.
class IcingSource : public Element { public:
  
    IcingSource();		
    ~IcingSource();		

    const char *port_count() const      { return PORTS_0_1;   }
    const char *class_name() const	{ return "IcingSource"; }
    const char *processing() const	{ return PUSH; }

    int configure(Vector<String> &, ErrorHandler *);

    int initialize(ErrorHandler *);
    void cleanup(CleanupStage);

    void run_timer(Timer*);

private:
 
    struct StupidPayload {
	char data[16];
    };

    static const uint8_t _nrealms = 3;
    static const uint32_t _payloadlen = 50;

    bool setup_packet();

    bool init_secrets(const String*);
    bool init_realmids(const String*);

    void cleanup_packet(Packet** p);

    Packet* _packet;
    Timer _timer;
    uint32_t _n;
    uint32_t _interval;

    RealmSecret  _k[_nrealms];
    RealmID _realmid[_nrealms];
	uint32_t _g[_nrealms];
	uint32_t _v[_nrealms];
	ecdh_priv _privk;

    static const uint32_t INTERVAL_DEFAULT = 5;
};

CLICK_ENDDECLS
#endif
