#ifndef ICINGSOURCE2_HH 
#define ICINGSOURCE2_HH 

#include <click/element.hh>
#include <click/timer.hh>
#include "ecdh.h"

CLICK_DECLS

// IcingSource2 takes as input just a path list and a list of tokens/turds/POCs.
// It constructs the icing header and the packet from this list. 
// Models a sending host that is given the needed information from a path 
// server. 
class IcingSource2 : public Element { public:
  
    IcingSource2();		
    ~IcingSource2();		

    const char *port_count() const      { return PORTS_0_1;   }
    const char *class_name() const	{ return "IcingSource2"; }
    const char *processing() const	{ return PUSH; }

    int configure(Vector<String> &, ErrorHandler *);

    int initialize(ErrorHandler *);
    void cleanup(CleanupStage);

    void run_timer(Timer*);

private:
 
    struct StupidPayload {
	char data[16];
    };

    static const uint32_t _payloadlen = sizeof(StupidPayload);

    bool validate_params(ErrorHandler*);
    bool setup_packet();

    Packet* _packet;

    Timer _timer;
    String _path_str;
    String _poc_str;

	ecdh_priv _privk;
};

CLICK_ENDDECLS
#endif
