#include <click/config.h>
#include "icingcommon.hh"
#include "linkaddr.hh"

CLICK_DECLS

void*
IcingLinkAddr::cast(const char* name)
{
    if (name && !strcmp(name, "IcingLinkAddr"))
	return this;
    else
	return 0;
}

void*
OverlayLinkAddr::cast(const char* name)
{
    if (name && !strcmp(name, "OverlayLinkAddr"))
	return (OverlayLinkAddr*)this;
    else 
	return IcingLinkAddr::cast(name);
}

void
OverlayLinkAddr::annotate(Packet* p) const
{
    p->set_dst_ip_anno(_ipaddr);
    set_port_anno(p, _udpport);
}

void*
L2LinkAddr::cast(const char* name)
{
    if (name && !strcmp(name, "L2LinkAddr"))
	return (L2LinkAddr*)this;
    else 
	return IcingLinkAddr::cast(name);
}

void
L2LinkAddr::annotate(Packet* p) const
{
    SET_PAINT_ANNO(p, _outputport);
    set_dstether_anno(p, _ea);
}

bool 
OverlayLinkAddr::parse(Vector<String>& words, Element* context)
{
    uint16_t port;

    if	(words.size() >= 2
          && cp_ip_address(words[0], &_ipaddr, context) 
	  && cp_tcpudp_port(words[1], IP_PROTO_UDP, &port, context)) {

	_udpport = htons(port);
	words.pop_front();
	words.pop_front();

	return true;
    }

    return false;
}

bool
L2LinkAddr::parse(Vector<String>& words, Element* context)
{
    if (words.size() >= 2
	&& cp_ethernet_address(words[0], &_ea, context)
	&& cp_integer(words[1], &_outputport)) {

	words.pop_front();
	words.pop_front();

	return true;
    }

    return false;

}

CLICK_ENDDECLS
ELEMENT_REQUIRES(icing)
ELEMENT_PROVIDES(IcingLinkAddrs)
