#ifndef _REALMID_HH_
#define _REALMID_HH_

#include "refcount.h"
#include "cryptkey.hh"

class rabin_pubkey;

String myhexdump(const void*, size_t);

class RealmID { public:

    RealmID();
    RealmID(const uint8_t* digest);
    RealmID(smart_ptr<RealmIDWire>);
    RealmID(const RealmIDWire&);
    explicit RealmID(const rabin_pubkey&);
    ~RealmID() {}

    // parse the string into ourselves. returns true if successful.
    bool parse(const String&);
    void parse(const RealmIDWire&);
    String unparse() const;
    smart_ptr<RealmIDWire> toWire() const;

    size_t hashcode() const;

    friend StringAccum& operator<<(StringAccum&, const RealmID&);
    friend bool operator==(const RealmID&, const RealmID&);
    friend bool operator!=(const RealmID&, const RealmID&);

    RealmID& operator=(const RealmID& r) {
	_wire = r._wire;
	return *this;
    }

    RealmID(const RealmID& r) : _wire(r._wire) {}
private:
    smart_ptr<RealmIDWire> _wire;

};

// precondition: String has been unquoted.
bool cp_icing_realmid(const String&, RealmID*);

inline
RealmID::RealmID() 
    : _wire(NULL) 
{}		 

inline
RealmID::RealmID(const uint8_t* digest) 
   : _wire(new RealmIDWire)
{ 
    memcpy(_wire->bytes, digest, RealmIDWire::SIZE); 
}

inline
RealmID::RealmID(smart_ptr<RealmIDWire> p)
 : _wire(p)
{}

inline
RealmID::RealmID(const RealmIDWire& w)
    : _wire(new RealmIDWire)
{
    memcpy(_wire->bytes, w.bytes, RealmIDWire::SIZE);
}

inline
RealmID::RealmID(const rabin_pubkey& pk) 
  :  _wire(new RealmIDWire())
{
    str pk_bytes = pk.to_bytes();
    sha1_hash(_wire->bytes, pk_bytes.cstr(), pk_bytes.len());
}

inline bool
RealmID::parse(const String& s)
{
    if (s.length() != static_cast<int32_t>(RealmIDWire::SIZE))
	return false;
    _wire = new RealmIDWire();
    memcpy(_wire->bytes, s.data(), RealmIDWire::SIZE);
    return true;
}

inline void
RealmID::parse(const RealmIDWire& w)
{
    _wire = new RealmIDWire();
    memcpy(_wire->bytes, w.bytes, RealmIDWire::SIZE);
}

inline String
RealmID::unparse() const
{
    return myhexdump(_wire->bytes, RealmIDWire::SIZE);
}

inline smart_ptr<RealmIDWire>
RealmID::toWire() const 
{ 
    return _wire;
}

inline size_t 
RealmID::hashcode() const 
{ 
    assert(_wire);
    return String::hashcode(_wire->bytes, _wire->bytes+RealmIDWire::SIZE); 
}


inline bool
operator==(const RealmID& r1, const RealmID& r2)
{
    return !memcmp(r1._wire->bytes, r2._wire->bytes, RealmIDWire::SIZE);
}

inline bool
operator!=(const RealmID& r1, const RealmID& r2)
{
    return !(r1 == r2);
}

inline StringAccum&
operator<<(StringAccum& sa, const RealmIDWire& w)
{
    for (uint32_t i = 0; i < sizeof(w.bytes); i++) 
	sa.snprintf(2, "%02x", w.bytes[i]);

    return sa;
}


inline StringAccum&
operator<<(StringAccum& sa, const RealmID& rid)
{
    return sa << *(rid._wire);
}

inline bool
cp_icing_realmid(const String& s, RealmID* realm_id)
{
    return realm_id->parse(s);
}

#endif
