#ifndef MWSMARTPTR_H_
#define MWSMARTPTR_H_

#include "assert.h"

template<class T>
class smart_ptr {
public:
    // smart_ptrs to other types can see our internals.
    template<class U> friend class smart_ptr;
    
    smart_ptr() : obj(NULL), cnt(NULL) { }

    smart_ptr(T *const t) : obj(t), cnt(new int(1)) { }

    template<class U>
    smart_ptr(U *const u) : obj(u), cnt(new int(1)) { }

    smart_ptr(const smart_ptr<T>& p) : obj(p.obj), cnt(p.cnt) { inc(); }

    // next line permits a smart_ptr<T> to be made from a smart_ptr<U>
    // if a T* could be made from a U*.
    template<class U>
    smart_ptr(const smart_ptr<U>& p) : obj(p.obj), cnt(p.cnt) { inc(); }

    ~smart_ptr() { dec(); }

    T* operator-> () const { return obj; }
    T& operator*  () const { return *obj; }
    operator T* () const   { return obj; }

    smart_ptr<T>& operator=(const smart_ptr<T>& p) {
	p.inc();
	dec();
	obj = p.obj;
	cnt = p.cnt;
	return *this;
    }

    // next line permits a smart_ptr<U> to be assigned to a smart_ptr<T>
    // if a U* could be assigned to a T*.
    template<class U>
    smart_ptr<T>& operator=(const smart_ptr<U>& p) {
	p.inc();
	dec();
	obj = p.obj;
	cnt = p.cnt;
	return *this;
    }
   
#if 0
    smart_ptr<T>& operator=(const T* & t) {
	if (obj)  { dec(); }
	obj = t;
	cnt = new int(1);
	t = NULL;
	return *this;
    }

    template<class U>
    smart_ptr<T>& operator=(const U* & u) {
	if (obj)  { dec(); }
	obj = u;
	cnt = new int(1);
	u = NULL;
	return *this;
    }
#endif

    bool operator== (const smart_ptr<T>& t) const { 
	return (t.obj == obj);
    }

    bool operator != (const smart_ptr<T>& t) const {
	return (t.obj != obj);
    }

    bool operator== (const T* const& p) const {
	return (p == obj);
    }
    bool operator!= (const T* const& p) const {
	return (p != obj);
    }

private:

    void inc() const { 
	if (cnt) *cnt = *cnt + 1; 
	else assert(obj == NULL); 
    }
    void dec() { 
	if (!cnt) {
	    assert(obj == NULL);
	    return;
	}
	*cnt = *cnt - 1;
	assert(*cnt >= 0);
	if (*cnt == 0) {
	    delete cnt;
	    delete obj;
	    cnt = NULL;
	    obj = NULL;
	}
    }

    T* obj;
    int* cnt;
};

#endif 
