#include <click/config.h>
#include "icingcommon.hh"
#include "regularswitch.hh"
#include "icingconfig.hh"
#include "realmtable.hh"

CLICK_DECLS

IcingRegularSwitch::IcingRegularSwitch() : _con(NULL)
{
    _errh = router()->chatter_channel("default");
}

void
IcingRegularSwitch::error_packet(Packet* p)
{
    if (noutputs() > 1) 
	output(1).push(p);
    else
	p->kill();
}

Packet*
IcingRegularSwitch::simple_action(Packet* p)
{
    // This method presumes that IcingCheck was called first
    const IcingHdr1* ice_hdr = 
	reinterpret_cast<const IcingHdr1*>(p->data());

    uint8_t curr_idx = ice_hdr->path_idx;
    uint8_t nrealms = ice_hdr->path_len;

    assert(curr_idx < nrealms);

    const Path* path = get_path(ice_hdr);
    const RealmVoq* rv = &path->realmvoqs[curr_idx];

    smart_ptr<const RealmInfo> rinfo;

    RealmID rid;
    rid.parse(rv->realmid);
// XXX: MM: After USEC11 submission, put this back in.
#if 0
    if(_con->use_pocomon()){
      // If we're using pocomon, we follow the following steps to
      // determine the IP address of the next hop:
      //
      // 1. Get the (realm, subrealm) pair of the next hop from the
      //    packet
      // 2. Use gethostbyname to query DNS for subrealm.realm.pocomon.net
      // 3. Create a new link address corresponding to the IP address found
      //    via DNS
      // 4. Use the link address to annotate the packet
      
      // XXX: MM: Formatting will be slow... this should be redone to be more performant
      // 40+8+14 = 62 bytes should be max length of format string. Putting 512 bytes on the stack to be safe.
      // probably not a huge perf loss, but worth noting.
      const char *formatString = "%040x.%08x.pocomon.net";
      char lookupString[512];
      if(snprintf(lookupString, 512, formatString, rv.voq, rv->realmid);
      struct hostent *result = gethostbyname()
      unsigned ipRaw = 0; //network order
      IPAddress ip(ipRaw);
      OverlayLinkAddr o(ip, _con->pocomon_port());
      o.annotate(p);
      return;
    }
#endif // #if 0
    rinfo = _realmtable->lookup_realm(rid);
    if (rinfo == NULL) {
	error_packet(p);
	return NULL;
    } 

    // XXX: Arun, we are going to be annotating even in the case of a
    // NON_NEIGH entry. Is that okay? Given the way that output 2 is
    // used, I'm guessing that this won't be a problem, but it feels a
    // bit wrong. Seems like we should only annotate if the packet was
    // handled properly, no?
	/* 
	 * AS: The plumbing to annotate exists, as a fake eth address and port is
	 * required when using NON_NEIGH entries (so as to not change the hw
	 * interface).  Handling it this way is analogous to the way that hw will
	 * handle the packet. 
	 */

    rinfo->linkaddr->annotate(p);
	
	if(rinfo->entry_kind == RealmInfo::NON_NEIGH)
	{
		if(noutputs() > 2)
			output(2).push(p);
		else
			p->kill();
		return NULL;
	}
	else
		return p;
}

int
IcingRegularSwitch::configure(Vector<String> &conf, ErrorHandler *errh)
{
    int before = errh->nerrors();

    Element* elem;

    if (cp_va_kparse(conf, this, errh,
		     "CONFIG", cpkP+cpkM, cpElement, &elem,
		     cpEnd) < 0)
	return -1;

    if ( (_con = static_cast<IcingConfig*>(elem->cast("IcingConfig"))) == NULL)
	errh->error("CONFIG needs to be an IcingConfig object");

    if ( (_realmtable = _con->realmtable()) == NULL)
	errh->error("This element requires CONFIG to specify a REALMTABLE");

    return (before == errh->nerrors() ? 0 : -1);
}


CLICK_ENDDECLS
ELEMENT_REQUIRES(icing)
EXPORT_ELEMENT(IcingRegularSwitch)
