#ifndef _TRANSLATIONTABLE_HH_
#define _TRANSLATIONTABLE_HH_

#include <click/element.hh>
#include "icingcommon.hh"

CLICK_DECLS

// MW: XXX: might be good to briefly comment this element. 
// MW: XXX: ditto with encap and dencap

struct TranslationInfo
{
	enum titype { PS, GLUE, IN, OUT }; // PS is used for our pathservers from bootstrap
	titype infotype;

	IPAddress localIP;
	IPAddress remoteIP;

	String domain; // This one I'm not sure about

	RealmID rid;
	Path *path;
	uint8_t pathlen;
	POC *pocs;
	POC *rpocs; // Return pocs

	String toString();
	friend StringAccum &operator<<(StringAccum &, const TranslationInfo &);

	TranslationInfo()
	{
		path = NULL;
		pocs = NULL;
		rpocs = NULL;
	}

	~TranslationInfo()
	{
		delete [] path;
		delete [] pocs;
		delete [] rpocs;
	}

	void
	initPathPocs()
	{
		delete [] path;
		delete [] pocs;

		path = reinterpret_cast<Path *>(new RealmVoq[pathlen]);
		pocs = new POC[pathlen - 1];
	}

	void
	initRPocs()
	{
		delete [] rpocs;
		rpocs = new POC[pathlen - 1];
	}
};

class TranslationTable : public Element
{
public:
	TranslationTable();
	virtual ~TranslationTable();

	const char *class_name() const { return "TranslationTable"; }
	const char *port_count() const { return PORTS_0_0; }

	//Main input to be IPAddr + Mask
	int configure(Vector<String> &, ErrorHandler *);
	void add_handlers();
	
	/*
	 * Add interface:
	 * Takes RealmID, Path, Perms; returns destIP
	 */
	void clear();
	const IPAddress &addEntry(TranslationInfo *info);
	TranslationInfo *lookup(const RealmID &);
	TranslationInfo *lookup(const IPAddress &);
	bool removeEntry(const RealmID &);
	bool removeEntry(const IPAddress &);

private:
	struct ht_entry
	{
		enum ht_status {FREE, INUSE};
		ht_status status;
		TranslationInfo *info;

		ht_entry()
		{
			status = FREE;
			info = NULL;
		}
		
		~ht_entry()
		{
			delete info;
		}
	};

	int32_t ipLoc(const IPAddress &) const;
	IPAddress locIP(const int32_t) const;
	void getNextIP(const int32_t = -1);

	DECLARE_WRITE_HANDLER(add_realm_handler);

	int32_t _tablesize;
	IPAddress _baseaddr, _mask;
	uint32_t _baseint, _masklen;

	typedef HashTable<RealmID, TranslationInfo *> HashTable_t;
	HashTable_t _ridtable;
	ht_entry *_iptable; // keyed on destIP
	int32_t _nextIP;

	void deleteEntry(int32_t, HashTable_t::iterator &);
};

inline StringAccum &
operator<<(StringAccum &sa, TranslationInfo &ti)
{
	sa 	<< "localIP: " 		<< ti.localIP.s()
		<< " remoteIP: " 		<< ti.remoteIP.s()
		<< " realm: "		<< ti.rid;
	return sa;
}

inline String
TranslationInfo::toString()
{
	StringAccum sa;
	sa << *(this);
	return sa.take_string();
}

CLICK_ENDDECLS
#endif /* _TRANSLATIONTABLE_HH_ */
