#include <click/config.h>
#include "icingcommon.hh"
#include "vtagverify.hh"
#include "icingconfig.hh"

CLICK_DECLS

VtagVerify::VtagVerify() : 
    _con(NULL)
{
    _errh = router()->chatter_channel("default");
}

VtagVerify::~VtagVerify()
{
}

void
VtagVerify::error_packet(Packet* p)
{
    if (noutputs() > 1) 
	output(1).push(p);
    else
	p->kill();
}

Packet* 
VtagVerify::simple_action(Packet* p)
{
    const IcingHdr1* hdr1 = 
	    reinterpret_cast<const IcingHdr1*>(p->data());

    //get vnode
    const Path *path = get_path(hdr1);
    const RealmVoq *rv = path->realmvoqs;
    uint32_t vnode = rv[hdr1->path_idx].voq;
    // derive key
    RealmSecret key;
    _con->derivekey()->derive(vnode, key);

    if (!validate_vtags(hdr1, &key, _con->privkey())) {
	_errh->debug("Vtag didn't validate");
	error_packet(p);
	return NULL;
    }

    return p;
}


int
VtagVerify::configure(Vector<String> &conf, ErrorHandler *errh)
{
    int before = errh->nerrors();

    Element* elem;

    if (cp_va_kparse(conf, this, errh,
		     "CONFIG", cpkP+cpkM, cpElement, &elem,
		     cpEnd) < 0)
	return -1;

    if ( (_con = static_cast<IcingConfig*>(elem->cast("IcingConfig"))) == NULL)
	errh->error("CONFIG needs to be an IcingConfig object");

    return (before == errh->nerrors() ? 0 : -1);
}

CLICK_ENDDECLS
ELEMENT_REQUIRES(icing)
EXPORT_ELEMENT(VtagVerify)
