#include "config.h"
#include "rxx.h"
#include "parseopt.h"
#include "ecdh.h"
#include "util.h"
#include <ctime>

void
add_memcert(keydb_t *keydb, icp_path *path, vec<str> args, str loc, bool *err)
{
	if(args.size() != 5)
	{
		warn << loc << ":Usage: memcert realm member vqueue mask\n"
			<< "vqueue can be in 0x format, while mask is the number of 1s (CIDR notation)\n";
		*err = true;
		return;
	}

	ptr<keydb_rec> *rec;
	//ptr<rabin_priv> realm, member;

	icp_memcert memcert;
	
	rec = (*keydb)[args[1]];
	if(!rec)
		return;
	//realm = (*rec)->priv;

	memcert.realm = (*rec)->key; //realm->n.getraw();

	rec = (*keydb)[args[2]];
	if(!rec)
		return;
	//member = (*rec)->priv;

	memcert.member = (*rec)->key; //member->n.getraw();

	memcert.vq = htonl((uint32_t)strtol(args[3], NULL, 0));
	uint32_t maskval = atoi(args[4]), mask = 0xFFFFFFFF;
	memcert.mask = htonl(mask << (32 - maskval));

	memcert.creation = time(NULL);
	memcert.expiry = memcert.creation + 120;

	//ptr<strbuf> buf = build_buf(memcert);
	//memcert.sig = realm->sign(*buf).getraw();

	path->memcerts.push_back(memcert);
}

void
add_cert(keydb_t *keydb, icp_path *path, vec<str> args, str loc, bool *err)
{
	ptr<keydb_rec> *rec;
	//ptr<rabin_priv> priv, realmk;
	if(args.size() != 7)
	{
		warn << loc << ": Usage: cert realm vqueue realm vqueue realm vqueue OR cert -0|1 0 realm vqueue -0|1 0\n"
			<< "vqueue can be in 0x format\n";
		*err = true;
		return;
	}

	icp_certificate cert;
	rxx dash("-");
	rec = (*keydb)[args[3]];
	if(!rec)
		return;
	//priv = (*rec)->priv;
	//realmk = priv;
	cert.realm.pubkey = (*rec)->key; //priv->n.getraw();
	cert.realm.vq = htonl((uint32_t)strtol(args[4], NULL, 0));

	int32_t keysize = cert.realm.pubkey.size();
	if(dash.search(args[1]))
	{
		if(! atoi(args[1]))
			memset(cert.prev.pubkey.base(), 0, keysize);
		else
		{
			memset(cert.prev.pubkey.base(), 0xFF, keysize);
		}
		cert.prev.vq = htonl(4062);
	}
	else
	{
		rec = (*keydb)[args[1]];
		if(!rec)
			return;
		//priv = (*rec)->priv;
		cert.prev.pubkey = (*rec)->key; //priv->n.getraw();
		cert.prev.vq = htonl((uint32_t)strtol(args[2], NULL, 0));
	}

	if(dash.search(args[5]))
	{
		if(! atoi(args[5]))
			memset(cert.next.pubkey.base(), 0, keysize);
		else
			memset(cert.next.pubkey.base(), 0xFF, keysize);
		cert.next.vq = htonl(4062);
	}
	else
	{
		rec = (*keydb)[args[5]];
		if(!rec)
			return;
		//priv = (*rec)->priv;
		cert.next.pubkey = (*rec)->key; //priv->n.getraw();
		cert.next.vq = htonl((uint32_t)strtol(args[6], NULL, 0));
	}

	cert.metric = 4062;

	cert.creation = time(NULL);
	cert.expiry = cert.creation + 120;

	//ptr<strbuf> buf = build_buf(cert);
	//cert.sig = realmk->sign(*buf).getraw();

	path->certs.push_back(cert);
}

void
add_waypoint(keydb_t *keydb, icp_path *path, vec<str> args, str loc, bool *err)
{
	if(args.size() < 2)
	{
		warn << loc << ": Usage: waypoint id\n";
		*err = true;
		return;
	}

	ptr<keydb_rec> *rec = (*keydb)[args[1]];
	if(!rec)
		return;

	//ptr<rabin_priv> priv = (*rec)->priv;
	icp_realm realm;
	realm.pubkey = (*rec)->key;
	
	if(args.size() == 3)
		realm.vq = (uint32_t)strtol(args[2], NULL, 0);
	else
		realm.vq = 4062;

	path->waypoints.push_back(realm);
}

void
add_path(keydb_t *keydb, ps_partialpath *partial, vec<str> args, str loc, bool *err)
{
	str realm_as_str;
	icp_path path;
	if(args.size() != 2)
	{
		warn << loc << ": Usage: path <filename>\n";
		*err = true;
		return;
	}

	conftab ct;
	ct.add("waypoint",	wrap(&add_waypoint, keydb, &path));
	ct.add("cert",		wrap(&add_cert, keydb, &path));
	ct.add("memcert", 	wrap(&add_memcert, keydb, &path));
	ct.run(args[1]);

	partial->paths.push_back(path);
}

void
add_key(keydb_t *keydb, vec<str> args, str loc, bool *err)
{
	if(args.size() != 2)
	{
		*err = true;
		return;
	}

	str id, pr, aes;

	conftab ct;
	ct.add("id",	&id);
	ct.add("pr",	&pr);
	ct.add("aes",	&aes);
	ct.run(args[1]);

	ptr<keydb_rec> rec = New refcounted<keydb_rec>();

	rec->id = id;

	char hex[3];
	hex[2] = '\0';
	for(uint32_t i = 0; i < id.len() / 2; i++)
	{
		memcpy(hex, id.cstr() + (i*2), 2);
		long byte = strtol(hex, NULL, 16);
		rec->key[i] = byte & 0xFF;
	}
	for(uint32_t i = 0; i < aes.len() / 2; i++)
	{
		memcpy(hex, aes.cstr() + (i*2), 2);
		long byte =strtol(hex, NULL, 16);
		rec->k[i] = byte & 0xFF;
	}

	rpc_bytes<> b;
	for(uint32_t i = 0; i < pr.len() / 2; i++)
	{
		memcpy(hex, pr.cstr() + (i*2), 2);
		long byte = strtol(hex, NULL, 16);
		b.push_back(byte & 0xFF);
	}
	str k(b.base(), b.size());
	bigint x;
	x.setraw(k);
	ecdh_priv privk(x);

	rec->privk = privk;

	keydb->insert(rec->id, rec);
	str rawkey(rec->key.base(), rec->key.size());
	keydb->insert(rawkey, rec);
	//keydb->insert(rec->priv->n.getraw(), rec);
}

void
add_privk(ecdh_priv *priv, vec<str> args, str loc, bool *err)
{
	if(args.size() != 2)
		warn << loc << ": Usage: privk <key>\n";

	str key = args[1];

	char hex[3];
	hex[2] = '\0';
	rpc_bytes<> b;
	for(uint32_t i = 0; i < key.len() / 2; i++)
	{
		memcpy(hex, key.cstr() + (i*2), 2);
		long byte = strtol(hex, NULL, 16);
		b.push_back(byte & 0xFF);
	}

	str k(b.base(), b.size());
	bigint x;
	x.setraw(k);
	ecdh_priv privk(x);
	*priv = privk;
}

vec<uint8_t>
convertFromHex(const char *hex, int len)
{
	//XXX: ask MW what the better what the better way to do this is.

	assert(len % 2 == 0);
	vec<uint8_t> bytes; 

	char raw[3];
	raw[2] = '\0';

	for(int i = 0; i < len / 2; i++)
	{
		memcpy(raw, &hex[i*2], 2);
		long byte = strtol(raw, NULL, 16);
		bytes.push_back(byte && 0xFF);
	}
}

void
convertFromHex(icp_pubkey &realmid, const char *hex, int len)
{
	assert(len == realmid.size());

	char raw[3];
	raw[2] = '\0';

	for(int i = 0; i < len / 2; i++)
	{
		memcpy(raw, &hex[i * 2], 2);
		long byte = strtol(raw, NULL, 16);
		realmid[i] = byte & 0xFF;
	}
}

