#ifndef UTIL_H_
#define UTIL_H_

#include "async.h"
#include "arpc.h"
#include "crypt.h"
//#include "aes.h"
#include "qhash.h"
#include "icp_ps.h"
#include "ecdh.h"

#define MAX_LINE_LENGTH 128
#define IMMEDIATE_CB(x) delaycb(0, 0, x)

struct keydb_rec
{
	str id;
	icp_pubkey key;
	rpc_opaque<16> k;
	ecdh_priv privk;
};
typedef qhash<str, ptr<keydb_rec> > keydb_t;

ptr<asrv> init_tcp_server(int fd, const rpc_program &prog);
ptr<asrv> init_udp_server(uint16_t, const rpc_program &prog);
ptr<aclnt> init_tcp_client(int32_t, const rpc_program &prog);

template<class T, size_t N>
inline void rpc_vec_append(rpc_vec<T, N> &dest, const rpc_vec<T, N> &src,
                int32_t start = 0)
{
	for(int32_t i = start; i < (int32_t)src.size(); i++)
		dest.push_back(src[i]);
}

template<class T, size_t N>
inline void rpc_vec_append_reverse(rpc_vec<T, N> &dest, const rpc_vec<T, N> &src,
				int32_t ignore = 0)
{
	for(int32_t i = src.size() - 1 - ignore; i >= 0; i--)
		dest.push_back(src[i]);
}

template<class T, size_t N>
inline void vec_append(vec<T, N> &dest, const vec<T, N> &src, int start = 0)
{
	for(int i = start; i < src.size(); i++)
		dest.push_back(src[i]);
}

inline void path_append(icp_path &dest, const icp_path &src)
{
	rpc_vec_append(dest.waypoints, src.waypoints);
	rpc_vec_append(dest.certs, src.certs);
	rpc_vec_append(dest.memcerts, src.memcerts);
}

inline void
strip_newline(char *string)
{
	char *nl = index(string, '\n');
	if(nl)
		*nl = '\0';
}


void print_paths(const icp_path_vec &vec, keydb_t * = NULL);
void print_path(const icp_path &path, keydb_t * = NULL);
void print_realmid(const icp_pubkey &);

bool icp_verify(const icp_memcert &);
bool icp_verify(const icp_certificate &);
bool icp_sig_verify(const str &, const icp_sig &, const icp_pubkey &);
void icp_sig_gen(icp_sig &sig, const str &msg, ecdh_priv &priv);

ptr<strbuf> build_buf(const icp_certificate &);
ptr<strbuf> build_buf(const icp_memcert &);

inline bool
operator== (const icp_realm &r1, const icp_realm &r2)
{
	return (r1.vq == r2.vq && r1.pubkey == r2.pubkey);
}
inline bool
operator!= (const icp_realm &r1, const icp_realm &r2)
{
	return (r1.vq != r2.vq || r1.pubkey != r2.pubkey);
}

template<>
struct hashfn<icp_realm>
{
	hashfn() {}
	hash_t operator() (const icp_realm &realm) const
	{
		hash_t strbits = hash_bytes(realm.pubkey.base(), realm.pubkey.size());
		return hash_bytes(&realm.vq, 4, strbits);
	}
};

#endif /* UTIL_H_ */
