// -*-c++-*-
/* $Id: dnsfmt.h,v 1.3 2004/04/21 07:06:34 max Exp $ */

/*
 *
 * Copyright (C) 2003 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#ifndef _ASYNC_DNSFMT_H_
#define _ASYNC_DNSFMT_H_ 1

#include "dnsparse.h"
#include "async.h"

#ifndef T_AAAA
# define T_AAAA 28
#endif /* !T_AAAA */
#ifndef T_A6
# define T_A6 38
#endif /* !T_A6 */
#ifndef T_DNAME
# define T_DNAME 39
#endif /* T_DNAME */

struct nsinfo {
  str mname;
  str rname;
  u_int32_t ttl;
  in_addr addr;
};

class dnsfmt {
public:
  enum section { QD = 0, AN = 1, NS = 2, AR = 3 };
private:
  struct adrec {
    str name;
    in_addr ip4addr;
    u_int32_t ttl;
  };
  enum { num_dnptrs = 20 };
  u_char *dnptrs[num_dnptrs];
  vec<adrec, 5> arvec;

  bool putname (u_char *&cp, const char *name);
  bool putbytes (u_char *&cp, const void *data, size_t len);
  bool putint (u_char *&cp, u_int32_t val);
  bool putshort (u_char *&cp, u_int16_t val);
  bool putrr (const resrec &rr);
  void sectioncheck (section s);
  void bumpcount (section s);

public:
  union {
    HEADER h;
    u_char buf[512];
  };
  u_int len;

  dnsfmt () { reset (); }
  void reset ();
  bool init (question *qp, const u_char *qbuf, u_int size);
  bool putquestion (const question &q);
  bool putresponse (section s, const resrec &rr) {
    assert (s != QD);
    sectioncheck (s);
    if (!putrr (rr))
      return false;
    bumpcount (s);
    return true;
  }
  bool putsoa (section s, const nsinfo &ni, str domain);
  bool putadrecs ();

  bool puta (section s, str domain, in_addr addr, u_int32_t ttl = 60);
  bool putns (section s, str domain, str server, u_int32_t ttl = 60,
	      const in_addr *addrp = NULL, u_int32_t addrttl = 0);
  bool putcname (str name, str cname, u_int32_t ttl = 60);
  bool putdname (str name, str dname, u_int32_t ttl = 60);
  bool dontknow ();
  bool fwd (const str &h);
  bool nxrec (const nsinfo &ni, str domain = NULL)
    { return putsoa (NS, ni, domain); }
  bool nxdomain (const nsinfo &ni, str domain = NULL)
    { h.rcode = NXDOMAIN; return nxrec (ni, domain); }
};

inline bool
dnsfmt::putint (u_char *&cp, u_int32_t val)
{
  if (cp + 4 > buf + sizeof (buf)) {
    h.tc = 1;
    return false;
  }
  PUTLONG (val, cp);
  return true;
}

inline bool
dnsfmt::putshort (u_char *&cp, u_int16_t val)
{
  if (cp + 2 > buf + sizeof (buf)) {
    h.tc = 1;
    return false;
  }
  PUTSHORT (val, cp);
  return true;
}

#endif /* !_ASYNC_DNSFMT_H_ */
