#include "pathserver.h"
#include "verify_path.h"
#include "tt.h"
#include <cstdlib>

enum {TIMES_TO_RUN = 1, MAX_PATH_LEN = 20};
FILE *out = NULL;

void
random_realm(uint32_t *realm)
{
	for(int i = 0; i < 5; i++)
		realm[i] = rand();
}

icp_path
build_path(uint32_t size)
{
	icp_path path;

	for(uint32_t i = 0; i < size; i++)
	{
		icp_realm r;
		random_realm(reinterpret_cast<uint32_t *>(r.pubkey.base()));
		r.vq = 0;

		path.waypoints.push_back(r);
	}

	for(uint32_t i = 1; i < size - 1; i++)
	{
		icp_certificate c;

		c.prev = path.waypoints[i - 1];
		c.realm = path.waypoints[i];
		c.next = path.waypoints[i + 1];
		path.certs.push_back(c);

		c.prev = path.waypoints[i + 1];
		c.realm = path.waypoints[i];
		c.next = path.waypoints[i - 1];
		path.certs.push_back(c);
	}

	return path;
}

int
main(int argc, char **argv)
{
	srand((unsigned) time(NULL));
	if(argc > 1)
		out = fopen(argv[1], "w+");

	icp_path vec[30];

	for(uint32_t i = 3; i <= MAX_PATH_LEN; i++)
	{
		uint64_t max = 0;
		printf("Building paths of length %u ... ", i);
		fflush(stdout);
		for(int j = 0; j < 30; j++)
			vec[j] = build_path(i);
		printf("DONE\n");
		printf("Running experiment for path length %u ... ", i);
		fflush(stdout);
		for(uint32_t j = 0; j < TIMES_TO_RUN; j++)
		{
			uint64_t end = get_now() + 30000000;
			uint64_t count = 0;
			while( get_now() < end)
			{
				bool b = icp_verify_path(vec[count % 30]);
				if(!b)
					fatal << "NO!\n";
				count++;
			}
				
			if(max == 0 || max < count)
				max = count;
			if(out)
			{
				fprintf(out, "%u, %llu\n", i, count);
				fflush(out);
			}
		}
		printf("max %llu \n", max);
		fflush(stdout);
	}

	return 0;
}
