#include "async.h"
#include "util.h"
#include "parseopt.h"
#include "icp_config.h"
#include "icinghdr.h"
#include "icingsec.h"

extern "C"
{
#include <sys/socket.h>
}

uint16_t _port, _myport=0;
int32_t _sendfd, _recvfd, _payload_len, _fd;
sockaddr_in _addr;
ecdh_priv _privk;

void
build_packet(Path *path, uint8_t pathlen, POC *pocs, uint8_t **buf, uint32_t *bufsize)
{
	uint32_t hdrlen = icing_hdrlen(pathlen);
	*bufsize = 1500;
	*buf = New uint8_t[*bufsize];

	IcingHdr1 *hdr1 = reinterpret_cast<IcingHdr1 *>(*buf);
	hdr1->icing_ver = ICING_CURRENT_VERSION;
	hdr1->path_len = pathlen;
	hdr1->path_idx = 0;
	for(uint32_t i = 0; i < IV_SIZE; i++)
		hdr1->init_vec[i] = (i+1) & 0xFF;

	IcingHdr2 *hdr2 = get_icinghdr2(hdr1);
	hdr2->proto = 8;
	hdr2->pkt_len = htons(*bufsize);

	Path *wirepath = get_path(hdr1);
	memcpy(wirepath, path, pathlen * sizeof(RealmVoq));

	uint8_t *payload = *buf + hdrlen;
	memset(payload, 0x46, *bufsize - hdrlen - 1);

	init_vtags(hdr1, pocs, _privk);
	hdr1->path_idx = 1;
}

void
send_packet(uint8_t *buf, uint32_t bufsize)
{
	warn << "Sending response...\n";
	if(static_cast<int32_t>(bufsize) > (sendto(_fd, buf, bufsize, 0, 
				reinterpret_cast<const sockaddr *>(&_addr), sizeof(_addr)))) 
		warn("Did not send the whole response: %d - %m\n", errno);

	warn << "Response sent.\n";
}

void
send_response(Path *recvpath, uint8_t pathlen, uint8_t *packet_payload)
{
	RealmVoq *path = New RealmVoq[pathlen];
	for(int8_t i = pathlen - 1, j = 0; i >= 0; i--, j++)
		memcpy(&path[j], &recvpath->realmvoqs[i], sizeof(RealmVoq));

	POC *recvpocs = reinterpret_cast<POC *>(packet_payload);
	POC *pocs = New POC[pathlen - 1];
	for(int8_t i = 0; i < pathlen - 1; i++)
	{
		memcpy(pocs[i], recvpocs[i], sizeof(POC));
	}

	uint8_t *buf;
	uint32_t bufsize;
	build_packet(reinterpret_cast<Path *>(path), pathlen, recvpocs, &buf, &bufsize); 
	send_packet(buf, bufsize);

	delete [] buf;
	delete [] path;
	delete [] pocs;
}

void
get_packet()
{
	uint8_t buf[1500];
	bzero(buf, sizeof(buf));

	size_t recvsize = recvfrom(_fd, buf, sizeof(buf), 0, NULL, NULL);

	if(recvsize < 0)
	{
		warn << "ERROR, negative packet size\n";
		return;
	}
	
	warn << "Received packet (" << recvsize << "), preparing response...\n";

	IcingHdr1 *hdr1 = reinterpret_cast<IcingHdr1 *>(buf);

	if(hdr1->icing_ver != ICING_CURRENT_VERSION)
		fatal << "version wrong!\n";

	uint32_t hdrlen = icing_hdrlen(hdr1);
	uint8_t nrealms = hdr1->path_len;
	uint8_t curr_idx = hdr1->path_idx;
	uint16_t total_len = icing_pktlen(hdr1);

	if(recvsize < hdrlen || curr_idx >= nrealms || total_len < hdrlen || recvsize < total_len)
		fatal << "Packet not well formed\n";
	
	RealmSecret k;
	memset(k.bytes, 37, RealmSecret::SIZE);
	if(!validate_vtags(hdr1, &k, _privk));

	send_response(get_path(hdr1), hdr1->path_len, buf + icing_hdrlen(hdr1->path_len));
	warn << "Waiting...\n";
}

void
config(str filename)
{
	str node_addr;
	conftab ct;
	ct.add("privk",			wrap(&add_privk, &_privk));
	ct.add("payload_len", 	&_payload_len, 0, 65535);
	ct.add("i2_addr", 		&node_addr);
	ct.add("i2_port",		&_port, 0, 65535);
	ct.add("my_port",		&_myport, 0, 65535);
	ct.run(filename);

	_addr.sin_family = AF_INET;
	_addr.sin_port = htons(_port);
	_addr.sin_addr.s_addr = inet_addr(node_addr.cstr());
	bzero(_addr.sin_zero, sizeof(_addr.sin_zero));
}

int32_t 
main()
{
	config("../eval/config/nsdi10/i2rx.config");
//	_sendfd = inetsocket(SOCK_DGRAM);
	if(_myport)
		_fd = inetsocket(SOCK_DGRAM, _myport, INADDR_ANY);
	else
		_fd = inetsocket(SOCK_DGRAM, _port, INADDR_ANY);
//	recvfd = socket(AF_INET, SOCK_DGRAM, 0);

#if 0
	struct sockaddr_in addr;
	bzero(&addr, sizeof(addr));

	addr.sin_family = AF_INET;
	addr.sin_port = htons(0x2345);
	addr.sin_addr.s_addr = inet_addr("192.168.39.1");
#endif


	if(_fd < 0)
		fatal("Could not open socket: %d - %m\n", errno);
#if 0
	int32_t bindret = bind(recvfd, (sockaddr *)&addr, sizeof(addr));
	if(bindret < 0)
		fatal("NO BIND! %d - %m\n", errno);
#endif

	warn << "Waiting...\n";
	fdcb(_fd, selread, wrap(get_packet));
	amain();
}
