#include "async.h"
#include "util.h"
#include "parseopt.h"
#include "config.h"
#include "icinghdr.h"
#include "icingsec.h"

extern "C"
{
#include <sys/socket.h>
}

char *ip = "192.168.39.254";
uint16_t port = 0x1234;
int32_t sendfd, recvfd;

#define PAYLOAD_SIZE 300

void
build_packet(Path *path, uint8_t pathlen, POC *pocs, uint8_t **buf, uint32_t *bufsize)
{
	uint32_t hdrlen = icing_hdrlen(pathlen);
	*bufsize = hdrlen + PAYLOAD_SIZE;
	*buf = New uint8_t[*bufsize];

	IcingHdr1 *hdr1 = reinterpret_cast<IcingHdr1 *>(*buf);
	hdr1->icing_ver = ICING_CURRENT_VERSION;
	hdr1->path_len = pathlen;
	hdr1->path_idx = 0;
	for(uint32_t i = 0; i < IV_SIZE; i++)
		hdr1->init_vec[i] = (i+1) & 0xFF;

	IcingHdr2 *hdr2 = get_icinghdr2(hdr1);
	hdr2->proto = 8;
	hdr2->pkt_len = htons(*bufsize);

	Path *wirepath = get_path(hdr1);
	memcpy(wirepath, path, pathlen * sizeof(RealmVoq));

	uint8_t *payload = *buf + hdrlen;
	memset(payload, 0x46, PAYLOAD_SIZE);

	// XXX:
	ecdh_priv p;
	init_vtags(hdr1, pocs, p);
	hdr1->path_idx = 1;

	delete[] path;
	//delete[] pocs;
}

void
send_packet(uint8_t *buf, uint32_t bufsize)
{
	warn << "Sending response...\n";
	struct sockaddr_in sin;
	bzero(&sin, sizeof(sin));

	sin.sin_family = AF_INET;
	sin.sin_port = htons(port);
	sin.sin_addr.s_addr = inet_addr(ip);

	if(static_cast<int32_t>(bufsize) > (sendto(sendfd, buf, bufsize, 0, 
				reinterpret_cast<const sockaddr *>(&sin), sizeof(sin)))) 
		warn("Did not send the whole response: %d - %m\n", errno);

	warn << "Response sent.\n";
	delete[] buf;
}

void
send_response(Path *recvpath, uint8_t pathlen, uint8_t *packet_payload)
{
	RealmVoq *path = New RealmVoq[pathlen];
	for(int8_t i = pathlen - 1, j = 0; i >= 0; i--, j++)
		memcpy(&path[j], &recvpath->realmvoqs[i], sizeof(RealmVoq));

	POC *recvpocs = reinterpret_cast<POC *>(packet_payload);
	POC *pocs = New POC[pathlen - 1];
	for(int8_t i = 0; i < pathlen - 1; i++)
	{
		memcpy(pocs[i], recvpocs[i], sizeof(POC));
	}

	uint8_t *buf;
	uint32_t bufsize;
	build_packet(reinterpret_cast<Path *>(path), pathlen, recvpocs, &buf, &bufsize); 
	send_packet(buf, bufsize);
}

void
get_packet()
{
	uint8_t buf[1500];
	bzero(buf, sizeof(buf));

	size_t recvsize = recvfrom(recvfd, buf, sizeof(buf), 0, NULL, NULL);

	if(recvsize < 0)
	{
		warn << "ERROR, negative packet size\n";
		return;
	}
	
	warn << "Received packet, preparing response...\n";

	IcingHdr1 *hdr1 = reinterpret_cast<IcingHdr1 *>(buf);

	send_response(get_path(hdr1), hdr1->path_len, buf + icing_hdrlen(hdr1->path_len));
	warn << "Waiting...\n";
}

int32_t 
main()
{
	sendfd = inetsocket(SOCK_DGRAM);
	recvfd = inetsocket(SOCK_DGRAM, 0x2345, ntohl(inet_addr("192.168.39.1")));
//	recvfd = socket(AF_INET, SOCK_DGRAM, 0);

#if 0
	struct sockaddr_in addr;
	bzero(&addr, sizeof(addr));

	addr.sin_family = AF_INET;
	addr.sin_port = htons(0x2345);
	addr.sin_addr.s_addr = inet_addr("192.168.39.1");
#endif


	if(sendfd < 0 || recvfd < 0)
		fatal("Could not open socket: %d - %m\n", errno);
#if 0
	int32_t bindret = bind(recvfd, (sockaddr *)&addr, sizeof(addr));
	if(bindret < 0)
		fatal("NO BIND! %d - %m\n", errno);
#endif

	warn << "Waiting...\n";
	fdcb(recvfd, selread, wrap(get_packet));
	amain();
}
