/*
 * Copyright (C) 2009 Arun Seehra (aseehra@cs.utexas.edu)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

%#include "icp_types.h"
%#include "icp_ps.h"

enum irp_reply_stat
{
	IRP_ACK = 0,
	IRP_OPEN_ACK = 1,
	IRP_MALFORMED_MSG = 2,
	IRP_UNKNOWN_NEIGHBOR = 3,
	IRP_PS_OPEN_ACK = 4
};

enum irp_neighbor_type
{
	IRP_PROVIDER = 0,
	IRP_PEER = 1,
	IRP_CUSTOMER = 2
};

typedef icp_certificate icp_cert_vec<>;
typedef icp_memcert icp_memcert_vec<>;

struct irp_open
{
	icp_pubkey realmid;
	irp_neighbor_type type;
};

struct irp_ps_open
{
	icp_pubkey realmid;
	irp_neighbor_type type;
	icp_cert_vec certs;
	icp_memcert_vec memcerts;
};

struct irp_update
{
	icp_pubkey realmid;
	unsigned hyper timestamp;
	icp_cert_vec certs;
	icp_memcert_vec memcerts;
};

union irp_reply switch(irp_reply_stat status)
{
	case IRP_ACK:
		void;
	case IRP_OPEN_ACK:
		irp_open open;
	case IRP_PS_OPEN_ACK: //We reply specially if we're the pathserver, since we have a constant list of partialpaths to give...
		irp_ps_open ps_open;
	default:
		void;
};

program ICING_SIRP
{
	version SIRP_VERS
	{
		irp_reply IRP_OPEN(irp_open) = 0;
		irp_reply IRP_UPDATE(irp_update) = 1;
		irp_reply IRP_KEEPALIVE(irp_open) = 2;
	} = 1;
} = 2;

program ICING_SIRP_INTERFACE
{
	version SIRP_INTERFACE_VERS
	{
		irp_reply SIRP_UPDATE(irp_update) = 0;
	} = 1;
} = 3;
