#include "util.h"
#include "pathserver.h"

char *file;
char *hostname;
bool ps_req;

void print_result(ptr<ps_reply> reply)
{
	switch(reply->status)
	{
		case PS_PATH_FOUND:
		{
			icp_path_vec &paths = *reply->paths;
			print_paths(paths);
			break;
		}
		case PS_SERVER_FOUND:
		{
			ps_nextps &res = *reply->nextps;

			warn << res.host << ":\n";

			print_paths(res.paths);
			break;
		}
		case PS_UNKNOWN_HOST:
			warn << "Unknown host\n";
			break;
		case PS_INVALID_ARG:
			warn << "Invalid arguments\n";
			break;
		case PS_NO_PATH:
			warn << "No path\n";
			break;
		case PS_UNKNOWN_DOMAIN:
			warn << "Unknown domain\n";
			break;
		default:
			warn << "Unknown error\n";
	}
	exit(0);
}

void get_callback(ptr<ps_reply> reply, clnt_stat err)
{
	switch(err)
	{
		case RPC_SUCCESS:
			print_result(reply);
			break;
		default:
			fatal << "ERROR\n";
	}
}

void make_call(ptr<aclnt> clnt)
{
	ptr<icp_path> clpath = build_path_from_file(file);

	ps_partialpath_vec partialpaths;
	ps_partialpath partial;
	partial.realm = pub_from_file("e");
	partial.paths.push_back(*clpath);
	partialpaths.push_back(partial);

	ptr<ps_reply> reply = New refcounted<ps_reply>();
	
	ptr<ps_getargs> args = New refcounted <ps_getargs>();
	args->host = str("master.h.com");
	args->partialpaths = partialpaths;

	if(ps_req)
		clnt->call(
				FIND_PATHTO_PS,
				args,
				reply,
				wrap(get_callback, reply));
	else
		clnt->call(
				FIND_PATHTO_HOST,
				args,
				reply,
				wrap(get_callback, reply));
}

void connected(int32_t fd)
{
	if(fd < 0) 
		fatal("FD BAD\nerrno: %d %m\n", errno);

	ptr<aclnt> clnt = init_tcp_client(fd, icing_pathserver_1); 

	make_call(clnt);
}

int main(int32_t argc, char **argv)
{
#if 0
	int32_t fd = inetsocket(SOCK_STREAM);
	if(!fd)
		fatal << "UNABLE TO INITIALIZE SOCKET\n";

	struct sockaddr_in addr;
	addr.sin_family = AF_INET;
	addr.sin_port = htons(1025);
	addr.sin_addr.s_addr = inet_addr("127.0.0.1");
	bzero(&addr.sin_zero, sizeof(addr.sin_zero));

	if(connect(fd, (struct sockaddr *)&addr, sizeof(struct sockaddr_in)) != 0)
		fatal << "CONNECT FAILED\n";

	connected(fd);
#endif

	if(argc < 6)
	{
		warn << "Usage: test_client <ip addr> <port> <path file> <PS|HOST> <host>\n";
		return 1;
	}
	uint16_t port = atoi(argv[2]);
	hostname = argv[5];
	file = argv[3];
	if(strncmp(argv[4], "PS", 2) == 0)
		ps_req = true;
	else
		ps_req = false;
	tcpconnect(argv[1], port, wrap(&connected));
	amain();
	return 0;
}
