#include "async.h"
#include "arpc.h"
#include "crypt.h"
#include "icinghdr.h"
#include "icingsec.h"
#include <cstring>
#include <cstdio>

void
printPath(RealmVoq *path, int32_t numrealms)
{
	uint8_t *raw = reinterpret_cast<uint8_t *>(&numrealms);
	printf("Path len:\n");
	for(uint32_t i = 0; i < 4; i++)
		printf("%02X", raw[i] & 0xFF);
	printf("\n");

	raw = reinterpret_cast<uint8_t *>(path);
	printf("Path:\n");
	for(uint32_t i = 0; i < numrealms * sizeof(RealmVoq); i++)
		printf("%02X", raw[i] & 0xFF);
	printf("\n");
}

void
printPocs(POC *pocs, int32_t numrealms)
{
	uint8_t *raw = reinterpret_cast<uint8_t *>(pocs);
	printf("POCS:\n");
	for(uint32_t i = 0; i < numrealms * sizeof(POC); i++)
		printf("%02X", raw[i] & 0xFF);
	printf("\n");
}

void
build(int32_t numrealms)
{
	assert(numrealms > 0);

	RealmVoq *path = new RealmVoq[numrealms];
	for(int i = 0; i < numrealms; i++)
	{
		path[i].voq = 0;
		
		ecdh_priv pk;
		chopped_ecpk_raw pubraw;
		pk.to_chopped_ecpk_raw(pubraw);
		memcpy(path[i].realmid.bytes, pubraw, RealmIDWire::SIZE);
	}

	printPath(path, numrealms);

	POC *pocs = new POC[numrealms - 1];
	for(int i = 1; i < numrealms; i++)
	{
		RealmSecret secret, key;
		uint32_t v = 16, g = 1;
		memset(secret.bytes, i, RealmSecret::SIZE);
		derive_poc_key(secret, g, v, 0, key);
		poc_from_path(pocs[i - 1], &key, reinterpret_cast<Path *>(path), numrealms);
	}

	printPocs(pocs, numrealms);

	delete [] path;
	delete [] pocs;
}

int
main(int32_t argc, char **argv)
{
	if (argc < 2)
	{
		warn << "Usage: pathgen <number of realms>\n";
		return 1;
	}

	int32_t numrealms = atoi(argv[1]);
	build(numrealms);
}
