# This file ssh's into each AWS box, pulls from the repo, kills running experiments, then recompiles:
#    - The control "stuff"
#    - The click modules
#    - The "send" and "receive" programs
# It then commits and pushes up any unstaged changes to the repo.
# Then, ~/.ssh/config is setup from the ~/.ssh/config_template template
require 'rubygems'
require 'AWS'

#Takes in string of command(delimited by \ns)
# - Removes comments/empty lines
# - Strips off anything after a # in a line
def setupCommand(cmd)
  # commentsRemoved contains an array of commands, with everything after a # stripped off
  commentsRemoved = cmd.split("\n").map do |s|
    s = s.strip
    idx = s.index("#")    
    # If we have a #, return everything before it
    # else, return the original string
    if !idx.nil?
      s[0,idx]
    else
      s
    end        
  end
  
  # Remove empty commands
  emptyRemoved = commentsRemoved.reject do |s|
    s.length == 0
  end
  
  # Concatenate commands together to run on a single line w/ bash
  return emptyRemoved.join(" && ")
end

ec2 = AWS::EC2::Base.new(:access_key_id => ENV["AMAZON_ACCESS_KEY_ID"], :secret_access_key => ENV["AMAZON_SECRET_ACCESS_KEY"])



# Add in SSH keys
`(ssh-add && ssh-add ~/.ec2/ec2*.pem) 2>&1`

instance_private_ips = []
instance_public_ips = []

hash = ec2.describe_instances
#puts hash.inspect
reservationSet = hash['reservationSet']['item']
raise "No reservations!" if reservationSet.size == 0
reservationSet.each do |reservation|
  instancesSet = reservation['instancesSet']['item']
  instancesSet.each do |instance|
    next if instance['instanceState']['code'] != "16" or instance['ipAddress'] == "50.18.122.144"
    instance_private_ips << instance['privateIpAddress']
    instance_public_ips << instance['ipAddress']
  end
end

puts "Found #{instance_private_ips.size} instances"

puts "Private ips: #{instance_private_ips.inspect}"
puts "Public ips: #{instance_public_ips.inspect}"

#; sudo killall pathserver
bashScript = <<EOF
# Remote:
# Kill all running clicks/pathserver
sudo killall click  ; sudo killall send ; sudo killall receive ; true

# Stop linux from messing with tun/tap
sudo sysctl -w net.ipv4.conf.default.rp_filter=0
sudo sysctl -w net.ipv4.conf.all.rp_filter=0

# Make sure we're on the right branch, pull down
cd ~/icing
git checkout master
git pull origin master

# Build control
cd ~/icing/code/control/build
make -j 3 # Might need to cd into build...?
make install

# Build click modules
cd ~/icing/code/click
make -j 3

# Build send/receive
cd ~/controlplane/eval/config/nsdi10
clang -lrt send.c -lrt -o send
clang receive.c -lrt -o receive

# Get Zoom
#cd ~
#wget http://www.rotateright.com/Zoom_2.0.0_Linux-i386.tar.gz
#tar xfz Zoom_2.0.0_Linux-i386.tar.gz

# ------- Commit commands
cd ~/icing
git commit -am "Committing on aws" # Need to fill in machine
git pull origin mikemiller-dev
git push origin mikemiller-dev


EOF
sshCommand = setupCommand(bashScript)
puts sshCommand

instance_public_ips.each_with_index do |publicIP, index|
  puts "Setting up IP #{publicIP}"
  `ssh ubuntu@#{publicIP} -o StrictHostKeyChecking=no '#{sshCommand}'`
end

fileData = File.read(File.expand_path("~/.ssh/config_template"))
1.upto(instance_private_ips.size) do |i|
  fileData << "Host aws#{i}\n"
  fileData << "Hostname #{instance_public_ips[i-1]}\n"
end

fileData << "\n"
File.open(File.expand_path("~/.ssh/config"), 'w') { |f| f.write(fileData) }
