require 'rubygems'
require 'AWS'

KEYPAIR = "ec2-keypair"
INSTANCE_TYPE = "c1.medium"
NUM_INSTANCES = 6


#Boots instance of AMI 'ami' on EC2, and returns the instance ID
def runInstance(ec2, ami)
  response = ec2.run_instances :image_id => ami, :key_name => KEYPAIR, :instance_type => INSTANCE_TYPE
  puts response.inspect
  a = response["instancesSet"]["item"]
  puts a.inspect
  raise "More than one instance created" if a.size != 1
  return a[0]["instanceId"]
end

def setupInstances
  instanceArray = []
  ec2 = AWS::EC2::Base.new(:access_key_id => ENV["AMAZON_ACCESS_KEY_ID"], :secret_access_key => ENV["AMAZON_SECRET_ACCESS_KEY"])
  
  # Spawn 6 instances
  1.upto(NUM_INSTANCES) do |i|
    instanceArray << runInstance(ec2,"ami-95aafad0")
  end
  
  puts "Instances spawned... waiting for startup"
  
  # Keep looping until all instances are fully operational
  while true
    hash = ec2.describe_instances(:instance_id => instanceArray)
    reservationSet = hash['reservationSet']['item']
    raise "No reservations!" if reservationSet.size == 0
    break if reservationSet.all? do |reservation|
      instancesSet = reservation['instancesSet']['item']
      instancesSet.all? do |instance|
        instance['instanceState']['code'] == "16" 
      end
    end
  end
  
  
  
  puts "Instances up!"
  
  puts "Press CTRL+C to terminate!"
  
  trap('INT') do 
    puts "Terminating instances..."
    
    #Terminate the instances
    ec2.terminate_instances(:instance_id  => instanceArray).inspect
    
    puts "Instances terminated."
    #Exit
    exit
  end
  
  
  # Sit around
  while true
    sleep 1
  end
end

setupInstances