#include <stdio.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <net/if.h>
#include <sys/time.h>
#include <time.h>
#include <errno.h>

#define PORT 9990
#define NUM_MESSAGES 1e10 // 1e7 == 10 million

void ERROR(char *str){
  printf("%s\n",str);
  exit(-1);
}

int main(int argc, char *argv[]){
  setbuf(stdout, NULL);
  int sock;
  struct sockaddr_in client;

  if(argc != 1){
    printf("Got %d args\n",argc);
    ERROR("Usage: receive [no arguments]");
  }

  if((sock= socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0){
    ERROR("Couldn't create socket");
  }

  memset(&client, 0, sizeof(client));
  client.sin_family = AF_INET;
  //client.sin_addr.s_addr = inet_addr("1.0.0.1");
  client.sin_addr.s_addr = htonl(INADDR_ANY);
  client.sin_port = htons(PORT);

  struct ifreq ifr;
  memset(&ifr, 0, sizeof(ifr));
  snprintf(ifr.ifr_name, sizeof(ifr.ifr_name), "tun0");
  if(setsockopt(sock, SOL_SOCKET, SO_BINDTODEVICE, (void *)&ifr, sizeof(ifr)) <0){
    perror("Error binding to device");
    ERROR("Couldn't set sock opt to bind to device!");
  }

  if( bind(sock, (struct sockaddr *) &client, sizeof(client)) < 0)
    ERROR("Couldn't bind socket");

  
  char buf[1000];

  struct timespec t1;
  clock_gettime(CLOCK_REALTIME, &t1);

  for(uint32_t i=0; i<NUM_MESSAGES; i++){
    if(i%50 == 0){
      struct timespec t2;
      clock_gettime(CLOCK_REALTIME,&t2);
      
      time_t elapsedSeconds = t2.tv_sec - t1.tv_sec;
      long elapsedNanoseconds = t2.tv_nsec - t1.tv_nsec;
      double elapsed = (double)elapsedSeconds+1e-9*elapsedNanoseconds;
      printf("%lf seconds passed\n",elapsed);
      printf("Throughput: %f packets per second\n", 50/elapsed);

      t1 = t2;
    }
    const void *msg = buf;
    const int len = sizeof(buf);
    const int flags = 0;

    socklen_t clientSize = sizeof(client);
    if( recvfrom(sock, buf, len, flags, (struct sockaddr *) &client, &clientSize) < 0 ){
      ERROR("Size didn't match");
    }
    uint32_t recv = ntohl(*((uint32_t *)buf));
    if( recv != i){
      //printf("Got packet with value %d, expected %d\n", recv, i);
      if(recv >=NUM_MESSAGES)
	continue;
      //      i = recv;
      // ERROR("Invalid packet recieved");
    }
    //           printf("Hi there! %d\n",i);
  }
}
