#!/usr/bin/perl -w

use strict;
use Getopt::Long;

my $fwd = 0;
my $translator = 0;

my $icingdir = "/home/icing/nfs/icing/icing";
#my $icingdir = "/Users/aseehra/Documents/School/phd/walfish/icing";
my $configdir = "$icingdir/code/control/config";
my $sshcmd = "ssh";
my $setupfile = "$configdir/setup";

sub getRealm
{
	my $str = $_[0];
	$str =~ s/(..)/$1 /g;
	return $str
}

sub configTranslator
{
	my $configfile = "$configdir/translator/$translator";
	my $outputfile = "$configdir/translator/$translator.click";
	my $templatefile = "$configdir/translator.tmpl";

	my $key;
	my $realmid;
	my $neighbor;
	my $mac;

	open CONFIG, "$configfile" || die "Could not open config file\n";
	open TEMPLATE, "$templatefile" || die "Could not open template\n";
	open OUTFILE, ">$outputfile" || die "Could not open out file\n";

	while(my $line = <CONFIG>)
	{
		chomp $line;
		if($line =~ m/neighbor\s+(\S+)/) { $neighbor = $1; }
		if($line =~ m/realmid\s+(\S+)/) { $realmid = getRealm($1); }
		if($line =~ m/mac\s+(\S+)/) { $mac = $1; }
		if($line =~ m/key\s+(\S+)/) { $key = getRealm($1); }
	}

	local $/;
	my $template = <TEMPLATE>;
	$template =~ s/==HOST_MAC==/$mac/;
	$template =~ s/==NEIGHBOR_MAC==/$neighbor/;
	$template =~ s/==K==/$key/;
	$template =~ s/==REALM==/$realmid/;

	print OUTFILE "$template\n";
	
	close CONFIG;
	close TEMPLATE;
	close OUTFILE;
}

sub configFwd
{
	my $configfile = "$configdir/fwd/$fwd";
	my $outputfile = "$configdir/fwd/$fwd.click";
	my $templatefile = "$configdir/fwd.tmpl";

	my @neighbors;
	my @macs;
	my $key;
	my $realmid;

	open CONFIG, "$configfile" || die "Could not open config ile\n";
	open TEMPLATE, "$templatefile" || die "Could not open template\n";
	open OUTFILE, ">$outputfile" || die "Could not open out file\n";

	while(my $line = <CONFIG>)
	{
		chomp $line;
		if($line =~ m/neighbor\s+(\S+)\s+(\S+)/)
		{
			my @neighbor;
			push @neighbor, (getRealm($1), $2);
			push @neighbors, \@neighbor;
		}
		if($line =~ m/mac\s+(\S+)/)
		{
			push @macs, $1;
		}
		if($line =~ m/key\s+(\S+)/)
		{
			$key = getRealm($1);
		}
		if($line =~ m/realmid\s+(\S+)/)
		{
			$realmid = getRealm($1);
		}
	}


	local $/;
	my $template = <TEMPLATE>;
	for(my $i = 0; $i < scalar(@neighbors); $i++)
	{
		$template =~ s/==REALM_$i==/$neighbors[$i][0]/;
		$template =~ s/==ETH_$i==/$neighbors[$i][1]/;
	}

	for(my $j = 0; $j < scalar(@macs); $j++)
	{
		$template =~ s/==MAC_$j==/$macs[$j]/;
	}

	$template =~ s/==K==/$key/;
	$template =~ s/==REALM==/$realmid/;

	print OUTFILE "$template\n";

	close CONFIG;
	close TEMPLATE;
	close OUTFILE;
}

GetOptions(
		"fwd=i" => \$fwd,
		"translator=i" => \$translator);

if($fwd) {configFwd();}
if($translator) {configTranslator();}
