//  This is a quick-and-dirty way to translate between icing-in-udp and 
//      icing-in-ethernet. eventually, we will expand this
//      infrastructure to make it so that we will be able to send
//      unmodified UDP packets inside icing.
//
//  Host applications that want to *send* icing packets should package
//      their icing packets in a UDP datagram, call sendto, and address
//	the packets to "192.168.39.0/255", but *not* "192.168.39.1" [that
//	is a special address]. Doing so will cause the packets to be
//	processed by this click module, which will strip the IP and UDP headers,
//	and stick the unmodified icing packet in an appropriate ethernet
//	frame.
//
//  Host applications that want to *receive* icing packets should bind
//      to (inet_addr("192.168.39.1", htons(0x2345)). Any icing packets that 
//      are received on eth1 will go to host applications that are so bound.
//
//  This click module should be run on a box that is simulating an icing
//  end-host
//
require(icing)

define($HOST_MAC 00:00:ca:fe:00:aa,
       $NEIGH_MAC 00:00:ca:fe:00:35,
       $HOST_K \< BB 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15  \>,
       $HOST_REALMID \< BB 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19  \>)

con::IcingConfig(K $HOST_K, OUR_REALMID $HOST_REALMID, MODE L3)
 
tun::KernelTun(192.168.39.1/24)
    -> StripIPHeader()
    -> Strip(8) 
    -> ic1::IcingCheck()  // sanity
// NOTA BENE: the next two lines are for debugging. they check things
// that the *forwarder* should be checking
//    -> cc::CheckCurrentRealm(CONFIG con) 
//    -> vv::VtagVerify(CONFIG con)
//    -> IcingPrint()
    -> EtherEncap(0x9090, $HOST_MAC, $NEIGH_MAC)
    -> Queue()
    -> ToDevice(eth1)

FromDevice(eth1)
    -> cl::Classifier(12/9090, -)
    -> Strip(14)
    -> ic2::IcingCheck()
    -> cc::CheckCurrentRealm(CONFIG con)
    -> cd::CheckDestination()
    -> vv::VtagVerify(CONFIG con) 
    -> IcingPrint()
    -> UDPIPEncap(192.168.39.253, 0x8181, 192.168.39.1, 0x2345)
    -> tun

cl[1] -> Print('received non-icing packet; dropping') -> Discard

ic1[1] -> Print('bad icing packet / non-icing packet sent from host')
       -> Discard

ic2[1] -> Print('bad icing packet / non-icing packet received from network')
       -> Discard

cc[1] -> Print('realm check failed') -> Discard
cd[1] -> Print("Packet ought to be, but isn't, at last hop") -> Discard
vv[1] -> Print('vtag verification failed') -> Discard


