#!/usr/bin/perl

use IO::Socket;

my $file = shift || "config/runmaster.config";

my %ips;
my %paths;

my $sshcmd = "ssh -x -o StrictHostKeyChecking=no -o BatchMode=yes ";
my $scpcmd = "scp -o StrictHostKeyChecking=no -o BatchMode=yes ";

open FILE, $file || die "file not found";

while(my $line = <FILE>)
{
	chomp $line;
	if($line =~ m/la_ip\s+(\S+)/) { $ips{la} = $1; }
	if($line =~ m/ny_ip\s+(\S+)/) { $ips{ny} = $1; }
	if($line =~ m/houston_ip\s+(\S+)/) { $ips{houston} = $1; }
	if($line =~ m/la_path\s+(\S+)/) { $paths{la} = $1; }
	if($line =~ m/ny_path\s+(\S+)/) { $paths{ny} = $1; }
	if($line =~ m/houston_path\s+(\S+)/) { $paths{houston} = $1; }
	if($line =~ m/austin_ip\s+(\S+)/) { $ips{austin} = $1; }
	if($line =~ m/austin_path\s+(\S+)/) { $paths{austin} = $1; }
}

sub startNodes
{
	my $control = "~/controlplane/bin"
	my $continue = 'code/control/eval/config';
	system "$sshcmd $ips{la} 'sudo $paths{la}/$continue/sudo_ld click $paths{la}/$continue/la.click' > /dev/null 2>/dev/null &";
	system "$sshcmd $ips{ny} 'sudo $paths{ny}/$continue/sudo_ld click $paths{ny}/$continue/ny.click' > /dev/null 2>/dev/null &";
	system "$sshcmd $ips{houston} 'sudo $paths{houston}/$continue/sudo_ld click $paths{houston}/$continue/houston.click' > /dev/null 2>/dev/null &";
	system "$sshcmd $ips{austin} 'cd $control && ./i2rx' &";
}

sub stopNodes
{
	system "$sshcmd $ips{la} 'sudo killall click'";
	system "$sshcmd $ips{ny} 'sudo killall click'";
	system "$sshcmd $ips{houston} 'sudo killall click'";
	system "$sshcmd $ips{austin} 'sudo killall i2rx'";
}

sub runTrial
{
	my $controlplane = "/home/icing/controlplane";
	my $file = "$_[0]_p1.dat" || "out";
	my $file2 = "$_[0]_p2.dat" || "out";
	my $cmd = "cd $controlplane/bin && ./i2tx $controlplane/eval/results/$file";
	my $cmd2 = "cd $controlplane/bin && ./i2tx $controlplane/eval/results/$file2";
	my $clear = "write rt.remove_non_neigh\n";
	my $line;

	system $cmd;
	system $cmd2;
	print "Clearing Tables\n";
	my $s = new IO::Socket::INET(
		PeerAddr => $ips{la},
		PeerPort => 4662,
		Proto => 'tcp');
	$s or die 'no socket';
	print $s $clear;
	$line = <$s>;
	$line = <$s>;
	close $s;

	my $s = new IO::Socket::INET(
		PeerAddr => $ips{ny},
		PeerPort => 4662,
		Proto => 'tcp');
	$s or die 'no socket';
	print $s $clear;
	$line = <$s>;
	$line = <$s>;
	close $s;

	$s = new IO::Socket::INET(
		PeerAddr => $ips{houston},
		PeerPort => 4662,
		Proto => 'tcp');
	$s or die 'no socket';
	print $s $clear;
	$line = <$s>;
	$line = <$s>;
	close $s;
}

sub runExperiment
{
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
                                                localtime(time);
	$year += 1900;
	$mon += 1;
	my $file = "wan.$year.$mon.$mday\_$hour.$min.$sec";
	for(my $i = 0; $i < 1000; $i++)
	{
		print "Running Trial $i\n";
		runTrial($file);
	}
}

print "Starting nodes\n";
startNodes();
sleep(10);
runExperiment();
sleep(3);
print "Stopping nodes\n";
stopNodes();
