#include "async.h"
#include "icinghdr.h"
#include "icingsec.h"
#include "tt.h"
#include <cstdlib>

FILE *out = NULL;

enum { NUM_LENS = 6, TIME_TO_RUN = 60, TIMES_TO_RUN = 10, NUM_REALMS = 100, NUM_PACKETS = 20 };
uint32_t lens[] = { 3, 7, 12, 20, 30, 37 };
uint8_t pkts[NUM_PACKETS][1500];
POC *pocs[NUM_PACKETS];
ecdh_priv my_priv;

RealmIDWire all_realms[NUM_REALMS];

void
init_all_realms()
{
	for(uint32_t i = 0; i < NUM_REALMS; i++)
	{
		ecdh_priv p;
		chopped_ecpk_raw raw;
		ecdh_pub::to_chopped_ecpk_raw(raw, *(p.public_key()));
		memcpy(all_realms[i].bytes, raw, RealmIDWire::SIZE);
	}
}

void
generate_path(uint32_t len, RealmVoq *realms)
{
	for(uint32_t i = 0; i < len ; i++)
	{
		realms[i].voq = 0;
		uint32_t cur = rand() % NUM_REALMS;
		memcpy(realms[i].realmid.bytes, all_realms[cur].bytes, RealmIDWire::SIZE);
	}
}

void
generate_secret(RealmSecret &s)
{
	for(uint32_t i = 0; i < RealmSecret::SIZE; i++)
		s.bytes[i] = rand();
}

void
build_pkt(uint32_t len)
{
	for(int i = 0; i < NUM_PACKETS; i++)
	{
		uint8_t *buf = pkts[i];

		IcingHdr1 *hdr1 = reinterpret_cast<IcingHdr1 *>(buf);
		Path *path = get_path(hdr1);

		RealmVoq *realms = reinterpret_cast<RealmVoq *>(path);
		generate_path(len, realms);
		
		pocs[i] = new POC[len - 1];
		for(uint32_t j = 0; j < len - 1; j++)
		{
			RealmSecret s;
			generate_secret(s);
			poc_from_path(pocs[i][j], &s, path, len);
		}

		hdr1->icing_ver = ICING_CURRENT_VERSION;
		hdr1->path_len = static_cast<uint8_t>(len);
		hdr1->path_idx = 0;

		IcingHdr2 *hdr2 = get_icinghdr2(hdr1);
		hdr2->proto = 8;
		hdr2->pkt_len = htons(1500);

	}
}

void
singleTrial()
{
	uint8_t *buf = pkts[rand() % NUM_PACKETS];
	POC *poc = pocs[rand() % NUM_PACKETS];

	IcingHdr1 *h1 = reinterpret_cast<IcingHdr1 *>(buf);
	bzero(get_vtagvector(h1), sizeof(VtagVector) * (h1->path_len - 1));
	init_vtags(h1, poc, my_priv);
}

void
cleanup(uint32_t len)
{
	for(int i = 0; i < NUM_PACKETS; i++)
		delete pocs[i];
}

int main(int argc, char **argv)
{
	srand((unsigned) time(NULL));
	if(argc > 1)
		out = fopen(argv[1], "w+");

	printf("Generating realmids ... ");
	fflush(stdout);
	init_all_realms();
	printf("DONE\n");
	fflush(stdout);

	if(out)
	{
		fprintf(out, "#path create\n#path_length, throughput (packets/s)\n");
		fflush(out);
	}
	for(uint32_t i = 0; i < NUM_LENS; i++)
	{

		for(uint32_t j = 0; j < TIMES_TO_RUN; j++)
		{
			printf("Creating paths and pocs of length %d ... ", lens[i]);
			fflush(stdout);
			build_pkt(lens[i]);
			printf("DONE\n");
			fflush(stdout);
			printf("Running experiment for %d seconds ... ", TIME_TO_RUN);
			fflush(stdout);
			
			uint32_t count = 0;
			double tp;
			uint64_t start, end;
			start = get_now();
			end = start + (TIME_TO_RUN * 1000000);

			while(get_now() < end)
			{
				singleTrial();
				count++;
			}

			tp = count * 1.0;
			tp /= TIME_TO_RUN;

			if(out)
			{
				fprintf(out, "%d, %f\n", lens[i], tp);
				fflush(out);
			}

			printf("DONE: %d, %f\n", count, tp);
			fflush(stdout);

			printf("cleaning up ... ");
			fflush(stdout);
			cleanup(i);
			printf("DONE\n\n");
			fflush(stdout);
		}
	}
}
