#include "async.h"
#include "icinghdr.h"
#include "icingsec.h"
#include "tt.h"
#include <cstdlib>
#include "localkey.h"

FILE *out = NULL;

enum { NUM_LENS = 6, TIMES_TO_RUN = 100, NUM_REALMS = 100 };
uint32_t lens[] = { 3, 7, 12, 20, 30, 37 };
uint8_t buf[1500];
ecdh_priv my_priv;

keyhash_t hash;
RealmIDWire all_realms[NUM_REALMS];

void
key_calc(SharedKey &d, const RealmIDWire &rid, const RealmIDWire *l = NULL)
{
	LocalKey *k = hash[rid];
	assert(k);
	memcpy(d, k->k, sizeof(SharedKey));
}

void
init_all_realms()
{
	for(uint32_t i = 0; i < NUM_REALMS; i++)
	{
		ecdh_priv p;
		chopped_ecpk_raw raw;
		ecdh_pub::to_chopped_ecpk_raw(raw, *(p.public_key()));

		LocalKey *k = new LocalKey;
		memcpy(k->rid.bytes, all_realms[i].bytes, RealmIDWire::SIZE);
		set_sharedkey(k->k, all_realms[i], my_priv);
		hash.insert(k);
	}
}

void
generate_path(uint32_t len, RealmVoq *realms)
{
	for(uint32_t i = 0; i < len ; i++)
	{
		realms[i].voq = 0x7667;
		uint32_t cur = rand() % NUM_REALMS;
		memcpy(realms[i].realmid.bytes, all_realms[cur].bytes, RealmIDWire::SIZE);
	}
}

void
generate_secret(RealmSecret &s)
{
	for(uint32_t i = 0; i < RealmSecret::SIZE; i++)
		s.bytes[i] = rand();
}

void
build_pkt(uint32_t len, uint64_t &min)
//build_pkt(uint32_t len)
{
	IcingHdr1 *hdr1 = reinterpret_cast<IcingHdr1 *>(buf);
	Path *path = get_path(hdr1);

	RealmVoq *realms = reinterpret_cast<RealmVoq *>(path);
	generate_path(len, realms);
	
	POC *pocs = new POC[len - 1];
	for(uint32_t i = 0; i < len - 1; i++)
	{
		RealmSecret s;
		generate_secret(s);
		poc_from_path(pocs[i], &s, path, len);
	}

	hdr1->icing_ver = ICING_CURRENT_VERSION;
	hdr1->path_len = static_cast<uint8_t>(len);
	hdr1->path_idx = 0;

	IcingHdr2 *hdr2 = get_icinghdr2(hdr1);
	hdr2->proto = 8;
	hdr2->pkt_len = htons(1500);

	//uint8_t H[HASHLEN];
	//RealmSecret s;
	//POC p;
	//generate_secret(s);
	uint64_t before = get_now();
	init_vtags(hdr1, pocs, my_priv, &key_calc);
	hdr1->path_idx ++;
	//cbcmac_128(pocs[0], H, (uint8_t *)(&get_vtagvector(hdr1)[0]));
	//construct_hash_247(H, hdr1);
	//pmac(s.bytes, p, realms[2].realmid.bytes, RealmIDWire::SIZE);
	uint64_t after = get_now();
	uint64_t delta = after - before;
	if(min == 0 || delta < min)
		min = delta;
	if(out)
	{
		fprintf(out, "%d, %llu\n", len, delta);
		fflush(out);
	}

	delete[] pocs;
}

int main(int argc, char **argv)
{
	srand((unsigned) time(NULL));
	if(argc > 1)
		out = fopen(argv[1], "w+");

	printf("Generating realmids ... ");
	fflush(stdout);
	init_all_realms();
	printf("DONE\n");
	fflush(stdout);

	if(out)
	{
		fprintf(out, "#path create\n#path_length, latency (us)\n");
		fflush(out);
	}
	for(uint32_t i = 0; i < NUM_LENS; i++)
	{
		uint64_t min = 0;
		printf("Running experiment for path length %d ... ", lens[i]);
		fflush(stdout);
		for(uint32_t j = 0; j < TIMES_TO_RUN; j++)
			build_pkt(lens[i], min);

		printf("min =  %llu\n", min);
		fflush(stdout);
	}
}
