#include "async.h"
#include "icinghdr.h"
#include "icingsec.h"
#include "tt.h"
#include <cstdlib>

enum { NUM_REALMS = 10, TIMES_TO_RUN = 3000 };

RealmIDWire realms[NUM_REALMS];
FILE *out = NULL;

void
run_sharedkey(ecdh_priv &key, uint64_t &min)
{
	uint32_t cur = rand() % NUM_REALMS;
	assert(cur < NUM_REALMS);

	SharedKey k;

	uint64_t prev = get_now();
	set_sharedkey(k, realms[cur], key);
	uint64_t after = get_now();

	uint64_t delta = after - prev;
	if(min == 0 || delta < min)
		min = delta;

	if(out)
	{
		fprintf(out, "%llu\n", delta);
		fflush(out);
	}
}

int
main(int argc, char **argv)
{
	if(argc > 1)
		out = fopen(argv[1], "w+");

	srand((unsigned) time(NULL));

	printf("Generating realms ... ");
	fflush(stdout);
	for(uint32_t i = 0; i < NUM_REALMS; i++)
	{
		ecdh_priv p;
		chopped_ecpk_raw raw;
		ecdh_pub::to_chopped_ecpk_raw(raw, *(p.public_key()));
		memcpy(realms[i].bytes, raw, RealmIDWire::SIZE);
	}
	printf("DONE\n");
	fflush(stdout);

	printf("Generating my private key ... ");
	fflush(stdout);
	ecdh_priv key;
	printf("DONE\n");
	fflush(stdout);

	if(out)
	{
		fprintf(out, "#shared key gen time\n#latency (us)\n");
		fflush(out);
	}

	printf("Running set_sharedkey ... ");
	fflush(stdout);
	uint64_t min = 0;
	for(uint32_t i = 0; i < TIMES_TO_RUN; i++)
		run_sharedkey(key, min);
	printf("DONE\n");
	fflush(stdout);

	printf("Min cost: %llu\n", min);
	fflush(stdout);
	if(out)
		fclose(out);
}
