/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Module: aes_keyexpansion.v
 * Project: ICING
 * Author: Jad Naous <jnaous@stanford.edu>
 * Description: Performs the KeyExpansion operation of the aes algorithm
 *              
 ******************************************************************************/
`timescale 1ns/1ps

module aes_keyexpansion
  #(parameter WIDTH = 128,
    parameter INVERSE = 0)
    (input  [0:WIDTH - 1] prev_round_key,
     output [0:WIDTH - 1] round_key,
     input [3:0]          round,

     input                clk);

`include "aes_funcs.v.inc"

   wire [0:31] rotword_0, xorcon_0;
   reg [0:31]  subwords_0;
   reg [0:WIDTH-1] prev_round_key_latched;
   reg [3:0]       round_latched;

   assign xorcon_0   = subwords_0 ^ rcon(round_latched);
   always @(posedge clk) begin
      subwords_0 <= {sbox(rotword_0[0:7]),
                     sbox(rotword_0[8:15]),
                     sbox(rotword_0[16:23]),
                     sbox(rotword_0[24:31])};
      prev_round_key_latched <= prev_round_key;
      round_latched <= round;
   end
   
   generate
      if(!INVERSE) begin
         assign rotword_0           = rotword(prev_round_key[96:127]);
         assign round_key[0:31]     = prev_round_key_latched[0:31]   ^ xorcon_0;
         assign round_key[32:63]    = prev_round_key_latched[32:63]  ^ round_key[0:31];
         assign round_key[64:95]    = prev_round_key_latched[64:95]  ^ round_key[32:63];
         assign round_key[96:127]   = prev_round_key_latched[96:127] ^ round_key[64:95];
      end // if (!inverse)

      else begin
         assign rotword_0           = rotword(prev_round_key[96:127] ^ prev_round_key[64:95]);
         assign round_key[96:127]   = prev_round_key_latched[96:127] ^ prev_round_key_latched[64:95];
         assign round_key[64:95]    = prev_round_key_latched[64:95]  ^ prev_round_key_latched[32:63];
         assign round_key[32:63]    = prev_round_key_latched[32:63]  ^ prev_round_key_latched[0:31];
         assign round_key[0:31]     = prev_round_key_latched[0:31]   ^ xorcon_0;
      end // else: !if(!inverse)
   endgenerate
endmodule // aes_keyexpansion


