/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Module: aes_round.v
 * Project: ICING
 * Author: Jad Naous <jnaous@stanford.edu>
 * Description: Implements a single AES encryption round
 *              
 ******************************************************************************/
`timescale 1ns/1ps

module aes_round
  #(parameter WIDTH = 128,
    parameter INVERSE = 0,
    parameter ONLY_KEY_EXPANSION = 0)
    (input      [0:WIDTH-1] s_in,
     output     [0:WIDTH-1] s_out,

     input      [0:WIDTH-1] prev_round_key,
     output     [0:WIDTH-1] round_key,
     input      [3:0]       round_number,

     input                  clk);
   
`include "aes_funcs.v.inc"

   reg [3:0] round_number_latched;

   always @(posedge clk) begin
      round_number_latched <= round_number;
   end
   
   aes_keyexpansion #(.INVERSE(INVERSE)) 
     aes_keyexpansion_inst
       (.prev_round_key(prev_round_key),
        .round_key(round_key),
        .clk(clk),
        .round(round_number));

   generate
      if(!ONLY_KEY_EXPANSION) begin
         if(!INVERSE) begin
            wire [0:WIDTH-1] subbytes_out, shiftrows_out, mixcols_out;
            
            aes_subbytes #(.INVERSE (INVERSE)) 
              aes_subbytes
                (.s_in (s_in),
                 .s_out (subbytes_out),
                 .clk(clk));
            
            assign shiftrows_out    = aes_shiftrows(subbytes_out, 0);
            assign mixcols_out      = (round_number_latched != 9) ? aes_mixcolumns(shiftrows_out, 0) : shiftrows_out;
            assign s_out            = aes_addroundkey(mixcols_out, round_key);
         end // if (!INVERSE)
         
         else begin
            wire [0:WIDTH-1] subbytes_out, shiftrows_out, addkey_out;

            assign shiftrows_out = aes_shiftrows(s_in, 1);

            aes_subbytes #(.INVERSE (INVERSE))
              aes_subbytes
                (.s_in (shiftrows_out),
                 .s_out (subbytes_out),
                 .clk(clk));

            assign addkey_out    = aes_addroundkey(subbytes_out, round_key);
            assign s_out         = (round_number_latched != 0) ? aes_mixcolumns(addkey_out, 1) : addkey_out;
         end // else: !if(!INVERSE)
      end // if (!ONLY_KEY_EXPANSION)
      else begin
         reg [0:WIDTH-1] s_out_reg;
         always @(posedge clk) begin
            s_out_reg <= s_in;
         end
         assign s_out = s_out_reg;
      end
   endgenerate
endmodule // aes_round
