/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Module: aes_subbytes.v
 * Project: ICING
 * Author: Jad Naous <jnaous@stanford.edu>
 * Description: Performs the SubBytes operation of the aes-encipher algorithm
 * 
 * From the spec: The SubBytes transformation is a non-linear byte substitution
 * that operates independently on each byte of the State using the S-box...
 *              
 ******************************************************************************/
`timescale 1ns/1ps

module aes_subbytes
  #(parameter WIDTH = 128,
    parameter INVERSE = 0)
    (input      [0:WIDTH - 1] s_in,
     output reg [0:WIDTH - 1] s_out,
     input                    clk);

`include "aes_funcs.v.inc"
   
   generate
      genvar  i;
      for (i=0; i<WIDTH/8; i=i+1) begin:subgen
	 if(!INVERSE)
	   always @(posedge clk) s_out[i*8 +: 8] <= sbox(s_in[i*8 +: 8]);
	 else
	   always @(posedge clk) s_out[i*8 +: 8] <= invsbox(s_in[i*8 +: 8]);
   
/* -----\/----- EXCLUDED -----\/-----
         aes_sbox #(.INVERSE (INVERSE))
	   aes_sbox
	     (.byte_in_0  (s_in [i*16   +: 8]),
	      .byte_in_1  (s_in [i*16+8 +: 8]),
	      .byte_out_0 (s_out[i*16   +: 8]),
	      .byte_out_1 (s_out[i*16+8 +: 8]),
	      .clk        (clk));
 -----/\----- EXCLUDED -----/\----- */
      end
   endgenerate

endmodule // aes_subbytes

