///////////////////////////////////////////////////////////////////////////////
// $Id: udp_reg_grp.v 1947 2007-07-14 23:37:46Z grg $
//
// Module: udp_reg_grp.v
// Project: NF2.1
// Description: predecodes registers for the user datapath and multiplexes 
//              between them
//
///////////////////////////////////////////////////////////////////////////////
`timescale 1ns/1ps

  module udp_reg_grp
    #(parameter CPCI_NF2_DATA_WIDTH       = 32)
   
   ( input                                udp_reg_req,
     input                                udp_reg_rd_wr_L,
     input  [`UDP_REG_ADDR_WIDTH-1:0]      udp_reg_addr,
     input  [CPCI_NF2_DATA_WIDTH-1:0]     udp_reg_wr_data,
     
     output reg [CPCI_NF2_DATA_WIDTH-1:0] udp_reg_rd_data,
     output reg                           udp_reg_ack,

     // --- Interface to input_arbiter
     output reg                           in_arb_reg_req,
     output reg                           in_arb_reg_rd_wr_L,
     output reg [`IN_ARB_REG_ADDR_WIDTH-1:0] in_arb_reg_addr,
     output reg [CPCI_NF2_DATA_WIDTH-1:0] in_arb_reg_wr_data,
     
     input [CPCI_NF2_DATA_WIDTH-1:0]      in_arb_reg_rd_data,
     input                                in_arb_reg_ack,

     // --- Interface to output_port_lookup
     output reg                           op_lut_reg_req,
     output reg                           op_lut_reg_rd_wr_L,
     output reg [`OP_LUT_REG_ADDR_WIDTH-1:0] op_lut_reg_addr,
     output reg [CPCI_NF2_DATA_WIDTH-1:0] op_lut_reg_wr_data,
     
     input [CPCI_NF2_DATA_WIDTH-1:0]      op_lut_reg_rd_data,
     input                                op_lut_reg_ack,

     // --- Interface to output_port_lookup
     output reg                           oq_reg_req,
     output reg                           oq_reg_rd_wr_L,
     output reg [`OQ_REG_ADDR_WIDTH-1:0]   oq_reg_addr,
     output reg [CPCI_NF2_DATA_WIDTH-1:0] oq_reg_wr_data,
     
     input [CPCI_NF2_DATA_WIDTH-1:0]      oq_reg_rd_data,
     input                                oq_reg_ack,

     input                                clk,
     input                                reset
     );

   function integer log2;
      input integer number;
      begin
         log2=0;
         while(2**log2<number) begin
            log2=log2+1;
         end
      end
   endfunction // log2

   // ------------- Internal parameters --------------
   parameter OP_LUT_TAG_ADDR     = { {(`UDP_REG_ADDR_WIDTH - `OP_LUT_BLOCK_ADDR_WIDTH - `OP_LUT_REG_ADDR_WIDTH){1'b0}},
                                     `OP_LUT_BLOCK_ADDR,
                                     {`OP_LUT_REG_ADDR_WIDTH{1'b?}}};
   //parameter OQ_TAG_ADDR         = { //{(`UDP_REG_ADDR_WIDTH - `OQ_BLOCK_ADDR_WIDTH - `OQ_REG_ADDR_WIDTH){1'b0}}, has 0 width
   parameter OQ_TAG_ADDR         = { {(`UDP_REG_ADDR_WIDTH - `OQ_BLOCK_ADDR_WIDTH - `OQ_REG_ADDR_WIDTH){1'b0}},
                                     `OQ_BLOCK_ADDR,
                                     {`OQ_REG_ADDR_WIDTH{1'b?}}};
   parameter IN_ARB_TAG_ADDR     = { {(`UDP_REG_ADDR_WIDTH - `IN_ARB_BLOCK_ADDR_WIDTH - `IN_ARB_REG_ADDR_WIDTH){1'b0}},
                                     `IN_ARB_BLOCK_ADDR,
                                     {`IN_ARB_REG_ADDR_WIDTH{1'b?}}};
   
   // ------------ Mux the outputs ------------------

   always @(posedge clk) begin
      if(reset) begin
         udp_reg_rd_data        <= 32'hDEAD_BEEF;
         udp_reg_ack            <= udp_reg_req;
      end
      else begin
         casez(udp_reg_addr)
           IN_ARB_TAG_ADDR: begin
              udp_reg_ack            <= in_arb_reg_ack;
              udp_reg_rd_data        <= in_arb_reg_rd_data;
           end

           OP_LUT_TAG_ADDR: begin
              udp_reg_ack            <= op_lut_reg_ack;
              udp_reg_rd_data        <= op_lut_reg_rd_data;
           end
           
           OQ_TAG_ADDR: begin
              udp_reg_ack            <= oq_reg_ack;
              udp_reg_rd_data        <= oq_reg_rd_data;
           end
           
           default: begin
              udp_reg_rd_data        <= 32'hDEAD_BEEF;
              udp_reg_ack            <= udp_reg_req;
           end
         endcase // casez(udp_reg_addr)
      end // else: !if(reset)
   end // always @ (*)
   
   
   always @(posedge clk) begin
      if(reset) begin
         in_arb_reg_req         <= 0;
         in_arb_reg_rd_wr_L     <= 0;
         in_arb_reg_addr        <= 0;
         in_arb_reg_wr_data     <= 0;
         op_lut_reg_req         <= 0;
         op_lut_reg_rd_wr_L     <= 0;
         op_lut_reg_addr        <= 0;
         op_lut_reg_wr_data     <= 0;
         oq_reg_req             <= 0;
         oq_reg_rd_wr_L         <= 0;
         oq_reg_addr            <= 0;
         oq_reg_wr_data         <= 0;
      end // if (reset)
      else begin
         casez(udp_reg_addr)
           IN_ARB_TAG_ADDR: begin
              in_arb_reg_req         <= udp_reg_req;
              in_arb_reg_rd_wr_L     <= udp_reg_rd_wr_L;
              in_arb_reg_addr        <= udp_reg_addr[`IN_ARB_REG_ADDR_WIDTH-1:0];
              in_arb_reg_wr_data     <= udp_reg_wr_data;
           end

           OP_LUT_TAG_ADDR: begin
              op_lut_reg_req         <= udp_reg_req;
              op_lut_reg_rd_wr_L     <= udp_reg_rd_wr_L;
              op_lut_reg_addr        <= udp_reg_addr[`OP_LUT_REG_ADDR_WIDTH-1:0];
              op_lut_reg_wr_data     <= udp_reg_wr_data;
           end

           OQ_TAG_ADDR: begin
              oq_reg_req             <= udp_reg_req;
              oq_reg_rd_wr_L         <= udp_reg_rd_wr_L;
              oq_reg_addr            <= udp_reg_addr[`OQ_REG_ADDR_WIDTH-1:0];
              oq_reg_wr_data         <= udp_reg_wr_data;
           end

           default: begin
              in_arb_reg_req         <= 0;
              in_arb_reg_rd_wr_L     <= 0;
              in_arb_reg_addr        <= 0;
              in_arb_reg_wr_data     <= 0;
              op_lut_reg_req         <= 0;
              op_lut_reg_rd_wr_L     <= 0;
              op_lut_reg_addr        <= 0;
              op_lut_reg_wr_data     <= 0;
              oq_reg_req             <= 0;
              oq_reg_rd_wr_L         <= 0;
              oq_reg_addr            <= 0;
              oq_reg_wr_data         <= 0;
           end
         endcase // casez(udp_reg_addr)
      end // else: !if(reset)
   end // always @ (*)
   
endmodule // udp_reg_grp


