#####################################
# vim:set shiftwidth=2 softtabstop=2 expandtab:
#
# $Id$
# author: Jad Naous jnaous@stanford.edu
# Provides functions to insert, remove 
# and read entries
#####################################

package DecacheLib;
use strict;
use POSIX;

use NF2::PacketGen;
use NF2::TestLib;
use Time::HiRes qw( usleep );

# bytes consists of:
#  72 bytes rule
#  72 bytes rule_mask
#  72 bytes replacement data
#  72 bytes replacement mask
# The first 8 bytes corresponds to the IOQ module hdr
# The rest correspond to the pkt bytes.
sub insertEntry {
  my $idx  = shift;
  die "bytes input should be at least 72*4=288 bytes. Found ".scalar @_ if(scalar @_ != 72*4);
  die "Index should be between 0 and 31. Found $idx" unless ($idx >=0 && $idx < 32);
  print "insertEntry: to index $idx\n";
  my $cnt=0;
  foreach (@_) {
    printf("$cnt-%02x ", $_);
    $cnt++;
  }
  # write rule
  for(my $i=0; $i<9; $i++) {
    my $word = (shift() << 24) + (shift() << 16) + (shift() << 8) + shift();
    my $addr = $i*512 + $idx*16 + main::WILDCARD_LOOKUP_BASE_REG();
    NF2::PacketGen::nf_PCI_write32($addr, $word);
    printf "  Writing %08x into %08x\n", $word, $addr;
    $word = (shift() << 24) + (shift() << 16) + (shift() << 8) + shift();
    $addr += 4;
    NF2::PacketGen::nf_PCI_write32($addr, $word);
    printf "  Writing %08x into %08x\n", $word, $addr;
  }

  # write rule_mask
  for(my $i=0; $i<9; $i++) {
    my $word = (shift() << 24) + (shift() << 16) + (shift() << 8) + shift();
    my $addr = $i*512 + 8 + $idx*16 + main::WILDCARD_LOOKUP_BASE_REG();
    NF2::PacketGen::nf_PCI_write32($addr, $word);
    printf "  Writing %08x into %08x\n", $word, $addr;
    $word = (shift() << 24) + (shift() << 16) + (shift() << 8) + shift();
    $addr += 4;
    NF2::PacketGen::nf_PCI_write32($addr, $word);
    printf "  Writing %08x into %08x\n", $word, $addr;
  }

  # write data and data_mask
  for(my $i=0; $i<18*2; $i++) {
    my $word = (shift() << 24) + (shift() << 16) + (shift() << 8) + shift();
    my $addr = $idx*512 + $i*4 + main::WILDCARD_ACTION_BASE_REG();
    NF2::PacketGen::nf_PCI_write32($addr, $word);
    printf "  Writing %08x into %08x\n", $word, $addr;
  }

  # reset counters and timestamp
  my $addr = $idx*512 + main::WILDCARD_ACTION_BASE_REG() + main::WILDCARD_LOOKUP_PKTS_HIT_SKIP();
  NF2::PacketGen::nf_PCI_write32($addr, 0);
  NF2::PacketGen::nf_PCI_write32($addr+4, 0);
  NF2::PacketGen::nf_PCI_write32($addr+8, 0);
  printf "  Writing 0 into %08x\n", $addr;
  printf "  Writing 0 into %08x\n", $addr+4;
  printf "  Writing 0 into %08x\n", $addr+8;

  return 6*9+3;
}

sub readEntry {
  my $idx  = shift;
  die "bytes input should be at least 72*4=288 bytes. Found ".scalar @_ if(scalar @_ != 72*4);
  die "Index should be between 0 and 31. Found $idx" unless ($idx >=0 && $idx < 32);

  # read rule
  for(my $i=0; $i<9; $i++) {
    my $word = (shift() << 24) + (shift() << 16) + (shift() << 8) + shift();
    my $addr = $i*512 + $idx*16 + main::WILDCARD_LOOKUP_BASE_REG();
    NF2::PacketGen::nf_PCI_read32($addr, $word);
    $word = (shift() << 24) + (shift() << 16) + (shift() << 8) + shift();
    $addr += 4;
    NF2::PacketGen::nf_PCI_read32($addr, $word);
  }

  # read rule_mask
  for(my $i=0; $i<9; $i++) {
    my $word = (shift() << 24) + (shift() << 16) + (shift() << 8) + shift();
    my $addr = $i*512 + 8 + $idx*16 + main::WILDCARD_LOOKUP_BASE_REG();
    NF2::PacketGen::nf_PCI_read32($addr, $word);

    $word = (shift() << 24) + (shift() << 16) + (shift() << 8) + shift();
    $addr += 4;
    NF2::PacketGen::nf_PCI_read32($addr, $word);
  }

  # read data and data_mask
  for(my $i=0; $i<18*2; $i++) {
    my $word = (shift() << 24) + (shift() << 16) + (shift() << 8) + shift();
    my $addr = $idx*512 + $i*4 + main::WILDCARD_ACTION_BASE_REG();
    NF2::PacketGen::nf_PCI_read32($addr, $word);
  }

  return 6*9+3;
}

sub readCounters {
  my $idx = shift;
  my $pkt_count = shift;
  my $byte_count = shift;

  my $addr = $idx*512 + main::WILDCARD_ACTION_BASE_REG() + main::WILDCARD_LOOKUP_PKTS_HIT_SKIP();
  NF2::PacketGen::nf_PCI_read32($addr, $pkt_count);
  NF2::PacketGen::nf_PCI_read32($addr+4, $byte_count);
}

1;
