#!/usr/local/bin/perl -w
# make_pkts.pl
#
#
# 

$delay = '@12us';
$batch = 0;

use NF2::PacketGen;
use CPCI_21Lib;
use NF21RouterLib;
use NF2::PacketGen;
use Test::TestLib;

use NFUtils::SimplePacket;
use DecacheLib;
use List::Util qw(min);

require "reg_defines.ph";
require "reg_defines_casado_forwarder.ph";

nf_set_environment( { PORT_MODE => 'PHYSICAL', MAX_PORTS => 4 } );
nf_add_port_rule(1, 'UNORDERED');
nf_add_port_rule(2, 'UNORDERED');
nf_add_port_rule(3, 'UNORDERED');
nf_add_port_rule(4, 'UNORDERED');

# use strict AFTER the $delay, $batch and %reg are declared
use strict;
use vars qw($delay $batch %reg);

my %cpci_regs = CPCI_21Lib::get_register_addresses();

# write 0 to CPCI_INTERRUPT_MASK_REG()
nf_PCI_write32(0, $batch, CPCI_INTERRUPT_MASK_REG(), 0);

# Prepare the DMA and enable interrupts
prepare_DMA('@3.9us');
enable_interrupts(0);

##################
# Create entries and store pkts
my @entry_list = ();
my @len_list = ();
my $num_entries = 24;
my $pkt_send_delay = '@600us';
my $ram_check_delay = '@1100us';
#my $pkt_send_delay = '@100us';
#my $ram_check_delay = '@200us';
foreach (0..$num_entries-1) {
  my $pkt_len = int(rand(1500)) + 60;
  my $pkt = NFUtils::SimplePacket->new(NFUtils::SimplePacket::PKT_TYPE() => NFUtils::SimplePacket::PKT_TYPE_UDP(),
				      NFUtils::SimplePacket::PKT_LEN() => $pkt_len);
  my @pkt_bytes = $pkt->bytes();

  my $input_queue = 2*int(rand(3));
  my $output_queue = 2*int(rand(3));

  my @rule = @pkt_bytes[0..63];
  my @rule_mask = map(int(rand(0x100)), (1..64));
  my @pkt_replacement = map(int(rand(0x100)), (1..64));
  my @replacement_mask = map(int(rand(0x100)), (1..64));;
  my $idx = $_;

  $delay = '@12us';
  DecacheLib::insertEntry($idx,
			  # rule
			  0, 0, 0, 0, 0, $input_queue, 0, 0, # IOQ hdr
			  @rule,
			  # rule mask
			  0xff, 0xff, 0xff, 0xff, 0, 0, 0xff, 0xff, # IOQ hdr
			  @rule_mask,
			  # replacement
			  0, (1 << $output_queue), 0, 0, 0, 0, 0, 0, # IOQ hdr
			  @pkt_replacement,
			  # replacement mask
			  0xff, 0xff, 0, 0, 0, 0, 0, 0,	# IOQ hdr
			  @replacement_mask,
			 );

  my $input_port = ($input_queue/2) + 1;
  my $output_port = ($output_queue/2) + 1;

  # transform pkt
  my @exp_bytes = ();
  my $i;
  for ($i=0; $i<min(scalar @pkt_bytes, 64); $i++) {
    my $byte = ($pkt_bytes[$i] & ~$replacement_mask[$i]) | ($pkt_replacement[$i] & $replacement_mask[$i]);
    push @exp_bytes, $byte;
  }
  push @exp_bytes, @pkt_bytes[$i..$pkt->size()-1];

  my $exp_hex = join(' ', map(sprintf ("%02x",$_), @exp_bytes)).' ';
  print "Expecting: $exp_hex\n";

  nf_packet_in($input_port, $pkt->get(NFUtils::SimplePacket::PKT_LEN), $pkt_send_delay, $main::batch, $pkt->hexBytes());
  nf_expected_packet($output_port, $pkt->size(), $exp_hex);
  nf_packet_in($input_port, $pkt->get(NFUtils::SimplePacket::PKT_LEN), $pkt_send_delay, $main::batch, $pkt->hexBytes());
  nf_expected_packet($output_port, $pkt->size(), $exp_hex);

  push @entry_list, [$idx,
		     # rule
		     0, 0, 0, 0, 0, $input_queue, 0, 0, # IOQ hdr
		     @rule,
		     # rule mask
		     0xff, 0xff, 0xff, 0xff, 0, 0, 0xff, 0xff, # IOQ hdr
		     @rule_mask,
		     # replacement
		     0, (1 << $output_queue), 0, 0, 0, 0, 0, 0, # IOQ hdr
		     @pkt_replacement,
		     # replacement mask
		     0xff, 0xff, 0, 0, 0, 0, 0, 0,	# IOQ hdr
		     @replacement_mask];
  push @len_list, $pkt->size();
}

$delay = $ram_check_delay;
foreach (0..$num_entries-1) {
  DecacheLib::readEntry(@{shift @entry_list});
  DecacheLib::readCounters($_, 2, 2*(shift @len_list));
}


# *********** Finishing Up - need this in all scripts ! **********************
my $t = nf_write_sim_files();
print  "--- make_pkts.pl: Generated all configuration packets.\n";
printf "--- make_pkts.pl: Last packet enters system at approx %0d microseconds.\n",($t/1000);
if (nf_write_expected_files()) {
  die "Unable to write expected files\n";
}

nf_create_hardware_file('LITTLE_ENDIAN');
nf_write_hardware_file('LITTLE_ENDIAN');


