#!/usr/local/bin/perl -w
# make_pkts.pl
#
#
# 

$delay = '@12us';
$batch = 0;

use NF2::PacketGen;
use CPCI_21Lib;
use NF21RouterLib;
use NF2::PacketGen;
use Test::TestLib;

use NFUtils::SimplePacket;
use DecacheLib;
use List::Util qw(min);

require "reg_defines.ph";
require "reg_defines_casado_forwarder.ph";

nf_set_environment( { PORT_MODE => 'PHYSICAL', MAX_PORTS => 4 } );
nf_add_port_rule(1, 'UNORDERED');
nf_add_port_rule(2, 'UNORDERED');
nf_add_port_rule(3, 'UNORDERED');
nf_add_port_rule(4, 'UNORDERED');

# use strict AFTER the $delay, $batch and %reg are declared
use strict;
use vars qw($delay $batch %reg);

my %cpci_regs = CPCI_21Lib::get_register_addresses();

# write 0 to CPCI_INTERRUPT_MASK_REG()
nf_PCI_write32(0, $batch, CPCI_INTERRUPT_MASK_REG(), 0);

# Prepare the DMA and enable interrupts
prepare_DMA('@3.9us');
enable_interrupts(0);

##################
# Create entry
my $pkt = NFUtils::SimplePacket->new(NFUtils::SimplePacket::PKT_TYPE() => NFUtils::SimplePacket::PKT_TYPE_UDP(),
				    NFUtils::SimplePacket::PKT_LEN() => 60);
my @pkt_bytes = $pkt->bytes();

my $input_queue = 2;
my $output_queue = 4;

my @rule = @pkt_bytes[0..63];
my @rule_mask = (0xff) x 64;
my @pkt_replacement = (1..64);
my @replacement_mask = ((0xff) x 32, (0x00) x 16, (0xff) x 16);
my $idx = 2;

DecacheLib::insertEntry($idx,
			# rule
			0, 0, 0, 0, 0, $input_queue, 0, 0,              # IOQ hdr
			@rule,
			# rule mask
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,  # IOQ hdr
			@rule_mask,
			# replacement
			0, (1 << $output_queue), 0, 0, 0, 0, 0, 0,           # IOQ hdr
			@pkt_replacement,
			# replacement mask
			0xff, 0xff, 0, 0, 0, 0, 0, 0,        # IOQ hdr
			@replacement_mask,
		       );

my $input_port = ($input_queue/2) + 1;
my $output_port = ($output_queue/2) + 1;

# transform pkt
my @exp_bytes = ();
my $i;
for($i=0; $i<min(scalar @pkt_bytes, 64); $i++) {
  my $byte = ($pkt_bytes[$i] & ~$replacement_mask[$i]) | ($pkt_replacement[$i] & $replacement_mask[$i]);
  push @exp_bytes, $byte;
}
push @exp_bytes, @pkt_bytes[$i..$pkt->size()-1];

my $exp_hex = join(' ', map(sprintf ("%02x",$_), @exp_bytes)).' ';
print "Expecting: $exp_hex\n";

foreach (1..100) {
  nf_packet_in(1, $pkt->get(NFUtils::SimplePacket::PKT_LEN), '@35us', $main::batch, $pkt->hexBytes());
  nf_packet_in(2, $pkt->get(NFUtils::SimplePacket::PKT_LEN), '@35us', $main::batch, $pkt->hexBytes());
  nf_packet_in(3, $pkt->get(NFUtils::SimplePacket::PKT_LEN), '@35us', $main::batch, $pkt->hexBytes());
#  nf_packet_in(4, $pkt->get(NFUtils::SimplePacket::PKT_LEN), '@35us', $main::batch, $pkt->hexBytes());
  nf_expected_packet($output_port, $pkt->size(), $exp_hex);
  nf_expected_packet($output_port, $pkt->size(), $exp_hex);
  nf_expected_packet($output_port, $pkt->size(), $exp_hex);
#  nf_expected_packet($output_port, $pkt->size(), $exp_hex);
}

# *********** Finishing Up - need this in all scripts ! **********************
my $t = nf_write_sim_files();
print  "--- make_pkts.pl: Generated all configuration packets.\n";
printf "--- make_pkts.pl: Last packet enters system at approx %0d microseconds.\n",($t/1000);
if (nf_write_expected_files()) {
  die "Unable to write expected files\n";
}

nf_create_hardware_file('LITTLE_ENDIAN');
nf_write_hardware_file('LITTLE_ENDIAN');


