function [0:63] chi256_init;
      input [2:0] val;

      case(val)
         0: chi256_init = 64'h3C6EF372FE94F82B;
         1: chi256_init = 64'hE73980C0B9DB9068;
         2: chi256_init = 64'h21044ED7E744E4A3;
         3: chi256_init = 64'hF0D8D423A1831D2A;
         4: chi256_init = 64'h4ECFE162A7A4F6FE;
         5: chi256_init = 64'h068E08B6B7E304FE;
         default: chi256_init = 0;
      endcase // case(val)
endfunction // chi256_init

function [0:63] chi256_swap8;
      input [0:63] x;

      chi256_swap8 = {x[7*8 +: 8],
                      x[6*8 +: 8],
                      x[5*8 +: 8],
                      x[4*8 +: 8],
                      x[3*8 +: 8],
                      x[2*8 +: 8],
                      x[1*8 +: 8],
                      x[0*8 +: 8]};
endfunction // chi256_swap8

function [0:63] chi256_swap32;
      input [0:63] x;
      chi256_swap32 = {x[32:63], x[0:31]};
endfunction // chi256_swap32

function [0:31] chi256_msb32;
      input [0:63] x;
      chi256_msb32 = x[0:31];
endfunction // chi256_msb32

function [0:2] chi256_map;
      input [3:0] rstu;

      case(rstu)
         0: chi256_map    = 4;
         1: chi256_map    = 6;
         2: chi256_map    = 2;
         3: chi256_map    = 3;
         4: chi256_map    = 0;
         5: chi256_map    = 7;
         6: chi256_map    = 5;
         7: chi256_map    = 6;
         8: chi256_map    = 7;
         9: chi256_map    = 0;
         10: chi256_map   = 1;
         11: chi256_map   = 4;
         12: chi256_map   = 5;
         13: chi256_map   = 3;
         14: chi256_map   = 2;
         15: chi256_map   = 1;
         default: chi256_map = 0;
      endcase // case(rstu)
endfunction // chi256_map

function [0:3*64-1] chi256_map_full;
      input [0:63] R;
      input [0:63] S;
      input [0:63] T;
      input [0:63] U;

      begin
         chi256_map_full[0 : 63] = ((R & ~S & T & U)
                                    | (~S & ~T & ~U)
                                    | (~R & ~T & U)
                                    | (~R & S  & T)
                                    | ( R & ~T & ~U));
           
         chi256_map_full[64:127] = (( R & S & T & ~U)
                                    | ( R & ~S & ~T & ~U)
                                    | (~R & ~S &  T     )
                                    | (      S & ~T &  U)
                                    | (~R           &  U));
                                 
         chi256_map_full[128:191] = ((  ~R & ~S &  T &  U)
                                     | (~R &  S &  T & ~U)
                                     | (      S & ~T &  U)
                                     | ( R &      ~T & ~U)
                                     | ( R &  S      &  U)
                                     | ( R & ~S      & ~U));
      end
endfunction // chi256_map_full

