`include "aes_funcs.v.inc"
`include "grostl_def.v"

/****************************************************************
 * Add Round Constants
 ****************************************************************/
function [0:511] grostl_addroundconstant;
      input [7:0]   round_num;
      input         Q; // is this a Q or P permutation?
      input [0:511] state;

      begin
         grostl_addroundconstant = state;
         if(!Q) begin
            grostl_addroundconstant[0:7] = state[0:7] ^ round_num;
         end
         else begin
            grostl_addroundconstant[56 : 63] 
                = state[56 : 63] ^ round_num ^ 8'hff;
         end
      end
endfunction // grostl_addroundconstant

/****************************************************************
 * ShiftBytes: Rotates each row left by the row number
 ****************************************************************/
function [0:511] grostl_shiftbytes;
      input [0:511] s_in;

      reg [7:0] state_in [0:7][0:7];
      reg [7:0] state_out[0:7][0:7];

      integer i,j;
      
      begin
         /* Put the state in matrix form */
         for(i=0; i<8; i=i+1) begin
            for(j=0; j<8; j=j+1) begin
               state_in[j][i] = s_in[8 * (j + i*8) +: 8];
            end
         end
               
         /* Do the transformation */
         for(i=0; i<8; i=i+1) begin
            for(j=0; j<8; j=j+1) begin
               state_out[j][(i+8-j)%8] = state_in[j][i];
            end
         end
               
         /* linearize the output state matrix */
         for(i=0; i<8; i=i+1) begin
            for(j=0; j<8; j=j+1) begin
               grostl_shiftbytes[8 * (j + i*8) +: 8] = state_out[j][i];
            end
         end
      end
endfunction // grostl_shiftbytes

/****************************************************************
 * ShiftWords: Rotates each row left by the row number 
 *             where a word is 40bits
 ****************************************************************/
function [0:8*8*40-1] grostl_shiftwords;
      input [0:8*8*40-1] s_in;

      reg [39:0] state_in [0:7][0:7];
      reg [39:0] state_out[0:7][0:7];

      integer i,j;
      
      begin
         /* Put the state in matrix form */
         for(i=0; i<8; i=i+1) begin
            for(j=0; j<8; j=j+1) begin
               state_in[j][i] = s_in[40 * (j + i*8) +: 40];
            end
         end
               
         /* Do the transformation */
         for(i=0; i<8; i=i+1) begin
            for(j=0; j<8; j=j+1) begin
               state_out[j][(i+8-j)%8] = state_in[j][i];
            end
         end
               
         /* linearize the output state matrix */
         grostl_shiftwords
	   = {state_out[0][0],
	      state_out[1][0],
	      state_out[2][0],
	      state_out[3][0],
	      state_out[4][0],
	      state_out[5][0],
	      state_out[6][0],
	      state_out[7][0],
	      state_out[0][1],
	      state_out[1][1],
	      state_out[2][1],
	      state_out[3][1],
	      state_out[4][1],
	      state_out[5][1],
	      state_out[6][1],
	      state_out[7][1],
	      state_out[0][2],
	      state_out[1][2],
	      state_out[2][2],
	      state_out[3][2],
	      state_out[4][2],
	      state_out[5][2],
	      state_out[6][2],
	      state_out[7][2],
	      state_out[0][3],
	      state_out[1][3],
	      state_out[2][3],
	      state_out[3][3],
	      state_out[4][3],
	      state_out[5][3],
	      state_out[6][3],
	      state_out[7][3],
	      state_out[0][4],
	      state_out[1][4],
	      state_out[2][4],
	      state_out[3][4],
	      state_out[4][4],
	      state_out[5][4],
	      state_out[6][4],
	      state_out[7][4],
	      state_out[0][5],
	      state_out[1][5],
	      state_out[2][5],
	      state_out[3][5],
	      state_out[4][5],
	      state_out[5][5],
	      state_out[6][5],
	      state_out[7][5],
	      state_out[0][6],
	      state_out[1][6],
	      state_out[2][6],
	      state_out[3][6],
	      state_out[4][6],
	      state_out[5][6],
	      state_out[6][6],
	      state_out[7][6],
	      state_out[0][7],
	      state_out[1][7],
	      state_out[2][7],
	      state_out[3][7],
	      state_out[4][7],
	      state_out[5][7],
	      state_out[6][7],
	      state_out[7][7]	      
	      };
      end
endfunction // grostl_shiftwords


/****************************************************************
 * MixBytesCol: Multiplies a single column as specified by Grostl
 *              and returns the one byte result
 ****************************************************************/
/* -----\/----- EXCLUDED -----\/-----
function [7:0] grostl_mixbytescol;
      input [0:`GROSTL_COL_WIDTH-1]  col;     /-* col to transform *-/
      input [2:0]                    row_num; /-* which row in B *-/

      reg [0:`GROSTL_COL_WIDTH-1]    col_shifted;

      begin
         /-* shift the column *-/
         if(row_num != 0) begin
            col_shifted = {col[8*row_num : `GROSTL_COL_WIDTH - 1],
                           col[0 : 8*row_num - 1]};
         end else begin
            col_shifted = col;
         end

         /-* do the multiplication *-/
         grostl_mixbytescol = times2(col_shifted[0*8 +: 8]) ^
                               times2(col_shifted[1*8 +: 8]) ^
                               times3(col_shifted[2*8 +: 8]) ^
                               times4(col_shifted[3*8 +: 8]) ^
                               times5(col_shifted[4*8 +: 8]) ^
                               times3(col_shifted[5*8 +: 8]) ^
                               times5(col_shifted[6*8 +: 8]) ^
                               times7(col_shifted[7*8 +: 8]);
      end
endfunction // grostl_mixbytescol
 -----/\----- EXCLUDED -----/\----- */

/*******************************************************************
 * Implements the sbox transformation and times simultaneously.
 * It outputs {times2(sbox(s_in)), times3(sbox(s_in)), 
 *             times5(sbox(s_in)), times7(sbox(s_in))}
 *******************************************************************/
function [0:31] sboxtimes2357;
      input [7:0] s_in;

      case(s_in)
         8'h00: sboxtimes2357 = {8'hc6, 8'ha5, 8'hf4, 8'h32};
         8'h01: sboxtimes2357 = {8'hf8, 8'h84, 8'h97, 8'h6f};
         8'h02: sboxtimes2357 = {8'hee, 8'h99, 8'hb0, 8'h5e};
         8'h03: sboxtimes2357 = {8'hf6, 8'h8d, 8'h8c, 8'h7a};
         8'h04: sboxtimes2357 = {8'hff, 8'h0d, 8'h17, 8'he8};
         8'h05: sboxtimes2357 = {8'hd6, 8'hbd, 8'hdc, 8'h0a};
         8'h06: sboxtimes2357 = {8'hde, 8'hb1, 8'hc8, 8'h16};
         8'h07: sboxtimes2357 = {8'h91, 8'h54, 8'hfc, 8'h6d};
         8'h08: sboxtimes2357 = {8'h60, 8'h50, 8'hf0, 8'h90};
         8'h09: sboxtimes2357 = {8'h02, 8'h03, 8'h05, 8'h07};
         8'h0a: sboxtimes2357 = {8'hce, 8'ha9, 8'he0, 8'h2e};
         8'h0b: sboxtimes2357 = {8'h56, 8'h7d, 8'h87, 8'hd1};
         8'h0c: sboxtimes2357 = {8'he7, 8'h19, 8'h2b, 8'hcc};
         8'h0d: sboxtimes2357 = {8'hb5, 8'h62, 8'ha6, 8'h13};
         8'h0e: sboxtimes2357 = {8'h4d, 8'he6, 8'h31, 8'h7c};
         8'h0f: sboxtimes2357 = {8'hec, 8'h9a, 8'hb5, 8'h59};
         8'h10: sboxtimes2357 = {8'h8f, 8'h45, 8'hcf, 8'h40};
         8'h11: sboxtimes2357 = {8'h1f, 8'h9d, 8'hbc, 8'ha3};
         8'h12: sboxtimes2357 = {8'h89, 8'h40, 8'hc0, 8'h49};
         8'h13: sboxtimes2357 = {8'hfa, 8'h87, 8'h92, 8'h68};
         8'h14: sboxtimes2357 = {8'hef, 8'h15, 8'h3f, 8'hd0};
         8'h15: sboxtimes2357 = {8'hb2, 8'heb, 8'h26, 8'h94};
         8'h16: sboxtimes2357 = {8'h8e, 8'hc9, 8'h40, 8'hce};
         8'h17: sboxtimes2357 = {8'hfb, 8'h0b, 8'h1d, 8'he6};
         8'h18: sboxtimes2357 = {8'h41, 8'hec, 8'h2f, 8'h6e};
         8'h19: sboxtimes2357 = {8'hb3, 8'h67, 8'ha9, 8'h1a};
         8'h1a: sboxtimes2357 = {8'h5f, 8'hfd, 8'h1c, 8'h43};
         8'h1b: sboxtimes2357 = {8'h45, 8'hea, 8'h25, 8'h60};
         8'h1c: sboxtimes2357 = {8'h23, 8'hbf, 8'hda, 8'hf9};
         8'h1d: sboxtimes2357 = {8'h53, 8'hf7, 8'h02, 8'h51};
         8'h1e: sboxtimes2357 = {8'he4, 8'h96, 8'ha1, 8'h45};
         8'h1f: sboxtimes2357 = {8'h9b, 8'h5b, 8'hed, 8'h76};
         8'h20: sboxtimes2357 = {8'h75, 8'hc2, 8'h5d, 8'h28};
         8'h21: sboxtimes2357 = {8'he1, 8'h1c, 8'h24, 8'hc5};
         8'h22: sboxtimes2357 = {8'h3d, 8'hae, 8'he9, 8'hd4};
         8'h23: sboxtimes2357 = {8'h4c, 8'h6a, 8'hbe, 8'hf2};
         8'h24: sboxtimes2357 = {8'h6c, 8'h5a, 8'hee, 8'h82};
         8'h25: sboxtimes2357 = {8'h7e, 8'h41, 8'hc3, 8'hbd};
         8'h26: sboxtimes2357 = {8'hf5, 8'h02, 8'h06, 8'hf3};
         8'h27: sboxtimes2357 = {8'h83, 8'h4f, 8'hd1, 8'h52};
         8'h28: sboxtimes2357 = {8'h68, 8'h5c, 8'he4, 8'h8c};
         8'h29: sboxtimes2357 = {8'h51, 8'hf4, 8'h07, 8'h56};
         8'h2a: sboxtimes2357 = {8'hd1, 8'h34, 8'h5c, 8'h8d};
         8'h2b: sboxtimes2357 = {8'hf9, 8'h08, 8'h18, 8'he1};
         8'h2c: sboxtimes2357 = {8'he2, 8'h93, 8'hae, 8'h4c};
         8'h2d: sboxtimes2357 = {8'hab, 8'h73, 8'h95, 8'h3e};
         8'h2e: sboxtimes2357 = {8'h62, 8'h53, 8'hf5, 8'h97};
         8'h2f: sboxtimes2357 = {8'h2a, 8'h3f, 8'h41, 8'h6b};
         8'h30: sboxtimes2357 = {8'h08, 8'h0c, 8'h14, 8'h1c};
         8'h31: sboxtimes2357 = {8'h95, 8'h52, 8'hf6, 8'h63};
         8'h32: sboxtimes2357 = {8'h46, 8'h65, 8'haf, 8'he9};
         8'h33: sboxtimes2357 = {8'h9d, 8'h5e, 8'he2, 8'h7f};
         8'h34: sboxtimes2357 = {8'h30, 8'h28, 8'h78, 8'h48};
         8'h35: sboxtimes2357 = {8'h37, 8'ha1, 8'hf8, 8'hcf};
         8'h36: sboxtimes2357 = {8'h0a, 8'h0f, 8'h11, 8'h1b};
         8'h37: sboxtimes2357 = {8'h2f, 8'hb5, 8'hc4, 8'heb};
         8'h38: sboxtimes2357 = {8'h0e, 8'h09, 8'h1b, 8'h15};
         8'h39: sboxtimes2357 = {8'h24, 8'h36, 8'h5a, 8'h7e};
         8'h3a: sboxtimes2357 = {8'h1b, 8'h9b, 8'hb6, 8'had};
         8'h3b: sboxtimes2357 = {8'hdf, 8'h3d, 8'h47, 8'h98};
         8'h3c: sboxtimes2357 = {8'hcd, 8'h26, 8'h6a, 8'ha7};
         8'h3d: sboxtimes2357 = {8'h4e, 8'h69, 8'hbb, 8'hf5};
         8'h3e: sboxtimes2357 = {8'h7f, 8'hcd, 8'h4c, 8'h33};
         8'h3f: sboxtimes2357 = {8'hea, 8'h9f, 8'hba, 8'h50};
         8'h40: sboxtimes2357 = {8'h12, 8'h1b, 8'h2d, 8'h3f};
         8'h41: sboxtimes2357 = {8'h1d, 8'h9e, 8'hb9, 8'ha4};
         8'h42: sboxtimes2357 = {8'h58, 8'h74, 8'h9c, 8'hc4};
         8'h43: sboxtimes2357 = {8'h34, 8'h2e, 8'h72, 8'h46};
         8'h44: sboxtimes2357 = {8'h36, 8'h2d, 8'h77, 8'h41};
         8'h45: sboxtimes2357 = {8'hdc, 8'hb2, 8'hcd, 8'h11};
         8'h46: sboxtimes2357 = {8'hb4, 8'hee, 8'h29, 8'h9d};
         8'h47: sboxtimes2357 = {8'h5b, 8'hfb, 8'h16, 8'h4d};
         8'h48: sboxtimes2357 = {8'ha4, 8'hf6, 8'h01, 8'ha5};
         8'h49: sboxtimes2357 = {8'h76, 8'h4d, 8'hd7, 8'ha1};
         8'h4a: sboxtimes2357 = {8'hb7, 8'h61, 8'ha3, 8'h14};
         8'h4b: sboxtimes2357 = {8'h7d, 8'hce, 8'h49, 8'h34};
         8'h4c: sboxtimes2357 = {8'h52, 8'h7b, 8'h8d, 8'hdf};
         8'h4d: sboxtimes2357 = {8'hdd, 8'h3e, 8'h42, 8'h9f};
         8'h4e: sboxtimes2357 = {8'h5e, 8'h71, 8'h93, 8'hcd};
         8'h4f: sboxtimes2357 = {8'h13, 8'h97, 8'ha2, 8'hb1};
         8'h50: sboxtimes2357 = {8'ha6, 8'hf5, 8'h04, 8'ha2};
         8'h51: sboxtimes2357 = {8'hb9, 8'h68, 8'hb8, 8'h01};
         8'h52: sboxtimes2357 = {8'h00, 8'h00, 8'h00, 8'h00};
         8'h53: sboxtimes2357 = {8'hc1, 8'h2c, 8'h74, 8'hb5};
         8'h54: sboxtimes2357 = {8'h40, 8'h60, 8'ha0, 8'he0};
         8'h55: sboxtimes2357 = {8'he3, 8'h1f, 8'h21, 8'hc2};
         8'h56: sboxtimes2357 = {8'h79, 8'hc8, 8'h43, 8'h3a};
         8'h57: sboxtimes2357 = {8'hb6, 8'hed, 8'h2c, 8'h9a};
         8'h58: sboxtimes2357 = {8'hd4, 8'hbe, 8'hd9, 8'h0d};
         8'h59: sboxtimes2357 = {8'h8d, 8'h46, 8'hca, 8'h47};
         8'h5a: sboxtimes2357 = {8'h67, 8'hd9, 8'h70, 8'h17};
         8'h5b: sboxtimes2357 = {8'h72, 8'h4b, 8'hdd, 8'haf};
         8'h5c: sboxtimes2357 = {8'h94, 8'hde, 8'h79, 8'hed};
         8'h5d: sboxtimes2357 = {8'h98, 8'hd4, 8'h67, 8'hff};
         8'h5e: sboxtimes2357 = {8'hb0, 8'he8, 8'h23, 8'h93};
         8'h5f: sboxtimes2357 = {8'h85, 8'h4a, 8'hde, 8'h5b};
         8'h60: sboxtimes2357 = {8'hbb, 8'h6b, 8'hbd, 8'h06};
         8'h61: sboxtimes2357 = {8'hc5, 8'h2a, 8'h7e, 8'hbb};
         8'h62: sboxtimes2357 = {8'h4f, 8'he5, 8'h34, 8'h7b};
         8'h63: sboxtimes2357 = {8'hed, 8'h16, 8'h3a, 8'hd7};
         8'h64: sboxtimes2357 = {8'h86, 8'hc5, 8'h54, 8'hd2};
         8'h65: sboxtimes2357 = {8'h9a, 8'hd7, 8'h62, 8'hf8};
         8'h66: sboxtimes2357 = {8'h66, 8'h55, 8'hff, 8'h99};
         8'h67: sboxtimes2357 = {8'h11, 8'h94, 8'ha7, 8'hb6};
         8'h68: sboxtimes2357 = {8'h8a, 8'hcf, 8'h4a, 8'hc0};
         8'h69: sboxtimes2357 = {8'he9, 8'h10, 8'h30, 8'hd9};
         8'h6a: sboxtimes2357 = {8'h04, 8'h06, 8'h0a, 8'h0e};
         8'h6b: sboxtimes2357 = {8'hfe, 8'h81, 8'h98, 8'h66};
         8'h6c: sboxtimes2357 = {8'ha0, 8'hf0, 8'h0b, 8'hab};
         8'h6d: sboxtimes2357 = {8'h78, 8'h44, 8'hcc, 8'hb4};
         8'h6e: sboxtimes2357 = {8'h25, 8'hba, 8'hd5, 8'hf0};
         8'h6f: sboxtimes2357 = {8'h4b, 8'he3, 8'h3e, 8'h75};
         8'h70: sboxtimes2357 = {8'ha2, 8'hf3, 8'h0e, 8'hac};
         8'h71: sboxtimes2357 = {8'h5d, 8'hfe, 8'h19, 8'h44};
         8'h72: sboxtimes2357 = {8'h80, 8'hc0, 8'h5b, 8'hdb};
         8'h73: sboxtimes2357 = {8'h05, 8'h8a, 8'h85, 8'h80};
         8'h74: sboxtimes2357 = {8'h3f, 8'had, 8'hec, 8'hd3};
         8'h75: sboxtimes2357 = {8'h21, 8'hbc, 8'hdf, 8'hfe};
         8'h76: sboxtimes2357 = {8'h70, 8'h48, 8'hd8, 8'ha8};
         8'h77: sboxtimes2357 = {8'hf1, 8'h04, 8'h0c, 8'hfd};
         8'h78: sboxtimes2357 = {8'h63, 8'hdf, 8'h7a, 8'h19};
         8'h79: sboxtimes2357 = {8'h77, 8'hc1, 8'h58, 8'h2f};
         8'h7a: sboxtimes2357 = {8'haf, 8'h75, 8'h9f, 8'h30};
         8'h7b: sboxtimes2357 = {8'h42, 8'h63, 8'ha5, 8'he7};
         8'h7c: sboxtimes2357 = {8'h20, 8'h30, 8'h50, 8'h70};
         8'h7d: sboxtimes2357 = {8'he5, 8'h1a, 8'h2e, 8'hcb};
         8'h7e: sboxtimes2357 = {8'hfd, 8'h0e, 8'h12, 8'hef};
         8'h7f: sboxtimes2357 = {8'hbf, 8'h6d, 8'hb7, 8'h08};
         8'h80: sboxtimes2357 = {8'h81, 8'h4c, 8'hd4, 8'h55};
         8'h81: sboxtimes2357 = {8'h18, 8'h14, 8'h3c, 8'h24};
         8'h82: sboxtimes2357 = {8'h26, 8'h35, 8'h5f, 8'h79};
         8'h83: sboxtimes2357 = {8'hc3, 8'h2f, 8'h71, 8'hb2};
         8'h84: sboxtimes2357 = {8'hbe, 8'he1, 8'h38, 8'h86};
         8'h85: sboxtimes2357 = {8'h35, 8'ha2, 8'hfd, 8'hc8};
         8'h86: sboxtimes2357 = {8'h88, 8'hcc, 8'h4f, 8'hc7};
         8'h87: sboxtimes2357 = {8'h2e, 8'h39, 8'h4b, 8'h65};
         8'h88: sboxtimes2357 = {8'h93, 8'h57, 8'hf9, 8'h6a};
         8'h89: sboxtimes2357 = {8'h55, 8'hf2, 8'h0d, 8'h58};
         8'h8a: sboxtimes2357 = {8'hfc, 8'h82, 8'h9d, 8'h61};
         8'h8b: sboxtimes2357 = {8'h7a, 8'h47, 8'hc9, 8'hb3};
         8'h8c: sboxtimes2357 = {8'hc8, 8'hac, 8'hef, 8'h27};
         8'h8d: sboxtimes2357 = {8'hba, 8'he7, 8'h32, 8'h88};
         8'h8e: sboxtimes2357 = {8'h32, 8'h2b, 8'h7d, 8'h4f};
         8'h8f: sboxtimes2357 = {8'he6, 8'h95, 8'ha4, 8'h42};
         8'h90: sboxtimes2357 = {8'hc0, 8'ha0, 8'hfb, 8'h3b};
         8'h91: sboxtimes2357 = {8'h19, 8'h98, 8'hb3, 8'haa};
         8'h92: sboxtimes2357 = {8'h9e, 8'hd1, 8'h68, 8'hf6};
         8'h93: sboxtimes2357 = {8'ha3, 8'h7f, 8'h81, 8'h22};
         8'h94: sboxtimes2357 = {8'h44, 8'h66, 8'haa, 8'hee};
         8'h95: sboxtimes2357 = {8'h54, 8'h7e, 8'h82, 8'hd6};
         8'h96: sboxtimes2357 = {8'h3b, 8'hab, 8'he6, 8'hdd};
         8'h97: sboxtimes2357 = {8'h0b, 8'h83, 8'h9e, 8'h95};
         8'h98: sboxtimes2357 = {8'h8c, 8'hca, 8'h45, 8'hc9};
         8'h99: sboxtimes2357 = {8'hc7, 8'h29, 8'h7b, 8'hbc};
         8'h9a: sboxtimes2357 = {8'h6b, 8'hd3, 8'h6e, 8'h05};
         8'h9b: sboxtimes2357 = {8'h28, 8'h3c, 8'h44, 8'h6c};
         8'h9c: sboxtimes2357 = {8'ha7, 8'h79, 8'h8b, 8'h2c};
         8'h9d: sboxtimes2357 = {8'hbc, 8'he2, 8'h3d, 8'h81};
         8'h9e: sboxtimes2357 = {8'h16, 8'h1d, 8'h27, 8'h31};
         8'h9f: sboxtimes2357 = {8'had, 8'h76, 8'h9a, 8'h37};
         8'ha0: sboxtimes2357 = {8'hdb, 8'h3b, 8'h4d, 8'h96};
         8'ha1: sboxtimes2357 = {8'h64, 8'h56, 8'hfa, 8'h9e};
         8'ha2: sboxtimes2357 = {8'h74, 8'h4e, 8'hd2, 8'ha6};
         8'ha3: sboxtimes2357 = {8'h14, 8'h1e, 8'h22, 8'h36};
         8'ha4: sboxtimes2357 = {8'h92, 8'hdb, 8'h76, 8'he4};
         8'ha5: sboxtimes2357 = {8'h0c, 8'h0a, 8'h1e, 8'h12};
         8'ha6: sboxtimes2357 = {8'h48, 8'h6c, 8'hb4, 8'hfc};
         8'ha7: sboxtimes2357 = {8'hb8, 8'he4, 8'h37, 8'h8f};
         8'ha8: sboxtimes2357 = {8'h9f, 8'h5d, 8'he7, 8'h78};
         8'ha9: sboxtimes2357 = {8'hbd, 8'h6e, 8'hb2, 8'h0f};
         8'haa: sboxtimes2357 = {8'h43, 8'hef, 8'h2a, 8'h69};
         8'hab: sboxtimes2357 = {8'hc4, 8'ha6, 8'hf1, 8'h35};
         8'hac: sboxtimes2357 = {8'h39, 8'ha8, 8'he3, 8'hda};
         8'had: sboxtimes2357 = {8'h31, 8'ha4, 8'hf7, 8'hc6};
         8'hae: sboxtimes2357 = {8'hd3, 8'h37, 8'h59, 8'h8a};
         8'haf: sboxtimes2357 = {8'hf2, 8'h8b, 8'h86, 8'h74};
         8'hb0: sboxtimes2357 = {8'hd5, 8'h32, 8'h56, 8'h83};
         8'hb1: sboxtimes2357 = {8'h8b, 8'h43, 8'hc5, 8'h4e};
         8'hb2: sboxtimes2357 = {8'h6e, 8'h59, 8'heb, 8'h85};
         8'hb3: sboxtimes2357 = {8'hda, 8'hb7, 8'hc2, 8'h18};
         8'hb4: sboxtimes2357 = {8'h01, 8'h8c, 8'h8f, 8'h8e};
         8'hb5: sboxtimes2357 = {8'hb1, 8'h64, 8'hac, 8'h1d};
         8'hb6: sboxtimes2357 = {8'h9c, 8'hd2, 8'h6d, 8'hf1};
         8'hb7: sboxtimes2357 = {8'h49, 8'he0, 8'h3b, 8'h72};
         8'hb8: sboxtimes2357 = {8'hd8, 8'hb4, 8'hc7, 8'h1f};
         8'hb9: sboxtimes2357 = {8'hac, 8'hfa, 8'h15, 8'hb9};
         8'hba: sboxtimes2357 = {8'hf3, 8'h07, 8'h09, 8'hfa};
         8'hbb: sboxtimes2357 = {8'hcf, 8'h25, 8'h6f, 8'ha0};
         8'hbc: sboxtimes2357 = {8'hca, 8'haf, 8'hea, 8'h20};
         8'hbd: sboxtimes2357 = {8'hf4, 8'h8e, 8'h89, 8'h7d};
         8'hbe: sboxtimes2357 = {8'h47, 8'he9, 8'h20, 8'h67};
         8'hbf: sboxtimes2357 = {8'h10, 8'h18, 8'h28, 8'h38};
         8'hc0: sboxtimes2357 = {8'h6f, 8'hd5, 8'h64, 8'h0b};
         8'hc1: sboxtimes2357 = {8'hf0, 8'h88, 8'h83, 8'h73};
         8'hc2: sboxtimes2357 = {8'h4a, 8'h6f, 8'hb1, 8'hfb};
         8'hc3: sboxtimes2357 = {8'h5c, 8'h72, 8'h96, 8'hca};
         8'hc4: sboxtimes2357 = {8'h38, 8'h24, 8'h6c, 8'h54};
         8'hc5: sboxtimes2357 = {8'h57, 8'hf1, 8'h08, 8'h5f};
         8'hc6: sboxtimes2357 = {8'h73, 8'hc7, 8'h52, 8'h21};
         8'hc7: sboxtimes2357 = {8'h97, 8'h51, 8'hf3, 8'h64};
         8'hc8: sboxtimes2357 = {8'hcb, 8'h23, 8'h65, 8'hae};
         8'hc9: sboxtimes2357 = {8'ha1, 8'h7c, 8'h84, 8'h25};
         8'hca: sboxtimes2357 = {8'he8, 8'h9c, 8'hbf, 8'h57};
         8'hcb: sboxtimes2357 = {8'h3e, 8'h21, 8'h63, 8'h5d};
         8'hcc: sboxtimes2357 = {8'h96, 8'hdd, 8'h7c, 8'hea};
         8'hcd: sboxtimes2357 = {8'h61, 8'hdc, 8'h7f, 8'h1e};
         8'hce: sboxtimes2357 = {8'h0d, 8'h86, 8'h91, 8'h9c};
         8'hcf: sboxtimes2357 = {8'h0f, 8'h85, 8'h94, 8'h9b};
         8'hd0: sboxtimes2357 = {8'he0, 8'h90, 8'hab, 8'h4b};
         8'hd1: sboxtimes2357 = {8'h7c, 8'h42, 8'hc6, 8'hba};
         8'hd2: sboxtimes2357 = {8'h71, 8'hc4, 8'h57, 8'h26};
         8'hd3: sboxtimes2357 = {8'hcc, 8'haa, 8'he5, 8'h29};
         8'hd4: sboxtimes2357 = {8'h90, 8'hd8, 8'h73, 8'he3};
         8'hd5: sboxtimes2357 = {8'h06, 8'h05, 8'h0f, 8'h09};
         8'hd6: sboxtimes2357 = {8'hf7, 8'h01, 8'h03, 8'hf4};
         8'hd7: sboxtimes2357 = {8'h1c, 8'h12, 8'h36, 8'h2a};
         8'hd8: sboxtimes2357 = {8'hc2, 8'ha3, 8'hfe, 8'h3c};
         8'hd9: sboxtimes2357 = {8'h6a, 8'h5f, 8'he1, 8'h8b};
         8'hda: sboxtimes2357 = {8'hae, 8'hf9, 8'h10, 8'hbe};
         8'hdb: sboxtimes2357 = {8'h69, 8'hd0, 8'h6b, 8'h02};
         8'hdc: sboxtimes2357 = {8'h17, 8'h91, 8'ha8, 8'hbf};
         8'hdd: sboxtimes2357 = {8'h99, 8'h58, 8'he8, 8'h71};
         8'hde: sboxtimes2357 = {8'h3a, 8'h27, 8'h69, 8'h53};
         8'hdf: sboxtimes2357 = {8'h27, 8'hb9, 8'hd0, 8'hf7};
         8'he0: sboxtimes2357 = {8'hd9, 8'h38, 8'h48, 8'h91};
         8'he1: sboxtimes2357 = {8'heb, 8'h13, 8'h35, 8'hde};
         8'he2: sboxtimes2357 = {8'h2b, 8'hb3, 8'hce, 8'he5};
         8'he3: sboxtimes2357 = {8'h22, 8'h33, 8'h55, 8'h77};
         8'he4: sboxtimes2357 = {8'hd2, 8'hbb, 8'hd6, 8'h04};
         8'he5: sboxtimes2357 = {8'ha9, 8'h70, 8'h90, 8'h39};
         8'he6: sboxtimes2357 = {8'h07, 8'h89, 8'h80, 8'h87};
         8'he7: sboxtimes2357 = {8'h33, 8'ha7, 8'hf2, 8'hc1};
         8'he8: sboxtimes2357 = {8'h2d, 8'hb6, 8'hc1, 8'hec};
         8'he9: sboxtimes2357 = {8'h3c, 8'h22, 8'h66, 8'h5a};
         8'hea: sboxtimes2357 = {8'h15, 8'h92, 8'had, 8'hb8};
         8'heb: sboxtimes2357 = {8'hc9, 8'h20, 8'h60, 8'ha9};
         8'hec: sboxtimes2357 = {8'h87, 8'h49, 8'hdb, 8'h5c};
         8'hed: sboxtimes2357 = {8'haa, 8'hff, 8'h1a, 8'hb0};
         8'hee: sboxtimes2357 = {8'h50, 8'h78, 8'h88, 8'hd8};
         8'hef: sboxtimes2357 = {8'ha5, 8'h7a, 8'h8e, 8'h2b};
         8'hf0: sboxtimes2357 = {8'h03, 8'h8f, 8'h8a, 8'h89};
         8'hf1: sboxtimes2357 = {8'h59, 8'hf8, 8'h13, 8'h4a};
         8'hf2: sboxtimes2357 = {8'h09, 8'h80, 8'h9b, 8'h92};
         8'hf3: sboxtimes2357 = {8'h1a, 8'h17, 8'h39, 8'h23};
         8'hf4: sboxtimes2357 = {8'h65, 8'hda, 8'h75, 8'h10};
         8'hf5: sboxtimes2357 = {8'hd7, 8'h31, 8'h53, 8'h84};
         8'hf6: sboxtimes2357 = {8'h84, 8'hc6, 8'h51, 8'hd5};
         8'hf7: sboxtimes2357 = {8'hd0, 8'hb8, 8'hd3, 8'h03};
         8'hf8: sboxtimes2357 = {8'h82, 8'hc3, 8'h5e, 8'hdc};
         8'hf9: sboxtimes2357 = {8'h29, 8'hb0, 8'hcb, 8'he2};
         8'hfa: sboxtimes2357 = {8'h5a, 8'h77, 8'h99, 8'hc3};
         8'hfb: sboxtimes2357 = {8'h1e, 8'h11, 8'h33, 8'h2d};
         8'hfc: sboxtimes2357 = {8'h7b, 8'hcb, 8'h46, 8'h3d};
         8'hfd: sboxtimes2357 = {8'ha8, 8'hfc, 8'h1f, 8'hb7};
         8'hfe: sboxtimes2357 = {8'h6d, 8'hd6, 8'h61, 8'h0c};
         8'hff: sboxtimes2357 = {8'h2c, 8'h3a, 8'h4e, 8'h62};
      endcase // case(s_in)
endfunction // sboxtimes2357
