/********************************************************************
 * File: nf_icing3_low_level.h
 * Author: Jad Naous
 * Description: low-level header file for interacting with the hardware
 ********************************************************************/
#ifndef _NF_ICING3_LOW_LEVEL_H_
#define _NF_ICING3_LOW_LEVEL_H_

#include <nf2util.h>
#include <stdint.h>

/* write an array into the netfpga skipping stride
 * 32-bit words between each address */
void nf_write_array(const uint8_t *array, uint8_t len, 
                    uint32_t base_addr, int stride,
                    struct nf2device *nf2);

/* reads an array from the netfpga skipping stride
 * 32-bit words between each address */
void nf_read_array(uint8_t *array, uint8_t len, 
                   uint32_t base_addr, int stride,
                   struct nf2device *nf2);

/* use these function to read/write the hash function for 
 * the shared keys. The functions either read or write 
 * arrays that are of length UHF_NUM_BYTES. These values
 * _MUST_ be randomly generated. The UHF must remain secret.*/
void nf_icing3_read_uhf_table(uint8_t *uhf, struct nf2device *dev);
void nf_icing3_write_uhf_table(const uint8_t *uhf, int full_uhf,
                               struct nf2device *dev);

/* Returns a 16-bit hash using the preset UHF table. */
uint16_t nf_icing3_uhf(const uint8_t *realm_id, const uint8_t* uhf);

/* Set/read the mac addresses of all ports. mac must be uint8_t[4][6] */
void nf_icing3_write_port_mac_addrs(const uint8_t *mac, struct nf2device *dev);
void nf_icing3_read_port_mac_addrs(uint8_t *mac, struct nf2device *dev);

/* Set/read the realm ID. Currently not needed. */
void nf_icing3_write_realm_id(const uint8_t *realm_id, struct nf2device *dev);
void nf_icing3_read_realm_id(uint8_t *realm_id, struct nf2device *dev);

#endif /* _NF_ICING3_LOW_LEVEL_H_ */
