/********************************************************************
 * File: nf_icing3_types.h
 * Author: Jad Naous
 * Description: Contains types used to interact with hardware
 ********************************************************************/
#ifndef _NF_ICING3_TYPES_H_
#define _NF_ICING3_TYPES_H_
#include <stdint.h>
#include <nf2util.h>
#include "reg_defines_icing_forwarder.h"


typedef struct realm_id {
  uint8_t id[20];
} realm_id_t;

typedef struct vnode {
  uint8_t vnode[4];
} vnode_t;

/* use to pass realm ids */
struct nf_icing_realm_id {
  realm_id_t realm_id;
  vnode_t vnode;
} __attribute__((__packed__));
typedef struct nf_icing_realm_id nf_icing_realm_id_t;

typedef struct poc_key {
  uint8_t key[16];
} poc_key_t;

/* use to configure the hardware */
typedef struct nf_icing_dev_config {
  char *dev_name;          /* The interface name (e.g. "nf2c0") */
  uint8_t macs[24];
  realm_id_t realm_id;             /* This realm's id */
} nf_icing_dev_config_t;

/* This is an opaque struct that should not be modified by
 * the user. */
#define UHF_NUM_ENTRIES = 256;
typedef struct nf_icing_dev {
  uint8_t uhf[ICING_UHF_NUM_BYTES];
  realm_id_t realm_id;
  struct nf2device nf2;
} nf_icing_dev_t;

/* Use to read/write realm info in the HW */
struct nf_icing_realm_info {
  realm_id_t realm_id;
  uint8_t pad[4];          // set to zero
  uint8_t shared_key[16];
  uint8_t my_id_is_less;   // strictly less
  uint8_t output_mask;
  uint8_t next_hop_mac[6];
} __attribute__((__packed__));
typedef struct nf_icing_realm_info nf_icing_realm_info_t;

#endif /* _NF_ICING3_TYPES_H_ */
