#!/usr/bin/perl -w

use strict;
use NF2::TestLib;
use ICING3Packet;
use ICING3Realm;
use ICING3Dev;

require "reg_defines.ph";
require "reg_defines_icing_forwarder.ph";

my @interfaces = ("eth6", "nf2c0", "eth7", "nf2c1", "eth8", "nf2c2", "eth9", "nf2c3");
my $total_errors = 0;

nftest_init(\@ARGV,\@interfaces,);
nftest_start(\@interfaces);

srand(2131);

sub someRandom {
  rand(4);
  return 2;
}

# *********** Write config info *****************
my $full_uhf = 1;
my $write_uhf = 1;
my $full_vnode = 0;

my @source_id = ( 0xE9, 0xA3, 0x46, 0x7B, 0x53, 0x26, 0xC9, 0x18, 0x85, 0x26, 0x5F, 0x1C, 0x36, 0x81, 0x21, 0x84, 0xFA, 0x07, 0x19, 0x32 );
my @realm_id = ( 0x3A, 0xCA, 0xA2, 0x61, 0xE5, 0xD8, 0x3D, 0xBC, 0xC1, 0x56, 0x94, 0xD3, 0x73, 0x3B, 0x5F, 0xDD, 0x97, 0x45, 0xE2, 0x02 );
my @dest_id = ( 0xDC, 0xB5, 0x07, 0xF7, 0x13, 0xE9, 0xCB, 0x73, 0x4A, 0xD5, 0x81, 0x3C, 0x56, 0x74, 0xA2, 0x17, 0xDC, 0xF3, 0x12, 0x7B );
my @port_macs = ( [0x00, 0x00, 0xca, 0xfe, 0x00, 0x01],
                  [0x00, 0x00, 0xca, 0xfe, 0x00, 0x02],
                  [0x00, 0x00, 0xca, 0xfe, 0x00, 0x03],
                  [0x00, 0x00, 0xca, 0xfe, 0x00, 0x04],);


my $dev = ICING3Dev->new(ICING3Dev::ID() => \@realm_id,
                         ICING3Dev::MACS() => [@{$port_macs[0]},
                                               @{$port_macs[1]},
                                               @{$port_macs[2]},
                                               @{$port_macs[3]},
                                              ],
			 ICING3Dev::HW() => 1,
                        );

print "initializing Device\n";
$dev->initDevSim($full_uhf, $write_uhf);
print "Filling poc key table Device\n";
$dev->fillPocKeyTable($full_vnode);

# *********** Create turd server *****************
my $num_realms = 0;
my $tokenserver = CTokenServer->new(CTokenServer::NUM_REALMS() => $num_realms,
				    CTokenServer::FULL_VNODE() => $full_vnode,
				    CTokenServer::FULL_UHF() => $full_uhf,
				   );
my $source_realm = Realm->new(Realm::ID() => \@source_id );#, Realm::OUTPUT_QUEUE() => 0);
$tokenserver->addRealm($source_realm);

my $dest_realm = Realm->new(Realm::ID() => \@dest_id , Realm::OUTPUT_QUEUE() => 6);
$tokenserver->addRealm($dest_realm);

my $my_realm = Realm->new(Realm::ID() => \@realm_id);
$tokenserver->addRealm($my_realm);

# **** Write shared keys and other crap to HW ******
my @realm_list = $tokenserver->getOtherRealms($my_realm);
print "Other realm list: @realm_list\n";
map {printf "     %s\n", unpack("H*", pack("C*", @{$_->{Realm::ID()}}))} @realm_list;

foreach (0..scalar(@realm_list) - 1) {
  my $realm = $realm_list[$_];
  printf("Writing Realm %s: outq %u\n",
	 unpack("H*", pack("C*", @{$realm->{Realm::ID()}})),
	 $realm->{Realm::OUTPUT_QUEUE()});
  $dev->writeRealmSim($realm->{Realm::ID()},
                      [$my_realm->getMySharedKey($realm)],
                      $realm->{Realm::NEXT_HOP_MAC()},
                      (1 << ($realm->{Realm::OUTPUT_QUEUE()})),
                     );
}

# *********** Send packet *****************
my $num_pkts = 1;
for(my $pkt_cnt = 0; $pkt_cnt<$num_pkts; $pkt_cnt++) {
  my $path_len = 3;
  my $path_idx = 1;
  my $input_port = someRandom();

#  if($pkt_cnt == 0) {
#    $path_idx = 1;
#    $path_len = 3;
#  } elsif ($pkt_cnt == 2) {
#    $path_idx = 2;
#    $path_len = 4;
#  }

  my $pkt_delay = 100 + 3*$num_realms + 5;


my $which = 0;
my @path;
if($which == 0)
{
  printf("My path:\n");
  my @path_ids = ( \@source_id, \@realm_id, \@dest_id );

  foreach my $id (@path_ids) {
    printf "    (%s)\n", join(":", map(sprintf("%02x", $_), @$id));
  }

  @path = map(RealmToken->new(RealmToken::ID() => $_), @path_ids);
}
else
{
  @path = $tokenserver->generateRandomPath($my_realm, $path_idx, $path_len);
}

  my $pkt = ICING3Packet->newSenderPkt(ICING3Packet::CTOKEN_SERVER() => $tokenserver,
                                       ICING3Packet::ICING_PATH() => \@path,
				       ICING3Packet::PAYLOAD_LEN() => 50,
				       ICING3Packet::ETH_SRC() => [0x00, 0x00, 0x76, 0x62, 0x72, 0x65],
                                       ICING3Packet::ETH_DST() => $port_macs[$input_port]);
  for (my $i=1; $i<$path_idx; $i++) {
    printf "***** Forwarding through $i\n";
    # get Realm
    my $r = $tokenserver->getRealm($path[$i]);
    # forward through that realm
    $pkt = $r->forwardPkt($pkt);
  }
  die "Why pkt not defined no mo'?" unless defined $pkt;

  printf "==============Packet sent in:\n%s\n", join(" ",$pkt->hexBytes());
  nftest_send($interfaces[$input_port * 2], $pkt->packed());

  # *********** Expect packet *****************
  printf "***** Forwarding through my_realm\n";
  my $exp_pkt = $my_realm->forwardPkt($pkt);
  my $nxt_hop_rlm = $tokenserver->getRealm($path[$path_idx+1]);
  die "Why pkt not defined no mo'?" unless defined $exp_pkt;

  my $out_port = $nxt_hop_rlm->{Realm::OUTPUT_QUEUE()}/2 + 1;
  my $out_q = $nxt_hop_rlm->{Realm::OUTPUT_QUEUE()};

  $pkt->{ICING3Packet::ETH_SRC()} = $port_macs[$out_port-1];
  $pkt->{ICING3Packet::ETH_DST()} = $nxt_hop_rlm->{Realm::NEXT_HOP_MAC()};

  print "Pkt goes: $input_port --> $out_port\n";
  printf "==============Packet expected, length %d:\n%s\n\n", (scalar($pkt->tobytes()), join(" ",$pkt->hexBytes()));
  nftest_expect($interfaces[$out_q], $pkt->packed());
}


sleep 2;
my $unmatched_hoh = nftest_finish();
$total_errors += nftest_print_errors($unmatched_hoh);
$total_errors += nftest_get_badReads();

if ($total_errors==0) {
  print "SUCCESS!\n";
        exit 0;
}
else {
  print "FAIL: $total_errors errors\n";
        exit 1;
}

