/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Module: arbitrated_aes.v
 * Project: ICING v3
 * Author: Jad Naous <jnaous@stanford.edu>
 *
 ******************************************************************************/
`timescale 1ns/1ps

module arbitrated_aes
  (input [0:3*128-1]      block_in,
   input [0:2]            block_in_vld,
   
   output     [0:3*128-1] block_out,
   output     [0:2]       block_out_vld,
   output     [0:2]       block_in_ack,
   
   input [0:3*128-1]      key,
   
   input                  clk,
   input                  reset);

   wire [0:127] aes_key, aes_block_in, aes_block_out;
   wire [0:127] client_rslt;

   wire [0:3*128-1] block_buf;
   wire [0:2]       block_buf_vld;
   wire [0:2]       block_buf_ack;
   wire [0:3*128-1] key_buf;

   reg [0:127]      aes_block_out_reg;
   reg              aes_block_out_vld_reg;
   wire             aes_block_out_vld;

   assign block_out = {3{client_rslt}};

   generate
      genvar i;
      for(i=0; i<3; i=i+1) begin:gen_buffer
         buffer #(.WIDTH(256)) buffer
           (.din          ({block_in[128*i:128*i+127], key[128*i:128*i+127]}),
            .din_ack      (block_in_ack[i]),
            .din_vld      (block_in_vld[i]),
                          
            .dout         ({block_buf[128*i:128*i+127], key_buf[128*i:128*i+127]}),
            .dout_ack     (block_buf_ack[i]),
            .dout_vld     (block_buf_vld[i]),

            .reset        (reset),
            .clk          (clk));
      end // block: gen_buffer
   endgenerate

   arbitrator
     #(.SERV_DATA_WIDTH (128),
       .SUPP_DATA_WIDTH (128),
       .FIFO_DEPTH_BITS (4),
       .NUM_CLIENTS     (4)) arbitrator
       (.client_supp_data ({key_buf, 128'h0}),
        .client_serv_data ({block_buf, 128'h0}),
        .client_req       ({block_buf_vld, 1'b0}),
        .client_ack       ({block_buf_ack, tmp1}),
        .client_vld       ({block_out_vld, tmp2}),
        .client_rslt_data (client_rslt),
        .server_supp_data (aes_key),
        .server_serv_data (aes_block_in),
        .server_req       (aes_block_in_vld),
        .server_ack       (aes_block_in_ack),
        .server_vld       (aes_block_out_vld_reg),
        .server_rslt_data (aes_block_out_reg),
        .clk              (clk),
        .reset            (reset));

   always @(posedge clk) begin
      aes_block_out_vld_reg    <= aes_block_out_vld;
      aes_block_out_reg        <= aes_block_out;
   end

   aes aes
     (// Inputs
      .key                              (aes_key),
      .block_in                         (aes_block_in),
      .block_in_vld                     (aes_block_in_vld),

      // Outputs
      .block_out                        (aes_block_out),
      .block_out_vld                    (aes_block_out_vld),
      .last_round_key                   (),
      .block_in_ack                     (aes_block_in_ack),
      .rdy                              (),

      .clk                              (clk),
      .reset                            (reset));
endmodule // arbitrated_aes
