/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Module: arbitrated_mac2.v
 * Project: ICING v3
 * Author: Jad Naous <jnaous@stanford.edu>
 *
 ******************************************************************************/
`timescale 1ns/1ps

module arbitrated_mac2
  (input [0:3*256-1]      block,
   input [0:2]            block_vld,
   output     [0:2]       block_ack,
   input [0:3*128-1]      key,
   
   output     [0:3*96-1]  tag,
   output     [0:2]       tag_vld,
   
   // --- Interface to shared AES core
   output     [0:127]     aes_enc_plain_0,
   output     [0:127]     aes_enc_key_0,
   output                 aes_enc_plain_vld_0,
   input                  aes_enc_plain_ack_0,
   input      [0:127]     aes_enc_cipher_0,
   input                  aes_enc_cipher_vld_0,

   // --- Interface to shared AES core
   output     [0:127]     aes_enc_plain_1,
   output     [0:127]     aes_enc_key_1,
   output                 aes_enc_plain_vld_1,
   input                  aes_enc_plain_ack_1,
   input      [0:127]     aes_enc_cipher_1,
   input                  aes_enc_cipher_vld_1,

   input                  clk,
   input                  reset);

   wire [0:127] mac2_key, mac2_tag;
   wire [0:255] mac2_block;
   wire [0:95] client_rslt;

   assign tag = {3{client_rslt}};

   arbitrator
     #(.SERV_DATA_WIDTH (256),
       .RSLT_DATA_WIDTH (96),
       .SUPP_DATA_WIDTH (128),
       .FIFO_DEPTH_BITS (4),
       .NUM_CLIENTS     (4)) arbitrator
       (.client_supp_data ({key, 128'h0}),
        .client_serv_data ({block, 256'h0}),
        .client_req       ({block_vld, 1'b0}),
        .client_ack       ({block_ack, tmp1}),
        .client_vld       ({tag_vld, tmp2}),
        .client_rslt_data (client_rslt),
        .server_supp_data (mac2_key),
        .server_serv_data (mac2_block),
        .server_req       (mac2_block_vld),
        .server_ack       (mac2_block_ack),
        .server_vld       (mac2_tag_vld),
        .server_rslt_data (mac2_tag[0:95]),
        .clk              (clk),
        .reset            (reset));

    cbc_mac2_128 cbc_mac2_128
     (.key                    (mac2_key),
      .block                  (mac2_block),
      .block_vld              (mac2_block_vld),
      .block_ack              (mac2_block_ack),
      .tag                    (mac2_tag),
      .tag_vld                (mac2_tag_vld),
      .aes_enc_cipher_0       (aes_enc_cipher_0),
      .aes_enc_cipher_vld_0   (aes_enc_cipher_vld_0),
      .aes_enc_plain_0        (aes_enc_plain_0),
      .aes_enc_plain_ack_0    (aes_enc_plain_ack_0),
      .aes_enc_key_0          (aes_enc_key_0),
      .aes_enc_plain_vld_0    (aes_enc_plain_vld_0),
      .aes_enc_cipher_1       (aes_enc_cipher_1),
      .aes_enc_cipher_vld_1   (aes_enc_cipher_vld_1),
      .aes_enc_plain_1        (aes_enc_plain_1),
      .aes_enc_plain_ack_1    (aes_enc_plain_ack_1),
      .aes_enc_key_1          (aes_enc_key_1),
      .aes_enc_plain_vld_1    (aes_enc_plain_vld_1),
      .clk                    (clk),
      .reset                  (reset));

endmodule // arbitrated_aes
