/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Module: calculate_A.v
 * Project: ICING v3
 * Author: Jad Naous <jnaous@stanford.edu>
 * Description:
 *   Finds A[i] = mac(poc, 9'h0 || H[0..246])
 ******************************************************************************/

module calculate_A
  (// --- Interface to hash_pkt
   input      [0:246] hash,
   input              hash_vld,

   // --- Interface to calculate_poc
   input      [0:127] poc,
   input              poc_vld,

   // --- Interface to shared 2 block mac core
   output     [0:127] mac2_key,
   output     [0:255] mac2_block,
   output             mac2_block_vld,
   input              mac2_block_ack,
   input      [0:95]  mac2_tag,
   input              mac2_tag_vld,

   // --- Interface to results
   output     [0:95]  A,
   output             A_vld,

   // --- Misc
   input              clk,
   input              reset);

   //------------------------ Wires/Regs -----------------------------
   wire [0:246]       hash_dout;
   wire [0:127]       poc_dout;
   
   //------------------------- Modules -------------------------------

   fallthrough_small_fifo 
     #(.WIDTH(247),
       .MAX_DEPTH_BITS(2))
      hash_fifo
        (.din           (hash),
         .wr_en         (hash_vld),
         .rd_en         (mac2_block_ack),
         .dout          (hash_dout),
         .full          (hash_fifo_full),
         .prog_full     (),
         .nearly_full   (),
         .empty         (hash_fifo_empty),
         .reset         (reset),
         .clk           (clk)
         );

   fallthrough_small_fifo 
     #(.WIDTH(128),
       .MAX_DEPTH_BITS(3))
      poc_fifo
        (.din           (poc),
         .wr_en         (poc_vld),
         .rd_en         (mac2_block_ack),
         .dout          (poc_dout),
         .full          (poc_fifo_full),
         .prog_full     (),
         .nearly_full   (),
         .empty         (poc_fifo_empty),
         .reset         (reset),
         .clk           (clk)
         );
      
   //-------------------------- Logic --------------------------------

   /******************************************************************
    * Wait till all inputs to the mac are available, then do it.
    ******************************************************************/
   assign mac2_key         = poc_dout;
   assign mac2_block       = {9'h0, hash_dout};
   assign mac2_block_vld   = (!hash_fifo_empty & !poc_fifo_empty);
   assign A                = mac2_tag;
   assign A_vld            = mac2_tag_vld;

   // synthesis translate_off
   integer A_count = 0;
   always @(posedge clk) begin
      if(A_vld === 1) begin
         A_count <= A_count + 1;
      end
      if(A_vld === 1) begin
         $display($time, " %m Num A's calc'ed  :  %u", (A_count + 1));
      end
   end
   // synthesis translate_on

endmodule // calculate_A
