module chi_hash_pipelined2
  (input  [0:2*512-1] msg_block_in,
   input  [0:1]       msg_block_vld,
   input  [0:1]       last_block,
   output [0:1]       msg_block_ack,

   output [0:2*256-1] hash_out,
   output [0:1]       hash_vld,

   input              clk,
   input              reset);

   wire [0 : 6*64-1]  new_abcdef;
   wire [0 : 6*64-1]  abcdef[0:1];
   wire [0:255]       hash[0:1];
   wire [0:511]       msg_block[0:1];
   wire [0:63]        W_2t[0:1];
   wire [0:63]        W_2t1[0:1];
   wire [0:63]        K_2t[0:1];
   wire [0:63]        K_2t1[0:1];
   wire [1:0]         tmod3[0:1];
   reg                toggle;

   generate 
      genvar          i;
      for(i=0; i<2; i=i+1) begin:gen_assign
         assign msg_block[i]              = msg_block_in[i*512:i*512+511];
         assign hash_out[i*256:i*256+255] = hash[i];
      end
   endgenerate

   always @(posedge clk) begin
      toggle <= reset ? 0 : ~toggle;
   end
   
   chi_round_pipelined chi_round
     (// Outputs
      .new_abcdef                       (new_abcdef),
      // Inputs
      .abcdef                           (abcdef[toggle]),
      .W_2t                             (W_2t[toggle]),
      .W_2t1                            (W_2t1[toggle]),
      .K_2t                             (K_2t[toggle]),
      .K_2t1                            (K_2t1[toggle]),
      .tmod3                            (tmod3[toggle]),
      .clk                              (clk)
      );

   chi_hash_pipelined_ext chi_hash_pipelined_ext_0
     (// Outputs
      .msg_block_ack                    (msg_block_ack[0]),
      .hash                             (hash[0]),
      .hash_vld                         (hash_vld[0]),
      .abcdef                           (abcdef[0]),
      .W_2t                             (W_2t[0]),
      .W_2t1                            (W_2t1[0]),
      .K_2t                             (K_2t[0]),
      .K_2t1                            (K_2t1[0]),
      .tmod3                            (tmod3[0]),
      // Inputs
      .msg_block                        (msg_block[0]),
      .msg_block_vld                    (msg_block_vld[0]),
      .last_block                       (last_block[0]),
      .new_abcdef                       (new_abcdef),
      .round_rdy_nxt                    (toggle),
      .clk                              (clk),
      .reset                            (reset));

   chi_hash_pipelined_ext chi_hash_pipelined_ext_1
     (// Outputs
      .msg_block_ack                    (msg_block_ack[1]),
      .hash                             (hash[1]),
      .hash_vld                         (hash_vld[1]),
      .abcdef                           (abcdef[1]),
      .W_2t                             (W_2t[1]),
      .W_2t1                            (W_2t1[1]),
      .K_2t                             (K_2t[1]),
      .K_2t1                            (K_2t1[1]),
      .tmod3                            (tmod3[1]),
      // Inputs
      .msg_block                        (msg_block[1]),
      .msg_block_vld                    (msg_block_vld[1]),
      .last_block                       (last_block[1]),
      .new_abcdef                       (new_abcdef),
      .round_rdy_nxt                    (~toggle),
      .clk                              (clk),
      .reset                            (reset));
   
endmodule // chi_hash_pipelined

module chi_hash_pipelined_ext
  (input  [0:511] msg_block,
   input          msg_block_vld,
   input          last_block,
   output reg     msg_block_ack,

   output [0:255] hash,
   output reg     hash_vld,

   input [0 : 6*64-1] new_abcdef,
   output reg [0 : 6*64-1]  abcdef,
   output [0:63]       W_2t,
   output [0:63]       W_2t1,
   output [0:63]       K_2t,
   output [0:63]       K_2t1,
   output reg [1:0]    tmod3,
   input               round_rdy_nxt,

   input          clk,
   input          reset);

`include "chi.v.inc"

   //------------------ Internal Parameter ---------------------------
   localparam NUM_ROUNDS       = 20;
   localparam WAIT_FOR_START   = 1;
   localparam LATCH_RESULT     = 2;
   localparam DO_ROUNDS        = 3;

   //---------------------- Wires/Regs -------------------------------
   reg [0 : 6*64-1]  abcdef_nxt;
   reg [1:0]         tmod3_nxt;
   reg [4:0]         round, round_nxt;
   reg               ld_msg_block;
   reg               next_W;
   wire [0 : 6*64-1] rotate_H0, H0, rotated_hash_reg, rotated_abcdef;
   wire [0 : 6*64-1] new_hash_reg;
   
   reg [0 : 6*64-1]  hash_reg, hash_reg_nxt;
   reg [0:2]         state, state_nxt;
   reg               last_block_nxt, last_block_reg;
   reg               hash_vld_nxt;
   
   //----------------------- Modules ---------------------------------
   chi_cst chi_cst
     (.kt    (K_2t),
      .kt1   (K_2t1),
      .t     ({1'b0, round_nxt}),
      .clk   (clk));

   chi_stepinput chi_stepinput
     (.msg_block       (msg_block),
      .ld_msg_block    (ld_msg_block),
      .next_W          (next_W),
      .W_2t            (W_2t),
      .W_2t1           (W_2t1),
      .clk             (clk),
      .reset           (reset)
      );

   generate
      genvar  i;
      for(i=0; i<6; i=i+1) begin:rotate
         chi_rotr64 chi_rotr64_0 (chi256_init(i), 1, rotate_H0[i*64 +: 64]);
         assign H0[i*64 +: 64] = chi256_init(i);

         chi_rotr64 chi_rotr64_1 (hash_reg[i*64 +: 64], 1, rotated_hash_reg[i*64 +: 64]); 

         chi_rotr64 chi_rotr64_2 (new_hash_reg[i*64 +: 64],
                                1, rotated_abcdef[i*64 +: 64]);
      end
   endgenerate
   
   //------------------------ Logic ----------------------------------
   
   assign       hash = {hash_reg[0:127], hash_reg[64*3 +: 128]};
   assign       new_hash_reg = rotated_hash_reg ^ new_abcdef;
   
   always @(*) begin
      msg_block_ack    = 0;
      ld_msg_block     = 0;
      round_nxt        = round;
      tmod3_nxt        = tmod3;
      next_W           = 0;
      hash_reg_nxt     = hash_reg;
      abcdef_nxt       = abcdef;
      state_nxt        = state;
      last_block_nxt   = last_block_reg;
      hash_vld_nxt     = 0;
      
      case(state)
         WAIT_FOR_START: begin
            if(msg_block_vld) begin
               if(round_rdy_nxt) begin
                  msg_block_ack   = 1'b1;
                  ld_msg_block    = 1'b1;
                  round_nxt       = 0;
                  tmod3_nxt       = 0;
                  hash_reg_nxt    = H0;
                  if(last_block) begin
                     abcdef_nxt   = rotate_H0;
                  end else begin
                     abcdef_nxt   = H0;
                  end
                  state_nxt       = LATCH_RESULT;
                  last_block_nxt  = last_block;
               end // if (round_rdy_nxt)
            end // if (msg_block_vld)
         end // case: WAIT_FOR_START

         LATCH_RESULT: begin
            state_nxt = DO_ROUNDS;
         end

         DO_ROUNDS: begin
            if(round == NUM_ROUNDS - 1'b1) begin
               /* if this is the last block then emit the hash the next
                * clock cycle and go to start. We can save a cycle here
                * by not going to start if we need to. */
               if(last_block_reg) begin
                  hash_vld_nxt  = 1'b1;
                  state_nxt     = WAIT_FOR_START;
                  /* latch the intermediate value */
                  hash_reg_nxt    = new_hash_reg;
               end
               /* If there are more blocks coming, then wait for a new
                * block to be available. */
               else begin
                  if(msg_block_vld && round_rdy_nxt) begin
                     /* initialize everything */
                     msg_block_ack    = 1'b1;
                     ld_msg_block     = 1'b1;
                     round_nxt        = 0;
                     tmod3_nxt        = 0;
                     last_block_nxt   = last_block;
                     /* latch the intermediate value */
                     hash_reg_nxt     = new_hash_reg;
                     
                     state_nxt        = LATCH_RESULT;

                     /* If this new block is the last, then we need to rotate
                      * the new hash_reg. */
                     if(last_block) begin
                        abcdef_nxt   = rotated_abcdef;
                     end
                     /* otherwise just use the new hash_reg */
                     else begin
                        abcdef_nxt   = new_hash_reg;
                     end

                  end // if (msg_block_vld)
               end // else: !if(last_block_reg)
            end // if (round == NUM_ROUNDS - 1'b1)
            else begin
               /* do a round */
               round_nxt     = round + 1'b1;
               tmod3_nxt     = tmod3 == 2 ? 0 : tmod3 + 1'b1;
               next_W        = 1'b1;
               abcdef_nxt    = new_abcdef;
               state_nxt     = LATCH_RESULT;
            end // else: !if(round == NUM_ROUNDS - 1'b1)
         end // case: DO_ROUNDS
      endcase // case(state)
   end // always @ (*)

   always @(posedge clk) begin
      round             <= round_nxt;
      tmod3             <= tmod3_nxt;
      abcdef            <= abcdef_nxt;
      hash_reg          <= hash_reg_nxt;
      last_block_reg    <= last_block_nxt;
      if(reset) begin
         hash_vld    <= 0;
         state       <= WAIT_FOR_START;
      end
      else begin
         hash_vld    <= hash_vld_nxt;
         state       <= state_nxt;
      end
   end // always @ (posedge clk)

endmodule // chi_hash
