/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Module: chi_round.v
 * Project: CHI
 * Author: Jad Naous <jnaous@stanford.edu>
 * Description: Implements a single round of CHI (SHA-3 candidate)
 * 
 ******************************************************************************/
`timescale 1ns/1ps

module chi_round
  (input [0 : 6*64-1]  abcdef,
   input [0 : 63]      W_2t,
   input [0 : 63]      W_2t1,
   input [0 : 63]      K_2t,
   input [0 : 63]      K_2t1,

   input [1:0]         tmod3,

   output [0 : 6*64-1] new_abcdef);

`include "chi.v.inc"
   
   wire [0:63] preR, preS, preT, preU, V_0, V_1,
               R, S, T, U, X, Y, Z, XX, YY, ZZ, AA,
               DD, newA, newD, A, B, C, D, E, F;

   wire [0:63] drotr88_B, drotr51_swap_D, drotr1817_swap_D,
               drotr726_swap_A, drotr1422_D, drotr1712_A,
               drotr223_swap_E, theta0_V_0, theta1_V_1,
               drotr55_Y;

   assign {A, B, C, D, E, F} = abcdef;
   
   chi_drotr32 chi_drotr32_0(B, 8, 8, drotr88_B);
   chi_drotr32 chi_drotr32_1(chi256_swap32(D), 5, 1, drotr51_swap_D);
   assign preR = drotr88_B ^ drotr51_swap_D;

   chi_drotr32 chi_drotr32_2(chi256_swap32(D), 18, 17, drotr1817_swap_D);
   assign preS = A ^ drotr1817_swap_D;

   chi_drotr32 chi_drotr32_3(chi256_swap32(A), 7, 26, drotr726_swap_A);
   chi_drotr32 chi_drotr32_4(D, 14, 22, drotr1422_D);
   assign preT = drotr726_swap_A ^ drotr1422_D;

   chi_drotr32 chi_drotr32_5(A, 17, 12, drotr1712_A);
   chi_drotr32 chi_drotr32_6(chi256_swap32(E), 2, 23, drotr223_swap_E);
   assign preU = drotr1712_A ^ drotr223_swap_E;

   assign V_0 = W_2t ^ K_2t;
   
   assign V_1 = W_2t1 ^ K_2t1;

   assign R = preR ^ V_0;
   
   assign S = preS ^ V_1;

   chi_theta256_0 chi_theta256_0_0(V_0, theta0_V_0);
   assign T = preT ^ theta0_V_0;

   chi_theta256_1 chi_theta256_1_0(V_1, theta1_V_1);
   assign U = preU ^ theta1_V_1;

   chi_map64 chi_map64_0(R, S, T, U, tmod3, X, Y, Z);

   assign XX = X;

   chi_drotr32 chi_drotr32_7(Y, 5, 5, drotr55_Y);
   assign YY = chi256_swap8(drotr55_Y);

   assign ZZ = chi256_swap32(Z);

   chi_rotr64 chi_rotr64_0(XX+YY, 16, AA);

   chi_rotr64 chi_rotr64_1(YY+ZZ, 48, DD);

   assign newA = AA ^ F;

   assign newD = DD ^ C;

   assign new_abcdef = {newA, A, B, newD, D, E};

endmodule // chi_round
