/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Modules: chi_rotr32, chi_rotr64, chi_drotr32, chi_mu256_0, chi_mu256_1,
 *          chi_theta256_0, chi_theta256_1, chi_cst, chi_stepinput
 * Project: CHI (SHA-3 candidate)
 * Author: Jad Naous <jnaous@stanford.edu>
 * Description: Implements a number of support modules for CHI
 * 
 ******************************************************************************/
`timescale 1ns/1ps

module chi_rotr32
  (input [0:31] din,
   input [31:0]  rotation,
   output [0:31] dout);

   rotate #(.UNIT_SIZE (1),
            .NUM_UNITS (32),
            .SHIFT_LEFT (0))
     rotate
       (.din      (din),
        .rotation (rotation[4:0]),
        .dout     (dout));
endmodule // chi_rotr32

module chi_rotr64
  (input [0:63] din,
   input [31:0]  rotation,
   output [0:63] dout);

   rotate #(.UNIT_SIZE (1),
            .NUM_UNITS (64),
            .SHIFT_LEFT (0))
     rotate
       (.din      (din),
        .rotation (rotation[5:0]),
        .dout     (dout));
endmodule // chi_rotr64

module chi_drotr32
  (input [0:63] din,
   input [31:0]  rotation_u,
   input [31:0]  rotation_l,
   output [0:63] dout);
   chi_rotr32 rotate_u (din[0:31], rotation_u, dout[0:31]);
   chi_rotr32 rotate_l (din[32:63], rotation_l, dout[32:63]);
endmodule

module chi_mu256_0
  (input [0:63] din,
   output [0:63] dout);

   wire [0:63]   t1, t2, t3;

   chi_rotr64 r0(din, 36, t1);
   chi_rotr64 r1(din, 18, t2);
   assign t3 = din >> 1;

   assign dout = t1 ^ t2 ^ t3;
   
endmodule // chi_mu256_0

module chi_mu256_1
  (input [0:63] din,
   output [0:63] dout);

   wire [0:63]   t1, t2, t3;

   chi_rotr64 r0(din, 59, t1);
   chi_rotr64 r1(din, 37, t2);
   assign t3 = din >> 10;

   assign dout = t1 ^ t2 ^ t3;
   
endmodule // chi_mu256_1

module chi_theta256_0
  (input [0:63] din,
   output [0:63] dout);

   wire [0:63]   t1, t2, t3;

   chi_drotr32 a(din, 21, 21, t1);
   chi_drotr32 b(din, 26, 26, t2);
   chi_drotr32 c(din, 30, 30, t3);

   assign dout = t1 ^ t2 ^ t3;
   
endmodule // chi_theta256_0

module chi_theta256_1
  (input [0:63] din,
   output [0:63] dout);

   wire [0:63]   t1, t2, t3;

   chi_drotr32 a(din, 14, 14, t1);
   chi_drotr32 b(din, 24, 24, t2);
   chi_drotr32 c(din, 31, 31, t3);

   assign dout = t1 ^ t2 ^ t3;
   
endmodule // chi_theta256_1

/* -----\/----- EXCLUDED -----\/-----
module chi_cst
  (output reg [0:63] kt,
   output reg [0:63] kt1,
   input      [5:0]  t,
   input             clk
   );

   RAMB16_S36_S36 ram (.DOA    (kt),
                       .DOPA   (),
                       .ADDRA  ({3'h0, t),
                       .CLKA   (clk),
                       .DIA    (din_words[2*i]),
                       .DIPA   (4'h0),
                       .ENA    (1'b1),
                       .SSRA   (reset),
                       .WEA    (wr_en[2*i]),
      
                       .DOB    (dout_words[2*i+1]),
                       .DOPB   (),
                       .ADDRB  ({1'b1, addr}),
                       .CLKB   (clk),
                       .DIB    (din_words[2*i+1]),
                       .DIPB   (4'h0),
                       .ENB    (1'b1),
                       .SSRB   (reset),
                       .WEB    (wr_en[2*i+1]));

endmodule // chi_cst
 -----/\----- EXCLUDED -----/\----- */


module chi_cst
  (output reg [0:63] kt,
   output reg [0:63] kt1,
   input      [5:0]  t,
   input             clk
   );

   wire [0:64*80 - 1] constants;
   wire [0:63]        cst_words_0[0:39];
   wire [0:63]        cst_words_1[0:39];

   assign constants
          = {64'h9B05688C2B3E6C1F, 64'hDBD99E6FF3C90BDC,
             64'h4DBC64712A5BB168, 64'h767E27C3CF76C8E7,
             64'h21EE9AC5EF4C823A, 64'hB36CCFC1204A9BD8,
             64'h754C8A7FB36BD941, 64'hCF20868F04A825E9,
             64'hABB379E0A8838BB0, 64'h12D8B70A5959E391,
             64'hB59168FA9F69E181, 64'h15C7D4918739F18F,
             64'h4B547673EA8D68E0, 64'h3CED7326FCD0EF81,
             64'h09F1D77309998460, 64'h1AF3937415E91F32,
             64'h1F83D9ABFB41BD6B, 64'h23C4654C2A217583,
             64'h2842012131573F2A, 64'hA59916ACA3991FCA,
             64'hEC1B7C06FD19D256, 64'h1EC785BCDC8BAF26,
             64'h69CA4E0FF2E6BDD8, 64'hCA2575EE6C950D0E,
             64'h5BCF66D2FB3D99F6, 64'h9D6D08C7BBCA18F3,
             64'hEEF64039F2175E83, 64'h00ED5AEBAA2AB6E0,
             64'h5040712FC29AD308, 64'h6DAFE433438D2C43,
             64'hBD7FAA3F06C71F15, 64'h03B5AA8CE9B6A4DD,
             64'h5BE0CD19137E2179, 64'h867F5E3B72221265,
             64'h43B6CBE0D67F4A20, 64'hDB99D767CB0E4933,
             64'hDC450DBC469248BD, 64'hFE1E5E4876100D6F,
             64'hB799D29EA1733137, 64'h16EA7ABCF92053C4,
             64'hBE3ECE968DBA92AC, 64'h18538F84D82C318B,
             64'h38D79F4E9C8A18C0, 64'hA8BBC28F1271F1F7,
             64'h2796E71067D2C8CC, 64'hDE1BF2334EDB3FF6,
             64'hB094D782A857F9C1, 64'h639B484B0C1DAED1,
             64'hCBBB9D5DC1059ED8, 64'hE7730EAFF25E24A3,
             64'hF367F2FC266A0373, 64'hFE7A4D34486D08AE,
             64'hD41670A136851F32, 64'h663914B66B4B3C23,
             64'h1B9E3D7740A60887, 64'h63C11D86D446CB1C,
             64'hD167D2469049D628, 64'hDDDBB606B9A49E38,
             64'h557F1C8BEE68A7F7, 64'hF99DC58B50F924BD,
             64'h205ACC9F653512A5, 64'h67C66344E4BAB193,
             64'h18026E467960D0C8, 64'hA2F5D84DAECA8980,
             64'h629A292A367CD507, 64'h98E67012D90CBB6D,
             64'hEED758D1D18C7E35, 64'h031C02E4437DC71E,
             64'h79B63D6482198EB7, 64'h936A9D7E8C9E4B33,
             64'hB30CA682C3E6C65D, 64'hCC442382BA4262FA,
             64'h51179BA5A1D37FF6, 64'h7202BDE7A98EEA51,
             64'h2B9F65D1DF9C610F, 64'hF56B742B0AF1CE83,
             64'hF9989D199B75848B, 64'hD142F19D8B46D578,
             64'h7A7580514D75EA33, 64'hB74F9690808E704D};

   generate
      genvar i;
      for(i=0; i<40; i=i+1) begin:gen_words
         assign cst_words_0[i] = constants[128*i      +: 64];
         assign cst_words_1[i] = constants[128*i + 64 +: 64];
      end
   endgenerate

   always @(posedge clk) begin
      kt   <= cst_words_0[t];
      kt1  <= cst_words_1[t];
   end
endmodule // chi_cst

/*******************************************************************
 * This module takes as input R, S, T, U and outputs X, Y, Z
 *******************************************************************/
module chi_map64
  (input [0:63]  R,
   input [0:63]  S,
   input [0:63]  T,
   input [0:63]  U,

   input [1:0]   tmod3,

   output reg [0:63] X,
   output reg [0:63] Y,
   output reg [0:63] Z);

`include "chi.v.inc"
   
   wire [0:63]   temp0, temp1, temp2;

   assign {temp0, temp1, temp2} = chi256_map_full(R, S, T, U);

   always @(*) begin
      case(tmod3)
         0: {X, Y, Z} = {temp0, temp1, temp2};
         1: {X, Y, Z} = {temp1, temp2, temp0};
         default: {X, Y, Z} = {temp2, temp0, temp1};
      endcase
   end
endmodule // chi_map64

module chi_map64_old
  (input [0:63]  R,
   input [0:63]  S,
   input [0:63]  T,
   input [0:63]  U,

   input [1:0]   tmod3,

   output reg [0:63] X,
   output reg [0:63] Y,
   output reg [0:63] Z);

`include "chi.v.inc"
   
   wire [0:63]   temp0, temp1, temp2;

   always @(*) begin
      case(tmod3)
         0: {X, Y, Z} = {temp0, temp1, temp2};
         1: {X, Y, Z} = {temp1, temp2, temp0};
         2: {X, Y, Z} = {temp2, temp0, temp1};
         default: {X, Y, Z} = {temp0, temp1, temp2};
      endcase
   end
   
   generate
      genvar i;
      for(i=0; i<64; i=i+1) begin:gen_maps
         assign {temp0[i], temp1[i], temp2[i]} 
           = chi256_map({R[i], S[i], T[i], U[i]});
      end
   endgenerate
endmodule // chi_map64

/********************************************************
 * This module provides the step inputs W_t
 * First, load the msg block. The W_0 and W_1 will be
 * available the next clock cycle. For each subsequent
 * step, assert next_W and the output will be available
 * the next cycle.
 * 
 * The module internally has a 512-bit register that
 * stores the future values of Wt. As they are consumed,
 * two new values are appended to the end of the queue.
 * 
 * For purposes of calculation, the 8 values in the queue
 * can be numbered t=0..7. The two new values are t=8,9.
 * So if a calculation asks for W_{t-7}, then if t=9, the
 * calculation would be asking for W_1 in the register.
 ********************************************************/
module chi_stepinput
  (input [0:511] msg_block,
   input         ld_msg_block,
   input         next_W,
   output [0:63] W_2t,
   output [0:63] W_2t1,

   input         clk,
   input         reset);

   reg [0:511]   msg_reg;
   wire [0:63]   mu0_0, mu0_1;
   wire [0:63]   mu1_0, mu1_1;

   assign W_2t  = msg_reg[0:63];
   assign W_2t1 = msg_reg[64:127];

   /* calculate mu_0(W_{8-7}) */
   chi_mu256_0 a(msg_reg[64 : 127], mu0_0);

   /* calculate mu_1(W_{8-2}) */
   chi_mu256_1 b(msg_reg[6*64 : 7*64-1], mu0_1);

   /* calculate mu_0(W_{9-7}) */
   chi_mu256_0 c(msg_reg[2*64 : 3*64-1], mu1_0);

   /* calculate mu_1(W_{9-2}) */
   chi_mu256_1 d(msg_reg[7*64 : 8*64-1], mu1_1);
   
   always @(posedge clk) begin
      if(ld_msg_block) begin
         msg_reg <= msg_block;
      end else if(next_W) begin
         /* shift and store t=8,9 */
         msg_reg[0    : 6*64 - 1] <= msg_reg[2*64 : 8*64-1];
         msg_reg[6*64 : 7*64 - 1] <= msg_reg[0    : 63 ] ^ mu0_0 ^ mu0_1;
         msg_reg[7*64 : 8*64 - 1] <= msg_reg[64   : 127] ^ mu1_0 ^ mu1_1;
      end
   end

   // synthesis translate_off
   wire [0:63]  msg_reg_words[0:7];
   generate
      genvar    i;
      for(i=0; i<8; i=i+1) begin:genwords
         assign msg_reg_words[i] = msg_reg[i*64 : i*64 + 63];
      end
   endgenerate
   // synthesis translate_on

endmodule // chi_stepinput

/********************************************************
 * This module is the same as chi_stepinput, but it 
 * provides 4 words instead of 2 at a time.
 ********************************************************/
module chi_stepinput2
  (input [0:511] msg_block,
   input         ld_msg_block,
   input         next_W,
   output [0:63] W_2t,
   output [0:63] W_2t1,
   output [0:63] W_2t2,
   output [0:63] W_2t3,

   input         clk,
   input         reset);

   reg [0:511]   msg_reg;
   wire [0:63]   mu0_0, mu0_1;
   wire [0:63]   mu1_0, mu1_1;
   wire [0:63]   mu2_0, mu2_1;
   wire [0:63]   mu3_0, mu3_1;

   assign W_2t  = msg_reg[0:63];
   assign W_2t1 = msg_reg[64:127];

   assign W_2t2 = msg_reg[128:191];
   assign W_2t3 = msg_reg[192:255];

   wire [0:63]  W[0:11];
   generate
      genvar    i;
      for(i=0; i<8; i=i+1) begin:genwords
         assign W[i] = msg_reg[i*64 : i*64 + 63];
      end
   endgenerate

   /* calculate mu_0(W_{8-7}) */
   chi_mu256_0 a(W[1], mu0_0);

   /* calculate mu_1(W_{8-2}) */
   chi_mu256_1 b(W[6], mu0_1);

   /* calculate mu_0(W_{9-7}) */
   chi_mu256_0 c(W[2], mu1_0);

   /* calculate mu_1(W_{9-2}) */
   chi_mu256_1 d(W[7], mu1_1);
   
   /* calculate mu_0(W_{10-7}) */
   chi_mu256_0 e(W[3], mu2_0);

   /* calculate mu_1(W_{10-2}) */
   chi_mu256_1 f(W[8], mu2_1);

   /* calculate mu_0(W_{11-7}) */
   chi_mu256_0 g(W[4], mu3_0);

   /* calculate mu_1(W_{11-2}) */
   chi_mu256_1 h(W[9], mu3_1);

   assign W[8]  = W[0] ^ mu0_0 ^ mu0_1;
   assign W[9]  = W[1] ^ mu1_0 ^ mu1_1;
   assign W[10] = W[2] ^ mu2_0 ^ mu2_1;
   assign W[11] = W[3] ^ mu3_0 ^ mu3_1;

   always @(posedge clk) begin
      if(ld_msg_block) begin
         msg_reg <= msg_block;
      end else if(next_W) begin
         /* shift and store t=8,9,10,11 */
         msg_reg <= {msg_reg[4*64 : 8*64-1],
                     W[8], W[9], W[10], W[11]};
      end
   end
endmodule // chi_stepinput2
