/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Module: grostl_mixbytes.v
 * Project: Grostl
 * Author: Jad Naous <jnaous@stanford.edu>
 * Description: implements the MixBytes phase of a Grostl round
 * 
 ******************************************************************************/
`timescale 1ns/1ps

module grostl_mixbytes
  (input [0:511]  s_in,
   output [0:511] s_out);

`include "grostl.v.inc"

   wire [0:63] state_cols [0:7];
   wire [7:0] state_out[0:7][0:7];

   generate
      genvar i, j;
      for(i=0; i<8; i=i+1) begin:gen_cols
         /* Put the state into cols */
         assign state_cols[i] = s_in[64*i : 64*i + 63];
         
         for(j=0; j<8; j=j+1) begin: gen_rows
            wire [0:63] col_shifted, col;

            assign col = state_cols[i];
            
            /* shift the column */
            if(j != 0) begin
               assign col_shifted = {col[8*j : 63],
                                     col[0 : 8*j - 1]};
            end else begin
               assign col_shifted = col;
            end

            /* do the multiplication */
            assign state_out[j][i] = times2(col_shifted[0*8 : 0*8 + 7]) ^
                               times2(col_shifted[1*8 : 1*8 + 7]) ^
                               times3(col_shifted[2*8 : 2*8 + 7]) ^
                               times4(col_shifted[3*8 : 3*8 + 7]) ^
                               times5(col_shifted[4*8 : 4*8 + 7]) ^
                               times3(col_shifted[5*8 : 5*8 + 7]) ^
                               times5(col_shifted[6*8 : 6*8 + 7]) ^
                               times7(col_shifted[7*8 : 7*8 + 7]);

            /* linearize the output state matrix */
            assign s_out[8 * (j + i*8) : 8 * (j + i*8) + 7] = state_out[j][i];
         end // block: gen_rows
      end // block: gen_cols
   endgenerate
endmodule // grostl_mixbytes
