/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Module: grostl_mixbytes23457.v
 * Project: Grostl
 * Author: Jad Naous <jnaous@stanford.edu>
 * Description: implements the MixBytes phase of a Grostl round using
 *              results of the grostl_subtimes23457 module
 ******************************************************************************/
`timescale 1ns/1ps

module grostl_mixbytes23457
  (input [0:8*8*40-1]  s_in,
   output [0:8*8*8-1]  s_out);

`include "grostl.v.inc"

   wire [0:8*40-1] state_cols [0:7];
   wire [7:0]      state_out[0:7][0:7];

   localparam      TIMES2 = 0,
                   TIMES3 = 1,
                   TIMES4 = 2,
                   TIMES5 = 3,
                   TIMES7 = 4;

   generate
      genvar i, j, k, l;
      for(i=0; i<8; i=i+1) begin:gen_cols
         /* Put the state into cols */
         assign state_cols[i] = s_in[40*8*i : 40*8*i + 8*40 - 1];
         
         for(j=0; j<8; j=j+1) begin: gen_rows
            wire [0 : 8*40 - 1] col_shifted, col;
            wire [7 : 0]        col_shifted_bytes[0:7][0:4];

            assign col = state_cols[i];
            
            /* shift the column */
            if(j != 0) begin
               assign col_shifted = {col[40*j : 8*40 - 1],
                                     col[0 : 40*j - 1]};
            end else begin
               assign col_shifted = col;
            end

            for(k=0; k<8; k=k+1) begin: gen_shifted_words
               wire [0:39] col_shifted_word = col_shifted[k*40 : k*40 + 39];
               for(l=0; l<5; l=l+1) begin: gen_col_shifted_bytes
                  assign col_shifted_bytes[k][l] = col_shifted_word[l*8 : l*8 + 7];
               end
            end

            /* do the multiplication */
            assign state_out[j][i] = col_shifted_bytes[0][TIMES2] ^
                                     col_shifted_bytes[1][TIMES2] ^
                                     col_shifted_bytes[2][TIMES3] ^
                                     col_shifted_bytes[3][TIMES4] ^
                                     col_shifted_bytes[4][TIMES5] ^
                                     col_shifted_bytes[5][TIMES3] ^
                                     col_shifted_bytes[6][TIMES5] ^
                                     col_shifted_bytes[7][TIMES7];

            /* linearize the output state matrix */
            assign s_out[8 * (j + i*8) +: 8] = state_out[j][i];
         end // block: gen_rows
      end // block: gen_cols
   endgenerate
endmodule // grostl_mixbytes
