/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Module: grostl_permute.v
 * Project: Grostl
 * Author: Jad Naous <jnaous@stanford.edu>
 * Description: implements the P and Q permutations of grostl
 * 
 ******************************************************************************/
`timescale 1ns/1ps

`ifndef GROSTL_BLOCK_SIZE
 `include "grostl_def.v"
`endif

module grostl_permute
  (input [0:`GROSTL_BLOCK_SIZE-1]  s_in,
   input                           s_in_vld,
   output reg                      s_in_ack,
   
   output [0:`GROSTL_BLOCK_SIZE-1] s_out,
   output reg                      s_out_vld,

   input                           Q,
   
   input                           clk,
   input                           reset);

   parameter NUM_ROUNDS = 10;

   localparam WAIT_FOR_INPUT = 0,
              DO_ROUNDS      = 1;
   
   reg [0:`GROSTL_BLOCK_SIZE-1] s_sel;
   reg [3:0]                    round_num, round_num_nxt;
   reg                          state, state_nxt;
   reg                          Q_in;

   grostl_round_23457 grostl_round
     (.s_in        (s_sel),
      .s_in_vld    (s_in_vld),
      .s_out       (s_out),
      .s_out_vld   (),
      .round_num   (round_num_nxt),
      .Q_in        (Q_in),
      .Q_out       (Q_out),
      .clk         (clk),
      .reset       (reset));

   always @(*) begin
      s_sel           = s_out;
      round_num_nxt   = round_num;
      state_nxt       = state;
      s_in_ack        = 0;
      s_out_vld       = 0;
      Q_in            = Q_out;
      
      case(state)
         WAIT_FOR_INPUT: begin
            s_sel = s_in;
            if(s_in_vld) begin
               s_in_ack        = 1;
               Q_in            = Q;
               state_nxt       = DO_ROUNDS;
               round_num_nxt   = 0;
            end
         end

         DO_ROUNDS: begin
            if(round_num == NUM_ROUNDS-1) begin
               s_sel           = s_in;
               round_num_nxt   = 0;
               s_out_vld       = 1;
               if(s_in_vld) begin
                  s_in_ack = 1;
                  Q_in     = Q;
               end else begin
                  state_nxt = WAIT_FOR_INPUT;
               end
            end // if (round_num == NUM_ROUNDS-1)
            else begin
               round_num_nxt = round_num + 1'b1;
            end // else: !if(round_num == NUM_ROUNDS-1)
         end // case: DO_ROUNDS
      endcase // case(state)
   end // always @ (*)

   always @(posedge clk) begin
      if(reset) begin
         round_num <= 0;
         state     <= WAIT_FOR_INPUT;
      end
      else begin
         round_num <= round_num_nxt;
         state <= state_nxt;
      end
   end // always @ (posedge clk)
   
endmodule // grostl_permute

