/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Module: grostl_shiftbytes23457.v
 * Project: Grostl
 * Author: Jad Naous <jnaous@stanford.edu>
 * Description: Rotates each row left by the row number 
 *             where a word is 40bits
 ****************************************************************/

module grostl_shiftbytes23457
  (input [0:8*8*40-1]  s_in,
   output [0:8*8*40-1] s_out);

   wire [39:0] state_in [0:7][0:7];
   wire [39:0] state_out[0:7][0:7];

   generate
      genvar i,j;
      
      /* Put the state in matrix form */
      for(i=0; i<8; i=i+1) begin: gen_row_state_in
         for(j=0; j<8; j=j+1) begin: gen_col_state_in
            assign state_in[j][i] = s_in[40 * (j + i*8) : 40 * (j + i*8) + 39];
         end
      end
               
      /* Do the transformation */
      for(i=0; i<8; i=i+1) begin: gen_trans_rows
         for(j=0; j<8; j=j+1) begin: gen_trans_cols
            assign state_out[j][(i+8-j)%8] = state_in[j][i];
         end
      end

      for(i=0; i<8; i=i+1) begin: gen_row_state_out
         for(j=0; j<8; j=j+1) begin: gen_col_state_out
            assign s_out[40 * (j + i*8) : 40 * (j + i*8) + 39] = state_out[j][i];
         end
      end
   endgenerate
endmodule // grostl_shiftbytes23457
