/*******************************************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 *
 * Module: grostl_subtimes23457.v
 * Project: Grostl
 * Author: Jad Naous <jnaous@stanford.edu>
 * Description: implements the SubBytes phase of a Grostl round but outputs
 *              for each byte: {times2(sbox(s_in)), times3(sbox(s_in)), 
 *                              times4(sbox(s_in)), times5(sbox(s_in)),
 *                              times7(sbox(s_in))}
 *******************************************************************************/

module grostl_subtimes23457
  (input [0:8*8*8-1]       s_in,
   output     [0:8*8*40-1] s_out,
   input                   clk);

   reg [0:32*64-1] subbytes_times2357;

`include "grostl.v.inc"

   generate
      genvar i;
      for (i=0; i<512/8; i=i+1) begin:subgen
         always @(posedge clk) begin
            subbytes_times2357[i*32 : i*32 + 31] 
                   <= sboxtimes2357(s_in[i*8 : i*8 + 7]);
         end
         assign s_out[i*40 : i*40 + 39]
                    = {subbytes_times2357[i*32 : i*32 + 15],         // x2, x3
                       times2(subbytes_times2357[i*32 : i*32 + 7]),  // x4
                       subbytes_times2357[i*32 + 16 : i*32 + 31]};   // x5, x7
      end // block: subgen
   endgenerate
endmodule // grostl_subtimes23457
