/* ****************************************************************************
 * $Id: regdump.c 4677 2008-11-07 03:44:30Z jnaous $
 *
 * Module: regdump.c
 * Project: NetFPGA 2.1 reference
 * Description: Test program to dump the switch registers
 *
 * Change history:
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>

#include <net/if.h>

#include <time.h>

#include <reg_defines.h>
#include <nf2util.h>
#include "../lib/C/reg_defines_icing_forwarder.h"

#define PATHLEN         80

#define DEFAULT_IFACE   "nf2c0"

/* Global vars */
static struct nf2device nf2;

/* Function declarations */
void print (void);
void printMAC (unsigned, unsigned);
void printIP (unsigned);

int main(int argc, char *argv[])
{
  nf2.device_name = DEFAULT_IFACE;

  if (check_iface(&nf2))
    {
      exit(1);
    }
  if (openDescriptor(&nf2))
    {
      exit(1);
    }

  print();

  closeDescriptor(&nf2);

  return 0;
}

void print(void) {
  unsigned val;

  //      readReg(&nf2, UNET_ID, &val);
  //      printf("Board ID: Version %i, Device %i\n", GET_VERSION(val), GET_DEVICE(val));
  readReg(&nf2, MAC_GRP_0_CONTROL_REG, &val);
  printf("MAC 0 Control: 0x%08x ", val);
  if(val&(1<<TX_QUEUE_DISABLE_BIT_NUM)) {
    printf("TX disabled, ");
  }
  else {
    printf("TX enabled,  ");
  }
  if(val&(1<<RX_QUEUE_DISABLE_BIT_NUM)) {
    printf("RX disabled, ");
  }
  else {
    printf("RX enabled,  ");
  }
  if(val&(1<<RESET_MAC_BIT_NUM)) {
    printf("reset on\n");
  }
  else {
    printf("reset off\n");
  }
  printf("mac config 0x%02x\n", val>>MAC_DISABLE_TX_BIT_NUM);
  readReg(&nf2, RX_QUEUE_0_NUM_PKTS_STORED_REG, &val);
  printf("Num pkts enqueued to rx queue 0:        %u\n", val);
  readReg(&nf2, RX_QUEUE_0_NUM_PKTS_DROPPED_FULL_REG, &val);
  printf("Num pkts dropped (rx queue 0 full):     %u\n", val);
  readReg(&nf2, RX_QUEUE_0_NUM_PKTS_DROPPED_BAD_REG, &val);
  printf("Num pkts dropped (bad fcs q 0):         %u\n", val);
  readReg(&nf2, RX_QUEUE_0_NUM_WORDS_PUSHED_REG, &val);
  printf("Num words pushed out of rx queue 0:     %u\n", val);
  readReg(&nf2, RX_QUEUE_0_NUM_BYTES_PUSHED_REG, &val);
  printf("Num bytes pushed out of rx queue 0:     %u\n", val);
  readReg(&nf2, RX_QUEUE_0_NUM_PKTS_DEQUEUED_REG, &val);
  printf("Num pkts dequeued from rx queue 0:      %u\n", val);
  readReg(&nf2, RX_QUEUE_0_NUM_PKTS_IN_QUEUE_REG, &val);
  printf("Num pkts in rx queue 0:                 %u\n\n", val);

  readReg(&nf2, TX_QUEUE_0_NUM_PKTS_IN_QUEUE_REG, &val);
  printf("Num pkts in tx queue 0:                 %u\n", val);
  readReg(&nf2, TX_QUEUE_0_NUM_PKTS_SENT_REG, &val);
  printf("Num pkts dequeued from tx queue 0:      %u\n", val);
  readReg(&nf2, TX_QUEUE_0_NUM_WORDS_PUSHED_REG, &val);
  printf("Num words pushed out of tx queue 0:     %u\n", val);
  readReg(&nf2, TX_QUEUE_0_NUM_BYTES_PUSHED_REG, &val);
  printf("Num bytes pushed out of tx queue 0:     %u\n", val);
  readReg(&nf2, TX_QUEUE_0_NUM_PKTS_ENQUEUED_REG, &val);
  printf("Num pkts enqueued to tx queue 0:        %u\n\n", val);

  readReg(&nf2, MAC_GRP_1_CONTROL_REG, &val);
  printf("MAC 1 Control: 0x%08x ", val);
  if(val&(1<<TX_QUEUE_DISABLE_BIT_NUM)) {
    printf("TX disabled, ");
  }
  else {
    printf("TX enabled,  ");
  }
  if(val&(1<<RX_QUEUE_DISABLE_BIT_NUM)) {
    printf("RX disabled, ");
  }
  else {
    printf("RX enabled,  ");
  }
  if(val&(1<<RESET_MAC_BIT_NUM)) {
    printf("reset on\n");
  }
  else {
    printf("reset off\n");
  }
  printf("mac config 0x%02x\n", val>>MAC_DISABLE_TX_BIT_NUM);
  readReg(&nf2, RX_QUEUE_1_NUM_PKTS_STORED_REG, &val);
  printf("Num pkts enqueued to rx queue 1:        %u\n", val);
  readReg(&nf2, RX_QUEUE_1_NUM_PKTS_DROPPED_FULL_REG, &val);
  printf("Num pkts dropped (rx queue 1 full):     %u\n", val);
  readReg(&nf2, RX_QUEUE_1_NUM_PKTS_DROPPED_BAD_REG, &val);
  printf("Num pkts dropped (bad fcs q 1):         %u\n", val);
  readReg(&nf2, RX_QUEUE_1_NUM_WORDS_PUSHED_REG, &val);
  printf("Num words pushed out of rx queue 1:     %u\n", val);
  readReg(&nf2, RX_QUEUE_1_NUM_BYTES_PUSHED_REG, &val);
  printf("Num bytes pushed out of rx queue 1:     %u\n", val);
  readReg(&nf2, RX_QUEUE_1_NUM_PKTS_DEQUEUED_REG, &val);
  printf("Num pkts dequeued from rx queue 1:      %u\n", val);
  readReg(&nf2, RX_QUEUE_1_NUM_PKTS_IN_QUEUE_REG, &val);
  printf("Num pkts in rx queue 1:                 %u\n\n", val);

  readReg(&nf2, TX_QUEUE_1_NUM_PKTS_IN_QUEUE_REG, &val);
  printf("Num pkts in tx queue 1:                 %u\n", val);
  readReg(&nf2, TX_QUEUE_1_NUM_PKTS_SENT_REG, &val);
  printf("Num pkts dequeued from tx queue 1:      %u\n", val);
  readReg(&nf2, TX_QUEUE_1_NUM_WORDS_PUSHED_REG, &val);
  printf("Num words pushed out of tx queue 1:     %u\n", val);
  readReg(&nf2, TX_QUEUE_1_NUM_BYTES_PUSHED_REG, &val);
  printf("Num bytes pushed out of tx queue 1:     %u\n", val);
  readReg(&nf2, TX_QUEUE_1_NUM_PKTS_ENQUEUED_REG, &val);
  printf("Num pkts enqueued to tx queue 1:        %u\n\n", val);

  readReg(&nf2, MAC_GRP_2_CONTROL_REG, &val);
  printf("MAC 2 Control: 0x%08x ", val);
  if(val&(1<<TX_QUEUE_DISABLE_BIT_NUM)) {
    printf("TX disabled, ");
  }
  else {
    printf("TX enabled,  ");
  }
  if(val&(1<<RX_QUEUE_DISABLE_BIT_NUM)) {
    printf("RX disabled, ");
  }
  else {
    printf("RX enabled,  ");
  }
  if(val&(1<<RESET_MAC_BIT_NUM)) {
    printf("reset on\n");
  }
  else {
    printf("reset off\n");
  }
  printf("mac config 0x%02x\n", val>>MAC_DISABLE_TX_BIT_NUM);
  readReg(&nf2, RX_QUEUE_2_NUM_PKTS_STORED_REG, &val);
  printf("Num pkts enqueued to rx queue 2:        %u\n", val);
  readReg(&nf2, RX_QUEUE_2_NUM_PKTS_DROPPED_FULL_REG, &val);
  printf("Num pkts dropped (rx queue 2 full):     %u\n", val);
  readReg(&nf2, RX_QUEUE_2_NUM_PKTS_DROPPED_BAD_REG, &val);
  printf("Num pkts dropped (bad fcs q 2):         %u\n", val);
  readReg(&nf2, RX_QUEUE_2_NUM_WORDS_PUSHED_REG, &val);
  printf("Num words pushed out of rx queue 2:     %u\n", val);
  readReg(&nf2, RX_QUEUE_2_NUM_BYTES_PUSHED_REG, &val);
  printf("Num bytes pushed out of rx queue 2:     %u\n", val);
  readReg(&nf2, RX_QUEUE_2_NUM_PKTS_DEQUEUED_REG, &val);
  printf("Num pkts dequeued from rx queue 2:      %u\n", val);
  readReg(&nf2, RX_QUEUE_2_NUM_PKTS_IN_QUEUE_REG, &val);
  printf("Num pkts in rx queue 2:                 %u\n\n", val);

  readReg(&nf2, TX_QUEUE_2_NUM_PKTS_IN_QUEUE_REG, &val);
  printf("Num pkts in tx queue 2:                 %u\n", val);
  readReg(&nf2, TX_QUEUE_2_NUM_PKTS_SENT_REG, &val);
  printf("Num pkts dequeued from tx queue 2:      %u\n", val);
  readReg(&nf2, TX_QUEUE_2_NUM_WORDS_PUSHED_REG, &val);
  printf("Num words pushed out of tx queue 2:     %u\n", val);
  readReg(&nf2, TX_QUEUE_2_NUM_BYTES_PUSHED_REG, &val);
  printf("Num bytes pushed out of tx queue 2:     %u\n", val);
  readReg(&nf2, TX_QUEUE_2_NUM_PKTS_ENQUEUED_REG, &val);
  printf("Num pkts enqueued to tx queue 2:        %u\n\n", val);

  readReg(&nf2, MAC_GRP_3_CONTROL_REG, &val);
  printf("MAC 3 Control: 0x%08x ", val);
  if(val&(1<<TX_QUEUE_DISABLE_BIT_NUM)) {
    printf("TX disabled, ");
  }
  else {
    printf("TX enabled,  ");
  }
  if(val&(1<<RX_QUEUE_DISABLE_BIT_NUM)) {
    printf("RX disabled, ");
  }
  else {
    printf("RX enabled,  ");
  }
  if(val&(1<<RESET_MAC_BIT_NUM)) {
    printf("reset on\n");
  }
  else {
    printf("reset off\n");
  }
  printf("mac config 0x%02x\n", val>>MAC_DISABLE_TX_BIT_NUM);
  readReg(&nf2, RX_QUEUE_3_NUM_PKTS_STORED_REG, &val);
  printf("Num pkts enqueued to rx queue 3:        %u\n", val);
  readReg(&nf2, RX_QUEUE_3_NUM_PKTS_DROPPED_FULL_REG, &val);
  printf("Num pkts dropped (rx queue 3 full):     %u\n", val);
  readReg(&nf2, RX_QUEUE_3_NUM_PKTS_DROPPED_BAD_REG, &val);
  printf("Num pkts dropped (bad fcs q 3):         %u\n", val);
  readReg(&nf2, RX_QUEUE_3_NUM_WORDS_PUSHED_REG, &val);
  printf("Num words pushed out of rx queue 3:     %u\n", val);
  readReg(&nf2, RX_QUEUE_3_NUM_BYTES_PUSHED_REG, &val);
  printf("Num bytes pushed out of rx queue 3:     %u\n", val);
  readReg(&nf2, RX_QUEUE_3_NUM_PKTS_DEQUEUED_REG, &val);
  printf("Num pkts dequeued from rx queue 3:      %u\n", val);
  readReg(&nf2, RX_QUEUE_3_NUM_PKTS_IN_QUEUE_REG, &val);
  printf("Num pkts in rx queue 3:                 %u\n\n", val);

  readReg(&nf2, TX_QUEUE_3_NUM_PKTS_IN_QUEUE_REG, &val);
  printf("Num pkts in tx queue 3:                 %u\n", val);
  readReg(&nf2, TX_QUEUE_3_NUM_PKTS_SENT_REG, &val);
  printf("Num pkts dequeued from tx queue 3:      %u\n", val);
  readReg(&nf2, TX_QUEUE_3_NUM_WORDS_PUSHED_REG, &val);
  printf("Num words pushed out of tx queue 3:     %u\n", val);
  readReg(&nf2, TX_QUEUE_3_NUM_BYTES_PUSHED_REG, &val);
  printf("Num bytes pushed out of tx queue 3:     %u\n", val);
  readReg(&nf2, TX_QUEUE_3_NUM_PKTS_ENQUEUED_REG, &val);
  printf("Num pkts enqueued to tx queue 3:        %u\n\n", val);

  readReg(&nf2, ICING_NUM_BAD_MACS_REG, &val);
  printf("ICING_NUM_BAD_MACS_REG                 %u\n", val);
  readReg(&nf2, ICING_NUM_BAD_ETHTYPES_REG, &val);
  printf("ICING_NUM_BAD_ETHTYPES_REG             %u\n", val);
  readReg(&nf2, ICING_NUM_BAD_PATH_IDX_REG, &val);
  printf("ICING_NUM_BAD_PATH_IDX_REG             %u\n", val);
  readReg(&nf2, ICING_NUM_BAD_PATH_LEN_REG, &val);
  printf("ICING_NUM_BAD_PATH_LEN_REG             %u\n", val);
  readReg(&nf2, ICING_NUM_BAD_PKT_LEN_REG, &val);
  printf("ICING_NUM_BAD_PKT_LEN_REG              %u\n", val);
  readReg(&nf2, ICING_NUM_BAD_VERSION_REG, &val);
  printf("ICING_NUM_BAD_VERSION_REG              %u\n", val);
  readReg(&nf2, ICING_NUM_PKTS_AUTHORIZED_REG, &val);
  printf("ICING_NUM_PKTS_AUTHORIZED_REG          %u\n", val);
  readReg(&nf2, ICING_NUM_PKTS_NOT_AUTHORIZED_REG, &val);
  printf("ICING_NUM_PKTS_NOT_AUTHORIZED_REG      %u\n", val);
  readReg(&nf2, ICING_NUM_SNDRS_NOT_AUTHORIZED_REG, &val);
  printf("ICING_NUM_SNDRS_NOT_AUTHORIZED_REG     %u\n", val);
  readReg(&nf2, ICING_NUM_KEY_MISSES_REG, &val);
  printf("ICING_NUM_KEY_MISSES_REG               %u\n\n", val);

  readReg(&nf2, BRAM_OQ_NUM_PKT_BYTES_RECEIVED_0_REG, &val);
  printf("BRAM_OQ_NUM_PKT_BYTES_RECEIVED_0_REG     %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_RECEIVED_0_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_RECEIVED_0_REG          %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_DROPPED_0_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_DROPPED_0_REG           %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKT_BYTES_RECEIVED_1_REG, &val);
  printf("BRAM_OQ_NUM_PKT_BYTES_RECEIVED_1_REG     %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_RECEIVED_1_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_RECEIVED_1_REG          %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_DROPPED_1_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_DROPPED_1_REG           %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKT_BYTES_RECEIVED_2_REG, &val);
  printf("BRAM_OQ_NUM_PKT_BYTES_RECEIVED_2_REG     %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_RECEIVED_2_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_RECEIVED_2_REG          %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_DROPPED_2_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_DROPPED_2_REG           %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKT_BYTES_RECEIVED_3_REG, &val);
  printf("BRAM_OQ_NUM_PKT_BYTES_RECEIVED_3_REG     %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_RECEIVED_3_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_RECEIVED_3_REG          %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_DROPPED_3_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_DROPPED_3_REG           %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKT_BYTES_RECEIVED_4_REG, &val);
  printf("BRAM_OQ_NUM_PKT_BYTES_RECEIVED_4_REG     %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_RECEIVED_4_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_RECEIVED_4_REG          %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_DROPPED_4_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_DROPPED_4_REG           %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKT_BYTES_RECEIVED_5_REG, &val);
  printf("BRAM_OQ_NUM_PKT_BYTES_RECEIVED_5_REG     %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_RECEIVED_5_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_RECEIVED_5_REG          %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_DROPPED_5_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_DROPPED_5_REG           %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKT_BYTES_RECEIVED_6_REG, &val);
  printf("BRAM_OQ_NUM_PKT_BYTES_RECEIVED_6_REG     %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_RECEIVED_6_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_RECEIVED_6_REG          %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_DROPPED_6_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_DROPPED_6_REG           %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKT_BYTES_RECEIVED_7_REG, &val);
  printf("BRAM_OQ_NUM_PKT_BYTES_RECEIVED_7_REG     %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_RECEIVED_7_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_RECEIVED_7_REG          %u\n", val);
  readReg(&nf2, BRAM_OQ_NUM_PKTS_DROPPED_7_REG, &val);
  printf("BRAM_OQ_NUM_PKTS_DROPPED_7_REG           %u\n\n", val);
}

//
// printMAC: print a MAC address as a : separated value. eg:
//    00:11:22:33:44:55
//
void printMAC(unsigned hi, unsigned lo)
{
  printf("%02x:%02x:%02x:%02x:%02x:%02x", 
         ((hi>>8)&0xff), ((hi>>0)&0xff),
         ((lo>>24)&0xff), ((lo>>16)&0xff), ((lo>>8)&0xff), ((lo>>0)&0xff)
         );
}


//
// printIP: print an IP address in dotted notation. eg: 192.168.0.1
//
void printIP(unsigned ip)
{
  printf("%u.%u.%u.%u", 
         ((ip>>24)&0xff), ((ip>>16)&0xff), ((ip>>8)&0xff), ((ip>>0)&0xff)
         );
}
