#!/usr/local/bin/perl -w
# make_pkts.pl
#
#
# 

$delay = '@12us';
$batch = 0;

use NF2::PacketGen;
use CPCI_21Lib;
use NF21RouterLib;

print "Adding libs\n";

use ICING3Packet;
use ICING3Realm;
use ICING3Dev;

print "Adding requires\n";

require "reg_defines.ph";
require "reg_defines_icing_forwarder.ph";

nf_set_environment( { PORT_MODE => 'PHYSICAL', MAX_PORTS => 4 } );
nf_add_port_rule(1, 'UNORDERED');
nf_add_port_rule(2, 'UNORDERED');
nf_add_port_rule(3, 'UNORDERED');
nf_add_port_rule(4, 'UNORDERED');

# use strict AFTER the $delay, $batch and %reg are declared
use strict;
use vars qw($delay $batch %reg);

my %cpci_regs = CPCI_21Lib::get_register_addresses();

# write 0 to CPCI_INTERRUPT_MASK_REG()
nf_PCI_write32(0, $batch, CPCI_INTERRUPT_MASK_REG(), 0);

# Prepare the DMA and enable interrupts
prepare_DMA('@3.9us');
enable_interrupts(0);

srand(2131);

# *********** Write config info *****************
my $full_uhf = 0;
my $write_uhf = 1;
my $full_vnode = 0;

my @realm_id = map {$_ & 0x7} (1..20);
my @port_macs = ([0x00, 0xca, 0xfe, 0x00, 0x00, 0x01],
                 [0x00, 0xca, 0xfe, 0x00, 0x00, 0x02],
                 [0x00, 0xca, 0xfe, 0x00, 0x00, 0x03],
                 [0x00, 0xca, 0xfe, 0x00, 0x00, 0x04],);


my $dev = ICING3Dev->new(ICING3Dev::ID() => \@realm_id,
                         ICING3Dev::MACS() => [@{$port_macs[0]},
                                               @{$port_macs[1]},
                                               @{$port_macs[2]},
                                               @{$port_macs[3]},
                                              ],
                        );

print "Checking sim_tool connection\n";
$dev->checkSimTool();

print "initializing Device\n";
$dev->initDevSim($full_uhf, $write_uhf);
$dev->fillPocKeyTable($full_vnode);

# *********** Create turd server *****************
my $num_realms = 0;
my $tokenserver = CTokenServer->new(CTokenServer::NUM_REALMS() => $num_realms,
				    CTokenServer::FULL_VNODE() => $full_vnode,
				    CTokenServer::FULL_UHF() => $full_uhf,
				   );
my $my_realm = Realm->new(Realm::ID() => \@realm_id);
my $my_realm_token =   RealmToken->new(RealmToken::ID() => $my_realm->{Realm::ID()},
				       RealmToken::VOQ() => [0,int(rand(16)),0,0]);

my @realms = map(Realm->new(Realm::ID() => [($_) x 19, 0],
			    Realm::OUTPUT_QUEUE() => 2*$_), (0..3));
my @realm_tokens;
if($full_vnode) {
  @realm_tokens = map(RealmToken->new(RealmToken::ID() => $_->{Realm::ID()}), @realms);
} else {
  @realm_tokens = map(RealmToken->new(RealmToken::ID() => $_->{Realm::ID()},
				      RealmToken::VOQ() => [0,int(rand(16)),0,0]),
		      @realms);
}

$tokenserver->addRealm($my_realm);
map($tokenserver->addRealm($_), @realms);

# **** Write shared keys and other crap to HW ******
my @realm_list = $tokenserver->getOtherRealms($my_realm);
print "Other realm list: @realm_list\n";
map {printf "     %s\n", unpack("H*", pack("C*", @{$_->{Realm::ID()}}))} @realm_list;

foreach (0..scalar(@realm_list) - 1) {
  my $realm = $realm_list[$_];
  printf("Writing Realm %s: outq %u\n",
	 unpack("H*", pack("C*", @{$realm->{Realm::ID()}})),
	 $realm->{Realm::OUTPUT_QUEUE()});
  $dev->writeRealmSim($realm->{Realm::ID()},
                      [$my_realm->getMySharedKey($realm)],
                      $realm->{Realm::NEXT_HOP_MAC()},
                      (1 << ($realm->{Realm::OUTPUT_QUEUE()})),
                     );
}

# *********** Send packet *****************
my @p_long = (@realm_tokens[0..3]) x 8;

my $num_pkts = 400;
my @pkts = ();
my @exp = ();
my $pkt_delay = '@110us';
for(my $pkt_cnt = 0; $pkt_cnt<$num_pkts; $pkt_cnt++) {
  my $sel = $pkt_cnt % 4;
  my @path;
  my $path_idx;
  my $payload_len = 2048;

  @path = (@p_long, $realm_tokens[0], $my_realm_token, $realm_tokens[$sel]);
  $path_idx = 33;

  my $input_port = $sel;
  my $nxt_hop_rlm = $tokenserver->getRealm($path[$path_idx+1]);
  my $out_port = $nxt_hop_rlm->{Realm::OUTPUT_QUEUE()}/2 + 1;

  if($pkt_cnt == $num_pkts/3 or $pkt_cnt == 2*$num_pkts/3) {
    @pkts = ();
    @exp = ();
  }

  if(scalar @pkts < 4) {
    my $pkt = ICING3Packet->newSenderPkt(ICING3Packet::CTOKEN_SERVER() => $tokenserver,
					 ICING3Packet::ICING_PATH() => \@path,
					 ICING3Packet::ETH_DST() => $port_macs[$input_port],
					 ICING3Packet::PAYLOAD_LEN() => $payload_len,
					);

    for (my $i=1; $i<$path_idx; $i++) {
      printf "***** Forwarding through $i\n";
      # get Realm
      my $r = $tokenserver->getRealm($path[$i]);
      # forward through that realm
      $pkt = $r->forwardPkt($pkt);
    }
    die "Why pkt not defined no mo'?" unless defined $pkt;

    push @pkts, $pkt->hexBytes();

    printf "***** Forwarding through my_realm\n";
    my $exp_pkt = $my_realm->forwardPkt($pkt);
    die "Why pkt not defined no mo'?" unless defined $exp_pkt;

    $pkt->{ICING3Packet::ETH_SRC()} = $port_macs[$out_port-1];
    $pkt->{ICING3Packet::ETH_DST()} = $nxt_hop_rlm->{Realm::NEXT_HOP_MAC()};

    push @exp, $pkt->hexBytes();
  }

  nf_packet_in($input_port+1, 2040, $pkt_delay, $main::batch, $pkts[$sel]);

  # *********** Expect packet *****************
  print "Expecting output from port $out_port\n";
  if(($sel == 0 && $pkt_cnt < 28*4)
     || ($sel == 1 && $pkt_cnt < 28*4)
     || ($sel == 2 && $pkt_cnt < 27*4)
     || ($sel == 3 && $pkt_cnt < 27*4)) {
    nf_expected_packet($out_port, 2040, $exp[$sel]);
  }
}

# *********** Finishing Up - need this in all scripts ! **********************
my $t = nf_write_sim_files();
print  "--- make_pkts.pl: Generated all configuration packets.\n";
printf "--- make_pkts.pl: Last packet enters system at approx %0d microseconds.\n",($t/1000);
if (nf_write_expected_files()) {
  die "Unable to write expected files\n";
}

nf_create_hardware_file('LITTLE_ENDIAN');
nf_write_hardware_file('LITTLE_ENDIAN');
