/**
 * Filename: protocol.h
 * Purpose: define the protocol's packet formats
 */

#ifndef PROTOCOL_H
#define PROTOCOL_H

#include "common.h"
#include "reg_defines.h"

/** the number of filters to send */
#define FILTER_DEPTH 8

/* The port for this protocol is  */
#define SR_MONITOR_PORT 5776

/** maximum lenght of the group name */
#define MAX_GROUP_NAME_LEN 63

/** defined if we want to send ARP, filter, and interface info too */
/* #define USE_ALL_DATA */

#ifdef USE_ALL_DATA
/** stores an ARP table entry */
typedef struct {
    addr_mac_t mac;
    addr_ip_t ip;
} __attribute__ ((packed)) nf2_arp_t;

/** stores the flags from an interface (yes, it could use bitfields) */
typedef struct {
    byte tx_queue;
    byte rx_queue;
    byte reset_mac;
    byte tx_mac;
    byte rx_mac;
    byte tx_jumbo;
    byte rx_jumbo;
    byte crc_check;
    byte crc_gen;
} __attribute__ ((packed)) nf2_intf_flags_t;

/* flags and other interface counters */
typedef struct {
    addr_mac_t mac;
    nf2_intf_flags_t flags;
    uint32_t rx_stored;
    uint32_t rx_dropped;
    uint32_t rx_dropped_bad;
    uint32_t rx_words_pushed;
    uint32_t rx_bytes_pushed;
    uint32_t rx_packets_dequeued;
    uint32_t rx_packets_in_queue;
    uint32_t tx_packets_in_queue;
    uint32_t tx_packets_sent;
    uint32_t tx_words_pushed;
    uint32_t tx_bytes_pushed;
    uint32_t tx_packets_enqueued;
} __attribute__ ((packed)) nf2_intf_t;
#endif /* USE_ALL_DATA */

#define MAX_PING_COUNT 10 /* mask only supports up to 32 bits, so this i max this may be */
#define MAX_PING_COUNT_MASK 1023 /* sum(2^i) from i=0 to MAX_PING_COUNT-1 inclusive */
#define MAX_PING_TARGETS 32
#define MAX_PING_TARGETS_PER_SUBNET 5

/* ping test information */
typedef struct {
    addr_ip_t ip;       /* ip of the target */
    byte num_ok;        /* number which succeeded out of last MAX_PING_COUNT tries */
    int16_t streak;     /* number of successes or failures in a row */
} __attribute__ ((packed)) nf2_ping_t;

/* hardware route information */
typedef struct {
    addr_ip_t subnet;
    addr_ip_t next_hop;
    addr_ip_t mask;
    byte intf_hw_id;
    uint32_t timestamp;
} __attribute__ ((packed)) nf2_rt_t;

#define INTF_NF2_0 0x01
#define INTF_CPU_0 0x02
#define INTF_NF2_1 0x04
#define INTF_CPU_1 0x08
#define INTF_NF2_2 0x10
#define INTF_CPU_2 0x20
#define INTF_NF2_3 0x40
#define INTF_CPU_3 0x80

/** packet which contains all information about a NetFPGA which we monitor */
typedef struct {
    char        group[MAX_GROUP_NAME_LEN+1]; /* must be all 0s after term nul */
    byte        sr_is_alive;
    uint32_t    timestamp;
#ifdef USE_ALL_DATA
    nf2_arp_t   arp[ROUTER_ARP_SIZE];
    addr_ip_t   filter[FILTER_DEPTH];
    nf2_intf_t  intf[MAX_INTERFACES];
#endif
    nf2_ping_t  ping[MAX_PING_TARGETS];
    nf2_rt_t    rt[ROUTER_RT_SIZE];
} __attribute__ ((packed)) nf2_info_t;

/**
 * Creates and binds a socket for UDP on the port for this protocol.
 * @return socket file descriptor created (on error, the program is terminated).
 */
int monitor_init();

#endif /* PROTOCOL_H */
