/**
 * Filename: server.h
 * Purpose: defines a server which listens for updates from clients monitoring
 *          NetFPGA routers.  It consolidates these updates and generates a
 *          human-readable output and summary.
 */

#ifndef SERVER_H
#define SERVER_H

#include <pthread.h>
#include "protocol.h"
#include "server_pres.h"

#define SERVER_VERSION "0.0.79"
#define MAX_CLIENTS 8
#define TMP_FILE "/tmp/.sr_server_monitor_dgu"

#define DEFAULT_REFRESH_INTERVAL 5
#define DEFAULT_REFRESH_ON_CHANGE FALSE

typedef struct client_state_t {
    unsigned timestamp;           /* local timestamp of last update time */
    nf2_info_t info;              /* last update contents */
    struct client_state_t* next;  /* next client in the list */
} client_state_t;

#define MAX_NAME_LEN 32
typedef struct {
    addr_ip_t ip;
    char name[MAX_NAME_LEN];
} host_info_t;

typedef struct server_t {
    int fd;
    int num_clients;
    client_state_t* client_list_head;
    pthread_cond_t cond;
    pthread_mutex_t lock;
    pres_info_t info;

    unsigned num_hosts;
    host_info_t host[MAX_PING_TARGETS];
    unsigned refresh_interval;
    bool refresh_on_change;
} server_t;

#endif /* SERVER_H */
