#!/bin/bash
set -o nounset

# simple script which creates your ping_list file and then
# builds and launches the monitoring client.  Do not run
# as root; script will launch client with sudo.

# remove the old by-products
client=./sr_monitor_client

# show help if the user asked for it
def_path="/mnt/cs344/group3/default_server_ip"
def_ip=128.12.174.104
if [ $# -eq 1 ]; then
    if [ "$1" = "-?" -o "$1" = "?" -o "$1" = "help" -o "$1" = "--help" ]; then
        echo "usage: $0 [group_name [server_ip [group number]]]"
        echo "  -- group_name and number are chosen based on the current user's name if omitted"
        echo "  -- server_ip is chosen based on the contents of $def_path if it exists and otherwise uses $def_ip"
        exit 0
    fi
fi

# get the name to give to the client
if [ $# -eq 0 ]; then
    name=`whoami`
else
    name="$1"
    shift
fi

# get the team and server IP
team=`whoami`
if [ $# -ge 1 ]; then
    server_ip="$1"
    shift

    if [ $# -eq 1 ]; then
        team="group$1"
        shift
    elif [ $# -gt 1 ]; then
        echo "Error: too many arguments"
        exit 1
    fi
else
    if [ -f "$def_path" ]; then
        server_ip=`cat $def_path`
    else
        server_ip=$def_ip
    fi
fi

# launch the client
if [ -f $client ]; then
    echo "Launching client for group $client (server = $server_ip) ..."
    sudo $client "$name" "$server_ip"
else
    echo "Error: unable to find the client app $client"
fi
