/* VLAN defines */
`define VLAN_ETHERTYPE 16'h8100

/* Other Constants */
`define ETH_TYPE_IP 16'h0800
`define IP_PROTO_TCP 8'h6
`define IP_PROTO_UDP 8'h11

/* Each flow entry is specified by
 * input port     : 8  bits
 * MAC src port   : 48 bits
 * MAC dst port   : 48 bits
 * Ethernet type  : 16 bits
 * VLAN id        : 12 bits - 16 bits used
 * IP src address : 32 bits
 * IP dst address : 32 bits
 * IP protocol    : 8  bits
 * Transport src  : 16 bits
 * Transport dst  : 16 bits
 * 
 * TOTAL          : 240 bits
 */
`define OPENFLOW_ENTRY_TRANSP_DST_WIDTH       16
`define OPENFLOW_ENTRY_TRANSP_DST_POS         0
`define OPENFLOW_ENTRY_TRANSP_SRC_WIDTH       16
`define OPENFLOW_ENTRY_TRANSP_SRC_POS         `OPENFLOW_ENTRY_TRANSP_DST_POS+`OPENFLOW_ENTRY_TRANSP_DST_WIDTH
`define OPENFLOW_ENTRY_IP_DST_WIDTH           32
`define OPENFLOW_ENTRY_IP_DST_POS             `OPENFLOW_ENTRY_TRANSP_SRC_POS+`OPENFLOW_ENTRY_TRANSP_SRC_WIDTH
`define OPENFLOW_ENTRY_IP_SRC_WIDTH           32
`define OPENFLOW_ENTRY_IP_SRC_POS             `OPENFLOW_ENTRY_IP_DST_POS+`OPENFLOW_ENTRY_IP_DST_WIDTH
`define OPENFLOW_ENTRY_IP_PROTO_WIDTH         8
`define OPENFLOW_ENTRY_IP_PROTO_POS           `OPENFLOW_ENTRY_IP_SRC_POS+`OPENFLOW_ENTRY_IP_SRC_WIDTH
`define OPENFLOW_ENTRY_IP_VER_HL_WIDTH        8
`define OPENFLOW_ENTRY_IP_VER_HL_POS          `OPENFLOW_ENTRY_IP_PROTO_POS+`OPENFLOW_ENTRY_IP_PROTO_WIDTH
`define OPENFLOW_ENTRY_ETH_TYPE_WIDTH         16
`define OPENFLOW_ENTRY_ETH_TYPE_POS           `OPENFLOW_ENTRY_IP_VER_HL_POS+`OPENFLOW_ENTRY_IP_VER_HL_WIDTH
`define OPENFLOW_ENTRY_ETH_SRC_WIDTH          48
`define OPENFLOW_ENTRY_ETH_SRC_POS            `OPENFLOW_ENTRY_ETH_TYPE_POS+`OPENFLOW_ENTRY_ETH_TYPE_WIDTH
`define OPENFLOW_ENTRY_ETH_DST_WIDTH          48
`define OPENFLOW_ENTRY_ETH_DST_POS            `OPENFLOW_ENTRY_ETH_SRC_POS+`OPENFLOW_ENTRY_ETH_SRC_WIDTH
`define OPENFLOW_ENTRY_SRC_PORT_WIDTH         8
`define OPENFLOW_ENTRY_SRC_PORT_POS           `OPENFLOW_ENTRY_ETH_DST_POS+`OPENFLOW_ENTRY_ETH_DST_WIDTH
`define OPENFLOW_ENTRY_VLAN_ID_WIDTH          24
`define OPENFLOW_ENTRY_VLAN_ID_POS            `OPENFLOW_ENTRY_SRC_PORT_POS+`OPENFLOW_ENTRY_SRC_PORT_WIDTH
`define OPENFLOW_ENTRY_WIDTH                  10'd256

`define OPENFLOW_ACTION_WIDTH                     10'd64
`define OPENFLOW_FORWARD_BITMASK_WIDTH            16
`define OPENFLOW_FORWARD_BITMASK_POS              0

`define OPENFLOW_EXACT_ENTRY_PKT_COUNTER_WIDTH      24
`define OPENFLOW_EXACT_ENTRY_PKT_COUNTER_POS         0
`define OPENFLOW_EXACT_ENTRY_LAST_SEEN_WIDTH         8
`define OPENFLOW_EXACT_ENTRY_LAST_SEEN_POS          24
`define OPENFLOW_EXACT_ENTRY_BYTE_COUNTER_WIDTH     32
`define OPENFLOW_EXACT_ENTRY_BYTE_COUNTER_POS       32

/* location of words in the SRAM for each entry.
 * Entries in the SRAM are aligned to 16 words. That is
 * each entry takes up 16 words, although only 9 of 
 * them are used. */
`define OPENFLOW_EXACT_ENTRY_HDR_BASE_POS     4'h0
`define OPENFLOW_EXACT_ENTRY_COUNTERS_POS     4'h8
`define OPENFLOW_EXACT_ENTRY_ACTION_BASE_POS  4'ha

/* Register macros */
`define OPENFLOW_LOOKUP_REG_ADDR_WIDTH       `UDP_BLOCK_SIZE_64_REG_ADDR_WIDTH
`define OPENFLOW_LOOKUP_BLOCK_ADDR_WIDTH     `UDP_BLOCK_SIZE_64_BLOCK_ADDR_WIDTH
`define OPENFLOW_LOOKUP_BLOCK_ADDR           `OPENFLOW_LOOKUP_BLOCK_ADDR_WIDTH'h8
`define OPENFLOW_LOOKUP_BLOCK_TAG            ({`UDP_BLOCK_SIZE_64_TAG, `OPENFLOW_LOOKUP_BLOCK_ADDR})

`define OPENFLOW_PROCESSOR_REG_ADDR_WIDTH       `UDP_BLOCK_SIZE_64_REG_ADDR_WIDTH
`define OPENFLOW_PROCESSOR_BLOCK_ADDR_WIDTH     `UDP_BLOCK_SIZE_64_BLOCK_ADDR_WIDTH
`define OPENFLOW_PROCESSOR_BLOCK_ADDR           `OPENFLOW_PROCESSOR_BLOCK_ADDR_WIDTH'h9
`define OPENFLOW_PROCESSOR_BLOCK_TAG            ({`UDP_BLOCK_SIZE_64_TAG, `OPENFLOW_PROCESSOR_BLOCK_ADDR})

`define OPENFLOW_WILDCARD_LOOKUP_REG_ADDR_WIDTH       `UDP_BLOCK_SIZE_1k_REG_ADDR_WIDTH
`define OPENFLOW_WILDCARD_LOOKUP_BLOCK_ADDR_WIDTH     `UDP_BLOCK_SIZE_1k_BLOCK_ADDR_WIDTH
`define OPENFLOW_WILDCARD_LOOKUP_BLOCK_ADDR           `OPENFLOW_WILDCARD_LOOKUP_BLOCK_ADDR_WIDTH'h3
`define OPENFLOW_WILDCARD_LOOKUP_BLOCK_TAG            ({`UDP_BLOCK_SIZE_1k_TAG, `OPENFLOW_WILDCARD_LOOKUP_BLOCK_ADDR})

`define OPENFLOW_LOOKUP_WILDCARD_MISSES      `OPENFLOW_LOOKUP_REG_ADDR_WIDTH'h0
`define OPENFLOW_LOOKUP_WILDCARD_HITS        `OPENFLOW_LOOKUP_REG_ADDR_WIDTH'h1
`define OPENFLOW_LOOKUP_EXACT_MISSES         `OPENFLOW_LOOKUP_REG_ADDR_WIDTH'h2
`define OPENFLOW_LOOKUP_EXACT_HITS           `OPENFLOW_LOOKUP_REG_ADDR_WIDTH'h3
`define OPENFLOW_LOOKUP_TIMER                `OPENFLOW_LOOKUP_REG_ADDR_WIDTH'h4

`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_0   `OPENFLOW_PROCESSOR_REG_ADDR_WIDTH'h0
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_1   `OPENFLOW_PROCESSOR_REG_ADDR_WIDTH'h1
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_2   `OPENFLOW_PROCESSOR_REG_ADDR_WIDTH'h2
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_3   `OPENFLOW_PROCESSOR_REG_ADDR_WIDTH'h3
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_4   `OPENFLOW_PROCESSOR_REG_ADDR_WIDTH'h4
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_5   `OPENFLOW_PROCESSOR_REG_ADDR_WIDTH'h5
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_6   `OPENFLOW_PROCESSOR_REG_ADDR_WIDTH'h6
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_7   `OPENFLOW_PROCESSOR_REG_ADDR_WIDTH'h7

`define OPENFLOW_WILDCARD_TABLE_SIZE            10'd32
`define OPENFLOW_WILDCARD_NUM_DATA_WORDS_USED   `OPENFLOW_ACTION_WIDTH/10'd32
`define OPENFLOW_WILDCARD_NUM_CMP_WORDS_USED    `OPENFLOW_ENTRY_WIDTH/10'd32

`define OPENFLOW_WILDCARD_LOOKUP_ACTION_BASE    `OPENFLOW_WILDCARD_LOOKUP_REG_ADDR_WIDTH'h0
`define OPENFLOW_WILDCARD_LOOKUP_CMP_MASK_BASE  (`OPENFLOW_WILDCARD_LOOKUP_ACTION_BASE + `OPENFLOW_WILDCARD_NUM_DATA_WORDS_USED)
`define OPENFLOW_WILDCARD_LOOKUP_CMP_BASE       (`OPENFLOW_WILDCARD_LOOKUP_CMP_MASK_BASE + `OPENFLOW_WILDCARD_NUM_CMP_WORDS_USED)
`define OPENFLOW_WILDCARD_LOOKUP_READ_ADDR      (`OPENFLOW_WILDCARD_LOOKUP_CMP_BASE + `OPENFLOW_WILDCARD_NUM_CMP_WORDS_USED)
`define OPENFLOW_WILDCARD_LOOKUP_WRITE_ADDR     `OPENFLOW_WILDCARD_LOOKUP_READ_ADDR+`OPENFLOW_WILDCARD_LOOKUP_REG_ADDR_WIDTH'h1

`define OPENFLOW_WILDCARD_LOOKUP_BYTES_HIT_BASE (`OPENFLOW_WILDCARD_LOOKUP_WRITE_ADDR + `OPENFLOW_WILDCARD_LOOKUP_REG_ADDR_WIDTH'h1)
`define OPENFLOW_WILDCARD_LOOKUP_PKTS_HIT_BASE  (`OPENFLOW_WILDCARD_LOOKUP_BYTES_HIT_BASE + `OPENFLOW_WILDCARD_TABLE_SIZE)
`define OPENFLOW_WILDCARD_LOOKUP_LAST_SEEN_TS   (`OPENFLOW_WILDCARD_LOOKUP_PKTS_HIT_BASE + `OPENFLOW_WILDCARD_TABLE_SIZE)

`define OPENFLOW_LOOKUP_WILDCARD_MISSES_REG    (`UDP_BASE_ADDRESS | {`OPENFLOW_LOOKUP_BLOCK_TAG, `OPENFLOW_LOOKUP_WILDCARD_MISSES})
`define OPENFLOW_LOOKUP_WILDCARD_HITS_REG      (`UDP_BASE_ADDRESS | {`OPENFLOW_LOOKUP_BLOCK_TAG, `OPENFLOW_LOOKUP_WILDCARD_HITS})
`define OPENFLOW_LOOKUP_EXACT_MISSES_REG       (`UDP_BASE_ADDRESS | {`OPENFLOW_LOOKUP_BLOCK_TAG, `OPENFLOW_LOOKUP_EXACT_MISSES})
`define OPENFLOW_LOOKUP_EXACT_HITS_REG         (`UDP_BASE_ADDRESS | {`OPENFLOW_LOOKUP_BLOCK_TAG, `OPENFLOW_LOOKUP_EXACT_HITS})
`define OPENFLOW_LOOKUP_TIMER_REG              (`UDP_BASE_ADDRESS | {`OPENFLOW_LOOKUP_BLOCK_TAG, `OPENFLOW_LOOKUP_TIMER})
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_0_REG (`UDP_BASE_ADDRESS | {`OPENFLOW_PROCESSOR_BLOCK_TAG, `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_0})
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_1_REG (`UDP_BASE_ADDRESS | {`OPENFLOW_PROCESSOR_BLOCK_TAG, `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_1})
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_2_REG (`UDP_BASE_ADDRESS | {`OPENFLOW_PROCESSOR_BLOCK_TAG, `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_2})
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_3_REG (`UDP_BASE_ADDRESS | {`OPENFLOW_PROCESSOR_BLOCK_TAG, `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_3})
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_4_REG (`UDP_BASE_ADDRESS | {`OPENFLOW_PROCESSOR_BLOCK_TAG, `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_4})
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_5_REG (`UDP_BASE_ADDRESS | {`OPENFLOW_PROCESSOR_BLOCK_TAG, `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_5})
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_6_REG (`UDP_BASE_ADDRESS | {`OPENFLOW_PROCESSOR_BLOCK_TAG, `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_6})
`define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_7_REG (`UDP_BASE_ADDRESS | {`OPENFLOW_PROCESSOR_BLOCK_TAG, `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_7})

`define OPENFLOW_WILDCARD_LOOKUP_READ_ADDR_REG     (`UDP_BASE_ADDRESS | {`OPENFLOW_WILDCARD_LOOKUP_BLOCK_TAG, `OPENFLOW_WILDCARD_LOOKUP_READ_ADDR})
`define OPENFLOW_WILDCARD_LOOKUP_WRITE_ADDR_REG    (`UDP_BASE_ADDRESS | {`OPENFLOW_WILDCARD_LOOKUP_BLOCK_TAG, `OPENFLOW_WILDCARD_LOOKUP_WRITE_ADDR})
`define OPENFLOW_WILDCARD_LOOKUP_ACTION_BASE_REG   (`UDP_BASE_ADDRESS | {`OPENFLOW_WILDCARD_LOOKUP_BLOCK_TAG, `OPENFLOW_WILDCARD_LOOKUP_ACTION_BASE})
`define OPENFLOW_WILDCARD_LOOKUP_CMP_BASE_REG      (`UDP_BASE_ADDRESS | {`OPENFLOW_WILDCARD_LOOKUP_BLOCK_TAG, `OPENFLOW_WILDCARD_LOOKUP_CMP_BASE})
`define OPENFLOW_WILDCARD_LOOKUP_CMP_MASK_BASE_REG (`UDP_BASE_ADDRESS | {`OPENFLOW_WILDCARD_LOOKUP_BLOCK_TAG, `OPENFLOW_WILDCARD_LOOKUP_CMP_MASK_BASE})

`define OPENFLOW_WILDCARD_LOOKUP_BYTES_HIT_BASE_REG (`UDP_BASE_ADDRESS | {`OPENFLOW_WILDCARD_LOOKUP_BLOCK_TAG, `OPENFLOW_WILDCARD_LOOKUP_BYTES_HIT_BASE})
`define OPENFLOW_WILDCARD_LOOKUP_PKTS_HIT_BASE_REG  (`UDP_BASE_ADDRESS | {`OPENFLOW_WILDCARD_LOOKUP_BLOCK_TAG, `OPENFLOW_WILDCARD_LOOKUP_PKTS_HIT_BASE})
`define OPENFLOW_WILDCARD_LOOKUP_LAST_SEEN_TS_REG   (`UDP_BASE_ADDRESS | {`OPENFLOW_WILDCARD_LOOKUP_BLOCK_TAG, `OPENFLOW_WILDCARD_LOOKUP_LAST_SEEN_TS})

`define PRINT_USER_REG_ADDRESSES                                                                                                                   \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_WILDCARD_TABLE_SIZE                   %07d\n", `OPENFLOW_WILDCARD_TABLE_SIZE);                       \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_WILDCARD_NUM_DATA_WORDS_USED          %07d\n", `OPENFLOW_WILDCARD_NUM_DATA_WORDS_USED);              \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_WILDCARD_NUM_CMP_WORDS_USED           %07d\n\n", `OPENFLOW_WILDCARD_NUM_CMP_WORDS_USED);             \
                                                                                                                                                   \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_WIDTH                           %07d\n", `OPENFLOW_ENTRY_WIDTH);                               \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_TRANSP_DST_WIDTH                %07d\n", `OPENFLOW_ENTRY_TRANSP_DST_WIDTH);                    \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_TRANSP_DST_POS                  %07d\n", `OPENFLOW_ENTRY_TRANSP_DST_POS);                      \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_TRANSP_SRC_WIDTH                %07d\n", `OPENFLOW_ENTRY_TRANSP_SRC_WIDTH);                    \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_TRANSP_SRC_POS                  %07d\n", `OPENFLOW_ENTRY_TRANSP_SRC_POS);                      \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_IP_PROTO_WIDTH                  %07d\n", `OPENFLOW_ENTRY_IP_PROTO_WIDTH);                      \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_IP_PROTO_POS                    %07d\n", `OPENFLOW_ENTRY_IP_PROTO_POS);                        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_IP_DST_WIDTH                    %07d\n", `OPENFLOW_ENTRY_IP_DST_WIDTH);                        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_IP_DST_POS                      %07d\n", `OPENFLOW_ENTRY_IP_DST_POS);                          \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_IP_SRC_WIDTH                    %07d\n", `OPENFLOW_ENTRY_IP_SRC_WIDTH);                        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_IP_SRC_POS                      %07d\n", `OPENFLOW_ENTRY_IP_SRC_POS);                          \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_IP_VER_HL_WIDTH                 %07d\n", `OPENFLOW_ENTRY_IP_VER_HL_WIDTH);                     \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_IP_VER_HL_POS                   %07d\n", `OPENFLOW_ENTRY_IP_VER_HL_POS);                       \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_VLAN_ID_WIDTH                   %07d\n", `OPENFLOW_ENTRY_VLAN_ID_WIDTH);                       \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_VLAN_ID_POS                     %07d\n", `OPENFLOW_ENTRY_VLAN_ID_POS);                         \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_ETH_TYPE_WIDTH                  %07d\n", `OPENFLOW_ENTRY_ETH_TYPE_WIDTH);                      \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_ETH_TYPE_POS                    %07d\n", `OPENFLOW_ENTRY_ETH_TYPE_POS);                        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_ETH_DST_WIDTH                   %07d\n", `OPENFLOW_ENTRY_ETH_DST_WIDTH);                       \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_ETH_DST_POS                     %07d\n", `OPENFLOW_ENTRY_ETH_DST_POS);                         \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_ETH_SRC_WIDTH                   %07d\n", `OPENFLOW_ENTRY_ETH_SRC_WIDTH);                       \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_ETH_SRC_POS                     %07d\n", `OPENFLOW_ENTRY_ETH_SRC_POS);                         \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_SRC_PORT_WIDTH                  %07d\n", `OPENFLOW_ENTRY_SRC_PORT_WIDTH);                      \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ENTRY_SRC_PORT_POS                    %07d\n\n", `OPENFLOW_ENTRY_SRC_PORT_POS);                      \
                                                                                                                                                   \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_ACTION_WIDTH                          %07d\n", `OPENFLOW_ACTION_WIDTH);                              \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_FORWARD_BITMASK_WIDTH                 %07d\n", `OPENFLOW_FORWARD_BITMASK_WIDTH);                     \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_FORWARD_BITMASK_POS                   %07d\n", `OPENFLOW_FORWARD_BITMASK_POS);                       \
                                                                                                                                                   \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_EXACT_ENTRY_HDR_BASE_POS              %07d\n", `OPENFLOW_EXACT_ENTRY_HDR_BASE_POS);                  \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_EXACT_ENTRY_COUNTERS_POS              %07d\n", `OPENFLOW_EXACT_ENTRY_COUNTERS_POS);                  \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_EXACT_ENTRY_ACTION_BASE_POS           %07d\n\n", `OPENFLOW_EXACT_ENTRY_ACTION_BASE_POS);             \
                                                                                                                                                   \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_EXACT_ENTRY_PKT_COUNTER_WIDTH         %07d\n", `OPENFLOW_EXACT_ENTRY_PKT_COUNTER_WIDTH);             \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_EXACT_ENTRY_BYTE_COUNTER_WIDTH        %07d\n", `OPENFLOW_EXACT_ENTRY_BYTE_COUNTER_WIDTH);            \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_EXACT_ENTRY_LAST_SEEN_WIDTH           %07d\n", `OPENFLOW_EXACT_ENTRY_LAST_SEEN_WIDTH);               \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_EXACT_ENTRY_PKT_COUNTER_POS           %07d\n", `OPENFLOW_EXACT_ENTRY_PKT_COUNTER_POS);               \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_EXACT_ENTRY_BYTE_COUNTER_POS          %07d\n", `OPENFLOW_EXACT_ENTRY_BYTE_COUNTER_POS);              \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_EXACT_ENTRY_LAST_SEEN_POS             %07d\n\n", `OPENFLOW_EXACT_ENTRY_LAST_SEEN_POS);               \
                                                                                                                                                   \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_WILDCARD_MISSES_REG            0x%07x\n", `OPENFLOW_LOOKUP_WILDCARD_MISSES_REG<<2);           \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_WILDCARD_HITS_REG              0x%07x\n", `OPENFLOW_LOOKUP_WILDCARD_HITS_REG<<2);             \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_EXACT_MISSES_REG               0x%07x\n", `OPENFLOW_LOOKUP_EXACT_MISSES_REG<<2);              \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_EXACT_HITS_REG                 0x%07x\n", `OPENFLOW_LOOKUP_EXACT_HITS_REG<<2);                \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_0_REG         0x%07x\n", `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_0_REG<<2);        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_1_REG         0x%07x\n", `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_1_REG<<2);        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_2_REG         0x%07x\n", `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_2_REG<<2);        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_3_REG         0x%07x\n", `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_3_REG<<2);        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_4_REG         0x%07x\n", `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_4_REG<<2);        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_5_REG         0x%07x\n", `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_5_REG<<2);        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_6_REG         0x%07x\n", `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_6_REG<<2);        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_7_REG         0x%07x\n", `OPENFLOW_LOOKUP_NUM_PKTS_DROPPED_7_REG<<2);        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_LOOKUP_TIMER_REG                      0x%07x\n\n", `OPENFLOW_LOOKUP_TIMER_REG<<2);                   \
                                                                                                                                                   \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_WILDCARD_LOOKUP_READ_ADDR_REG         0x%07x\n", `OPENFLOW_WILDCARD_LOOKUP_READ_ADDR_REG<<2);        \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_WILDCARD_LOOKUP_WRITE_ADDR_REG        0x%07x\n", `OPENFLOW_WILDCARD_LOOKUP_WRITE_ADDR_REG<<2);       \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_WILDCARD_LOOKUP_ACTION_BASE_REG       0x%07x\n", `OPENFLOW_WILDCARD_LOOKUP_ACTION_BASE_REG<<2);      \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_WILDCARD_LOOKUP_CMP_BASE_REG          0x%07x\n", `OPENFLOW_WILDCARD_LOOKUP_CMP_BASE_REG<<2);         \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_WILDCARD_LOOKUP_CMP_MASK_BASE_REG     0x%07x\n", `OPENFLOW_WILDCARD_LOOKUP_CMP_MASK_BASE_REG<<2);    \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_WILDCARD_LOOKUP_BYTES_HIT_BASE_REG    0x%07x\n", `OPENFLOW_WILDCARD_LOOKUP_BYTES_HIT_BASE_REG<<2);   \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_WILDCARD_LOOKUP_PKTS_HIT_BASE_REG     0x%07x\n", `OPENFLOW_WILDCARD_LOOKUP_PKTS_HIT_BASE_REG<<2);    \
  $fwrite(c_reg_defines_fd, "#define OPENFLOW_WILDCARD_LOOKUP_LAST_SEEN_TS_REG      0x%07x\n\n", `OPENFLOW_WILDCARD_LOOKUP_LAST_SEEN_TS_REG<<2)
