/*********** THESE ARE ALL WORD ADDRESSES (note the <<2 when printing external addresses) ***********/

`define PKT_CAP_DA_HI			16'h 00_00
`define PKT_CAP_DA_LO			32'h 00_00_00_01

`define PKT_CAP_SA_HI			16'h 00_00
`define PKT_CAP_SA_LO			32'h 00_00_00_02

`define PKT_CAP_ETHERTYPE		16'h 90_01

// General packet generator control registers
`define PKT_GEN_CTRL_REG_ADDR_WIDTH     `UDP_BLOCK_SIZE_64_REG_ADDR_WIDTH
`define PKT_GEN_CTRL_BLOCK_ADDR_WIDTH   `UDP_BLOCK_SIZE_64_BLOCK_ADDR_WIDTH
`define PKT_GEN_CTRL_BLOCK_ADDR         `PKT_GEN_CTRL_BLOCK_ADDR_WIDTH'h8
`define PKT_GEN_CTRL_BLOCK_TAG          ({`UDP_BLOCK_SIZE_64_TAG, `PKT_GEN_CTRL_BLOCK_ADDR})

// Delay module registers
`define DELAY_REG_ADDR_WIDTH     	`UDP_BLOCK_SIZE_64_REG_ADDR_WIDTH
`define DELAY_BLOCK_ADDR_WIDTH   	`UDP_BLOCK_SIZE_64_BLOCK_ADDR_WIDTH
`define DELAY_BLOCK_ADDR         	`DELAY_BLOCK_ADDR_WIDTH'h9
`define DELAY_BLOCK_TAG          	({`UDP_BLOCK_SIZE_64_TAG, `DELAY_BLOCK_ADDR})

// Rate limiter registers
`define RATE_LIMIT_0_BLOCK_ADDR           `RATE_LIMIT_BLOCK_ADDR_WIDTH'h10
`define RATE_LIMIT_0_BLOCK_TAG            ({`UDP_BLOCK_SIZE_64_TAG, `RATE_LIMIT_0_BLOCK_ADDR})

`define RATE_LIMIT_1_BLOCK_ADDR           `RATE_LIMIT_BLOCK_ADDR_WIDTH'h11
`define RATE_LIMIT_1_BLOCK_TAG            ({`UDP_BLOCK_SIZE_64_TAG, `RATE_LIMIT_1_BLOCK_ADDR})

`define RATE_LIMIT_2_BLOCK_ADDR           `RATE_LIMIT_BLOCK_ADDR_WIDTH'h12
`define RATE_LIMIT_2_BLOCK_TAG            ({`UDP_BLOCK_SIZE_64_TAG, `RATE_LIMIT_2_BLOCK_ADDR})

`define RATE_LIMIT_3_BLOCK_ADDR           `RATE_LIMIT_BLOCK_ADDR_WIDTH'h13
`define RATE_LIMIT_3_BLOCK_TAG            ({`UDP_BLOCK_SIZE_64_TAG, `RATE_LIMIT_3_BLOCK_ADDR})

`define RATE_LIMIT_4_BLOCK_ADDR           `RATE_LIMIT_BLOCK_ADDR_WIDTH'h14
`define RATE_LIMIT_4_BLOCK_TAG            ({`UDP_BLOCK_SIZE_64_TAG, `RATE_LIMIT_4_BLOCK_ADDR})

`define RATE_LIMIT_5_BLOCK_ADDR           `RATE_LIMIT_BLOCK_ADDR_WIDTH'h15
`define RATE_LIMIT_5_BLOCK_TAG            ({`UDP_BLOCK_SIZE_64_TAG, `RATE_LIMIT_5_BLOCK_ADDR})

`define RATE_LIMIT_6_BLOCK_ADDR           `RATE_LIMIT_BLOCK_ADDR_WIDTH'h16
`define RATE_LIMIT_6_BLOCK_TAG            ({`UDP_BLOCK_SIZE_64_TAG, `RATE_LIMIT_6_BLOCK_ADDR})

`define RATE_LIMIT_7_BLOCK_ADDR           `RATE_LIMIT_BLOCK_ADDR_WIDTH'h17
`define RATE_LIMIT_7_BLOCK_TAG            ({`UDP_BLOCK_SIZE_64_TAG, `RATE_LIMIT_7_BLOCK_ADDR})


/*******************************************************************
 -- Internal addresses -- these are used inside the modules
 *******************************************************************/

// Packet generator output queue registers
`define PKT_GEN_OQ_NUM_WORDS_LEFT               `OQ_REG_HELPER_ADDR_WIDTH'h0
`define PKT_GEN_OQ_NUM_PKT_BYTES_STORED         `OQ_REG_HELPER_ADDR_WIDTH'h1
`define PKT_GEN_OQ_NUM_OVERHEAD_BYTES_STORED    `OQ_REG_HELPER_ADDR_WIDTH'h2
`define PKT_GEN_OQ_NUM_PKTS_STORED              `OQ_REG_HELPER_ADDR_WIDTH'h3
`define PKT_GEN_OQ_NUM_PKTS_DROPPED             `OQ_REG_HELPER_ADDR_WIDTH'h4
`define PKT_GEN_OQ_NUM_PKT_BYTES_REMOVED        `OQ_REG_HELPER_ADDR_WIDTH'h5
`define PKT_GEN_OQ_NUM_OVERHEAD_BYTES_REMOVED   `OQ_REG_HELPER_ADDR_WIDTH'h6
`define PKT_GEN_OQ_NUM_PKTS_REMOVED             `OQ_REG_HELPER_ADDR_WIDTH'h7
`define PKT_GEN_OQ_ADDRESS_HI                   `OQ_REG_HELPER_ADDR_WIDTH'h8
`define PKT_GEN_OQ_ADDRESS_LO                   `OQ_REG_HELPER_ADDR_WIDTH'h9
`define PKT_GEN_OQ_WR_ADDRESS                   `OQ_REG_HELPER_ADDR_WIDTH'ha
`define PKT_GEN_OQ_RD_ADDRESS                   `OQ_REG_HELPER_ADDR_WIDTH'hb
`define PKT_GEN_OQ_NUM_PKTS_IN_Q                `OQ_REG_HELPER_ADDR_WIDTH'hc
`define PKT_GEN_OQ_MAX_PKTS_IN_Q                `OQ_REG_HELPER_ADDR_WIDTH'hd
`define PKT_GEN_OQ_CONTROL                      `OQ_REG_HELPER_ADDR_WIDTH'he
`define PKT_GEN_OQ_NUM_WORDS_IN_Q               `OQ_REG_HELPER_ADDR_WIDTH'hf
`define PKT_GEN_OQ_FULL_THRESH                  `OQ_REG_HELPER_ADDR_WIDTH'h10
`define PKT_GEN_OQ_CURR_ITER                    `OQ_REG_HELPER_ADDR_WIDTH'h11
`define PKT_GEN_OQ_MAX_ITER                     `OQ_REG_HELPER_ADDR_WIDTH'h12

`define PKT_GEN_ITER_WIDTH		 32

// --- Packet generator control registers
`define PKT_GEN_CTRL_ENABLE             `PKT_GEN_CTRL_REG_ADDR_WIDTH'h0

`define PKT_GEN_CTRL_PKT_CNT_0          `PKT_GEN_CTRL_REG_ADDR_WIDTH'h1
`define PKT_GEN_CTRL_BYTE_CNT_HI_0      `PKT_GEN_CTRL_REG_ADDR_WIDTH'h2
`define PKT_GEN_CTRL_BYTE_CNT_LO_0      `PKT_GEN_CTRL_REG_ADDR_WIDTH'h3
`define PKT_GEN_CTRL_TIME_FIRST_HI_0    `PKT_GEN_CTRL_REG_ADDR_WIDTH'h4
`define PKT_GEN_CTRL_TIME_FIRST_LO_0    `PKT_GEN_CTRL_REG_ADDR_WIDTH'h5
`define PKT_GEN_CTRL_TIME_LAST_HI_0     `PKT_GEN_CTRL_REG_ADDR_WIDTH'h6
`define PKT_GEN_CTRL_TIME_LAST_LO_0     `PKT_GEN_CTRL_REG_ADDR_WIDTH'h7

`define PKT_GEN_CTRL_PKT_CNT_1          `PKT_GEN_CTRL_REG_ADDR_WIDTH'h8
`define PKT_GEN_CTRL_BYTE_CNT_HI_1      `PKT_GEN_CTRL_REG_ADDR_WIDTH'h9
`define PKT_GEN_CTRL_BYTE_CNT_LO_1      `PKT_GEN_CTRL_REG_ADDR_WIDTH'ha
`define PKT_GEN_CTRL_TIME_FIRST_HI_1    `PKT_GEN_CTRL_REG_ADDR_WIDTH'hb
`define PKT_GEN_CTRL_TIME_FIRST_LO_1    `PKT_GEN_CTRL_REG_ADDR_WIDTH'hc
`define PKT_GEN_CTRL_TIME_LAST_HI_1     `PKT_GEN_CTRL_REG_ADDR_WIDTH'hd
`define PKT_GEN_CTRL_TIME_LAST_LO_1     `PKT_GEN_CTRL_REG_ADDR_WIDTH'he

`define PKT_GEN_CTRL_PKT_CNT_2          `PKT_GEN_CTRL_REG_ADDR_WIDTH'hf
`define PKT_GEN_CTRL_BYTE_CNT_HI_2      `PKT_GEN_CTRL_REG_ADDR_WIDTH'h10
`define PKT_GEN_CTRL_BYTE_CNT_LO_2      `PKT_GEN_CTRL_REG_ADDR_WIDTH'h11
`define PKT_GEN_CTRL_TIME_FIRST_HI_2    `PKT_GEN_CTRL_REG_ADDR_WIDTH'h12
`define PKT_GEN_CTRL_TIME_FIRST_LO_2    `PKT_GEN_CTRL_REG_ADDR_WIDTH'h13
`define PKT_GEN_CTRL_TIME_LAST_HI_2     `PKT_GEN_CTRL_REG_ADDR_WIDTH'h14
`define PKT_GEN_CTRL_TIME_LAST_LO_2     `PKT_GEN_CTRL_REG_ADDR_WIDTH'h15

`define PKT_GEN_CTRL_PKT_CNT_3          `PKT_GEN_CTRL_REG_ADDR_WIDTH'h16
`define PKT_GEN_CTRL_BYTE_CNT_HI_3      `PKT_GEN_CTRL_REG_ADDR_WIDTH'h17
`define PKT_GEN_CTRL_BYTE_CNT_LO_3      `PKT_GEN_CTRL_REG_ADDR_WIDTH'h18
`define PKT_GEN_CTRL_TIME_FIRST_HI_3    `PKT_GEN_CTRL_REG_ADDR_WIDTH'h19
`define PKT_GEN_CTRL_TIME_FIRST_LO_3    `PKT_GEN_CTRL_REG_ADDR_WIDTH'h1a
`define PKT_GEN_CTRL_TIME_LAST_HI_3     `PKT_GEN_CTRL_REG_ADDR_WIDTH'h1b
`define PKT_GEN_CTRL_TIME_LAST_LO_3     `PKT_GEN_CTRL_REG_ADDR_WIDTH'h1c

// --- Rate limiter registers
`define NUM_RATE_LIMITERS		8

`define RATE_LIMIT_CTRL                 `RATE_LIMIT_REG_ADDR_WIDTH'h0
`define RATE_LIMIT_TOKEN_INTERVAL       `RATE_LIMIT_REG_ADDR_WIDTH'h1
`define RATE_LIMIT_TOKEN_INC            `RATE_LIMIT_REG_ADDR_WIDTH'h2

`define RATE_LIMIT_ENABLE_BIT_NUM		0
`define RATE_LIMIT_INCLUDE_OVERHEAD_BIT_NUM	1

// --- Delay control registers
`define DELAY_RESET			`DELAY_REG_ADDR_WIDTH'h0


/************************
  --- External addresses
 ************************/

`define PKT_GEN_CTRL_ENABLE_REG             (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_ENABLE})

`define PKT_GEN_CTRL_PKT_CNT_0_REG          (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_PKT_CNT_0})
`define PKT_GEN_CTRL_BYTE_CNT_HI_0_REG      (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_BYTE_CNT_HI_0})
`define PKT_GEN_CTRL_BYTE_CNT_LO_0_REG      (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_BYTE_CNT_LO_0})
`define PKT_GEN_CTRL_TIME_FIRST_HI_0_REG    (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_FIRST_HI_0})
`define PKT_GEN_CTRL_TIME_FIRST_LO_0_REG    (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_FIRST_LO_0})
`define PKT_GEN_CTRL_TIME_LAST_HI_0_REG     (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_LAST_HI_0})
`define PKT_GEN_CTRL_TIME_LAST_LO_0_REG     (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_LAST_LO_0})

`define PKT_GEN_CTRL_PKT_CNT_1_REG          (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_PKT_CNT_1})
`define PKT_GEN_CTRL_BYTE_CNT_HI_1_REG      (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_BYTE_CNT_HI_1})
`define PKT_GEN_CTRL_BYTE_CNT_LO_1_REG      (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_BYTE_CNT_LO_1})
`define PKT_GEN_CTRL_TIME_FIRST_HI_1_REG    (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_FIRST_HI_1})
`define PKT_GEN_CTRL_TIME_FIRST_LO_1_REG    (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_FIRST_LO_1})
`define PKT_GEN_CTRL_TIME_LAST_HI_1_REG     (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_LAST_HI_1})
`define PKT_GEN_CTRL_TIME_LAST_LO_1_REG     (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_LAST_LO_1})

`define PKT_GEN_CTRL_PKT_CNT_2_REG          (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_PKT_CNT_2})
`define PKT_GEN_CTRL_BYTE_CNT_HI_2_REG      (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_BYTE_CNT_HI_2})
`define PKT_GEN_CTRL_BYTE_CNT_LO_2_REG      (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_BYTE_CNT_LO_2})
`define PKT_GEN_CTRL_TIME_FIRST_HI_2_REG    (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_FIRST_HI_2})
`define PKT_GEN_CTRL_TIME_FIRST_LO_2_REG    (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_FIRST_LO_2})
`define PKT_GEN_CTRL_TIME_LAST_HI_2_REG     (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_LAST_HI_2})
`define PKT_GEN_CTRL_TIME_LAST_LO_2_REG     (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_LAST_LO_2})

`define PKT_GEN_CTRL_PKT_CNT_3_REG          (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_PKT_CNT_3})
`define PKT_GEN_CTRL_BYTE_CNT_HI_3_REG      (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_BYTE_CNT_HI_3})
`define PKT_GEN_CTRL_BYTE_CNT_LO_3_REG      (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_BYTE_CNT_LO_3})
`define PKT_GEN_CTRL_TIME_FIRST_HI_3_REG    (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_FIRST_HI_3})
`define PKT_GEN_CTRL_TIME_FIRST_LO_3_REG    (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_FIRST_LO_3})
`define PKT_GEN_CTRL_TIME_LAST_HI_3_REG     (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_LAST_HI_3})
`define PKT_GEN_CTRL_TIME_LAST_LO_3_REG     (`UDP_BASE_ADDRESS | {`PKT_GEN_CTRL_BLOCK_TAG, `PKT_GEN_CTRL_TIME_LAST_LO_3})

`define RATE_LIMIT_BASE_ADDR                (`UDP_BASE_ADDRESS | {`RATE_LIMIT_0_BLOCK_TAG, `RATE_LIMIT_CTRL})
`define RATE_LIMIT_OFFSET		    ({{(`CPCI_NF2_ADDR_WIDTH-`UDP_BLOCK_SIZE_64_BLOCK_ADDR_WIDTH-`UDP_BLOCK_SIZE_64_REG_ADDR_WIDTH){1'b0}},  \
				              `RATE_LIMIT_BLOCK_ADDR_WIDTH'h1,                                                                       \
				              `RATE_LIMIT_REG_ADDR_WIDTH'h0})
`define RATE_LIMIT_CTRL_0_REG               (`UDP_BASE_ADDRESS | {`RATE_LIMIT_0_BLOCK_TAG, `RATE_LIMIT_CTRL})
`define RATE_LIMIT_TOKEN_INTERVAL_0_REG     (`UDP_BASE_ADDRESS | {`RATE_LIMIT_0_BLOCK_TAG, `RATE_LIMIT_TOKEN_INTERVAL})
`define RATE_LIMIT_TOKEN_INC_0_REG          (`UDP_BASE_ADDRESS | {`RATE_LIMIT_0_BLOCK_TAG, `RATE_LIMIT_TOKEN_INC})

`define OQ_OFFSET			    ({{(`CPCI_NF2_ADDR_WIDTH-`OQ_REG_ADDR_WIDTH){1'b0}}, \
				              `OQ_REG_HELPER_BLOCK_ADDR_WIDTH'h1,                \
				              `OQ_REG_HELPER_ADDR_WIDTH'h0})

`define PKT_GEN_OQ_CURR_ITER_REG_0          (`UDP_BASE_ADDRESS | {`OQ_BLOCK_TAG, {`OQ_REG_HELPER_BLOCK_ADDR_WIDTH'h0}, `PKT_GEN_OQ_CURR_ITER})
`define PKT_GEN_OQ_MAX_ITER_REG_0           (`UDP_BASE_ADDRESS | {`OQ_BLOCK_TAG, {`OQ_REG_HELPER_BLOCK_ADDR_WIDTH'h0}, `PKT_GEN_OQ_MAX_ITER})

`define DELAY_RESET_REG                     (`UDP_BASE_ADDRESS | {`DELAY_BLOCK_TAG, `DELAY_RESET})

/************************
  --- Print the registers
 ************************/
`define PRINT_USER_REG_ADDRESSES                                                                                                                   \
    $fwrite(c_reg_defines_fd, "#define PKT_CAP_DA_HI                           0x%04x\n", `PKT_CAP_DA_HI);                                         \
    $fwrite(c_reg_defines_fd, "#define PKT_CAP_DA_LO                           0x%08x\n", `PKT_CAP_DA_LO);                                         \
    $fwrite(c_reg_defines_fd, "#define PKT_CAP_SA_HI                           0x%04x\n", `PKT_CAP_SA_HI);                                         \
    $fwrite(c_reg_defines_fd, "#define PKT_CAP_SA_LO                           0x%08x\n", `PKT_CAP_SA_LO);                                         \
    $fwrite(c_reg_defines_fd, "#define PKT_CAP_ETHERTYPE                       0x%04x\n\n", `PKT_CAP_ETHERTYPE);                                   \
                                                                                                                                                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_MAX_ITER                        %d\n\n", 33'd2 ** `PKT_GEN_ITER_WIDTH - 1);                             \
                                                                                                                                                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_ENABLE_REG                 0x%07x\n\n", `PKT_GEN_CTRL_ENABLE_REG<<2);                          \
                                                                                                                                                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_PKT_CNT_0_REG              0x%07x\n", `PKT_GEN_CTRL_PKT_CNT_0_REG<<2);                         \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_BYTE_CNT_HI_0_REG          0x%07x\n", `PKT_GEN_CTRL_BYTE_CNT_HI_0_REG<<2);                     \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_BYTE_CNT_LO_0_REG          0x%07x\n", `PKT_GEN_CTRL_BYTE_CNT_LO_0_REG<<2);                     \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_FIRST_HI_0_REG        0x%07x\n", `PKT_GEN_CTRL_TIME_FIRST_HI_0_REG<<2);                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_FIRST_LO_0_REG        0x%07x\n", `PKT_GEN_CTRL_TIME_FIRST_LO_0_REG<<2);                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_LAST_HI_0_REG         0x%07x\n", `PKT_GEN_CTRL_TIME_LAST_HI_0_REG<<2);                    \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_LAST_LO_0_REG         0x%07x\n\n", `PKT_GEN_CTRL_TIME_LAST_LO_0_REG<<2);                  \
                                                                                                                                                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_PKT_CNT_1_REG              0x%07x\n", `PKT_GEN_CTRL_PKT_CNT_1_REG<<2);                         \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_BYTE_CNT_HI_1_REG          0x%07x\n", `PKT_GEN_CTRL_BYTE_CNT_HI_1_REG<<2);                     \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_BYTE_CNT_LO_1_REG          0x%07x\n", `PKT_GEN_CTRL_BYTE_CNT_LO_1_REG<<2);                     \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_FIRST_HI_1_REG        0x%07x\n", `PKT_GEN_CTRL_TIME_FIRST_HI_1_REG<<2);                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_FIRST_LO_1_REG        0x%07x\n", `PKT_GEN_CTRL_TIME_FIRST_LO_1_REG<<2);                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_LAST_HI_1_REG         0x%07x\n", `PKT_GEN_CTRL_TIME_LAST_HI_1_REG<<2);                    \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_LAST_LO_1_REG         0x%07x\n\n", `PKT_GEN_CTRL_TIME_LAST_LO_1_REG<<2);                  \
                                                                                                                                                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_PKT_CNT_2_REG              0x%07x\n", `PKT_GEN_CTRL_PKT_CNT_2_REG<<2);                         \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_BYTE_CNT_HI_2_REG          0x%07x\n", `PKT_GEN_CTRL_BYTE_CNT_HI_2_REG<<2);                     \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_BYTE_CNT_LO_2_REG          0x%07x\n", `PKT_GEN_CTRL_BYTE_CNT_LO_2_REG<<2);                     \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_FIRST_HI_2_REG        0x%07x\n", `PKT_GEN_CTRL_TIME_FIRST_HI_2_REG<<2);                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_FIRST_LO_2_REG        0x%07x\n", `PKT_GEN_CTRL_TIME_FIRST_LO_2_REG<<2);                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_LAST_HI_2_REG         0x%07x\n", `PKT_GEN_CTRL_TIME_LAST_HI_2_REG<<2);                    \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_LAST_LO_2_REG         0x%07x\n\n", `PKT_GEN_CTRL_TIME_LAST_LO_2_REG<<2);                  \
                                                                                                                                                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_PKT_CNT_3_REG              0x%07x\n", `PKT_GEN_CTRL_PKT_CNT_3_REG<<2);                         \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_BYTE_CNT_HI_3_REG          0x%07x\n", `PKT_GEN_CTRL_BYTE_CNT_HI_3_REG<<2);                     \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_BYTE_CNT_LO_3_REG          0x%07x\n", `PKT_GEN_CTRL_BYTE_CNT_LO_3_REG<<2);                     \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_FIRST_HI_3_REG        0x%07x\n", `PKT_GEN_CTRL_TIME_FIRST_HI_3_REG<<2);                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_FIRST_LO_3_REG        0x%07x\n", `PKT_GEN_CTRL_TIME_FIRST_LO_3_REG<<2);                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_LAST_HI_3_REG         0x%07x\n", `PKT_GEN_CTRL_TIME_LAST_HI_3_REG<<2);                    \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_CTRL_TIME_LAST_LO_3_REG         0x%07x\n\n", `PKT_GEN_CTRL_TIME_LAST_LO_3_REG<<2);                  \
                                                                                                                                                   \
    $fwrite(c_reg_defines_fd, "#define NUM_RATE_LIMITERS                       %d\n\n", `NUM_RATE_LIMITERS);                                       \
    $fwrite(c_reg_defines_fd, "#define RATE_LIMIT_ENABLE_BIT_NUM               %d\n", `RATE_LIMIT_ENABLE_BIT_NUM);                                 \
    $fwrite(c_reg_defines_fd, "#define RATE_LIMIT_INCLUDE_OVERHEAD_BIT_NUM     %d\n\n", `RATE_LIMIT_INCLUDE_OVERHEAD_BIT_NUM);                     \
                                                                                                                                                   \
    $fwrite(c_reg_defines_fd, "#define RATE_LIMIT_BASE_ADDR                    0x%07x\n", `RATE_LIMIT_BASE_ADDR<<2);                               \
    $fwrite(c_reg_defines_fd, "#define RATE_LIMIT_OFFSET                       0x%07x\n", `RATE_LIMIT_OFFSET<<2);                                  \
    $fwrite(c_reg_defines_fd, "#define RATE_LIMIT_CTRL_0_REG                   0x%07x\n", `RATE_LIMIT_CTRL_0_REG<<2);                              \
    $fwrite(c_reg_defines_fd, "#define RATE_LIMIT_TOKEN_INTERVAL_0_REG         0x%07x\n", `RATE_LIMIT_TOKEN_INTERVAL_0_REG<<2);                    \
    $fwrite(c_reg_defines_fd, "#define RATE_LIMIT_TOKEN_INC_0_REG              0x%07x\n\n", `RATE_LIMIT_TOKEN_INC_0_REG<<2);                       \
                                                                                                                                                   \
    $fwrite(c_reg_defines_fd, "#define OQ_OFFSET                               0x%07x\n\n", `OQ_OFFSET<<2);                                        \
                                                                                                                                                   \
    $fwrite(c_reg_defines_fd, "#define DELAY_RESET_REG                         0x%07x\n\n", `DELAY_RESET_REG<<2);                                  \
                                                                                                                                                   \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_OQ_CURR_ITER_REG_0              0x%07x\n", `PKT_GEN_OQ_CURR_ITER_REG_0<<2);                         \
    $fwrite(c_reg_defines_fd, "#define PKT_GEN_OQ_MAX_ITER_REG_0               0x%07x\n\n", `PKT_GEN_OQ_MAX_ITER_REG_0<<2)



