#!/usr/local/bin/perl -w
# make_pkts.pl
#

use NF2::PacketGen;
use NF2::PacketLib;
use NF21RouterLib;
use POSIX;
require "reg_defines.ph";
require $ENV{'NF2_DESIGN_DIR'} . '/lib/Perl5/reg_defines_packet_generator.ph';

$delay = '@4us';
$batch = 0;
nf_set_environment( { PORT_MODE => 'PHYSICAL', MAX_PORTS => 4 } );

# use strict AFTER the $delay, $batch and %reg are declared
use strict;
use vars qw($delay $batch %reg);

# Prepare the DMA and enable interrupts
prepare_DMA('@3.9us');
enable_interrupts(0);

my $defaultQSize = floor(0x80000 / 12);
my $qAddrOffset = OQ_OFFSET();

# Construct a packet and place in SRAM
my $len = 60;
my @pkt = (1 .. 60);

my $wordLen = POSIX::ceil($len / 8);
my $srcPort = 0;
my $dstPort = 0x100;

my $qOffset = 8 * $defaultQSize * 16;
my $addr = SRAM_BASE_ADDR() + $qOffset;

# Write the header to SRAM
nf_PCI_write32($delay, $batch, $addr + 0x4, 0x000000ff);
nf_PCI_write32($delay, $batch, $addr + 0x8, $wordLen | ($dstPort << 16));
nf_PCI_write32($delay, $batch, $addr + 0xc, $len | ($srcPort << 16));
$addr += 0x10;

# Write the actual data
my @paddedPkt = @pkt;
push @paddedPkt, ((0) x (8 - $len % 8)) if ($len % 8 != 0);
for (my $i = 0; $i < scalar(@paddedPkt); $i += 8) {
	my $ctrl = 0x0;
	if ($i / 8 == $wordLen - 1) {
		$ctrl = 0x100 >> ($len % 8);
		$ctrl = $ctrl & 0xff | ($ctrl == 0x100);
	}
	my $word1 = ($paddedPkt[$i + 0] << 24) | ($paddedPkt[$i + 1] << 16) |
	            ($paddedPkt[$i + 2] <<  8) | ($paddedPkt[$i + 3] <<  0);
	my $word2 = ($paddedPkt[$i + 4] << 24) | ($paddedPkt[$i + 5] << 16) |
	            ($paddedPkt[$i + 6] <<  8) | ($paddedPkt[$i + 7] <<  0);

	nf_PCI_write32($delay, $batch, $addr + 0x4, $ctrl);
	nf_PCI_write32($delay, $batch, $addr + 0x8, $word1);
	nf_PCI_write32($delay, $batch, $addr + 0xc, $word2);

	$addr += 0x10;
}


# Attempt to send the packet
nf_PCI_write32($delay, $batch, 
	OQ_ADDRESS_HI_REG_0() + 8 * $qAddrOffset, 
	scalar(@paddedPkt) / 8 + 8 * $defaultQSize);
nf_PCI_write32($delay, $batch, OQ_CONTROL_REG_0() + 8 * $qAddrOffset, 0x03);
nf_PCI_write32($delay, $batch, PKT_GEN_OQ_MAX_ITER_REG_0() + 8 * $qAddrOffset, 2);
nf_PCI_write32($delay, $batch, &PKT_GEN_CTRL_ENABLE_REG(), 0x0000000f);

# Expect the packet
nf_expected_packet(1, $len, join(' ', map {sprintf("%02x", $_)} @pkt));
nf_expected_packet(1, $len, join(' ', map {sprintf("%02x", $_)} @pkt));


# *********** Finishing Up - need this in all scripts ! ****************************
my $t = nf_write_sim_files();
print  "--- make_pkts.pl: Generated all configuration packets.\n";
printf "--- make_pkts.pl: Last packet enters system at approx %0d microseconds.\n",($t/1000);
if (nf_write_expected_files()) {
  die "Unable to write expected files\n";
}

nf_create_hardware_file('LITTLE_ENDIAN');
nf_write_hardware_file('LITTLE_ENDIAN');
