/******************************************************
 * vim:set shiftwidth=3 softtabstop=3 expandtab:
 * $Id$
 * filename:  debug_counters.v
 * author:    Jad Naous
 * Summary:   Counts the number of packets passing by
 ******************************************************/

`timescale 1ns/1ps
  module debug_counters
    #(parameter DATA_WIDTH = 64,
      parameter CTRL_WIDTH=DATA_WIDTH/8,
      parameter UDP_REG_SRC_WIDTH = 2,
      parameter NUM_INPUTS = 4)

   (// --- data path interface
    input  [NUM_INPUTS*CTRL_WIDTH-1:0] in_ctrl,
    input  [NUM_INPUTS-1:0]            in_wr,

    // --- Register interface
    input                              reg_req_in,
    input                              reg_ack_in,
    input                              reg_rd_wr_L_in,
    input  [`UDP_REG_ADDR_WIDTH-1:0]   reg_addr_in,
    input  [`CPCI_NF2_DATA_WIDTH-1:0]  reg_data_in,
    input  [UDP_REG_SRC_WIDTH-1:0]     reg_src_in,

    output                             reg_req_out,
    output                             reg_ack_out,
    output                             reg_rd_wr_L_out,
    output  [`UDP_REG_ADDR_WIDTH-1:0]  reg_addr_out,
    output  [`CPCI_NF2_DATA_WIDTH-1:0] reg_data_out,
    output  [UDP_REG_SRC_WIDTH-1:0]    reg_src_out,

    // --- Misc
    input                              clk,
    input                              reset);

   //------------------- wires/regs -------------------
   wire [NUM_INPUTS-1:0] eop_seen;

   //-------------------- Modules ---------------------
   generic_regs
     #(.UDP_REG_SRC_WIDTH (UDP_REG_SRC_WIDTH),
       .TAG (`DEBUG_COUNTERS_BLOCK_TAG),
       .REG_ADDR_WIDTH (`DEBUG_COUNTERS_REG_ADDR_WIDTH),
       .NUM_COUNTERS (NUM_INPUTS),
       .NUM_SOFTWARE_REGS (0),
       .NUM_HARDWARE_REGS (0),
       .COUNTER_INPUT_WIDTH (1)
       )
   generic_regs
     (
      .reg_req_in        (reg_req_in),
      .reg_ack_in        (reg_ack_in),
      .reg_rd_wr_L_in    (reg_rd_wr_L_in),
      .reg_addr_in       (reg_addr_in),
      .reg_data_in       (reg_data_in),
      .reg_src_in        (reg_src_in),
                         
      .reg_req_out       (reg_req_out),
      .reg_ack_out       (reg_ack_out),
      .reg_rd_wr_L_out   (reg_rd_wr_L_out),
      .reg_addr_out      (reg_addr_out),
      .reg_data_out      (reg_data_out),
      .reg_src_out       (reg_src_out),
                         
      // --- counters interface
      .counter_updates   (eop_seen),
      .counter_decrement ({NUM_INPUTS{1'b0}}),

      // --- SW regs interface
      .software_regs     (),
                         
      // --- HW regs interface
      .hardware_regs     (1'b0),

      .clk               (clk),
      .reset             (reset));

   generate
      genvar i;
      for (i=0; i<NUM_INPUTS; i=i+1) begin:eop_gen
         eop_signal eop_signal
           (.in_wr   (in_wr[i]),
            .in_ctrl (in_ctrl[CTRL_WIDTH*(i+1)-1:CTRL_WIDTH*i]),
            .eop     (eop_seen[i]),
            .clk     (clk),
            .reset   (reset));
      end
   endgenerate
   
endmodule // debug_counters
