
  module encap_control
    #(parameter DATA_WIDTH = 64,
      parameter CTRL_WIDTH = DATA_WIDTH/8
      )
   (
    input [CTRL_WIDTH-1:0]             out_ctrl,
    input                              out_rdy,
    input                              encap_enable,
    input                              out_we,
    
    output reg [1:0]                   sel,
    output reg                         mask_fifo_rd,
    output [7:0]                       out_state,

    // --- Misc   
    input                              reset,
    input                              clk
   );

   function integer log2;
      input integer number;
      begin
         log2=0;
         while(2**log2<number) begin
            log2=log2+1;
         end
      end
   endfunction // log2
   
   //------------------ Internal Parameter ---------------------------

   localparam WAIT             = 1;
   localparam WORD_1           = 2;
   localparam WORD_2           = 4;
   localparam WRITE_WORD_1      = 8;
   localparam WRITE_WORD_2      = 32;
   localparam WRITE_WORD_3      = 64;   

   //---------------------- Wires/Regs -------------------------------
   reg [7:0]                            state, state_next;
   //------------------------ Logic ----------------------------------

   assign out_state = state;
   
   always @(*) begin
      case(state) 
        WAIT: begin
	   sel = 2'b00;
	   mask_fifo_rd = 1'b1;
	   if((out_ctrl == 8'hFF) & out_we) 
	     state_next = WORD_1;
	   else
	     state_next = WAIT;     
	end
	WORD_1:begin
	   sel = 2'b00;
	   mask_fifo_rd = 1'b1;
	   if(out_we)
	     state_next = WORD_2;
	   else
	     state_next = WORD_1;
	end
	WORD_2:begin
	   sel = 2'b00;
	   mask_fifo_rd = 1'b1;
	   if(out_we)
	     if(encap_enable)
	       state_next = WRITE_WORD_1;
	     else
	       state_next = WAIT;
	   else
	     state_next = WORD_2;
	end
	WRITE_WORD_1:begin
	   if(out_we) begin
	      sel = 2'b01;
	      mask_fifo_rd = 1'b0;
	      state_next = WRITE_WORD_2;	      
	   end
	   else begin
	      sel = 2'b00;
	      mask_fifo_rd = 1'b1;
	      state_next = WRITE_WORD_1;
	   end
	end
	WRITE_WORD_2:begin
	   if(out_we) begin
	      sel = 2'b10;
	      mask_fifo_rd = 1'b0;
	      state_next = WRITE_WORD_3;	      
	   end
	   else begin
	      sel = 2'b00;
	      mask_fifo_rd = 1'b1;
	      state_next = WRITE_WORD_2;
	   end
	end
	WRITE_WORD_3:begin
	   if(out_we) begin
	      sel = 2'b11;
	      mask_fifo_rd = 1'b0;
	      state_next = WAIT;
	   end
	   else begin
	      sel = 2'b00;
	      mask_fifo_rd = 1'b1;
	      state_next = WRITE_WORD_3;
	   end
	end
	default: begin
	   sel = 2'b00;
	   mask_fifo_rd = 1'b0;
	   state_next = WAIT;
	end
      endcase
      
   end // always @ (*)
   
   
   always@(posedge clk) begin
      if(reset) begin
         state <= WAIT;
      end
      else begin
         state <= state_next;
      end
   end
   

endmodule // encap_control



