
  module encap_ip_control
    #(parameter DATA_WIDTH = 64,
      parameter CTRL_WIDTH = DATA_WIDTH/8
      )
   (
    input [CTRL_WIDTH-1:0]             in_ctrl,
    input                              in_we,
    input                              packet_in,
 
    output reg                         ip_header_enable_1,
    output reg                         ip_header_enable_2,
    output reg                         mask_fifo_rd_2,      
    
    // --- Misc   
    input                              reset,
    input                              clk
   );

   function integer log2;
      input integer number;
      begin
         log2=0;
         while(2**log2<number) begin
            log2=log2+1;
         end
      end
   endfunction // log2
   
   //------------------ Internal Parameter ---------------------------

   localparam WAIT             = 1;
   localparam WORD_1           = 2;
   localparam WORD_2           = 4;
   localparam WORD_3           = 8;
   localparam WORD_4           = 32;
   localparam WORD_5           = 64;   
   localparam SEND             = 128;   

   //---------------------- Wires/Regs -------------------------------
   reg [7:0]                            state, state_next;
   //------------------------ Logic ----------------------------------

   assign out_state = state;
   
   always @(*) begin
      ip_header_enable_1   = 0;
      ip_header_enable_2   = 0;
      mask_fifo_rd_2       = 1;
      state_next           = state;
      case(state) 
        WAIT: begin
           ip_header_enable_1 = 1'b0;
           ip_header_enable_2 = 1'b0;   
           mask_fifo_rd_2 = 1'b1;      
           if((in_ctrl == 8'hFF) & in_we) begin
             mask_fifo_rd_2 = 1'b0;      
             state_next = WORD_1;
           end
           else
             state_next = WAIT;     
        end
        WORD_1:begin
           mask_fifo_rd_2 = 1'b0;
           ip_header_enable_1 = 1'b0;
           ip_header_enable_2 = 1'b0;         
           if(in_we)
             state_next = WORD_2;
           else
             state_next = WORD_1;
        end
        WORD_2:begin
           mask_fifo_rd_2 = 1'b0;      
           ip_header_enable_1 = 1'b0;
           ip_header_enable_2 = 1'b0;         
           if(in_we)
             state_next = WORD_3;
           else
             state_next = WORD_2;
        end
        WORD_3:begin
           mask_fifo_rd_2 = 1'b0;      
           ip_header_enable_1 = 1'b0;
           ip_header_enable_2 = 1'b0;                 
           if(in_we)
             state_next = WORD_4;
           else
             state_next = WORD_3;
        end

        WORD_4:begin
           ip_header_enable_2 = 1'b0;   
           mask_fifo_rd_2 = 1'b0;            
           if(in_we) begin
              state_next = WORD_5;
              ip_header_enable_1 = 1'b1;              
           end
           else
             state_next = WORD_4;
        end

        WORD_5:begin
           ip_header_enable_1 = 1'b0;
           mask_fifo_rd_2 = 1'b0;             
           if(in_we) begin
              state_next = WAIT;
              ip_header_enable_2 = 1'b1;              
           end
           else
             state_next = WORD_5;
        end

        SEND:begin
           ip_header_enable_1 = 1'b0;
           ip_header_enable_2 = 1'b0;
           mask_fifo_rd_2 = 1'b1;    
           if(!packet_in)
             state_next = WAIT;
           else
             state_next = SEND;
        end
        
        default: begin
           ip_header_enable_1 = 1'b0;
           ip_header_enable_2 = 1'b0;
           mask_fifo_rd_2 = 1'b1;             
           state_next = WAIT;
        end
      endcase
      
   end // always @ (*)
   
   
   always@(posedge clk) begin
      if(reset) begin
         state <= WAIT;
      end
      else begin
         state <= state_next;
      end
   end
   

endmodule // encap_ip_control




