 module encap
    #(parameter DATA_WIDTH = 64,
      parameter CTRL_WIDTH=DATA_WIDTH/8,
      parameter UDP_REG_SRC_WIDTH = 2)

   (// --- data path interface
    output [DATA_WIDTH-1:0]        out_data,
    output [CTRL_WIDTH-1:0]        out_ctrl,
    output reg                          out_wr,
    input                               out_rdy,
    
    input  [DATA_WIDTH-1:0]            in_data,
    input  [CTRL_WIDTH-1:0]            in_ctrl,
    input                              in_wr,
    output                             in_rdy,

    // --- Register interface
    input                              reg_req_in,
    input                              reg_ack_in,
    input                              reg_rd_wr_L_in,
    input  [`UDP_REG_ADDR_WIDTH-1:0]   reg_addr_in,
    input  [`CPCI_NF2_DATA_WIDTH-1:0]  reg_data_in,
    input  [UDP_REG_SRC_WIDTH-1:0]     reg_src_in,

    output                             reg_req_out,
    output                             reg_ack_out,
    output                             reg_rd_wr_L_out,
    output  [`UDP_REG_ADDR_WIDTH-1:0]  reg_addr_out,
    output  [`CPCI_NF2_DATA_WIDTH-1:0] reg_data_out,
    output  [UDP_REG_SRC_WIDTH-1:0]    reg_src_out,

    // --- Misc
    
    input                              clk,
    input                              reset);


   small_fifo
     #(.WIDTH    (DATA_WIDTH),
       .MAX_DEPTH_BITS (1))rcp_fifo_data
       
       (
        
        .din         (in_data),       // Data in
        .wr_en       (fifo_wr),       // Write enable
        
        .rd_en       (fifo_rd),       // Read the next word
        
        .dout         (out_data),     // Data out  
        .full         (rcp_data_full),
        .nearly_full  (rcp_data_nearly_full),
        .empty        (rcp_data_empty),
        
        .reset        (reset),
        .clk          (clk)
        );

     small_fifo
     #(.WIDTH        (CTRL_WIDTH),
       .MAX_DEPTH_BITS (1))rcp_fifo_ctrl
       
       (
        .din         (in_ctrl),       // Data in
        .wr_en       (fifo_wr),       // Write enable
        
        .rd_en       (fifo_rd),       // Read the next word
        
        .dout         (out_ctrl),     // Data out 
        .full         (rcp_ctrl_full),
        .nearly_full  (rcp_ctrl_nearly_full),
        .empty        (rcp_ctrl_empty),
        
        .reset        (reset),
        .clk          (clk)
        );
   

   assign                          fifo_wr = in_wr;
   assign                          fifo_rd = out_rdy & !rcp_data_empty;
   assign                          in_rdy = !rcp_data_nearly_full;
   
   always @(posedge clk) begin
      if(fifo_rd)
	out_wr <= 1;
      
      else
	out_wr <= 0;
   end


    generic_regs
   #( 
      .UDP_REG_SRC_WIDTH    (UDP_REG_SRC_WIDTH),
      .TAG                  (`ENCAP_BLOCK_TAG),
      .REG_ADDR_WIDTH       (`ENCAP_REG_ADDR_WIDTH),  
      .NUM_COUNTERS         (2),
      .NUM_SOFTWARE_REGS    (2),
      .NUM_HARDWARE_REGS    (2),
      .COUNTER_INPUT_WIDTH  (2)
   ) generic_regs
   (
    .reg_req_in      (reg_req_in),
    .reg_ack_in      (reg_ack_in),
    .reg_rd_wr_L_in  (reg_rd_wr_L_in),
    .reg_addr_in     (reg_addr_in),
    .reg_data_in     (reg_data_in),
    .reg_src_in      (reg_src_in),
    
    .reg_req_out     (reg_req_out),
    .reg_ack_out     (reg_ack_out),
    .reg_rd_wr_L_out (reg_rd_wr_L_out),
    .reg_addr_out    (reg_addr_out),
    .reg_data_out    (reg_data_out),
    .reg_src_out     (reg_src_out),

    // --- counters interface
    .counter_updates  (4'b0),   // all the counter updates are concatenated
    .counter_decrement(2'b0), // if 1 then subtract the update, else add.
    
    // --- SW regs interface
    //.software_regs    ({temp, soft_reg}), // signals from the software
    
    // --- HW regs interface
    //.hardware_regs  (),
    
    .clk           (clk),
    .reset         (reset));



endmodule // encap
