
#!/usr/local/bin/perl -w
# make_pkts.pl
#
#
# 

use NF2::PacketGen;
use NF2::PacketLib;
use NF21RouterLib;
use POSIX qw(ceil floor);
$batch=0;
$delay=2000;

require "reg_defines.ph";
require "reg_defines_router_ip_tuneling.ph";

nf_set_environment( { PORT_MODE => 'PHYSICAL', MAX_PORTS => 4 } );

# use strict AFTER the $delay, $batch and %reg are declared
use strict;
use vars qw($delay $batch %reg);

my $ROUTER_PORT_1_MAC = '00:ca:fe:00:00:01';
my $ROUTER_PORT_2_MAC = '00:ca:fe:00:00:02';
my $ROUTER_PORT_3_MAC = '00:ca:fe:00:00:03';
my $ROUTER_PORT_4_MAC = '00:ca:fe:00:00:04';

my $ROUTER_PORT_1_IP = '192.168.1.1';
my $ROUTER_PORT_2_IP = '192.168.2.1';
my $ROUTER_PORT_3_IP = '192.168.3.1';
my $ROUTER_PORT_4_IP = '192.168.4.1';

my $next_hop_1_DA = '00:fe:ed:01:d0:65';
my $next_hop_2_DA = '00:fe:ed:02:d0:65';

# Prepare the DMA and enable interrupts
prepare_DMA('@3.9us');
enable_interrupts(0);

# Write the ip addresses and mac addresses, routing table, filter, ARP entries
my $delay = '@10us';
set_router_MAC(1, $ROUTER_PORT_1_MAC);
set_router_MAC(2, $ROUTER_PORT_2_MAC);
set_router_MAC(3, $ROUTER_PORT_3_MAC);
set_router_MAC(4, $ROUTER_PORT_4_MAC);

add_dst_ip_filter_entry(0,$ROUTER_PORT_1_IP);
add_dst_ip_filter_entry(1,$ROUTER_PORT_2_IP);
add_dst_ip_filter_entry(2,$ROUTER_PORT_3_IP);
add_dst_ip_filter_entry(3,$ROUTER_PORT_4_IP);

add_LPM_table_entry(0,'171.64.2.0', '255.255.255.0', '171.64.2.1', 0x04);
add_LPM_table_entry(15, '0.0.0.0', '0.0.0.0', '171.64.1.1', 0x01);

# Add the ARP table entries
add_ARP_table_entry(0, '171.64.1.7', $next_hop_1_DA);
add_ARP_table_entry(1, '171.64.2.1', $next_hop_2_DA);

my $TTL = 30;
my $DA = 0;
my $SA = 0;
my $dst_ip = 0;
my $src_ip = 0;
my $pkt;
my $num_packet = 10;
$delay = '@40us';
my $tunel_dst_ip = '198.64.2.1';
my $tunel_src_ip = '198.62.2.2';
my $length = 0;
my $init_length = 100;

## make the tunneling ip header which is going to be added
my $IP_hdr = NF2::IP_hdr->new(ttl => $TTL-1,
			   src_ip => $tunel_src_ip,
			   dst_ip => $tunel_dst_ip,
			   proto  => 0xf4,
			   dgram_len => $init_length +24 - 14, # 14 is the length of the MAC header, 24 is the ip header plus the padding
			  );
$IP_hdr->checksum(0);  # make sure its zero before we calculate it.
$IP_hdr->checksum($IP_hdr->calc_checksum);

my @unpacked_ip_h= unpack('H*', $IP_hdr->packed);
my @unpacked_ip= unpack('C*', $IP_hdr->packed);

printf  @unpacked_ip_h, "\n";

my $data_0 = (@unpacked_ip[2]<<24)+ (@unpacked_ip[3]<<16)+ (@unpacked_ip[4]<<8)+(@unpacked_ip[5]);
my $data_1 = (@unpacked_ip[6]<<24)+   (@unpacked_ip[7]<<16)+ (@unpacked_ip[8]<<8)+(@unpacked_ip[9]);
my $data_2 = (@unpacked_ip[10]<<24)+ (@unpacked_ip[11]<<16)+ (@unpacked_ip[12]<<8)+(@unpacked_ip[13]);
my $data_3 = (@unpacked_ip[14]<<24)+ (@unpacked_ip[15]<<16)+ (@unpacked_ip[16]<<8)+(@unpacked_ip[17]);
my $data_4 = (@unpacked_ip[18]<<24)+ (@unpacked_ip[19]<<16)+ (0xFFFF);

printf "\n %x \n", $data_0;
printf "%x \n", $data_1;
printf "%x \n", $data_2;
printf "%x \n", $data_3;
printf "%x \n", $data_4;

nf_PCI_write32($delay, 0, ENCAP_ENABLE_REG(),     0x1);
nf_PCI_write32($delay, 0, ENCAP_PORT_MASK_REG(),  0xab400207);
nf_PCI_write32($delay, 0, ENCAP_IP_DATA_0_REG(),  $data_0);
nf_PCI_write32($delay, 0, ENCAP_IP_DATA_1_REG(),  $data_1);
nf_PCI_write32($delay, 0, ENCAP_IP_DATA_2_REG(),  $data_2);
nf_PCI_write32($delay, 0, ENCAP_IP_DATA_3_REG(),  $data_3);
nf_PCI_write32($delay, 0, ENCAP_IP_DATA_4_REG(),  $data_4);
nf_PCI_write32($delay, 0, ENCAP_IP_DATA_5_REG(),  0xffff4500);

$delay = '@90us';
#
###############################
#

for (my $i=0; $i<$num_packet; $i++)
  {

    $SA = '01:55:55:55:55:55';
    $dst_ip = '171.64.2.7';
    $src_ip = '171.64.8.1';

    $length = $init_length;
    $DA = $ROUTER_PORT_1_MAC;    
    $pkt = make_IP_pkt($length, $DA, $SA, $TTL, $dst_ip, $src_ip);
    nf_packet_in(1, $length, $delay, $batch,  $pkt);

    $length = $init_length;
    $DA = $ROUTER_PORT_2_MAC;    
    $pkt = make_IP_pkt($length, $DA, $SA, $TTL, $dst_ip, $src_ip);
    nf_packet_in(2, $length, $delay, $batch,  $pkt);

    $length = $init_length;
    $DA = $ROUTER_PORT_3_MAC;    
    $pkt = make_IP_pkt($length, $DA, $SA, $TTL, $dst_ip, $src_ip);
    nf_packet_in(3, $length, $delay, $batch,  $pkt);

    $length = $init_length;
    $DA = $ROUTER_PORT_4_MAC;    
    $pkt = make_IP_pkt($length, $DA, $SA, $TTL, $dst_ip, $src_ip);
    nf_packet_in(4, $length, $delay, $batch,  $pkt);

    ### make the expected packet
    $length = $init_length+20+4;
    $DA = $next_hop_2_DA;
    $SA = $ROUTER_PORT_2_MAC;
    $pkt = make_IP_IP_pkt($length, $DA, $SA, $TTL-1, $tunel_dst_ip, $tunel_src_ip,$dst_ip, $src_ip);
    print "expected packet", "\n";
    print $pkt, "\n";
    nf_expected_packet(2, $length, $pkt);
    nf_expected_packet(2, $length, $pkt);
    nf_expected_packet(2, $length, $pkt);
    nf_expected_packet(2, $length, $pkt);

    $delay = 0;
  }
#
###############################
#
$delay = '@120us';
nf_PCI_read32($delay, 0, ENCAP_NUM_PACKET_IN_REG(),  $num_packet*4);
nf_PCI_read32($delay, 0, ENCAP_NUM_PACKET_OUT_REG(), $num_packet*4);
nf_PCI_read32($delay, 0, ENCAP_NUM_BYTES_IN_REG(),   $num_packet*4*ceil(($init_length+8)/8));
nf_PCI_read32($delay, 0, ENCAP_NUM_BYTES_OUT_REG(),  $num_packet*4*ceil(($init_length+8+24)/8));

# *********** Finishing Up - need this in all scripts ! ****************************
my $t = nf_write_sim_files();
print  "--- make_pkts.pl: Generated all configuration packets.\n";
printf "--- make_pkts.pl: Last packet enters system at approx %0d microseconds.\n",($t/1000);
if (nf_write_expected_files()) {
  die "Unable to write expected files\n";
}

nf_create_hardware_file('LITTLE_ENDIAN');
nf_write_hardware_file('LITTLE_ENDIAN');
