///////////////////////////////////////////////////////////////////////////////
//
//      Project:  Aurora Module Generator version 2.6
//
//         Date:  $Date: 2006/12/28 05:14:15 $
//          Tag:  $Name: i+IP+121336 $
//         File:  $RCSfile: tx_ll.ejava,v $
//          Rev:  $Revision: 1.1.2.3 $
//
//      Company:  Xilinx
// Contributors:  R. K. Awalt, B. L. Woodard, N. Gulstone
//
//   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
//                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
//                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
//                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
//                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
//                APPLICATION OR STANDARD, XILINX IS MAKING NO
//                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
//                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
//                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
//                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
//                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
//                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
//                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
//                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
//                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//                PURPOSE.
//
//                (c) Copyright 2004 Xilinx, Inc.
//                All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////
//
//  TX_LL
//
//  Author: Nigel Gulstone
//          Xilinx - Embedded Networking System Engineering Group
//
//  Description: The TX_LL module converts user data from the LocalLink interface
//               to Aurora Data, then sends it to the Aurora Channel for transmission.
//               It also handles NFC and UFC messages.
//
//               This module supports 1 2-byte lane designs
//
//               This module supports Immediate Mode Native Flow Control
//

`timescale 1 ns / 10 ps

module aurora_framing_TX_LL
(
    // LocalLink PDU Interface
    TX_D,
    TX_REM,
    TX_SRC_RDY_N,
    TX_SOF_N,
    TX_EOF_N,

    TX_DST_RDY_N,

    // NFC Interface
    NFC_REQ_N,
    NFC_NB,

    NFC_ACK_N,



    // Clock Compensation Interface
    WARN_CC,
    DO_CC,


    // Global Logic Interface
    CHANNEL_UP,


    // Aurora Lane Interface
    GEN_SCP,
    GEN_ECP,
    GEN_SNF,
    FC_NB,
    TX_PE_DATA_V,
    GEN_PAD,
    TX_PE_DATA,
    GEN_CC,

    // RX_LL Interface
    TX_WAIT,

    DECREMENT_NFC,

    // System Interface
    USER_CLK


);

`define DLY #1


//***********************************Port Declarations*******************************


    // LocalLink PDU Interface
    input   [0:15]     TX_D;
    input              TX_REM;
    input              TX_SRC_RDY_N;
    input              TX_SOF_N;
    input              TX_EOF_N;

    output             TX_DST_RDY_N;

    // NFC Interface
    input              NFC_REQ_N;
    input   [0:3]      NFC_NB;

    output             NFC_ACK_N;

    // Clock Compensation Interface
    input              WARN_CC;
    input              DO_CC;


    // Global Logic Interface
    input              CHANNEL_UP;


    // Aurora Lane Interface
    output             GEN_SCP;
    output             GEN_ECP;
    output             GEN_SNF;
    output  [0:3]      FC_NB;
    output             TX_PE_DATA_V;
    output             GEN_PAD;
    output  [0:15]     TX_PE_DATA;
    output             GEN_CC;

    // RX_LL Interface
    input              TX_WAIT;

    output             DECREMENT_NFC;

    // System Interface
    input              USER_CLK;



//*********************************Wire Declarations**********************************

    wire               halt_c_i;
    wire               tx_dst_rdy_n_i;


//*********************************Main Body of Code**********************************


    // TX_DST_RDY_N is generated by TX_LL_CONTROL and used by TX_LL_DATAPATH and
    // external modules to regulate incoming pdu data signals.
    assign  TX_DST_RDY_N    =   tx_dst_rdy_n_i;


    // TX_LL_Datapath module
    aurora_framing_TX_LL_DATAPATH aurora_framing_tx_ll_datapath_i
    (
        // LocalLink PDU Interface
        .TX_D(TX_D),
        .TX_REM(TX_REM),
        .TX_SRC_RDY_N(TX_SRC_RDY_N),
        .TX_SOF_N(TX_SOF_N),
        .TX_EOF_N(TX_EOF_N),



        // Aurora Lane Interface
        .TX_PE_DATA_V(TX_PE_DATA_V),
        .GEN_PAD(GEN_PAD),
        .TX_PE_DATA(TX_PE_DATA),


        // TX_LL Control Module Interface
        .HALT_C(halt_c_i),
        .TX_DST_RDY_N(tx_dst_rdy_n_i),


        // System Interface
        .CHANNEL_UP(CHANNEL_UP),
        .USER_CLK(USER_CLK)

    );



    // TX_LL_Control module

    aurora_framing_TX_LL_CONTROL aurora_framing_tx_ll_control_i
    (
        // LocalLink PDU Interface
        .TX_SRC_RDY_N(TX_SRC_RDY_N),
        .TX_SOF_N(TX_SOF_N),
        .TX_EOF_N(TX_EOF_N),
        .TX_REM(TX_REM),

        .TX_DST_RDY_N(tx_dst_rdy_n_i),

        // NFC Interface
        .NFC_REQ_N(NFC_REQ_N),
        .NFC_NB(NFC_NB),

        .NFC_ACK_N(NFC_ACK_N),

        // Clock Compensation Interface
        .WARN_CC(WARN_CC),
        .DO_CC(DO_CC),

        // Global Logic Interface
        .CHANNEL_UP(CHANNEL_UP),

        // TX_LL Control Module Interface
        .HALT_C(halt_c_i),


        // Aurora Lane Interface
        .GEN_SCP(GEN_SCP),
        .GEN_ECP(GEN_ECP),
        .GEN_SNF(GEN_SNF),
        .FC_NB(FC_NB),
        .GEN_CC(GEN_CC),

        // RX_LL Interface
        .TX_WAIT(TX_WAIT),

        .DECREMENT_NFC(DECREMENT_NFC),

        // System Interface
        .USER_CLK(USER_CLK)

    );

endmodule
