///////////////////////////////////////////////////////////////////////////////
// $Id: testbench.v 1887 2007-06-19 21:33:32Z grg $
//
// Module: testbench.v
// Project: event capture
// Description: Tests the evt_rcrdr module
//
///////////////////////////////////////////////////////////////////////////////
`timescale 1 ns/1 ps
module testbench();

   function integer log2;
      input integer number;
      begin
         log2=0;
         while(2**log2<number) begin
            log2=log2+1;
         end
      end
   endfunction // log2

   parameter NUM_MONITORED_SIGS = 8;
   parameter SIG_VALUE_SIZE = 12;
   parameter DATA_WIDTH = 64;
   parameter TIMER_RES_SIZE = 3;
   parameter ALL_SIG_VALUES_SIZE = SIG_VALUE_SIZE*NUM_MONITORED_SIGS;
   parameter NUM_MON_SIGS_SIZE = log2(NUM_MONITORED_SIGS+2);
	     
   reg 	                         reset_timers;
   reg 	                         enable_events;
   reg [NUM_MONITORED_SIGS-1:0]  monitor_mask;
   reg [TIMER_RES_SIZE-1:0] 	 tmr_resolution;
   wire [NUM_MON_SIGS_SIZE-1:0]    evts_dropped;   // number of events dropped this cycle

   // interface to events
   reg [NUM_MONITORED_SIGS-1:0]  signals;
   reg [SIG_VALUE_SIZE-1:0]      signal_values[NUM_MONITORED_SIGS-1:0];
   
   // interface to evt_pkt_wrtr.v
   wire [DATA_SIZE-1:0] 	 event_data;
   wire 			 event_avail;
   reg 				 evt_wr_rdy;
   reg 				 new_evt_pkt;    // pulses high
   
   // misc
   reg 				 reset;
   reg				 clk;
   
   evt_rcrdr
     #(.NUM_MONITORED_SIGS (NUM_MONITORED_SIGS),
       .SIG_VALUE_SIZE(SIG_VALUE_SIZE),
       .DATA_WIDTH(DATA_WIDTH),
       .TIMER_RES_SIZE(TIMER_RES_SIZE))
     evt_rcrdr(
	       // Outputs
	       .evts_dropped		(evts_dropped),
	       .event_data		(event_data[DATA_SIZE-1:0]),
	       .event_avail		(event_avail),
	       // Inputs
	       .reset_timers            (reset_timers),
	       .enable_events		(enable_events),
	       .monitor_mask		(monitor_mask[NUM_MONITORED_SIGS-1:0]),
	       .tmr_resolution		(tmr_resolution[TIMER_RES_SIZE-1:0]),
	       .signals			(signals[NUM_MONITORED_SIGS-1:0]),
	       .signal_values		({signal_values[7],
					  signal_values[6],
					  signal_values[5],
					  signal_values[4],
					  signal_values[3],
					  signal_values[2],
					  signal_values[1],
					  signal_values[0]}),
	       .evt_wr_rdy		(evt_wr_rdy),
	       .new_evt_pkt		(new_evt_pkt),
	       .reset			(reset),
	       .clk			(clk));

   reg [31:0] 			 clk_cntr;

   always #8 clk = (clk===1) ? 0 : 1;
   
   always @(posedge clk) begin
      clk_cntr <= (clk_cntr==='hx) ? 0 : clk_cntr + 1;
      #1 case (clk_cntr)
	    0: begin
	       enable_events <= 1;
	       reset_timers<=0;
	       monitor_mask<='b1000001;
	       tmr_resolution<=0;
	       signal_values[0] <= 0;
	       signal_values[1] <= 1;
	       signal_values[2] <= 2;
	       signal_values[3] <= 3;
	       signal_values[4] <= 4;
	       signal_values[5] <= 5;
	       signal_values[6] <= 6;
	       signal_values[7] <= 7;
	       signals <= 0;
	       evt_wr_rdy <= 1;
	       new_evt_pkt <= 0;
	       reset <= 1;
	    end // case: 0
	    
	    1: reset <= 0;
	    // send 2 events
	    2: begin
	       monitor_mask<='b10000001;
	       signals <= 'b11111111;
	       signal_values[0] <= 8;
	       signal_values[7] <= 6+8;
	    end // case: 2

	    // send all events
	    3:begin
	       monitor_mask<='b11111111;
	       signals <=    'b11111110;
	       signal_values[3] <= 8+3;
	       signal_values[5] <= 8+5;
	    end
	    4:begin
	       monitor_mask<='b11111111;
	       signals <=    0;
	    end
	    // wait until cleared we should have one event unsent yet
	    // because the number of events is odd
	    5:begin
	    end
	    6:;
	    7: begin
	    end
	    8:;
	    // send an odd number of events, test new_evt_pkt signal
	    9: begin
	       monitor_mask<='b11111111;
	       signals <=    'b00010000;
	       new_evt_pkt <= 1;
	    end
	    10:begin
	       monitor_mask<='b11111111;
	       signals <=    'b00000000;
	       new_evt_pkt <= 0;
	    end
	    11:;
	    12:;
	    13:;
	    // all events should be sent now, try to drop events
	    14:begin
	       monitor_mask<='b11111111;
	       signals <=    'b11111111;
	    end
	    15:;
	    16:;
	    17:;
	    18:;
	    19:;
	    24:begin
	       monitor_mask<='b11111111;
	       signals <=    'b00000000;
	    end
	 endcase
   end

endmodule
     