// dumb sender skeleton. this code would benefit from an overhaul.
#include "async.h"
#include "arpc.h"
#include "dns.h"
#include "sha1.h"
#include "ps_iface.h"
#include "util.h"
#include "icinghdr.h"
#include "tame.h"
#include "pathclient.h"
#include "tt.h"

const uint32_t MAX_UNSIGNED32 = 0xffffffff;
const char* const PAYLOAD = "HELLO!";
const uint32_t EXPIRY_HEADROOM_S = 2;
const uint64_t EXPIRY_HEADROOM_US = EXPIRY_HEADROOM_S * MILLION_LL;
const uint32_t INTERVAL = 10;

int sock_g = -1;
ptr<packet> hdrpack_g = NULL;
ptr<packet> flowpack_g = NULL;
ptr<aclnt> aclient_g = NULL;
ptr<reachability> reach_g = NULL;
bool dump_g = false;

const char* ps_host = NULL;
uint16_t ps_port = 0;

typedef callback<void, ptr<reachability> >::ref reachability_info_cb;

ptr<reachability> process_rpc_result(str, clnt_stat, ptr<icingps_get_res>);
const precord_ent& choose_entry(const precord&);
const precord_ent& get_winning_entry(const precord&);
ptr<reachability> ent2reach(const precord_ent&);

tamed static void
create_udp_client(const char* host, uint16_t port,
		  callback<void, ptr<aclnt> >::ref cb)
{
    tvars {
	clnt_stat stat;
	ptr<aclnt> aclient;
    }

    twait {
	aclntudp_create(host, port, icingps_program_1, 
			mkevent(aclient, stat));
    }
    if (!aclient) 
	fatal("Couldn't find host name %s\n", host);

    (*cb)(aclient);
}

tamed static void
get_reachability(str name, reachability_info_cb cb)
{
     tvars {
	ptr<aclnt> local_aclient;
	clnt_stat err;
	icingps_get_arg arg;
	ptr<icingps_get_res> res;
    }

    if (!aclient_g) {
	twait { create_udp_client(ps_host, ps_port, mkevent(local_aclient)); }
	aclient_g = local_aclient;
    }

    res = New refcounted<icingps_get_res>();
    arg.name = name;

    twait { aclient_g->timedcall((time_t)5, ICINGPS_PROC_GET, &arg,
			     res, mkevent(err)); }

    ptr<reachability> reach = process_rpc_result(name, err, res);
    (*cb)(reach);
}


ptr<reachability> 
process_rpc_result(str name, clnt_stat err, ptr<icingps_get_res> res)
{	
    bool success = false;

    switch(err) {

	case RPC_SUCCESS:
	    success = true;
	    break;

	case RPC_TIMEDOUT:
	    warnx << "RPC timed out\n";
	    break;

	default:
	    warnx << "get call failed\n";
    } 

    if (!success) 
	return NULL;

    success = false;

    switch (res->status) {

	case ICINGPS_OK:
	    success = true;
#if 0
	    rpc_print(warn, *(res->prec));
//		    warn << *(res->prec);
#endif
	    break;

	case ICINGPS_NOTFOUND:
	    warnx("p-record for name \"%s\" not found\n", name.cstr());
	    break;

	default:
	    warnx << "Something went wrong. Unexpected status code\n";
    }

    if (!success) 
	return NULL;
    
    const precord_ent& ent = choose_entry(*(res->prec));
    ptr<reachability> reach = ent2reach(ent);
    return reach;
}   

ptr<reachability>
ent2reach(const precord_ent& ent)
{
    ptr<reachability> reach = New refcounted<reachability>();

    reach->set_waypoint_name(ent.waypoint.waypoint_name);
    reach->set_waypoint_port(ent.waypoint.waypoint_udp_port);
    reach->set_caplist(ent.path_from_waypoint);
    reach->set_ncaps(ent.path_from_waypoint);
    reach->set_expiry(ent.path_from_waypoint);

    return reach;
}

inline bool
nopacket()
{
    return (hdrpack_g == NULL);
}

inline bool
near_expiry()
{
    assert(reach_g);

    return (get_now() + EXPIRY_HEADROOM_US > reach_g->_expiry);
}

ptr<icing_packet>
make_hdr_packet(ptr<reachability> reach)
{
    ptr<icing_packet> hdrpack = 
	New refcounted<icing_packet>(reach->_ncaps, 
				     strlen(PAYLOAD),
				     &reach->_cl);
    
    uint8_t* payloadp = hdrpack->icing_payload();
    strcpy(reinterpret_cast<char*>(payloadp), PAYLOAD);

    return hdrpack;
}

ptr<icing_flow_packet>
make_flow_packet(ptr<icing_packet> hdrpack)
{
    ptr<icing_flow_packet> flowpack = 
	New refcounted<icing_flow_packet>(hdrpack->const_icing_slow_hdr(),
					  strlen(PAYLOAD));

    uint8_t* payloadp = flowpack->icing_payload();
    strcpy(reinterpret_cast<char*>(payloadp), PAYLOAD);

    return flowpack;
}

int
setup_socket()
{
    int sock;
    
    if ( (sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
	fatal("couldn't allocate socket: %m\n");

    make_async(sock);

    return sock;
}


const precord_ent&
choose_entry(const precord& prec)
{
    const precord_ent& entry = get_winning_entry(prec);

    if (dump_g) {
	const waypoint_info& wp_info = entry.waypoint;

	warnx << "The waypoint for this entry has\n";
	warnx << "(id,name,port) = ";
	warnx << "(" << wp_info.waypoint_id
	      << "," << wp_info.waypoint_name
	      << "," << ntohl(wp_info.waypoint_udp_port)
	      << ")\n";

	byte_print(entry);
    }

    return entry;
}

const precord_ent& 
get_winning_entry(const precord& prec)
{
    size_t nentries = prec.entries.size();

    uint32_t winning_metric = MAX_UNSIGNED32;
    size_t winning_entry = nentries;

    // XXX: revisit. eventually, the metric from the server will have to
    // be combined with the client's candidate states and notion of a
    // metric. for now, just pick the entry with the lowest metric.
    for (size_t i = 0; i < nentries; i++) {

	const precord_ent* entp = &prec.entries[i];
	if (entp->metric < winning_metric) {
	    winning_entry = i;
	    winning_metric = entp->metric;
	}

    }

#ifdef CHATTY
    warnx("The winning entry is %zu\n", winning_entry);
#endif

    return prec.entries[winning_entry];
}

tamed static void
resolve_waypoint_name(ptr<reachability> reach, cbv cb)
{
    tvars {
	ptr<hostent> he;
	int err;
    }

    twait {
	dns_hostbyname(reach->_waypoint_name, mkevent(he, err));
    }

    if (err) 
	fatal("couldn't resolve %s: %s\n", 
	      reach->_waypoint_name.cstr(), strerror(err));

    struct in_addr addr;
    addr = *reinterpret_cast<in_addr*>(he->h_addr_list[0]);
    reach->set_waypoint_ip(addr.s_addr);

    (*cb)();
}

void
send_packet(ptr<packet> pack)
{
    if (sock_g == -1) {
	sock_g = setup_socket();
    } 

    // XXX: probably we should only be writing if socket is writable
    int rc;
    sockaddr_in addr;
    memset(&addr, 0, sizeof(sockaddr_in));
    addr.sin_family = AF_INET;
    addr.sin_port = reach_g->_waypoint_port;
    addr.sin_addr.s_addr = reach_g->_waypoint_ip;

    if ( (rc = sendto(sock_g, pack->dat(), pack->length(), 0, 
		      reinterpret_cast<sockaddr*>(&addr),
		      sizeof(addr))) != 
			    static_cast<ssize_t>(pack->length())) {
	if (rc >= 0) 
	    fatal("rc %d confuses me\n", rc);
	else
	    warnx("couldn't send: %m\n");
    }
}


tamed static void
maintenanceloop(str name)
{
    tvars {
	ptr<reachability> localreach;
    }

    while (1) {

	if (nopacket() || near_expiry()) {

	    twait { get_reachability (name, mkevent(localreach)); }
	    if (!localreach) 
		fatal("couldn't get reachability info for %s\n", name.cstr());

	    twait { resolve_waypoint_name (localreach, mkevent());  }
	    ptr<icing_packet> localhdrpacket = make_hdr_packet(localreach);
	    ptr<icing_flow_packet> localflowpacket = 
		    make_flow_packet(localhdrpacket);

	    // packet now consistent with reachability info and both are 
	    // the latest. thus, now safe to replace the global state with 
	    // the one we just developed.
	    reach_g = localreach;
	    hdrpack_g = localhdrpacket;
	    flowpack_g = localflowpacket;
	}

	twait { delaycb(1, 0, mkevent()); }
    }

}

tamed static void
sendloop()
{
    tvars {
	// empty for now
	int n;
    }

    while (1) {
	if (!nopacket())
	    break;
	twait { delaycb(1,0,mkevent());}
    }

    for (n = 0; ; n++) {

	if (!nopacket()) {

	    ptr<packet> which_packet;
	    if ( !(n % INTERVAL) ) {
		which_packet = hdrpack_g;
	    } else {
		which_packet = flowpack_g;
	    }

	    send_packet(which_packet);
	}

	twait { delaycb(1, 0, mkevent()); }
    }
}

void
usage()
{
    warnx << "Usage: " << progname << " lookup_name ps_name ps_port [dump]\n";
    exit(1);

}

int 
main(int argc, char** argv)
{
    setprogname(argv[0]);
    if (argc != 4 && argc != 5)
	usage();

    str name = argv[1];
    ps_host = argv[2];
    ps_port = atoi(argv[3]);

    // XXX: use getopt
    if (argc == 5) 
	dump_g = true;

// XXX: switch to TCP
//    aclnttcp_create("127.0.0.1", 9009, icingps_program_1, wrap(foo));

    maintenanceloop(name);
    sendloop();
    amain();

    return 0;
}


