#include "async.h"
#include "arpc.h"
#include "ps_iface.h"
#include "util.h"
#include "sha1.h"
#include "qhash.h"
#include "icinghdr.h"
#include "icingmacker.h"
#include "pathclient.h"

// global state
qhash<str, ptr<precord> > crapmap;
ptr<asrv> srv;

// hacked crap for hard-coding the precord
const char* foo = "the hash of this is the dest realm id";
size_t nentries = 2;
size_t nnodes = 5; 

// config variables
str hmackey = "abcdef";
sha1hmac hmac;
uint16_t svc_port;
const char * hack_name = "";
uint16_t hack_port;

const char* realm0 = "dfda2b959663e6a4208e15c8a252c4511a7610a0";
const char* realm1 = "19566d9ec2ea6080b8bb0dcb34a1a05d3d999531";

macker_assembler_wholepath macker;

#define sassert(x) \
    switch (x) {   \
	case 0:    \
	    break; \
	case x:    \
	    break; 	   \
    }			

void
bogus_set_realmid(Capability* c, 
		  const realm_id* waypoint_id, 
		  bool first_node,
		  int j)
{
    if (first_node) {
	memcpy(c->realm.bytes, waypoint_id->base(), waypoint_id->size());
    } else {
	// XXX: eventually, we will need to get the public key out of band
	// and SHA1 it into the waypoint_id
	str realm_bytes = hex2bytes(realm1);
	assert(realm_bytes.len() == c->realm.SIZE);
	memcpy(c->realm.bytes, realm_bytes.cstr(), realm_bytes.len());
    }
}

void
bogus_set_expiry(TimestampWire* tw)
{
    struct timespec ts;
    clock_gettime(CLOCK_REALTIME , &ts);

    tw->sec = htonl(ts.tv_sec + 60);
    tw->usec = htonl(0);
}

void
bogus_set_patterns(Capability* c)
{
    // XXX: revisit
    memset(&c->source_path_patt, 
	   0x00,
	   sizeof(c->source_path_patt) + sizeof(c->dest_path_patt));
}

void
bogus_set_path_from_waypoint(rpc_vec<xdr_capmac, RPC_INFINITY>& path, 
			     const realm_id* waypoint_id)
{
    path.setsize(nnodes);
    for (size_t j = 0; j < nnodes; j++) {

	xdr_capmac* x = &path[j];
	x->setsize(sizeof(CapabilityMac));
	CapabilityMac* cm = reinterpret_cast<CapabilityMac*>(x->base());
	Capability* c = &cm->cap;

	bogus_set_realmid(c, waypoint_id, (j==0), j);

	bogus_set_expiry(&c->expiry);
	bogus_set_patterns(c);
    }

    // set up dummy header so we can produce the required MAC.
    caplist cl(path);
    icing_packet dummy_packet(nnodes, 0, &cl);

    for (size_t j = 0; j < nnodes; j++) {
	xdr_capmac* x = &path[j];
	CapabilityMac* cm = reinterpret_cast<CapabilityMac*>(x->base());
	produce_mac(dummy_packet.const_icing_slow_hdr(), cm, hmac, macker);
    }
}

void
bogus_set_waypoint(waypoint_info* infop, size_t entry)
{
    // XXX: eventually, we will need to get the public key out of band
    // and SHA1 it into the waypoint_id
    str realm_bytes = hex2bytes(realm0); 
    assert(realm_bytes.len() == infop->waypoint_id.size());
    memcpy(infop->waypoint_id.base(), realm_bytes.cstr(), realm_bytes.len());

    infop->waypoint_name  = hack_name;
    infop->waypoint_udp_port = htonl(static_cast<uint32_t>(hack_port));
}


void
bogus_construct_entry(precord_ent* entp, size_t entry)
{
    bogus_set_waypoint(&entp->waypoint, entry);
    bogus_set_path_from_waypoint(entp->path_from_waypoint, 
				 &entp->waypoint.waypoint_id);
    // XXX: revisit
    entp->metric = entry;
}

void
construct_bogus_prec(ptr<precord>& bogus_prec)
{
    bogus_prec = New refcounted<precord>();

    char bull[sha1::hashsize];
    sassert(sizeof(bull) == realm_id::nelm);
    sha1_hash(bogus_prec->dest.base(), foo, strlen(foo));

    bogus_prec->entries.setsize(nentries);
    for (size_t i = 0; i < nentries; i++) {
	bogus_construct_entry(&bogus_prec->entries[i], i);
    }

    crapmap.insert("crap", bogus_prec);
}

void
reset_expiry(ptr<precord> prec)
{
    for (size_t i = 0; i < prec->entries.size(); i++) {
	precord_ent* entp = &prec->entries[i];

	for (size_t j = 0; j < entp->path_from_waypoint.size(); j++) {
	    xdr_capmac* x = &entp->path_from_waypoint[j];
	    CapabilityMac* cm = reinterpret_cast<CapabilityMac*>(x->base());
	    Capability* cap = &cm->cap;
	
	    bogus_set_expiry(&cap->expiry);
	}

	// set up dummy header so we can produce the required mac
	caplist cl(entp->path_from_waypoint);
	icing_packet dummy_packet(nnodes, 0, &cl);

	for (size_t j = 0; j < entp->path_from_waypoint.size(); j++) {
	    xdr_capmac* x = &entp->path_from_waypoint[j];
	    CapabilityMac* cm = reinterpret_cast<CapabilityMac*>(x->base());
	    produce_mac(dummy_packet.const_icing_slow_hdr(), cm, hmac, macker);
	}
    }


}

void
handle_GET(svccb* rpc)
{
    const icingps_get_arg* arg = rpc->getarg<icingps_get_arg>();
    warnx("request for lookup of name \"%s\"\n", arg->name.cstr());

    icingps_get_res res;

    ptr<precord>* prec = crapmap[arg->name];

    if (prec == NULL) {
	res.set_status(ICINGPS_NOTFOUND);
    } else {
	res.set_status(ICINGPS_OK);
	reset_expiry(*prec);
	*(res.prec) = **prec;
    }

    rpc->reply(&res);
}

void
handle_SET(svccb* rpc)
{
    // XXX
}

void
dispatch(svccb* rpc)
{
    if (!rpc) {
	srv->setcb(NULL);
	return;
    }

    switch (rpc->proc()) {
   
	case ICINGPS_PROC_NULL:
	    warnx << "proc null called by " 
		 << *(const sockaddr_in*)(rpc->getsa())
		 << "\n";
	    rpc->reply(NULL);
	    break;

	case ICINGPS_PROC_GET:
	    handle_GET(rpc);
	    break;

	case ICINGPS_PROC_SET:
	    handle_SET(rpc);
	    break;

	default:
	    fatal("Procedure %d not handled\n", rpc->proc());
    }

}

void
usage()
{
    warnx << "Usage: " << progname 
	  << " ps_port waypoint_name waypoint_port [hmac_key]\n";

    exit(1);
}

int main(int argc, char** argv)
{
    setprogname(argv[0]);

    if (argc != 4 && argc != 5)
	usage();

    if (argc == 5)
	hmackey = argv[4];

    hmac.setkey(hmackey.cstr(), hmackey.len());

    svc_port = atoi(argv[1]);
    hack_name = argv[2];
    hack_port = atoi(argv[3]);

    ptr<precord> bogus_prec;
    construct_bogus_prec(bogus_prec);

// XXX: move to TCP if we need big packets, which we probably do.
//    srv = init_tcp_server(svc_port, icingps_program_1);
    srv = init_udp_server(svc_port, icingps_program_1);
    srv->setcb(wrap(dispatch));

    amain();

    return 0;
}
