/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef _UTIL_H_
#define _UTIL_H_

#include "async.h"
#include "arpc.h"
#include "ps_iface.h"

ptr<asrv> init_udp_server(uint16_t port, const rpc_program& program, 
			  size_t sockbufsize = axprt::defps);
ptr<aclnt> init_udp_client(uint16_t port, const rpc_program& program);
ptr<asrv> init_tcp_server(uint16_t port, const rpc_program& program);

inline const strbuf&
strbuf_cat (const strbuf &sb, const sockaddr_in& a)
{
    strbuf_cat(sb, inet_ntoa(a.sin_addr));
    strbuf_cat(sb, ':');
    strbuf_cat(sb, ntohs(a.sin_port));
    return sb;
}

inline const strbuf&
strbuf_cat (const strbuf &sb, const realm_id& rid)
{
    return sb << hexdump(rid.base(), rid.size());
}


#if 0
inline const strbuf&
strbuf_cat (const strbuf &sb, const precord& prec)
{
    sb << "destination realm id: " << prec.dest << "\n";

    size_t nentries = prec.entries.size();
    for (uint32_t i = 0; i < nentries; i++) {

	const precord_ent* entp = &prec.entries[i];

	sb << "entry " << i << "\n";
	sb << "\tsource: " << entp->source << "\n";
	sb << "\tpath:\n";
	size_t nnodes = entp->path_from_source.size();

	for (uint32_t j = 0; j < nnodes; j++) 
	    sb << "\t\t" << entp->path_from_source[j] << "\n";
	
	sb << "\n";
	sb << "\tmetric: " << entp->metric;
	sb << "\n";
    }

    return sb;
}

inline const strbuf&
strbuf_cat (const strbuf& sb, const capmac& cm)
{
    return sb << "capability: " << cm.cap << " mac: " << cm.mac;
}

inline const strbuf&
strbuf_cat (const strbuf& sb, const capability& cap)
{
    sb << hexdump(cap.realm.base(), cap.realm.size()) << " ";
    sb.fmt("%d,%d", cap.expiry.sec, cap.expiry.usec);
    sb << " " << cap.source_path_patt << " " << cap.dest_path_patt;
    return sb;
}

inline const strbuf&
strbuf_cat (const strbuf& sb, const path_pattern& pp)
{
    return sb << hexdump(pp.base(), pp.size());
}

inline const strbuf&
strbuf_cat (const strbuf& sb, const mactype& m)
{
    return sb << hexdump(m.base(), m.size());
}
#endif

// Recursively make all of the directories in the path. Returns 'false'
// on failure and 'true' on success.
bool makedirs(const char* path);

// random integer between a and b, inclusive
inline int get_rand_int(int a, int b)
{
    double rand_double = (rand() / (double)(RAND_MAX + 1.0));
    int scaled = (int)(((b+1)-a)*rand_double);
    int shifted = scaled + a;
    return shifted;
}

bool convertdouble(const char* cp, double* resp);


#endif
