/* $Id: $ */

/*
 *
 * Copyright (C) 2009 Antonio Nicolosi (nicolosi@cs.stevens.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "b163.h"

void
b163_set_to_zero (b163_elem sp) {
  bzero(sp, sizeof(u_int32_t)*B163_MAX_LEN);
}

void
b163_set_to_zero_dp (b163_elem_dp dp) {
  bzero(dp, sizeof(u_int32_t)*B163_MAX_LEN_DP);
}

void 
b163_copy (b163_elem sp1, b163_elem sp2) {
  memcpy(sp1, sp2, sizeof(u_int32_t)*B163_MAX_LEN);
}

void 
b163_copy_dp (b163_elem_dp dp1, b163_elem_dp dp2) {
  memcpy(dp1, dp2, sizeof(u_int32_t)*B163_MAX_LEN_DP);
}

void
b163_set_to_one (b163_elem sp) {
  b163_set_to_zero (sp);
  sp[0] = 0x1;
}

void
b163_set_to_one_dp (b163_elem_dp dp) {
  b163_set_to_zero_dp (dp);
  dp[0] = 0x1;
}

void
b163_invert (b163_elem inv, b163_elem sp) {
  b163_elem one;
  b163_set_to_one (one);
  b163_div (inv, one, sp);
}

/* Splits dp as sp1<<163 + sp2 */
void
b163_get_sp_pair (b163_elem sp1, b163_elem sp2, b163_elem_dp dp) {
  sp1[5]  =  (dp[10] >> 3);
  sp1[4]  = ((dp[10] & 0x7) << 29);
  sp1[4] |=  (dp[9] >> 3);
  sp1[3]  = ((dp[9] & 0x7) << 29);
  sp1[3] |=  (dp[8] >> 3);
  sp1[2]  = ((dp[8] & 0x7) << 29);
  sp1[2] |=  (dp[7] >> 3);
  sp1[1]  = ((dp[7] & 0x7) << 29);
  sp1[1] |=  (dp[6] >> 3);
  sp1[0]  = ((dp[6] & 0x7) << 29);
  sp1[0] |=  (dp[5] >> 3);
  memcpy(sp2, dp, sizeof(u_int32_t)*B163_MAX_LEN);
  sp2[5] &= 0x7;
}

void 
b163_dp_from_sp (b163_elem_dp dp, b163_elem sp) {
  b163_set_to_zero_dp (dp);
  b163_copy (dp, sp); 
}

void
b163_sp_from_dp (b163_elem sp, b163_elem_dp dp) {
  assert(!(B163_NEEDS_MOD(dp))); 
  b163_copy (sp, dp);
}

void 
b163_add_into (b163_elem sp1, b163_elem sp2) {
  sp1[0] ^= sp2[0];
  sp1[1] ^= sp2[1];
  sp1[2] ^= sp2[2];
  sp1[3] ^= sp2[3];
  sp1[4] ^= sp2[4];
  sp1[5] ^= sp2[5];
}

void
b163_add_into_dp (b163_elem_dp dp1, b163_elem_dp dp2) {
  b163_add_into (dp1, dp2);
  (dp1)[6] ^= (dp2)[6];
  (dp1)[7] ^= (dp2)[7];
  (dp1)[8] ^= (dp2)[8];
  (dp1)[9] ^= (dp2)[9];
  (dp1)[10] ^=(dp2)[10];
}

void
b163_add4g_into (b163_elem sp,u_int32_t v) {
  sp[0] ^= v;
}

void
b163_set_to_mod (b163_elem sp) {
  b163_set_to_one (sp);
  gf_shl (B163_MAX_LEN, sp, B163_MAX_DEGREE);
  b163_add4g_into (sp, B163_MOD_MOD);
}

void 
b163_clear_constant_term(b163_elem sp) {
  sp[0] &= 0xfffffffe;
}

void 
b163_cat (b163_elem sp) {
  printf ("0x%08x\t", (sp)[5]);
  printf ("0x%08x\t", (sp)[4]);
  printf ("0x%08x\t", (sp)[3]);
  printf ("0x%08x\t", (sp)[2]);
  printf ("0x%08x\t", (sp)[1]);
  printf ("0x%08x\n", (sp)[0]);
}

void 
b163_cat_dp (b163_elem_dp dp) {
  printf ("0x%08x\t", (dp)[10]);
  printf ("0x%08x\t", (dp)[9]);
  printf ("0x%08x\t", (dp)[8]);
  printf ("0x%08x\t", (dp)[7]);
  printf ("0x%08x\t", (dp)[6]);
  printf ("0x%08x\t", (dp)[5]);
  printf ("0x%08x\t", (dp)[4]);
  printf ("0x%08x\t", (dp)[3]);
  printf ("0x%08x\t", (dp)[2]);
  printf ("0x%08x\t", (dp)[1]);
  printf ("0x%08x\n", (dp)[0]);
}

/* Polynomial division with reminder for B163 (no mod. red.) */
/* Special case: Assume all polys have length B163_MAX_LEN   */
/*                                                           */
/* Input:  32-bit arrays U, V                                */ 
/* Output: 32-bit arrays Q, R, s.t. U = V*Q + R              */ 
/* Note:   Destroys pre-existing values of Q, R              */
void
b163_div_rem (b163_elem Q, b163_elem R, b163_elem U, b163_elem V) {
  b163_elem X, Y;
  u_int16_t h = gf_degree (B163_MAX_LEN, U);    /* dividend degree */
  u_int16_t k = gf_degree (B163_MAX_LEN, V);    /* divisor  degree */
  
  b163_copy (R,U);
  b163_set_to_zero(Q);
  
  /* XXX gf_degree of ZERO poly yields 0, so need to check for this */
  while ((h > k) || ((h == k) && !(B163_IS_ZERO(R)))) {  
    /* Cancel the leading term in the running dividend */
    b163_copy (X,V);
    gf_shl (B163_MAX_LEN, X, h-k); 
    b163_add_into (R,X);
    
    /* Keep track of the exponent used by adding monomial to quotient */
    b163_set_to_one (Y);     
    gf_shl (B163_MAX_LEN, Y, h-k); 
    b163_add_into (Q,Y);
    
    h = gf_degree (B163_MAX_LEN, R);  /* degree of running dividend */
  }
  
  b163_set_to_zero (X);
  b163_set_to_zero (Y);
}

/* Sets sp to sp1*sp2 modulo (x^163+x^7+x^6+x^3+1) 
 * Note: overwrites pre-existing value of sp     
 */
void 
b163_mul(b163_elem sp, b163_elem sp1, b163_elem sp2) {
  b163_elem_dp dp;
  b163_elem high;

  b163_set_to_zero_dp (dp);

  gf_multiply (dp, B163_MAX_LEN, sp1, B163_MAX_LEN, sp2);

  b163_get_sp_pair (high,sp,dp);

  /* When reducing dp by the irreducible polynomial of B-163,
   * we can replace x^163 by x^7+x^6+x^3+1, which is represented 
   * as a u_int32_t B163_MOD_MOD.  So we keep computing
   * high*B163_MOD_MOD+sp2_ into dp and then splitting dp into 
   * high+x^163+sp till degree drops below 163 
   */  
  do {
    b163_dp_from_sp(dp,sp);
    gf_mul_by4g (dp+2, &(dp[1]), &(dp[0]),
		 B163_MAX_LEN-1, high+1, high[0],
		 B163_MOD_MOD);
    b163_get_sp_pair (high,sp,dp); 
  } while (B163_NEEDS_MOD(dp));

  /* now sp contains dp as single precision, and high is zero */
  assert (B163_IS_ZERO(high));
}

/* Sets sp to sp1/sp2 modulo p(x)=(x^163+x^7+x^6+x^3+1) 
 * Note: overwrites pre-existing value of sp     
 */
void 
b163_div (b163_elem s0, b163_elem sp1, b163_elem sp2) {
  b163_elem s1, s2, q, r0, r1, r2;

  b163_copy (r1, sp2);
  b163_set_to_mod (r0);
  b163_copy (s1, sp1);
  b163_set_to_zero (s0);
  
  while (!(B163_IS_ZERO(r1))) {
    b163_div_rem (q, r2, r0, r1);

    /* s2 = s0 + s1*q mod p(x) */
    b163_mul (s2, q, s1);
    b163_add_into (s2, s0);

    b163_copy (r0, r1);         b163_copy (r1, r2);
    b163_copy (s0, s1);         b163_copy (s1, s2);
  }
}

/* Sets sp1 to sp2^exp modulo (x^163+x^7+x^6+x^3+1) 
 * Note: overwrites pre-existing value of sp1     
 */
void 
b163_powm(b163_elem sp1, b163_elem sp2, bigint *exp) {

  assert (sp1 != sp2);    /* XXX: should actually ckeck for disjointness */

  if (-1 == mpz_sgn (exp)) {
    b163_elem sp2_inv;
    mpz_t exp_neg;     mpz_init (exp_neg);

    b163_invert (sp2_inv, sp2);
    mpz_neg (exp_neg, exp);
    b163_powm (sp1, sp2_inv, exp_neg);

    mpz_clear (exp_neg);
  } else {
    size_t i = mpz_sizeinbase2 (exp) - 1;
    
    /* check for zero exponent */
    if (!mpz_getbit (exp, i)) {
      b163_set_to_one (sp1);
    } else {
      b163_copy (sp1, sp2);
      /* Vanilla square-and-multiply implementation */
      while (i-- > 0) {
	/* square ... */
	b163_square(sp1, sp1);
	if (mpz_getbit (exp, i))
	  /* ... multiply */
	  b163_mul (sp1, sp1, sp2);
      }
    }
  }
}

/* Sets sp1 to the square root of sp2 modulo (x^163+x^7+x^6+x^3+1)
 * Note: overwrites pre-existing value of sp1
 */
void 
b163_sqrt(b163_elem sp1, b163_elem sp2) {
  size_t i = (B163_MAX_DEGREE - 1);

  assert (sp1 != sp2);    /* XXX: should actually ckeck for disjointness */
  
  b163_copy (sp1, sp2);
  /* since the group order is 2^m, just square m-1 times */
  while (i-- > 0)
    b163_square(sp1, sp1);
}

/* Sets sp1 to the half-trace of sp2.
 * Note: overwrites pre-existing value of sp1     
 */
void
b163_halftrace(b163_elem sp1, b163_elem sp2) {
  size_t i = (B163_MAX_DEGREE - 1) / 2;

  assert (sp1 != sp2);    /* XXX: should actually ckeck for disjointness */

  b163_copy (sp1, sp2);
  /* Vanilla pow-4-and-add implementation */
    while (i-- > 0) {
      /* square twice---that's why it's half trace... */
      b163_square(sp1, sp1);
      b163_square(sp1, sp1);
      /* ... and add */
      b163_add_into (sp1, sp2);
    }
}

void 
b163_set_random (b163_elem sp) {
  prng_getbytes((void *)sp, sizeof(u_int32_t)*B163_MAX_LEN);
  sp[5] &= 0x00000007;
} 
