/* $Id: $ */

/*
 *
 * Copyright (C) 2009 Antonio Nicolosi (nicolosi@cs.stevens.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "b163.h"
#include <stdio.h>

void test_b163_8 () {
  /* x-coordinate of a curve point */
  b163_elem a = { 0x4A3205FD,
		  0x512F7874,
		  0x1481EB10,
		  0xB8C953CA,
		  0x0A601907, 
		  0x00000002 };

  /* x-coordinate of any B-163 point should have trace 1 */
  printf("Verifying trace of x-coordinate of base point: %s",
	 B163_TRACE(a) ? "OK\n" : "XXX---Something is wrong\n");


  /* Half-trace should give solution to quadratic eq: z*(z+1) = (a+1) */
  b163_elem a1, b, c, should_be_one;

  b163_copy (a1, a);
  b163_add4g_into (a1, 0x1);

  b163_halftrace (b, a1);

  /* Now let's verify */
  b163_copy (c, b);
  b163_add4g_into (c, 0x1);
  b163_mul (c, c, b);
  b163_div (should_be_one, c, a1);
  b163_add4g_into (should_be_one, 0x1);

  if (B163_IS_ZERO(should_be_one))
    printf("Half-trace verified\n");
  else
    printf("XXX Problem with half-trace\n");
}

void test_b163_7 () {
  b163_elem a = { 0x4A3205FD,
		  0x512F7874,
		  0x1481EB10,
		  0xB8C953CA,
		  0x0A601907, 
		  0x00000002 };

  b163_elem a2, b, b2, should_be_one;

  b163_square(a2, a);

  b163_invert(b,a);

  b163_square(b2, b);

  b163_square(b2, b);

  b163_mul (should_be_one, a2, b2);

  b163_add4g_into (should_be_one, 0x1);
 
  if (B163_IS_ZERO(should_be_one))
    printf("Modular inversion verified\n");
  else
    printf("XXX Problem with modular inversion\n");
}

void test_b163_6 () {
  /* represents x^3 */
  b163_elem a = { 0x00000008,
		  0x00000000,
		  0x00000000,
		  0x00000000,
		  0x00000000,
		  0x00000000 };

  b163_elem b;

  /* should be x^160 + x^4 + x^3 + 1 */
  b163_invert(b,a);

  printf("Element\n\t");
  b163_cat(a);

  printf("Modular inverse in the field underlying B163\n\t");
  b163_cat(b);
}

void test_b163_5 () {
  b163_elem a = { 0x00000001,
		  0x00000000,
		  0x00000000,
		  0x00000000,
		  0x00000000,
		  0x00000004 };

  b163_elem b = { 0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0x00000003 };

  b163_elem c, d;
  

  b163_div_rem (c, d, a, b);

  printf("Dividend\n\t");
  b163_cat(a);

  printf("Divisor\n\t");
  b163_cat(b);

  printf("Quotient\n\t");
  b163_cat(c);

  printf("Reminder\n\t");
  b163_cat(d);
}

void test_b163_4 () {
  b163_elem a = { 0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0x00000007 };

  b163_elem b = { 0x00000003,
		  0x00000000,
		  0x00000000,
		  0x00000000,
		  0x00000000,
		  0x00000000 };

  b163_elem c, d;
  

  b163_div_rem (c, d, a, b);

  printf("Dividend\n\t");
  b163_cat(a);

  printf("Divisor\n\t");
  b163_cat(b);

  printf("Quotient\n\t");
  b163_cat(c);

  printf("Reminder\n\t");
  b163_cat(d);
}

void test_b163_3 () {
  b163_elem a = { 0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0x00000007 };

  b163_elem b = { 0x00000003,
		  0x00000000,
		  0x00000000,
		  0x00000000,
		  0x00000000,
		  0x00000000 };

  b163_elem_dp c;
  
  b163_set_to_zero_dp(c);

  b163_mul (c, a, b);

  int i;
  for (i=B163_MAX_LEN-1; i>=0; i--)
    printf ("0x%08x\t", c[i]);
  printf("\n");
}

void test_b163_2 () {
  b163_elem a = { 0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0x00000007 };

  b163_elem b = { 0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0xffffffff,
		  0x00000007 };

  b163_elem_dp c;
  
  b163_set_to_zero_dp(c);

  b163_mul (c, a, b);

  int i;
  for (i=B163_MAX_LEN-1; i>=0; i--)
    printf ("0x%08x\t", c[i]);
  printf("\n");
}

void test_b163_1 () {
  b163_elem a = { 0x00000001,
		  0x00000000,
		  0x00000000,
		  0x00000000,
		  0x00000000,
		  0x00000000 };

  /* constant term of EC equation for B-163 */
  b163_elem b = { 0x4A3205FD,
		  0x512F7874,
		  0x1481EB10,
		  0xB8C953CA,
		  0x0A601907, 
		  0x00000002 };

  /* x-coordinate of generator */
  b163_elem x = { 0xE8343E36,
		  0xD4994637,
		  0xA0991168,
		  0x86A2D57E,
		  0xF0EBA162,
		  0x00000003 };
	
  /* y-coordinate of generator */
  b163_elem y = { 0x797324F1,
		  0xB11C5C0C,
		  0xA2CDD545,
		  0x71A0094F, 
		  0xD51FBC6C,
		  0x00000000 };

  printf ("Verifying Weierstrass equation y^2 + x*y = x^3 + a*x^2 + b\n");
  printf ("\t a is\n");
  b163_cat(a);
  printf ("\t b is\n");
  b163_cat(b);
  printf ("\t x is\n");
  b163_cat(x);
  printf ("\t y is\n");
  b163_cat(y);
  
  b163_elem rhs, lhs;
  
  /* lhs is y*y + x*y = y*(y+x) */
  b163_elem sum;
  b163_copy(sum, y);
  b163_add_into(sum, x);
  b163_mul (lhs, y, sum);

  /* rhs is x^3 + a*x^2 + b = x^2*(x + a) + b */
  b163_elem x2, xa;
  b163_square(x2, x);
  b163_copy(xa, x);
  b163_add_into(xa, a);
  b163_mul (rhs, x2, xa);
  b163_add_into(rhs, b);

  /* check if lhs == rhs */
  b163_elem zero;

  b163_copy(zero, lhs);
  b163_add_into(zero, rhs);

  if (B163_IS_ZERO(zero)) {
    printf ("Success! Equation verified at generator\n");
  }
  else {
    printf ("Something went wrong :( Following should have been zero\n");
    
    int i;
    for (i=B163_MAX_LEN-1; i>=0; i--)
      printf ("0x%08x\t", zero[i]);
    printf("\n");
  }
}

/* tests for gf2m.c */

void test_gf2m_1() {
  u_int32_t F[1] = { 0xffffffff };
  u_int32_t G[1] = { 0xffffffff };
  u_int32_t FG[2] = { 0xeeeeeeee,
		      0x55555555 };

  gf_mul (FG+2, &(FG[1]), &(FG[0]), 
	  0, F+1, F[0], 
	  0, G+1, G[0]);

  int i;
  for (i=1; i>=0; i--)
    printf ("0x%08x\t", FG[i]);
  printf("\n");
}

void test_gf2m_1a() {
  u_int32_t F[1] = { 0xffffffff };
  u_int32_t G[1] = { 0xffffffff };
  u_int32_t FG[2] = { 0x00000000,
		      0x00000000 };

  gf_mul (FG+2, &(FG[1]), &(FG[0]), 
	  0, F+1, F[0], 
	  0, G+1, G[0]);

  int i;
  for (i=1; i>=0; i--)
    printf ("0x%08x\t", FG[i]);
  printf("\n");
}

void test_gf2m_2() {
  /* (x^7 + x + 1)^10 */
  u_int32_t F[3] = { 0x00404505,
		     0x05000000,
		     0x00000040 };
  /* (x^7 + x + 1)^5  */
  u_int32_t G[2] = { 0x300008b3,
		     0x00000008 };
  /* Preset to (x^7 + x + 1)^15  
     so output should be all zeros*/
  u_int32_t FG[5] = { 0xdec6957f,
		      0x7d0e24da,
		      0x28f22cea,
		      0x0000020c,
		      0x00000000 };

  gf_mul (FG+2, &(FG[1]), &(FG[0]), 
	  2, F+1, F[0], 
	  1, G+1, G[0]);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", FG[i]);
  printf("\n"); 
}

void test_gf2m_2d() {
  u_int32_t F[3] = { 0x00000401,
		     0x01000000,
		     0x00000040 };
  u_int32_t G[2] = { 0x10000021,
		     0x00000008 };
  u_int32_t FG[5] = { 0x00000000,
		      0x00000000,
		      0x00000000,
		      0x00000000,
		      0x00000000 };

  gf_mul (FG+2, &(FG[1]), &(FG[0]), 
	  2, F+1, F[0], 
	  1, G+1, G[0]);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", FG[i]);
  printf("\n"); 
}

void test_gf2m_2c() {
  u_int32_t F[3] = { 0xffffffff,
		     0x00000000,
		     0x00000000 };
  u_int32_t G[2] = { 0xffffffff,
		     0xffffffff };
  u_int32_t FG[5] = { 0xffffffff,
		      0xffffffff,
		      0x00000000,
		      0xffffffff,
		      0xffffffff };

  gf_mul (FG+2, &(FG[1]), &(FG[0]), 
	  2, F+1, F[0], 
	  1, G+1, G[0]);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", FG[i]);
  printf("\n"); 
}

void test_gf2m_2b() {
  u_int32_t F[3] = { 0x00000003,
		     0x00000000,
		     0x00000000 };
  u_int32_t G[2] = { 0xffffffff,
		     0xffffffff };
  u_int32_t FG[5] = { 0x00000000,
		      0x00000000,
		      0x00000000,
		      0x00000000,
		      0x00000000 };

  gf_mul (FG+2, &(FG[1]), &(FG[0]), 
	  2, F+1, F[0], 
	  1, G+1, G[0]);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", FG[i]);
  printf("\n"); 
}

void test_gf2m_2a() {
  u_int32_t F[3] = { 0x01234567, 
		     0xfedcba98,
		     0xf0f0f0f0 };
  u_int32_t G[2] = { 0x02468ace,
		     0xfdb97531 };
  u_int32_t FG[5] = { 0x00000000,
		      0x00000000,
		      0x00000000,
		      0x00000000,
		      0x00000000 };

  gf_mul (FG+2, &(FG[1]), &(FG[0]), 
	  2, F+1, F[0], 
	  1, G+1, G[0]);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", FG[i]);
  printf("\n"); 
} 

void test_gf2m_3a() {
  u_int32_t F[5] = { 0x89abcdef, 
		     0x01234567,
		     0x00000000,
		     0x00000000,
		     0x00000000 };
  
  gf_shl (5, F, 72);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", F[i]);
  printf("\n"); 
} 

void test_gf2m_3b() {
  u_int32_t F[5] = { 0x89abcdef, 
		     0x01234567,
		     0x00000000,
		     0x00000000,
		     0x00000000 };
  
  gf_shl (5, F, 73);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", F[i]);
  printf("\n"); 
} 

void test_gf2m_3c() {
  u_int32_t F[5] = { 0x89abcdef, 
		     0x01234567,
		     0x00000000,
		     0x00000000,
		     0x00000000 };
  
  gf_shl (5, F, 74);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", F[i]);
  printf("\n"); 
} 

void test_gf2m_3d() {
  u_int32_t F[5] = { 0x89abcdef, 
		     0x01234567,
		     0x00000000,
		     0x00000000,
		     0x00000000 };
  
  gf_shl (5, F, 75);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", F[i]);
  printf("\n"); 
} 

void test_gf2m_3e() {
  u_int32_t F[5] = { 0x89abcdef, 
		     0x01234567,
		     0x00000000,
		     0x00000000,
		     0x00000000 };
  
  gf_shl (5, F, 76);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", F[i]);
  printf("\n"); 
} 

void test_gf2m_3f() {
  u_int32_t F[5] = { 0x89abcdef, 
		     0x01234567,
		     0x00000000,
		     0x00000000,
		     0x00000000 };
  
  gf_shl (5, F, 77);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", F[i]);
  printf("\n"); 
} 

void test_gf2m_3g() {
  u_int32_t F[5] = { 0x89abcdef, 
		     0x01234567,
		     0x00000000,
		     0x00000000,
		     0x00000000 };
  
  gf_shl (5, F, 78);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", F[i]);
  printf("\n"); 
} 

void test_gf2m_3h() {
  u_int32_t F[5] = { 0x89abcdef, 
		     0x01234567,
		     0x00000000,
		     0x00000000,
		     0x00000000 };
  
  gf_shl (5, F, 79);

  int i;
  for (i=4; i>=0; i--)
    printf ("0x%08x\t", F[i]);
  printf("\n"); 
} 

int main (int argc, char**argv) {
  
  test_b163_1();
  test_b163_2();
  test_b163_3();
  test_b163_4();
  test_b163_5();
  test_b163_6();
  test_b163_7();
  test_b163_8();

  test_gf2m_1();
  test_gf2m_1a();
  test_gf2m_2();
  test_gf2m_2a();
  test_gf2m_2b();
  test_gf2m_2c();
  test_gf2m_2d();

  test_gf2m_3a();
  test_gf2m_3b();
  test_gf2m_3c();
  test_gf2m_3d();
  test_gf2m_3e();
  test_gf2m_3f();
  test_gf2m_3g();
  test_gf2m_3h();

  return 0;
}
