#include "async.h"
#include "icinghdr.h"
#include "icingsec.h"
#include "tt.h"
#include <cstdlib>

FILE *out = NULL;

enum { NUM_PATH_LENS = 6, TIMES_TO_RUN = 1000 };
uint32_t lens[NUM_PATH_LENS] = {3, 7, 12, 20, 30, 37};

void
generate_path(uint32_t len, RealmVoq *realms)
{
	for(uint32_t i = 0; i < len; i++)
	{
		realms[i].voq = 0x7667;

		for(uint32_t j = 0; j < RealmIDWire::SIZE; j++)
			realms[i].realmid.bytes[j] = rand();
	}
}

void
generate_secret(RealmSecret &s)
{
	for(uint32_t i = 0; i < RealmSecret::SIZE; i++)
		s.bytes[i] = rand();
}

void
build_poc(uint32_t len, uint64_t &min)
{
	RealmVoq *realms = new RealmVoq[len];
	generate_path(len, realms);
	RealmSecret s;
	generate_secret(s);

	POC poc;

	uint64_t start, end, delta;
	start = get_now();
	poc_from_path(poc, &s, reinterpret_cast<Path *>(realms), len);
	end = get_now();
	delta = end - start;

	if(out)
	{
		fprintf(out, "%d, %llu\n", len, delta);
		fflush(out);
	}

	if(min == 0 || delta < min)
		min = delta;

	delete[] realms;
}

int main(int argc, char **argv)
{
	srand((unsigned) time(NULL));
	if(argc > 1)
		out = fopen(argv[1], "w+");

	if(out)
	{
		fprintf(out, "#poc_create\n#path_len, latency (us)\n");
		fflush(out);
	}

	for(uint32_t i = 0; i < NUM_PATH_LENS; i++)
	{
		uint64_t min = 0;
		printf("Running experiment for length %u ... ", lens[i]);
		fflush(stdout);
		for(uint32_t j = 0; j < TIMES_TO_RUN; j++)
			build_poc(lens[i], min);
		printf("DONE: min = %llu\n", min);
		fflush(stdout);
	}
}
